
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity SinCos_Decoder_Inv is
port
(
   reset           : in  std_logic;
   clk             : in  std_logic;
   freq_multiplier : in  std_logic_vector(3 downto 0);
   sin_data        : out unsigned(15 downto 0);
   cos_data        : out unsigned(15 downto 0)
);
end SinCos_Decoder_Inv;


architecture Behavioral of SinCos_Decoder_Inv is

   signal theta_int      : unsigned(11 downto 0) := x"000";
   signal scaler         : unsigned(11 downto 0);
   signal sin_data_int   : unsigned(15 downto 0);
   signal cos_data_int   : unsigned(15 downto 0);

begin


scaler <= (unsigned(x"00" & freq_multiplier));


process(reset,clk)
begin
   if(reset = '1')then
       sin_data_int <= to_unsigned(0,16); 
       cos_data_int <= to_unsigned(0,16);
   elsif(rising_edge(clk)) then
    
     sin_data <= sin_data_int;
     cos_data <= cos_data_int;

     case theta_int is


when "000000000000" => sin_data_int <= to_unsigned(32760,16); cos_data_int <= to_unsigned(2760,16);
     when "000000000001" => sin_data_int <= to_unsigned(32714,16); cos_data_int <= to_unsigned(2760,16);
     when "000000000010" => sin_data_int <= to_unsigned(32668,16); cos_data_int <= to_unsigned(2760,16);
     when "000000000011" => sin_data_int <= to_unsigned(32622,16); cos_data_int <= to_unsigned(2760,16);
     when "000000000100" => sin_data_int <= to_unsigned(32576,16); cos_data_int <= to_unsigned(2761,16);
     when "000000000101" => sin_data_int <= to_unsigned(32530,16); cos_data_int <= to_unsigned(2761,16);
     when "000000000110" => sin_data_int <= to_unsigned(32484,16); cos_data_int <= to_unsigned(2761,16);
     when "000000000111" => sin_data_int <= to_unsigned(32438,16); cos_data_int <= to_unsigned(2762,16);
     when "000000001000" => sin_data_int <= to_unsigned(32392,16); cos_data_int <= to_unsigned(2762,16);
     when "000000001001" => sin_data_int <= to_unsigned(32346,16); cos_data_int <= to_unsigned(2763,16);
     when "000000001010" => sin_data_int <= to_unsigned(32300,16); cos_data_int <= to_unsigned(2764,16);
     when "000000001011" => sin_data_int <= to_unsigned(32254,16); cos_data_int <= to_unsigned(2764,16);
     when "000000001100" => sin_data_int <= to_unsigned(32208,16); cos_data_int <= to_unsigned(2765,16);
     when "000000001101" => sin_data_int <= to_unsigned(32162,16); cos_data_int <= to_unsigned(2766,16);
     when "000000001110" => sin_data_int <= to_unsigned(32116,16); cos_data_int <= to_unsigned(2767,16);
     when "000000001111" => sin_data_int <= to_unsigned(32070,16); cos_data_int <= to_unsigned(2768,16);
     when "000000010000" => sin_data_int <= to_unsigned(32024,16); cos_data_int <= to_unsigned(2769,16);
     when "000000010001" => sin_data_int <= to_unsigned(31978,16); cos_data_int <= to_unsigned(2770,16);
     when "000000010010" => sin_data_int <= to_unsigned(31932,16); cos_data_int <= to_unsigned(2771,16);
     when "000000010011" => sin_data_int <= to_unsigned(31886,16); cos_data_int <= to_unsigned(2773,16);
     when "000000010100" => sin_data_int <= to_unsigned(31840,16); cos_data_int <= to_unsigned(2774,16);
     when "000000010101" => sin_data_int <= to_unsigned(31794,16); cos_data_int <= to_unsigned(2776,16);
     when "000000010110" => sin_data_int <= to_unsigned(31748,16); cos_data_int <= to_unsigned(2777,16);
     when "000000010111" => sin_data_int <= to_unsigned(31702,16); cos_data_int <= to_unsigned(2779,16);
     when "000000011000" => sin_data_int <= to_unsigned(31656,16); cos_data_int <= to_unsigned(2780,16);
     when "000000011001" => sin_data_int <= to_unsigned(31610,16); cos_data_int <= to_unsigned(2782,16);
     when "000000011010" => sin_data_int <= to_unsigned(31564,16); cos_data_int <= to_unsigned(2784,16);
     when "000000011011" => sin_data_int <= to_unsigned(31518,16); cos_data_int <= to_unsigned(2786,16);
     when "000000011100" => sin_data_int <= to_unsigned(31472,16); cos_data_int <= to_unsigned(2788,16);
     when "000000011101" => sin_data_int <= to_unsigned(31426,16); cos_data_int <= to_unsigned(2790,16);
     when "000000011110" => sin_data_int <= to_unsigned(31380,16); cos_data_int <= to_unsigned(2792,16);
     when "000000011111" => sin_data_int <= to_unsigned(31334,16); cos_data_int <= to_unsigned(2794,16);
     when "000000100000" => sin_data_int <= to_unsigned(31288,16); cos_data_int <= to_unsigned(2796,16);
     when "000000100001" => sin_data_int <= to_unsigned(31242,16); cos_data_int <= to_unsigned(2798,16);
     when "000000100010" => sin_data_int <= to_unsigned(31196,16); cos_data_int <= to_unsigned(2801,16);
     when "000000100011" => sin_data_int <= to_unsigned(31150,16); cos_data_int <= to_unsigned(2803,16);
     when "000000100100" => sin_data_int <= to_unsigned(31104,16); cos_data_int <= to_unsigned(2806,16);
     when "000000100101" => sin_data_int <= to_unsigned(31058,16); cos_data_int <= to_unsigned(2808,16);
     when "000000100110" => sin_data_int <= to_unsigned(31012,16); cos_data_int <= to_unsigned(2811,16);
     when "000000100111" => sin_data_int <= to_unsigned(30966,16); cos_data_int <= to_unsigned(2814,16);
     when "000000101000" => sin_data_int <= to_unsigned(30920,16); cos_data_int <= to_unsigned(2816,16);
     when "000000101001" => sin_data_int <= to_unsigned(30874,16); cos_data_int <= to_unsigned(2819,16);
     when "000000101010" => sin_data_int <= to_unsigned(30829,16); cos_data_int <= to_unsigned(2822,16);
     when "000000101011" => sin_data_int <= to_unsigned(30783,16); cos_data_int <= to_unsigned(2825,16);
     when "000000101100" => sin_data_int <= to_unsigned(30737,16); cos_data_int <= to_unsigned(2828,16);
     when "000000101101" => sin_data_int <= to_unsigned(30691,16); cos_data_int <= to_unsigned(2831,16);
     when "000000101110" => sin_data_int <= to_unsigned(30645,16); cos_data_int <= to_unsigned(2835,16);
     when "000000101111" => sin_data_int <= to_unsigned(30599,16); cos_data_int <= to_unsigned(2838,16);
     when "000000110000" => sin_data_int <= to_unsigned(30553,16); cos_data_int <= to_unsigned(2841,16);
     when "000000110001" => sin_data_int <= to_unsigned(30507,16); cos_data_int <= to_unsigned(2845,16);
     when "000000110010" => sin_data_int <= to_unsigned(30461,16); cos_data_int <= to_unsigned(2848,16);
     when "000000110011" => sin_data_int <= to_unsigned(30415,16); cos_data_int <= to_unsigned(2852,16);
     when "000000110100" => sin_data_int <= to_unsigned(30370,16); cos_data_int <= to_unsigned(2855,16);
     when "000000110101" => sin_data_int <= to_unsigned(30324,16); cos_data_int <= to_unsigned(2859,16);
     when "000000110110" => sin_data_int <= to_unsigned(30278,16); cos_data_int <= to_unsigned(2863,16);
     when "000000110111" => sin_data_int <= to_unsigned(30232,16); cos_data_int <= to_unsigned(2867,16);
     when "000000111000" => sin_data_int <= to_unsigned(30186,16); cos_data_int <= to_unsigned(2871,16);
     when "000000111001" => sin_data_int <= to_unsigned(30140,16); cos_data_int <= to_unsigned(2875,16);
     when "000000111010" => sin_data_int <= to_unsigned(30094,16); cos_data_int <= to_unsigned(2879,16);
     when "000000111011" => sin_data_int <= to_unsigned(30049,16); cos_data_int <= to_unsigned(2883,16);
     when "000000111100" => sin_data_int <= to_unsigned(30003,16); cos_data_int <= to_unsigned(2887,16);
     when "000000111101" => sin_data_int <= to_unsigned(29957,16); cos_data_int <= to_unsigned(2891,16);
     when "000000111110" => sin_data_int <= to_unsigned(29911,16); cos_data_int <= to_unsigned(2896,16);
     when "000000111111" => sin_data_int <= to_unsigned(29865,16); cos_data_int <= to_unsigned(2900,16);
     when "000001000000" => sin_data_int <= to_unsigned(29819,16); cos_data_int <= to_unsigned(2904,16);
     when "000001000001" => sin_data_int <= to_unsigned(29774,16); cos_data_int <= to_unsigned(2909,16);
     when "000001000010" => sin_data_int <= to_unsigned(29728,16); cos_data_int <= to_unsigned(2914,16);
     when "000001000011" => sin_data_int <= to_unsigned(29682,16); cos_data_int <= to_unsigned(2918,16);
     when "000001000100" => sin_data_int <= to_unsigned(29636,16); cos_data_int <= to_unsigned(2923,16);
     when "000001000101" => sin_data_int <= to_unsigned(29591,16); cos_data_int <= to_unsigned(2928,16);
     when "000001000110" => sin_data_int <= to_unsigned(29545,16); cos_data_int <= to_unsigned(2933,16);
     when "000001000111" => sin_data_int <= to_unsigned(29499,16); cos_data_int <= to_unsigned(2938,16);
     when "000001001000" => sin_data_int <= to_unsigned(29453,16); cos_data_int <= to_unsigned(2943,16);
     when "000001001001" => sin_data_int <= to_unsigned(29408,16); cos_data_int <= to_unsigned(2948,16);
     when "000001001010" => sin_data_int <= to_unsigned(29362,16); cos_data_int <= to_unsigned(2953,16);
     when "000001001011" => sin_data_int <= to_unsigned(29316,16); cos_data_int <= to_unsigned(2958,16);
     when "000001001100" => sin_data_int <= to_unsigned(29270,16); cos_data_int <= to_unsigned(2964,16);
     when "000001001101" => sin_data_int <= to_unsigned(29225,16); cos_data_int <= to_unsigned(2969,16);
     when "000001001110" => sin_data_int <= to_unsigned(29179,16); cos_data_int <= to_unsigned(2974,16);
     when "000001001111" => sin_data_int <= to_unsigned(29133,16); cos_data_int <= to_unsigned(2980,16);
     when "000001010000" => sin_data_int <= to_unsigned(29088,16); cos_data_int <= to_unsigned(2986,16);
     when "000001010001" => sin_data_int <= to_unsigned(29042,16); cos_data_int <= to_unsigned(2991,16);
     when "000001010010" => sin_data_int <= to_unsigned(28996,16); cos_data_int <= to_unsigned(2997,16);
     when "000001010011" => sin_data_int <= to_unsigned(28951,16); cos_data_int <= to_unsigned(3003,16);
     when "000001010100" => sin_data_int <= to_unsigned(28905,16); cos_data_int <= to_unsigned(3009,16);
     when "000001010101" => sin_data_int <= to_unsigned(28859,16); cos_data_int <= to_unsigned(3015,16);
     when "000001010110" => sin_data_int <= to_unsigned(28814,16); cos_data_int <= to_unsigned(3021,16);
     when "000001010111" => sin_data_int <= to_unsigned(28768,16); cos_data_int <= to_unsigned(3027,16);
     when "000001011000" => sin_data_int <= to_unsigned(28723,16); cos_data_int <= to_unsigned(3033,16);
     when "000001011001" => sin_data_int <= to_unsigned(28677,16); cos_data_int <= to_unsigned(3039,16);
     when "000001011010" => sin_data_int <= to_unsigned(28631,16); cos_data_int <= to_unsigned(3045,16);
     when "000001011011" => sin_data_int <= to_unsigned(28586,16); cos_data_int <= to_unsigned(3052,16);
     when "000001011100" => sin_data_int <= to_unsigned(28540,16); cos_data_int <= to_unsigned(3058,16);
     when "000001011101" => sin_data_int <= to_unsigned(28495,16); cos_data_int <= to_unsigned(3065,16);
     when "000001011110" => sin_data_int <= to_unsigned(28449,16); cos_data_int <= to_unsigned(3071,16);
     when "000001011111" => sin_data_int <= to_unsigned(28404,16); cos_data_int <= to_unsigned(3078,16);
     when "000001100000" => sin_data_int <= to_unsigned(28358,16); cos_data_int <= to_unsigned(3085,16);
     when "000001100001" => sin_data_int <= to_unsigned(28313,16); cos_data_int <= to_unsigned(3091,16);
     when "000001100010" => sin_data_int <= to_unsigned(28267,16); cos_data_int <= to_unsigned(3098,16);
     when "000001100011" => sin_data_int <= to_unsigned(28222,16); cos_data_int <= to_unsigned(3105,16);
     when "000001100100" => sin_data_int <= to_unsigned(28176,16); cos_data_int <= to_unsigned(3112,16);
     when "000001100101" => sin_data_int <= to_unsigned(28131,16); cos_data_int <= to_unsigned(3119,16);
     when "000001100110" => sin_data_int <= to_unsigned(28085,16); cos_data_int <= to_unsigned(3126,16);
     when "000001100111" => sin_data_int <= to_unsigned(28040,16); cos_data_int <= to_unsigned(3134,16);
     when "000001101000" => sin_data_int <= to_unsigned(27994,16); cos_data_int <= to_unsigned(3141,16);
     when "000001101001" => sin_data_int <= to_unsigned(27949,16); cos_data_int <= to_unsigned(3148,16);
     when "000001101010" => sin_data_int <= to_unsigned(27903,16); cos_data_int <= to_unsigned(3156,16);
     when "000001101011" => sin_data_int <= to_unsigned(27858,16); cos_data_int <= to_unsigned(3163,16);
     when "000001101100" => sin_data_int <= to_unsigned(27813,16); cos_data_int <= to_unsigned(3171,16);
     when "000001101101" => sin_data_int <= to_unsigned(27767,16); cos_data_int <= to_unsigned(3178,16);
     when "000001101110" => sin_data_int <= to_unsigned(27722,16); cos_data_int <= to_unsigned(3186,16);
     when "000001101111" => sin_data_int <= to_unsigned(27676,16); cos_data_int <= to_unsigned(3194,16);
     when "000001110000" => sin_data_int <= to_unsigned(27631,16); cos_data_int <= to_unsigned(3202,16);
     when "000001110001" => sin_data_int <= to_unsigned(27586,16); cos_data_int <= to_unsigned(3210,16);
     when "000001110010" => sin_data_int <= to_unsigned(27540,16); cos_data_int <= to_unsigned(3218,16);
     when "000001110011" => sin_data_int <= to_unsigned(27495,16); cos_data_int <= to_unsigned(3226,16);
     when "000001110100" => sin_data_int <= to_unsigned(27450,16); cos_data_int <= to_unsigned(3234,16);
     when "000001110101" => sin_data_int <= to_unsigned(27405,16); cos_data_int <= to_unsigned(3242,16);
     when "000001110110" => sin_data_int <= to_unsigned(27359,16); cos_data_int <= to_unsigned(3250,16);
     when "000001110111" => sin_data_int <= to_unsigned(27314,16); cos_data_int <= to_unsigned(3258,16);
     when "000001111000" => sin_data_int <= to_unsigned(27269,16); cos_data_int <= to_unsigned(3267,16);
     when "000001111001" => sin_data_int <= to_unsigned(27224,16); cos_data_int <= to_unsigned(3275,16);
     when "000001111010" => sin_data_int <= to_unsigned(27178,16); cos_data_int <= to_unsigned(3284,16);
     when "000001111011" => sin_data_int <= to_unsigned(27133,16); cos_data_int <= to_unsigned(3292,16);
     when "000001111100" => sin_data_int <= to_unsigned(27088,16); cos_data_int <= to_unsigned(3301,16);
     when "000001111101" => sin_data_int <= to_unsigned(27043,16); cos_data_int <= to_unsigned(3310,16);
     when "000001111110" => sin_data_int <= to_unsigned(26998,16); cos_data_int <= to_unsigned(3319,16);
     when "000001111111" => sin_data_int <= to_unsigned(26952,16); cos_data_int <= to_unsigned(3327,16);
     when "000010000000" => sin_data_int <= to_unsigned(26907,16); cos_data_int <= to_unsigned(3336,16);
     when "000010000001" => sin_data_int <= to_unsigned(26862,16); cos_data_int <= to_unsigned(3345,16);
     when "000010000010" => sin_data_int <= to_unsigned(26817,16); cos_data_int <= to_unsigned(3355,16);
     when "000010000011" => sin_data_int <= to_unsigned(26772,16); cos_data_int <= to_unsigned(3364,16);
     when "000010000100" => sin_data_int <= to_unsigned(26727,16); cos_data_int <= to_unsigned(3373,16);
     when "000010000101" => sin_data_int <= to_unsigned(26682,16); cos_data_int <= to_unsigned(3382,16);
     when "000010000110" => sin_data_int <= to_unsigned(26637,16); cos_data_int <= to_unsigned(3392,16);
     when "000010000111" => sin_data_int <= to_unsigned(26592,16); cos_data_int <= to_unsigned(3401,16);
     when "000010001000" => sin_data_int <= to_unsigned(26547,16); cos_data_int <= to_unsigned(3410,16);
     when "000010001001" => sin_data_int <= to_unsigned(26502,16); cos_data_int <= to_unsigned(3420,16);
     when "000010001010" => sin_data_int <= to_unsigned(26457,16); cos_data_int <= to_unsigned(3430,16);
     when "000010001011" => sin_data_int <= to_unsigned(26412,16); cos_data_int <= to_unsigned(3439,16);
     when "000010001100" => sin_data_int <= to_unsigned(26367,16); cos_data_int <= to_unsigned(3449,16);
     when "000010001101" => sin_data_int <= to_unsigned(26322,16); cos_data_int <= to_unsigned(3459,16);
     when "000010001110" => sin_data_int <= to_unsigned(26277,16); cos_data_int <= to_unsigned(3469,16);
     when "000010001111" => sin_data_int <= to_unsigned(26232,16); cos_data_int <= to_unsigned(3479,16);
     when "000010010000" => sin_data_int <= to_unsigned(26187,16); cos_data_int <= to_unsigned(3489,16);
     when "000010010001" => sin_data_int <= to_unsigned(26142,16); cos_data_int <= to_unsigned(3499,16);
     when "000010010010" => sin_data_int <= to_unsigned(26097,16); cos_data_int <= to_unsigned(3509,16);
     when "000010010011" => sin_data_int <= to_unsigned(26052,16); cos_data_int <= to_unsigned(3519,16);
     when "000010010100" => sin_data_int <= to_unsigned(26007,16); cos_data_int <= to_unsigned(3530,16);
     when "000010010101" => sin_data_int <= to_unsigned(25963,16); cos_data_int <= to_unsigned(3540,16);
     when "000010010110" => sin_data_int <= to_unsigned(25918,16); cos_data_int <= to_unsigned(3551,16);
     when "000010010111" => sin_data_int <= to_unsigned(25873,16); cos_data_int <= to_unsigned(3561,16);
     when "000010011000" => sin_data_int <= to_unsigned(25828,16); cos_data_int <= to_unsigned(3572,16);
     when "000010011001" => sin_data_int <= to_unsigned(25783,16); cos_data_int <= to_unsigned(3582,16);
     when "000010011010" => sin_data_int <= to_unsigned(25739,16); cos_data_int <= to_unsigned(3593,16);
     when "000010011011" => sin_data_int <= to_unsigned(25694,16); cos_data_int <= to_unsigned(3604,16);
     when "000010011100" => sin_data_int <= to_unsigned(25649,16); cos_data_int <= to_unsigned(3615,16);
     when "000010011101" => sin_data_int <= to_unsigned(25605,16); cos_data_int <= to_unsigned(3626,16);
     when "000010011110" => sin_data_int <= to_unsigned(25560,16); cos_data_int <= to_unsigned(3637,16);
     when "000010011111" => sin_data_int <= to_unsigned(25515,16); cos_data_int <= to_unsigned(3648,16);
     when "000010100000" => sin_data_int <= to_unsigned(25471,16); cos_data_int <= to_unsigned(3659,16);
     when "000010100001" => sin_data_int <= to_unsigned(25426,16); cos_data_int <= to_unsigned(3670,16);
     when "000010100010" => sin_data_int <= to_unsigned(25381,16); cos_data_int <= to_unsigned(3682,16);
     when "000010100011" => sin_data_int <= to_unsigned(25337,16); cos_data_int <= to_unsigned(3693,16);
     when "000010100100" => sin_data_int <= to_unsigned(25292,16); cos_data_int <= to_unsigned(3704,16);
     when "000010100101" => sin_data_int <= to_unsigned(25248,16); cos_data_int <= to_unsigned(3716,16);
     when "000010100110" => sin_data_int <= to_unsigned(25203,16); cos_data_int <= to_unsigned(3727,16);
     when "000010100111" => sin_data_int <= to_unsigned(25159,16); cos_data_int <= to_unsigned(3739,16);
     when "000010101000" => sin_data_int <= to_unsigned(25114,16); cos_data_int <= to_unsigned(3751,16);
     when "000010101001" => sin_data_int <= to_unsigned(25070,16); cos_data_int <= to_unsigned(3762,16);
     when "000010101010" => sin_data_int <= to_unsigned(25025,16); cos_data_int <= to_unsigned(3774,16);
     when "000010101011" => sin_data_int <= to_unsigned(24981,16); cos_data_int <= to_unsigned(3786,16);
     when "000010101100" => sin_data_int <= to_unsigned(24936,16); cos_data_int <= to_unsigned(3798,16);
     when "000010101101" => sin_data_int <= to_unsigned(24892,16); cos_data_int <= to_unsigned(3810,16);
     when "000010101110" => sin_data_int <= to_unsigned(24847,16); cos_data_int <= to_unsigned(3822,16);
     when "000010101111" => sin_data_int <= to_unsigned(24803,16); cos_data_int <= to_unsigned(3834,16);
     when "000010110000" => sin_data_int <= to_unsigned(24759,16); cos_data_int <= to_unsigned(3847,16);
     when "000010110001" => sin_data_int <= to_unsigned(24714,16); cos_data_int <= to_unsigned(3859,16);
     when "000010110010" => sin_data_int <= to_unsigned(24670,16); cos_data_int <= to_unsigned(3871,16);
     when "000010110011" => sin_data_int <= to_unsigned(24626,16); cos_data_int <= to_unsigned(3884,16);
     when "000010110100" => sin_data_int <= to_unsigned(24581,16); cos_data_int <= to_unsigned(3896,16);
     when "000010110101" => sin_data_int <= to_unsigned(24537,16); cos_data_int <= to_unsigned(3909,16);
     when "000010110110" => sin_data_int <= to_unsigned(24493,16); cos_data_int <= to_unsigned(3922,16);
     when "000010110111" => sin_data_int <= to_unsigned(24449,16); cos_data_int <= to_unsigned(3934,16);
     when "000010111000" => sin_data_int <= to_unsigned(24404,16); cos_data_int <= to_unsigned(3947,16);
     when "000010111001" => sin_data_int <= to_unsigned(24360,16); cos_data_int <= to_unsigned(3960,16);
     when "000010111010" => sin_data_int <= to_unsigned(24316,16); cos_data_int <= to_unsigned(3973,16);
     when "000010111011" => sin_data_int <= to_unsigned(24272,16); cos_data_int <= to_unsigned(3986,16);
     when "000010111100" => sin_data_int <= to_unsigned(24228,16); cos_data_int <= to_unsigned(3999,16);
     when "000010111101" => sin_data_int <= to_unsigned(24184,16); cos_data_int <= to_unsigned(4012,16);
     when "000010111110" => sin_data_int <= to_unsigned(24140,16); cos_data_int <= to_unsigned(4025,16);
     when "000010111111" => sin_data_int <= to_unsigned(24096,16); cos_data_int <= to_unsigned(4038,16);
     when "000011000000" => sin_data_int <= to_unsigned(24051,16); cos_data_int <= to_unsigned(4052,16);
     when "000011000001" => sin_data_int <= to_unsigned(24007,16); cos_data_int <= to_unsigned(4065,16);
     when "000011000010" => sin_data_int <= to_unsigned(23963,16); cos_data_int <= to_unsigned(4079,16);
     when "000011000011" => sin_data_int <= to_unsigned(23919,16); cos_data_int <= to_unsigned(4092,16);
     when "000011000100" => sin_data_int <= to_unsigned(23875,16); cos_data_int <= to_unsigned(4106,16);
     when "000011000101" => sin_data_int <= to_unsigned(23832,16); cos_data_int <= to_unsigned(4119,16);
     when "000011000110" => sin_data_int <= to_unsigned(23788,16); cos_data_int <= to_unsigned(4133,16);
     when "000011000111" => sin_data_int <= to_unsigned(23744,16); cos_data_int <= to_unsigned(4147,16);
     when "000011001000" => sin_data_int <= to_unsigned(23700,16); cos_data_int <= to_unsigned(4161,16);
     when "000011001001" => sin_data_int <= to_unsigned(23656,16); cos_data_int <= to_unsigned(4175,16);
     when "000011001010" => sin_data_int <= to_unsigned(23612,16); cos_data_int <= to_unsigned(4189,16);
     when "000011001011" => sin_data_int <= to_unsigned(23568,16); cos_data_int <= to_unsigned(4203,16);
     when "000011001100" => sin_data_int <= to_unsigned(23525,16); cos_data_int <= to_unsigned(4217,16);
     when "000011001101" => sin_data_int <= to_unsigned(23481,16); cos_data_int <= to_unsigned(4231,16);
     when "000011001110" => sin_data_int <= to_unsigned(23437,16); cos_data_int <= to_unsigned(4245,16);
     when "000011001111" => sin_data_int <= to_unsigned(23393,16); cos_data_int <= to_unsigned(4260,16);
     when "000011010000" => sin_data_int <= to_unsigned(23350,16); cos_data_int <= to_unsigned(4274,16);
     when "000011010001" => sin_data_int <= to_unsigned(23306,16); cos_data_int <= to_unsigned(4289,16);
     when "000011010010" => sin_data_int <= to_unsigned(23262,16); cos_data_int <= to_unsigned(4303,16);
     when "000011010011" => sin_data_int <= to_unsigned(23219,16); cos_data_int <= to_unsigned(4318,16);
     when "000011010100" => sin_data_int <= to_unsigned(23175,16); cos_data_int <= to_unsigned(4332,16);
     when "000011010101" => sin_data_int <= to_unsigned(23131,16); cos_data_int <= to_unsigned(4347,16);
     when "000011010110" => sin_data_int <= to_unsigned(23088,16); cos_data_int <= to_unsigned(4362,16);
     when "000011010111" => sin_data_int <= to_unsigned(23044,16); cos_data_int <= to_unsigned(4377,16);
     when "000011011000" => sin_data_int <= to_unsigned(23001,16); cos_data_int <= to_unsigned(4392,16);
     when "000011011001" => sin_data_int <= to_unsigned(22957,16); cos_data_int <= to_unsigned(4407,16);
     when "000011011010" => sin_data_int <= to_unsigned(22914,16); cos_data_int <= to_unsigned(4422,16);
     when "000011011011" => sin_data_int <= to_unsigned(22870,16); cos_data_int <= to_unsigned(4437,16);
     when "000011011100" => sin_data_int <= to_unsigned(22827,16); cos_data_int <= to_unsigned(4452,16);
     when "000011011101" => sin_data_int <= to_unsigned(22783,16); cos_data_int <= to_unsigned(4467,16);
     when "000011011110" => sin_data_int <= to_unsigned(22740,16); cos_data_int <= to_unsigned(4483,16);
     when "000011011111" => sin_data_int <= to_unsigned(22697,16); cos_data_int <= to_unsigned(4498,16);
     when "000011100000" => sin_data_int <= to_unsigned(22653,16); cos_data_int <= to_unsigned(4514,16);
     when "000011100001" => sin_data_int <= to_unsigned(22610,16); cos_data_int <= to_unsigned(4529,16);
     when "000011100010" => sin_data_int <= to_unsigned(22567,16); cos_data_int <= to_unsigned(4545,16);
     when "000011100011" => sin_data_int <= to_unsigned(22523,16); cos_data_int <= to_unsigned(4560,16);
     when "000011100100" => sin_data_int <= to_unsigned(22480,16); cos_data_int <= to_unsigned(4576,16);
     when "000011100101" => sin_data_int <= to_unsigned(22437,16); cos_data_int <= to_unsigned(4592,16);
     when "000011100110" => sin_data_int <= to_unsigned(22394,16); cos_data_int <= to_unsigned(4608,16);
     when "000011100111" => sin_data_int <= to_unsigned(22351,16); cos_data_int <= to_unsigned(4624,16);
     when "000011101000" => sin_data_int <= to_unsigned(22307,16); cos_data_int <= to_unsigned(4640,16);
     when "000011101001" => sin_data_int <= to_unsigned(22264,16); cos_data_int <= to_unsigned(4656,16);
     when "000011101010" => sin_data_int <= to_unsigned(22221,16); cos_data_int <= to_unsigned(4672,16);
     when "000011101011" => sin_data_int <= to_unsigned(22178,16); cos_data_int <= to_unsigned(4688,16);
     when "000011101100" => sin_data_int <= to_unsigned(22135,16); cos_data_int <= to_unsigned(4704,16);
     when "000011101101" => sin_data_int <= to_unsigned(22092,16); cos_data_int <= to_unsigned(4721,16);
     when "000011101110" => sin_data_int <= to_unsigned(22049,16); cos_data_int <= to_unsigned(4737,16);
     when "000011101111" => sin_data_int <= to_unsigned(22006,16); cos_data_int <= to_unsigned(4754,16);
     when "000011110000" => sin_data_int <= to_unsigned(21963,16); cos_data_int <= to_unsigned(4770,16);
     when "000011110001" => sin_data_int <= to_unsigned(21920,16); cos_data_int <= to_unsigned(4787,16);
     when "000011110010" => sin_data_int <= to_unsigned(21877,16); cos_data_int <= to_unsigned(4803,16);
     when "000011110011" => sin_data_int <= to_unsigned(21834,16); cos_data_int <= to_unsigned(4820,16);
     when "000011110100" => sin_data_int <= to_unsigned(21792,16); cos_data_int <= to_unsigned(4837,16);
     when "000011110101" => sin_data_int <= to_unsigned(21749,16); cos_data_int <= to_unsigned(4854,16);
     when "000011110110" => sin_data_int <= to_unsigned(21706,16); cos_data_int <= to_unsigned(4871,16);
     when "000011110111" => sin_data_int <= to_unsigned(21663,16); cos_data_int <= to_unsigned(4888,16);
     when "000011111000" => sin_data_int <= to_unsigned(21620,16); cos_data_int <= to_unsigned(4905,16);
     when "000011111001" => sin_data_int <= to_unsigned(21578,16); cos_data_int <= to_unsigned(4922,16);
     when "000011111010" => sin_data_int <= to_unsigned(21535,16); cos_data_int <= to_unsigned(4939,16);
     when "000011111011" => sin_data_int <= to_unsigned(21492,16); cos_data_int <= to_unsigned(4956,16);
     when "000011111100" => sin_data_int <= to_unsigned(21450,16); cos_data_int <= to_unsigned(4974,16);
     when "000011111101" => sin_data_int <= to_unsigned(21407,16); cos_data_int <= to_unsigned(4991,16);
     when "000011111110" => sin_data_int <= to_unsigned(21365,16); cos_data_int <= to_unsigned(5009,16);
     when "000011111111" => sin_data_int <= to_unsigned(21322,16); cos_data_int <= to_unsigned(5026,16);
     when "000100000000" => sin_data_int <= to_unsigned(21279,16); cos_data_int <= to_unsigned(5044,16);
     when "000100000001" => sin_data_int <= to_unsigned(21237,16); cos_data_int <= to_unsigned(5061,16);
     when "000100000010" => sin_data_int <= to_unsigned(21195,16); cos_data_int <= to_unsigned(5079,16);
     when "000100000011" => sin_data_int <= to_unsigned(21152,16); cos_data_int <= to_unsigned(5097,16);
     when "000100000100" => sin_data_int <= to_unsigned(21110,16); cos_data_int <= to_unsigned(5115,16);
     when "000100000101" => sin_data_int <= to_unsigned(21067,16); cos_data_int <= to_unsigned(5132,16);
     when "000100000110" => sin_data_int <= to_unsigned(21025,16); cos_data_int <= to_unsigned(5150,16);
     when "000100000111" => sin_data_int <= to_unsigned(20983,16); cos_data_int <= to_unsigned(5168,16);
     when "000100001000" => sin_data_int <= to_unsigned(20940,16); cos_data_int <= to_unsigned(5187,16);
     when "000100001001" => sin_data_int <= to_unsigned(20898,16); cos_data_int <= to_unsigned(5205,16);
     when "000100001010" => sin_data_int <= to_unsigned(20856,16); cos_data_int <= to_unsigned(5223,16);
     when "000100001011" => sin_data_int <= to_unsigned(20813,16); cos_data_int <= to_unsigned(5241,16);
     when "000100001100" => sin_data_int <= to_unsigned(20771,16); cos_data_int <= to_unsigned(5260,16);
     when "000100001101" => sin_data_int <= to_unsigned(20729,16); cos_data_int <= to_unsigned(5278,16);
     when "000100001110" => sin_data_int <= to_unsigned(20687,16); cos_data_int <= to_unsigned(5297,16);
     when "000100001111" => sin_data_int <= to_unsigned(20645,16); cos_data_int <= to_unsigned(5315,16);
     when "000100010000" => sin_data_int <= to_unsigned(20603,16); cos_data_int <= to_unsigned(5334,16);
     when "000100010001" => sin_data_int <= to_unsigned(20561,16); cos_data_int <= to_unsigned(5352,16);
     when "000100010010" => sin_data_int <= to_unsigned(20519,16); cos_data_int <= to_unsigned(5371,16);
     when "000100010011" => sin_data_int <= to_unsigned(20477,16); cos_data_int <= to_unsigned(5390,16);
     when "000100010100" => sin_data_int <= to_unsigned(20435,16); cos_data_int <= to_unsigned(5409,16);
     when "000100010101" => sin_data_int <= to_unsigned(20393,16); cos_data_int <= to_unsigned(5428,16);
     when "000100010110" => sin_data_int <= to_unsigned(20351,16); cos_data_int <= to_unsigned(5447,16);
     when "000100010111" => sin_data_int <= to_unsigned(20309,16); cos_data_int <= to_unsigned(5466,16);
     when "000100011000" => sin_data_int <= to_unsigned(20267,16); cos_data_int <= to_unsigned(5485,16);
     when "000100011001" => sin_data_int <= to_unsigned(20225,16); cos_data_int <= to_unsigned(5504,16);
     when "000100011010" => sin_data_int <= to_unsigned(20183,16); cos_data_int <= to_unsigned(5523,16);
     when "000100011011" => sin_data_int <= to_unsigned(20142,16); cos_data_int <= to_unsigned(5543,16);
     when "000100011100" => sin_data_int <= to_unsigned(20100,16); cos_data_int <= to_unsigned(5562,16);
     when "000100011101" => sin_data_int <= to_unsigned(20058,16); cos_data_int <= to_unsigned(5582,16);
     when "000100011110" => sin_data_int <= to_unsigned(20017,16); cos_data_int <= to_unsigned(5601,16);
     when "000100011111" => sin_data_int <= to_unsigned(19975,16); cos_data_int <= to_unsigned(5621,16);
     when "000100100000" => sin_data_int <= to_unsigned(19933,16); cos_data_int <= to_unsigned(5640,16);
     when "000100100001" => sin_data_int <= to_unsigned(19892,16); cos_data_int <= to_unsigned(5660,16);
     when "000100100010" => sin_data_int <= to_unsigned(19850,16); cos_data_int <= to_unsigned(5680,16);
     when "000100100011" => sin_data_int <= to_unsigned(19809,16); cos_data_int <= to_unsigned(5700,16);
     when "000100100100" => sin_data_int <= to_unsigned(19767,16); cos_data_int <= to_unsigned(5720,16);
     when "000100100101" => sin_data_int <= to_unsigned(19726,16); cos_data_int <= to_unsigned(5739,16);
     when "000100100110" => sin_data_int <= to_unsigned(19684,16); cos_data_int <= to_unsigned(5760,16);
     when "000100100111" => sin_data_int <= to_unsigned(19643,16); cos_data_int <= to_unsigned(5780,16);
     when "000100101000" => sin_data_int <= to_unsigned(19602,16); cos_data_int <= to_unsigned(5800,16);
     when "000100101001" => sin_data_int <= to_unsigned(19560,16); cos_data_int <= to_unsigned(5820,16);
     when "000100101010" => sin_data_int <= to_unsigned(19519,16); cos_data_int <= to_unsigned(5840,16);
     when "000100101011" => sin_data_int <= to_unsigned(19478,16); cos_data_int <= to_unsigned(5861,16);
     when "000100101100" => sin_data_int <= to_unsigned(19436,16); cos_data_int <= to_unsigned(5881,16);
     when "000100101101" => sin_data_int <= to_unsigned(19395,16); cos_data_int <= to_unsigned(5901,16);
     when "000100101110" => sin_data_int <= to_unsigned(19354,16); cos_data_int <= to_unsigned(5922,16);
     when "000100101111" => sin_data_int <= to_unsigned(19313,16); cos_data_int <= to_unsigned(5943,16);
     when "000100110000" => sin_data_int <= to_unsigned(19272,16); cos_data_int <= to_unsigned(5963,16);
     when "000100110001" => sin_data_int <= to_unsigned(19231,16); cos_data_int <= to_unsigned(5984,16);
     when "000100110010" => sin_data_int <= to_unsigned(19190,16); cos_data_int <= to_unsigned(6005,16);
     when "000100110011" => sin_data_int <= to_unsigned(19148,16); cos_data_int <= to_unsigned(6026,16);
     when "000100110100" => sin_data_int <= to_unsigned(19107,16); cos_data_int <= to_unsigned(6047,16);
     when "000100110101" => sin_data_int <= to_unsigned(19067,16); cos_data_int <= to_unsigned(6068,16);
     when "000100110110" => sin_data_int <= to_unsigned(19026,16); cos_data_int <= to_unsigned(6089,16);
     when "000100110111" => sin_data_int <= to_unsigned(18985,16); cos_data_int <= to_unsigned(6110,16);
     when "000100111000" => sin_data_int <= to_unsigned(18944,16); cos_data_int <= to_unsigned(6131,16);
     when "000100111001" => sin_data_int <= to_unsigned(18903,16); cos_data_int <= to_unsigned(6152,16);
     when "000100111010" => sin_data_int <= to_unsigned(18862,16); cos_data_int <= to_unsigned(6173,16);
     when "000100111011" => sin_data_int <= to_unsigned(18821,16); cos_data_int <= to_unsigned(6195,16);
     when "000100111100" => sin_data_int <= to_unsigned(18781,16); cos_data_int <= to_unsigned(6216,16);
     when "000100111101" => sin_data_int <= to_unsigned(18740,16); cos_data_int <= to_unsigned(6238,16);
     when "000100111110" => sin_data_int <= to_unsigned(18699,16); cos_data_int <= to_unsigned(6259,16);
     when "000100111111" => sin_data_int <= to_unsigned(18659,16); cos_data_int <= to_unsigned(6281,16);
     when "000101000000" => sin_data_int <= to_unsigned(18618,16); cos_data_int <= to_unsigned(6302,16);
     when "000101000001" => sin_data_int <= to_unsigned(18578,16); cos_data_int <= to_unsigned(6324,16);
     when "000101000010" => sin_data_int <= to_unsigned(18537,16); cos_data_int <= to_unsigned(6346,16);
     when "000101000011" => sin_data_int <= to_unsigned(18496,16); cos_data_int <= to_unsigned(6368,16);
     when "000101000100" => sin_data_int <= to_unsigned(18456,16); cos_data_int <= to_unsigned(6390,16);
     when "000101000101" => sin_data_int <= to_unsigned(18416,16); cos_data_int <= to_unsigned(6412,16);
     when "000101000110" => sin_data_int <= to_unsigned(18375,16); cos_data_int <= to_unsigned(6434,16);
     when "000101000111" => sin_data_int <= to_unsigned(18335,16); cos_data_int <= to_unsigned(6456,16);
     when "000101001000" => sin_data_int <= to_unsigned(18294,16); cos_data_int <= to_unsigned(6478,16);
     when "000101001001" => sin_data_int <= to_unsigned(18254,16); cos_data_int <= to_unsigned(6500,16);
     when "000101001010" => sin_data_int <= to_unsigned(18214,16); cos_data_int <= to_unsigned(6522,16);
     when "000101001011" => sin_data_int <= to_unsigned(18174,16); cos_data_int <= to_unsigned(6545,16);
     when "000101001100" => sin_data_int <= to_unsigned(18133,16); cos_data_int <= to_unsigned(6567,16);
     when "000101001101" => sin_data_int <= to_unsigned(18093,16); cos_data_int <= to_unsigned(6590,16);
     when "000101001110" => sin_data_int <= to_unsigned(18053,16); cos_data_int <= to_unsigned(6612,16);
     when "000101001111" => sin_data_int <= to_unsigned(18013,16); cos_data_int <= to_unsigned(6635,16);
     when "000101010000" => sin_data_int <= to_unsigned(17973,16); cos_data_int <= to_unsigned(6657,16);
     when "000101010001" => sin_data_int <= to_unsigned(17933,16); cos_data_int <= to_unsigned(6680,16);
     when "000101010010" => sin_data_int <= to_unsigned(17893,16); cos_data_int <= to_unsigned(6703,16);
     when "000101010011" => sin_data_int <= to_unsigned(17853,16); cos_data_int <= to_unsigned(6726,16);
     when "000101010100" => sin_data_int <= to_unsigned(17813,16); cos_data_int <= to_unsigned(6749,16);
     when "000101010101" => sin_data_int <= to_unsigned(17773,16); cos_data_int <= to_unsigned(6772,16);
     when "000101010110" => sin_data_int <= to_unsigned(17733,16); cos_data_int <= to_unsigned(6795,16);
     when "000101010111" => sin_data_int <= to_unsigned(17694,16); cos_data_int <= to_unsigned(6818,16);
     when "000101011000" => sin_data_int <= to_unsigned(17654,16); cos_data_int <= to_unsigned(6841,16);
     when "000101011001" => sin_data_int <= to_unsigned(17614,16); cos_data_int <= to_unsigned(6864,16);
     when "000101011010" => sin_data_int <= to_unsigned(17574,16); cos_data_int <= to_unsigned(6887,16);
     when "000101011011" => sin_data_int <= to_unsigned(17535,16); cos_data_int <= to_unsigned(6911,16);
     when "000101011100" => sin_data_int <= to_unsigned(17495,16); cos_data_int <= to_unsigned(6934,16);
     when "000101011101" => sin_data_int <= to_unsigned(17455,16); cos_data_int <= to_unsigned(6957,16);
     when "000101011110" => sin_data_int <= to_unsigned(17416,16); cos_data_int <= to_unsigned(6981,16);
     when "000101011111" => sin_data_int <= to_unsigned(17376,16); cos_data_int <= to_unsigned(7005,16);
     when "000101100000" => sin_data_int <= to_unsigned(17337,16); cos_data_int <= to_unsigned(7028,16);
     when "000101100001" => sin_data_int <= to_unsigned(17297,16); cos_data_int <= to_unsigned(7052,16);
     when "000101100010" => sin_data_int <= to_unsigned(17258,16); cos_data_int <= to_unsigned(7076,16);
     when "000101100011" => sin_data_int <= to_unsigned(17219,16); cos_data_int <= to_unsigned(7099,16);
     when "000101100100" => sin_data_int <= to_unsigned(17179,16); cos_data_int <= to_unsigned(7123,16);
     when "000101100101" => sin_data_int <= to_unsigned(17140,16); cos_data_int <= to_unsigned(7147,16);
     when "000101100110" => sin_data_int <= to_unsigned(17101,16); cos_data_int <= to_unsigned(7171,16);
     when "000101100111" => sin_data_int <= to_unsigned(17062,16); cos_data_int <= to_unsigned(7195,16);
     when "000101101000" => sin_data_int <= to_unsigned(17022,16); cos_data_int <= to_unsigned(7219,16);
     when "000101101001" => sin_data_int <= to_unsigned(16983,16); cos_data_int <= to_unsigned(7244,16);
     when "000101101010" => sin_data_int <= to_unsigned(16944,16); cos_data_int <= to_unsigned(7268,16);
     when "000101101011" => sin_data_int <= to_unsigned(16905,16); cos_data_int <= to_unsigned(7292,16);
     when "000101101100" => sin_data_int <= to_unsigned(16866,16); cos_data_int <= to_unsigned(7316,16);
     when "000101101101" => sin_data_int <= to_unsigned(16827,16); cos_data_int <= to_unsigned(7341,16);
     when "000101101110" => sin_data_int <= to_unsigned(16788,16); cos_data_int <= to_unsigned(7365,16);
     when "000101101111" => sin_data_int <= to_unsigned(16749,16); cos_data_int <= to_unsigned(7390,16);
     when "000101110000" => sin_data_int <= to_unsigned(16710,16); cos_data_int <= to_unsigned(7414,16);
     when "000101110001" => sin_data_int <= to_unsigned(16671,16); cos_data_int <= to_unsigned(7439,16);
     when "000101110010" => sin_data_int <= to_unsigned(16632,16); cos_data_int <= to_unsigned(7464,16);
     when "000101110011" => sin_data_int <= to_unsigned(16594,16); cos_data_int <= to_unsigned(7489,16);
     when "000101110100" => sin_data_int <= to_unsigned(16555,16); cos_data_int <= to_unsigned(7513,16);
     when "000101110101" => sin_data_int <= to_unsigned(16516,16); cos_data_int <= to_unsigned(7538,16);
     when "000101110110" => sin_data_int <= to_unsigned(16477,16); cos_data_int <= to_unsigned(7563,16);
     when "000101110111" => sin_data_int <= to_unsigned(16439,16); cos_data_int <= to_unsigned(7588,16);
     when "000101111000" => sin_data_int <= to_unsigned(16400,16); cos_data_int <= to_unsigned(7613,16);
     when "000101111001" => sin_data_int <= to_unsigned(16362,16); cos_data_int <= to_unsigned(7638,16);
     when "000101111010" => sin_data_int <= to_unsigned(16323,16); cos_data_int <= to_unsigned(7664,16);
     when "000101111011" => sin_data_int <= to_unsigned(16285,16); cos_data_int <= to_unsigned(7689,16);
     when "000101111100" => sin_data_int <= to_unsigned(16246,16); cos_data_int <= to_unsigned(7714,16);
     when "000101111101" => sin_data_int <= to_unsigned(16208,16); cos_data_int <= to_unsigned(7739,16);
     when "000101111110" => sin_data_int <= to_unsigned(16169,16); cos_data_int <= to_unsigned(7765,16);
     when "000101111111" => sin_data_int <= to_unsigned(16131,16); cos_data_int <= to_unsigned(7790,16);
     when "000110000000" => sin_data_int <= to_unsigned(16093,16); cos_data_int <= to_unsigned(7816,16);
     when "000110000001" => sin_data_int <= to_unsigned(16055,16); cos_data_int <= to_unsigned(7842,16);
     when "000110000010" => sin_data_int <= to_unsigned(16016,16); cos_data_int <= to_unsigned(7867,16);
     when "000110000011" => sin_data_int <= to_unsigned(15978,16); cos_data_int <= to_unsigned(7893,16);
     when "000110000100" => sin_data_int <= to_unsigned(15940,16); cos_data_int <= to_unsigned(7919,16);
     when "000110000101" => sin_data_int <= to_unsigned(15902,16); cos_data_int <= to_unsigned(7944,16);
     when "000110000110" => sin_data_int <= to_unsigned(15864,16); cos_data_int <= to_unsigned(7970,16);
     when "000110000111" => sin_data_int <= to_unsigned(15826,16); cos_data_int <= to_unsigned(7996,16);
     when "000110001000" => sin_data_int <= to_unsigned(15788,16); cos_data_int <= to_unsigned(8022,16);
     when "000110001001" => sin_data_int <= to_unsigned(15750,16); cos_data_int <= to_unsigned(8048,16);
     when "000110001010" => sin_data_int <= to_unsigned(15712,16); cos_data_int <= to_unsigned(8075,16);
     when "000110001011" => sin_data_int <= to_unsigned(15674,16); cos_data_int <= to_unsigned(8101,16);
     when "000110001100" => sin_data_int <= to_unsigned(15637,16); cos_data_int <= to_unsigned(8127,16);
     when "000110001101" => sin_data_int <= to_unsigned(15599,16); cos_data_int <= to_unsigned(8153,16);
     when "000110001110" => sin_data_int <= to_unsigned(15561,16); cos_data_int <= to_unsigned(8180,16);
     when "000110001111" => sin_data_int <= to_unsigned(15523,16); cos_data_int <= to_unsigned(8206,16);
     when "000110010000" => sin_data_int <= to_unsigned(15486,16); cos_data_int <= to_unsigned(8232,16);
     when "000110010001" => sin_data_int <= to_unsigned(15448,16); cos_data_int <= to_unsigned(8259,16);
     when "000110010010" => sin_data_int <= to_unsigned(15411,16); cos_data_int <= to_unsigned(8286,16);
     when "000110010011" => sin_data_int <= to_unsigned(15373,16); cos_data_int <= to_unsigned(8312,16);
     when "000110010100" => sin_data_int <= to_unsigned(15336,16); cos_data_int <= to_unsigned(8339,16);
     when "000110010101" => sin_data_int <= to_unsigned(15298,16); cos_data_int <= to_unsigned(8366,16);
     when "000110010110" => sin_data_int <= to_unsigned(15261,16); cos_data_int <= to_unsigned(8392,16);
     when "000110010111" => sin_data_int <= to_unsigned(15223,16); cos_data_int <= to_unsigned(8419,16);
     when "000110011000" => sin_data_int <= to_unsigned(15186,16); cos_data_int <= to_unsigned(8446,16);
     when "000110011001" => sin_data_int <= to_unsigned(15149,16); cos_data_int <= to_unsigned(8473,16);
     when "000110011010" => sin_data_int <= to_unsigned(15112,16); cos_data_int <= to_unsigned(8500,16);
     when "000110011011" => sin_data_int <= to_unsigned(15074,16); cos_data_int <= to_unsigned(8527,16);
     when "000110011100" => sin_data_int <= to_unsigned(15037,16); cos_data_int <= to_unsigned(8555,16);
     when "000110011101" => sin_data_int <= to_unsigned(15000,16); cos_data_int <= to_unsigned(8582,16);
     when "000110011110" => sin_data_int <= to_unsigned(14963,16); cos_data_int <= to_unsigned(8609,16);
     when "000110011111" => sin_data_int <= to_unsigned(14926,16); cos_data_int <= to_unsigned(8636,16);
     when "000110100000" => sin_data_int <= to_unsigned(14889,16); cos_data_int <= to_unsigned(8664,16);
     when "000110100001" => sin_data_int <= to_unsigned(14852,16); cos_data_int <= to_unsigned(8691,16);
     when "000110100010" => sin_data_int <= to_unsigned(14815,16); cos_data_int <= to_unsigned(8719,16);
     when "000110100011" => sin_data_int <= to_unsigned(14778,16); cos_data_int <= to_unsigned(8746,16);
     when "000110100100" => sin_data_int <= to_unsigned(14742,16); cos_data_int <= to_unsigned(8774,16);
     when "000110100101" => sin_data_int <= to_unsigned(14705,16); cos_data_int <= to_unsigned(8802,16);
     when "000110100110" => sin_data_int <= to_unsigned(14668,16); cos_data_int <= to_unsigned(8829,16);
     when "000110100111" => sin_data_int <= to_unsigned(14631,16); cos_data_int <= to_unsigned(8857,16);
     when "000110101000" => sin_data_int <= to_unsigned(14595,16); cos_data_int <= to_unsigned(8885,16);
     when "000110101001" => sin_data_int <= to_unsigned(14558,16); cos_data_int <= to_unsigned(8913,16);
     when "000110101010" => sin_data_int <= to_unsigned(14522,16); cos_data_int <= to_unsigned(8941,16);
     when "000110101011" => sin_data_int <= to_unsigned(14485,16); cos_data_int <= to_unsigned(8969,16);
     when "000110101100" => sin_data_int <= to_unsigned(14449,16); cos_data_int <= to_unsigned(8997,16);
     when "000110101101" => sin_data_int <= to_unsigned(14412,16); cos_data_int <= to_unsigned(9025,16);
     when "000110101110" => sin_data_int <= to_unsigned(14376,16); cos_data_int <= to_unsigned(9053,16);
     when "000110101111" => sin_data_int <= to_unsigned(14339,16); cos_data_int <= to_unsigned(9081,16);
     when "000110110000" => sin_data_int <= to_unsigned(14303,16); cos_data_int <= to_unsigned(9110,16);
     when "000110110001" => sin_data_int <= to_unsigned(14267,16); cos_data_int <= to_unsigned(9138,16);
     when "000110110010" => sin_data_int <= to_unsigned(14231,16); cos_data_int <= to_unsigned(9166,16);
     when "000110110011" => sin_data_int <= to_unsigned(14194,16); cos_data_int <= to_unsigned(9195,16);
     when "000110110100" => sin_data_int <= to_unsigned(14158,16); cos_data_int <= to_unsigned(9223,16);
     when "000110110101" => sin_data_int <= to_unsigned(14122,16); cos_data_int <= to_unsigned(9252,16);
     when "000110110110" => sin_data_int <= to_unsigned(14086,16); cos_data_int <= to_unsigned(9280,16);
     when "000110110111" => sin_data_int <= to_unsigned(14050,16); cos_data_int <= to_unsigned(9309,16);
     when "000110111000" => sin_data_int <= to_unsigned(14014,16); cos_data_int <= to_unsigned(9338,16);
     when "000110111001" => sin_data_int <= to_unsigned(13978,16); cos_data_int <= to_unsigned(9367,16);
     when "000110111010" => sin_data_int <= to_unsigned(13942,16); cos_data_int <= to_unsigned(9396,16);
     when "000110111011" => sin_data_int <= to_unsigned(13907,16); cos_data_int <= to_unsigned(9424,16);
     when "000110111100" => sin_data_int <= to_unsigned(13871,16); cos_data_int <= to_unsigned(9453,16);
     when "000110111101" => sin_data_int <= to_unsigned(13835,16); cos_data_int <= to_unsigned(9482,16);
     when "000110111110" => sin_data_int <= to_unsigned(13799,16); cos_data_int <= to_unsigned(9511,16);
     when "000110111111" => sin_data_int <= to_unsigned(13764,16); cos_data_int <= to_unsigned(9541,16);
     when "000111000000" => sin_data_int <= to_unsigned(13728,16); cos_data_int <= to_unsigned(9570,16);
     when "000111000001" => sin_data_int <= to_unsigned(13693,16); cos_data_int <= to_unsigned(9599,16);
     when "000111000010" => sin_data_int <= to_unsigned(13657,16); cos_data_int <= to_unsigned(9628,16);
     when "000111000011" => sin_data_int <= to_unsigned(13622,16); cos_data_int <= to_unsigned(9658,16);
     when "000111000100" => sin_data_int <= to_unsigned(13586,16); cos_data_int <= to_unsigned(9687,16);
     when "000111000101" => sin_data_int <= to_unsigned(13551,16); cos_data_int <= to_unsigned(9716,16);
     when "000111000110" => sin_data_int <= to_unsigned(13516,16); cos_data_int <= to_unsigned(9746,16);
     when "000111000111" => sin_data_int <= to_unsigned(13480,16); cos_data_int <= to_unsigned(9775,16);
     when "000111001000" => sin_data_int <= to_unsigned(13445,16); cos_data_int <= to_unsigned(9805,16);
     when "000111001001" => sin_data_int <= to_unsigned(13410,16); cos_data_int <= to_unsigned(9835,16);
     when "000111001010" => sin_data_int <= to_unsigned(13375,16); cos_data_int <= to_unsigned(9864,16);
     when "000111001011" => sin_data_int <= to_unsigned(13340,16); cos_data_int <= to_unsigned(9894,16);
     when "000111001100" => sin_data_int <= to_unsigned(13305,16); cos_data_int <= to_unsigned(9924,16);
     when "000111001101" => sin_data_int <= to_unsigned(13270,16); cos_data_int <= to_unsigned(9954,16);
     when "000111001110" => sin_data_int <= to_unsigned(13235,16); cos_data_int <= to_unsigned(9984,16);
     when "000111001111" => sin_data_int <= to_unsigned(13200,16); cos_data_int <= to_unsigned(10014,16);
     when "000111010000" => sin_data_int <= to_unsigned(13165,16); cos_data_int <= to_unsigned(10044,16);
     when "000111010001" => sin_data_int <= to_unsigned(13130,16); cos_data_int <= to_unsigned(10074,16);
     when "000111010010" => sin_data_int <= to_unsigned(13095,16); cos_data_int <= to_unsigned(10104,16);
     when "000111010011" => sin_data_int <= to_unsigned(13060,16); cos_data_int <= to_unsigned(10134,16);
     when "000111010100" => sin_data_int <= to_unsigned(13026,16); cos_data_int <= to_unsigned(10164,16);
     when "000111010101" => sin_data_int <= to_unsigned(12991,16); cos_data_int <= to_unsigned(10195,16);
     when "000111010110" => sin_data_int <= to_unsigned(12957,16); cos_data_int <= to_unsigned(10225,16);
     when "000111010111" => sin_data_int <= to_unsigned(12922,16); cos_data_int <= to_unsigned(10255,16);
     when "000111011000" => sin_data_int <= to_unsigned(12888,16); cos_data_int <= to_unsigned(10286,16);
     when "000111011001" => sin_data_int <= to_unsigned(12853,16); cos_data_int <= to_unsigned(10316,16);
     when "000111011010" => sin_data_int <= to_unsigned(12819,16); cos_data_int <= to_unsigned(10347,16);
     when "000111011011" => sin_data_int <= to_unsigned(12784,16); cos_data_int <= to_unsigned(10378,16);
     when "000111011100" => sin_data_int <= to_unsigned(12750,16); cos_data_int <= to_unsigned(10408,16);
     when "000111011101" => sin_data_int <= to_unsigned(12716,16); cos_data_int <= to_unsigned(10439,16);
     when "000111011110" => sin_data_int <= to_unsigned(12682,16); cos_data_int <= to_unsigned(10470,16);
     when "000111011111" => sin_data_int <= to_unsigned(12647,16); cos_data_int <= to_unsigned(10501,16);
     when "000111100000" => sin_data_int <= to_unsigned(12613,16); cos_data_int <= to_unsigned(10531,16);
     when "000111100001" => sin_data_int <= to_unsigned(12579,16); cos_data_int <= to_unsigned(10562,16);
     when "000111100010" => sin_data_int <= to_unsigned(12545,16); cos_data_int <= to_unsigned(10593,16);
     when "000111100011" => sin_data_int <= to_unsigned(12511,16); cos_data_int <= to_unsigned(10624,16);
     when "000111100100" => sin_data_int <= to_unsigned(12477,16); cos_data_int <= to_unsigned(10656,16);
     when "000111100101" => sin_data_int <= to_unsigned(12443,16); cos_data_int <= to_unsigned(10687,16);
     when "000111100110" => sin_data_int <= to_unsigned(12409,16); cos_data_int <= to_unsigned(10718,16);
     when "000111100111" => sin_data_int <= to_unsigned(12376,16); cos_data_int <= to_unsigned(10749,16);
     when "000111101000" => sin_data_int <= to_unsigned(12342,16); cos_data_int <= to_unsigned(10780,16);
     when "000111101001" => sin_data_int <= to_unsigned(12308,16); cos_data_int <= to_unsigned(10812,16);
     when "000111101010" => sin_data_int <= to_unsigned(12275,16); cos_data_int <= to_unsigned(10843,16);
     when "000111101011" => sin_data_int <= to_unsigned(12241,16); cos_data_int <= to_unsigned(10875,16);
     when "000111101100" => sin_data_int <= to_unsigned(12207,16); cos_data_int <= to_unsigned(10906,16);
     when "000111101101" => sin_data_int <= to_unsigned(12174,16); cos_data_int <= to_unsigned(10938,16);
     when "000111101110" => sin_data_int <= to_unsigned(12141,16); cos_data_int <= to_unsigned(10969,16);
     when "000111101111" => sin_data_int <= to_unsigned(12107,16); cos_data_int <= to_unsigned(11001,16);
     when "000111110000" => sin_data_int <= to_unsigned(12074,16); cos_data_int <= to_unsigned(11033,16);
     when "000111110001" => sin_data_int <= to_unsigned(12040,16); cos_data_int <= to_unsigned(11064,16);
     when "000111110010" => sin_data_int <= to_unsigned(12007,16); cos_data_int <= to_unsigned(11096,16);
     when "000111110011" => sin_data_int <= to_unsigned(11974,16); cos_data_int <= to_unsigned(11128,16);
     when "000111110100" => sin_data_int <= to_unsigned(11941,16); cos_data_int <= to_unsigned(11160,16);
     when "000111110101" => sin_data_int <= to_unsigned(11908,16); cos_data_int <= to_unsigned(11192,16);
     when "000111110110" => sin_data_int <= to_unsigned(11875,16); cos_data_int <= to_unsigned(11224,16);
     when "000111110111" => sin_data_int <= to_unsigned(11842,16); cos_data_int <= to_unsigned(11256,16);
     when "000111111000" => sin_data_int <= to_unsigned(11809,16); cos_data_int <= to_unsigned(11288,16);
     when "000111111001" => sin_data_int <= to_unsigned(11776,16); cos_data_int <= to_unsigned(11320,16);
     when "000111111010" => sin_data_int <= to_unsigned(11743,16); cos_data_int <= to_unsigned(11352,16);
     when "000111111011" => sin_data_int <= to_unsigned(11710,16); cos_data_int <= to_unsigned(11385,16);
     when "000111111100" => sin_data_int <= to_unsigned(11677,16); cos_data_int <= to_unsigned(11417,16);
     when "000111111101" => sin_data_int <= to_unsigned(11645,16); cos_data_int <= to_unsigned(11449,16);
     when "000111111110" => sin_data_int <= to_unsigned(11612,16); cos_data_int <= to_unsigned(11482,16);
     when "000111111111" => sin_data_int <= to_unsigned(11579,16); cos_data_int <= to_unsigned(11514,16);
     when "001000000000" => sin_data_int <= to_unsigned(11547,16); cos_data_int <= to_unsigned(11547,16);
     when "001000000001" => sin_data_int <= to_unsigned(11514,16); cos_data_int <= to_unsigned(11579,16);
     when "001000000010" => sin_data_int <= to_unsigned(11482,16); cos_data_int <= to_unsigned(11612,16);
     when "001000000011" => sin_data_int <= to_unsigned(11449,16); cos_data_int <= to_unsigned(11645,16);
     when "001000000100" => sin_data_int <= to_unsigned(11417,16); cos_data_int <= to_unsigned(11677,16);
     when "001000000101" => sin_data_int <= to_unsigned(11385,16); cos_data_int <= to_unsigned(11710,16);
     when "001000000110" => sin_data_int <= to_unsigned(11352,16); cos_data_int <= to_unsigned(11743,16);
     when "001000000111" => sin_data_int <= to_unsigned(11320,16); cos_data_int <= to_unsigned(11776,16);
     when "001000001000" => sin_data_int <= to_unsigned(11288,16); cos_data_int <= to_unsigned(11809,16);
     when "001000001001" => sin_data_int <= to_unsigned(11256,16); cos_data_int <= to_unsigned(11842,16);
     when "001000001010" => sin_data_int <= to_unsigned(11224,16); cos_data_int <= to_unsigned(11875,16);
     when "001000001011" => sin_data_int <= to_unsigned(11192,16); cos_data_int <= to_unsigned(11908,16);
     when "001000001100" => sin_data_int <= to_unsigned(11160,16); cos_data_int <= to_unsigned(11941,16);
     when "001000001101" => sin_data_int <= to_unsigned(11128,16); cos_data_int <= to_unsigned(11974,16);
     when "001000001110" => sin_data_int <= to_unsigned(11096,16); cos_data_int <= to_unsigned(12007,16);
     when "001000001111" => sin_data_int <= to_unsigned(11064,16); cos_data_int <= to_unsigned(12040,16);
     when "001000010000" => sin_data_int <= to_unsigned(11033,16); cos_data_int <= to_unsigned(12074,16);
     when "001000010001" => sin_data_int <= to_unsigned(11001,16); cos_data_int <= to_unsigned(12107,16);
     when "001000010010" => sin_data_int <= to_unsigned(10969,16); cos_data_int <= to_unsigned(12141,16);
     when "001000010011" => sin_data_int <= to_unsigned(10938,16); cos_data_int <= to_unsigned(12174,16);
     when "001000010100" => sin_data_int <= to_unsigned(10906,16); cos_data_int <= to_unsigned(12207,16);
     when "001000010101" => sin_data_int <= to_unsigned(10875,16); cos_data_int <= to_unsigned(12241,16);
     when "001000010110" => sin_data_int <= to_unsigned(10843,16); cos_data_int <= to_unsigned(12275,16);
     when "001000010111" => sin_data_int <= to_unsigned(10812,16); cos_data_int <= to_unsigned(12308,16);
     when "001000011000" => sin_data_int <= to_unsigned(10780,16); cos_data_int <= to_unsigned(12342,16);
     when "001000011001" => sin_data_int <= to_unsigned(10749,16); cos_data_int <= to_unsigned(12376,16);
     when "001000011010" => sin_data_int <= to_unsigned(10718,16); cos_data_int <= to_unsigned(12409,16);
     when "001000011011" => sin_data_int <= to_unsigned(10687,16); cos_data_int <= to_unsigned(12443,16);
     when "001000011100" => sin_data_int <= to_unsigned(10656,16); cos_data_int <= to_unsigned(12477,16);
     when "001000011101" => sin_data_int <= to_unsigned(10624,16); cos_data_int <= to_unsigned(12511,16);
     when "001000011110" => sin_data_int <= to_unsigned(10593,16); cos_data_int <= to_unsigned(12545,16);
     when "001000011111" => sin_data_int <= to_unsigned(10562,16); cos_data_int <= to_unsigned(12579,16);
     when "001000100000" => sin_data_int <= to_unsigned(10531,16); cos_data_int <= to_unsigned(12613,16);
     when "001000100001" => sin_data_int <= to_unsigned(10501,16); cos_data_int <= to_unsigned(12647,16);
     when "001000100010" => sin_data_int <= to_unsigned(10470,16); cos_data_int <= to_unsigned(12682,16);
     when "001000100011" => sin_data_int <= to_unsigned(10439,16); cos_data_int <= to_unsigned(12716,16);
     when "001000100100" => sin_data_int <= to_unsigned(10408,16); cos_data_int <= to_unsigned(12750,16);
     when "001000100101" => sin_data_int <= to_unsigned(10378,16); cos_data_int <= to_unsigned(12784,16);
     when "001000100110" => sin_data_int <= to_unsigned(10347,16); cos_data_int <= to_unsigned(12819,16);
     when "001000100111" => sin_data_int <= to_unsigned(10316,16); cos_data_int <= to_unsigned(12853,16);
     when "001000101000" => sin_data_int <= to_unsigned(10286,16); cos_data_int <= to_unsigned(12888,16);
     when "001000101001" => sin_data_int <= to_unsigned(10255,16); cos_data_int <= to_unsigned(12922,16);
     when "001000101010" => sin_data_int <= to_unsigned(10225,16); cos_data_int <= to_unsigned(12957,16);
     when "001000101011" => sin_data_int <= to_unsigned(10195,16); cos_data_int <= to_unsigned(12991,16);
     when "001000101100" => sin_data_int <= to_unsigned(10164,16); cos_data_int <= to_unsigned(13026,16);
     when "001000101101" => sin_data_int <= to_unsigned(10134,16); cos_data_int <= to_unsigned(13060,16);
     when "001000101110" => sin_data_int <= to_unsigned(10104,16); cos_data_int <= to_unsigned(13095,16);
     when "001000101111" => sin_data_int <= to_unsigned(10074,16); cos_data_int <= to_unsigned(13130,16);
     when "001000110000" => sin_data_int <= to_unsigned(10044,16); cos_data_int <= to_unsigned(13165,16);
     when "001000110001" => sin_data_int <= to_unsigned(10014,16); cos_data_int <= to_unsigned(13200,16);
     when "001000110010" => sin_data_int <= to_unsigned(9984,16); cos_data_int <= to_unsigned(13235,16);
     when "001000110011" => sin_data_int <= to_unsigned(9954,16); cos_data_int <= to_unsigned(13270,16);
     when "001000110100" => sin_data_int <= to_unsigned(9924,16); cos_data_int <= to_unsigned(13305,16);
     when "001000110101" => sin_data_int <= to_unsigned(9894,16); cos_data_int <= to_unsigned(13340,16);
     when "001000110110" => sin_data_int <= to_unsigned(9864,16); cos_data_int <= to_unsigned(13375,16);
     when "001000110111" => sin_data_int <= to_unsigned(9835,16); cos_data_int <= to_unsigned(13410,16);
     when "001000111000" => sin_data_int <= to_unsigned(9805,16); cos_data_int <= to_unsigned(13445,16);
     when "001000111001" => sin_data_int <= to_unsigned(9775,16); cos_data_int <= to_unsigned(13480,16);
     when "001000111010" => sin_data_int <= to_unsigned(9746,16); cos_data_int <= to_unsigned(13516,16);
     when "001000111011" => sin_data_int <= to_unsigned(9716,16); cos_data_int <= to_unsigned(13551,16);
     when "001000111100" => sin_data_int <= to_unsigned(9687,16); cos_data_int <= to_unsigned(13586,16);
     when "001000111101" => sin_data_int <= to_unsigned(9658,16); cos_data_int <= to_unsigned(13622,16);
     when "001000111110" => sin_data_int <= to_unsigned(9628,16); cos_data_int <= to_unsigned(13657,16);
     when "001000111111" => sin_data_int <= to_unsigned(9599,16); cos_data_int <= to_unsigned(13693,16);
     when "001001000000" => sin_data_int <= to_unsigned(9570,16); cos_data_int <= to_unsigned(13728,16);
     when "001001000001" => sin_data_int <= to_unsigned(9541,16); cos_data_int <= to_unsigned(13764,16);
     when "001001000010" => sin_data_int <= to_unsigned(9511,16); cos_data_int <= to_unsigned(13799,16);
     when "001001000011" => sin_data_int <= to_unsigned(9482,16); cos_data_int <= to_unsigned(13835,16);
     when "001001000100" => sin_data_int <= to_unsigned(9453,16); cos_data_int <= to_unsigned(13871,16);
     when "001001000101" => sin_data_int <= to_unsigned(9424,16); cos_data_int <= to_unsigned(13907,16);
     when "001001000110" => sin_data_int <= to_unsigned(9396,16); cos_data_int <= to_unsigned(13942,16);
     when "001001000111" => sin_data_int <= to_unsigned(9367,16); cos_data_int <= to_unsigned(13978,16);
     when "001001001000" => sin_data_int <= to_unsigned(9338,16); cos_data_int <= to_unsigned(14014,16);
     when "001001001001" => sin_data_int <= to_unsigned(9309,16); cos_data_int <= to_unsigned(14050,16);
     when "001001001010" => sin_data_int <= to_unsigned(9280,16); cos_data_int <= to_unsigned(14086,16);
     when "001001001011" => sin_data_int <= to_unsigned(9252,16); cos_data_int <= to_unsigned(14122,16);
     when "001001001100" => sin_data_int <= to_unsigned(9223,16); cos_data_int <= to_unsigned(14158,16);
     when "001001001101" => sin_data_int <= to_unsigned(9195,16); cos_data_int <= to_unsigned(14194,16);
     when "001001001110" => sin_data_int <= to_unsigned(9166,16); cos_data_int <= to_unsigned(14231,16);
     when "001001001111" => sin_data_int <= to_unsigned(9138,16); cos_data_int <= to_unsigned(14267,16);
     when "001001010000" => sin_data_int <= to_unsigned(9110,16); cos_data_int <= to_unsigned(14303,16);
     when "001001010001" => sin_data_int <= to_unsigned(9081,16); cos_data_int <= to_unsigned(14339,16);
     when "001001010010" => sin_data_int <= to_unsigned(9053,16); cos_data_int <= to_unsigned(14376,16);
     when "001001010011" => sin_data_int <= to_unsigned(9025,16); cos_data_int <= to_unsigned(14412,16);
     when "001001010100" => sin_data_int <= to_unsigned(8997,16); cos_data_int <= to_unsigned(14449,16);
     when "001001010101" => sin_data_int <= to_unsigned(8969,16); cos_data_int <= to_unsigned(14485,16);
     when "001001010110" => sin_data_int <= to_unsigned(8941,16); cos_data_int <= to_unsigned(14522,16);
     when "001001010111" => sin_data_int <= to_unsigned(8913,16); cos_data_int <= to_unsigned(14558,16);
     when "001001011000" => sin_data_int <= to_unsigned(8885,16); cos_data_int <= to_unsigned(14595,16);
     when "001001011001" => sin_data_int <= to_unsigned(8857,16); cos_data_int <= to_unsigned(14631,16);
     when "001001011010" => sin_data_int <= to_unsigned(8829,16); cos_data_int <= to_unsigned(14668,16);
     when "001001011011" => sin_data_int <= to_unsigned(8802,16); cos_data_int <= to_unsigned(14705,16);
     when "001001011100" => sin_data_int <= to_unsigned(8774,16); cos_data_int <= to_unsigned(14742,16);
     when "001001011101" => sin_data_int <= to_unsigned(8746,16); cos_data_int <= to_unsigned(14778,16);
     when "001001011110" => sin_data_int <= to_unsigned(8719,16); cos_data_int <= to_unsigned(14815,16);
     when "001001011111" => sin_data_int <= to_unsigned(8691,16); cos_data_int <= to_unsigned(14852,16);
     when "001001100000" => sin_data_int <= to_unsigned(8664,16); cos_data_int <= to_unsigned(14889,16);
     when "001001100001" => sin_data_int <= to_unsigned(8636,16); cos_data_int <= to_unsigned(14926,16);
     when "001001100010" => sin_data_int <= to_unsigned(8609,16); cos_data_int <= to_unsigned(14963,16);
     when "001001100011" => sin_data_int <= to_unsigned(8582,16); cos_data_int <= to_unsigned(15000,16);
     when "001001100100" => sin_data_int <= to_unsigned(8555,16); cos_data_int <= to_unsigned(15037,16);
     when "001001100101" => sin_data_int <= to_unsigned(8527,16); cos_data_int <= to_unsigned(15074,16);
     when "001001100110" => sin_data_int <= to_unsigned(8500,16); cos_data_int <= to_unsigned(15112,16);
     when "001001100111" => sin_data_int <= to_unsigned(8473,16); cos_data_int <= to_unsigned(15149,16);
     when "001001101000" => sin_data_int <= to_unsigned(8446,16); cos_data_int <= to_unsigned(15186,16);
     when "001001101001" => sin_data_int <= to_unsigned(8419,16); cos_data_int <= to_unsigned(15223,16);
     when "001001101010" => sin_data_int <= to_unsigned(8392,16); cos_data_int <= to_unsigned(15261,16);
     when "001001101011" => sin_data_int <= to_unsigned(8366,16); cos_data_int <= to_unsigned(15298,16);
     when "001001101100" => sin_data_int <= to_unsigned(8339,16); cos_data_int <= to_unsigned(15336,16);
     when "001001101101" => sin_data_int <= to_unsigned(8312,16); cos_data_int <= to_unsigned(15373,16);
     when "001001101110" => sin_data_int <= to_unsigned(8286,16); cos_data_int <= to_unsigned(15411,16);
     when "001001101111" => sin_data_int <= to_unsigned(8259,16); cos_data_int <= to_unsigned(15448,16);
     when "001001110000" => sin_data_int <= to_unsigned(8232,16); cos_data_int <= to_unsigned(15486,16);
     when "001001110001" => sin_data_int <= to_unsigned(8206,16); cos_data_int <= to_unsigned(15523,16);
     when "001001110010" => sin_data_int <= to_unsigned(8180,16); cos_data_int <= to_unsigned(15561,16);
     when "001001110011" => sin_data_int <= to_unsigned(8153,16); cos_data_int <= to_unsigned(15599,16);
     when "001001110100" => sin_data_int <= to_unsigned(8127,16); cos_data_int <= to_unsigned(15637,16);
     when "001001110101" => sin_data_int <= to_unsigned(8101,16); cos_data_int <= to_unsigned(15674,16);
     when "001001110110" => sin_data_int <= to_unsigned(8075,16); cos_data_int <= to_unsigned(15712,16);
     when "001001110111" => sin_data_int <= to_unsigned(8048,16); cos_data_int <= to_unsigned(15750,16);
     when "001001111000" => sin_data_int <= to_unsigned(8022,16); cos_data_int <= to_unsigned(15788,16);
     when "001001111001" => sin_data_int <= to_unsigned(7996,16); cos_data_int <= to_unsigned(15826,16);
     when "001001111010" => sin_data_int <= to_unsigned(7970,16); cos_data_int <= to_unsigned(15864,16);
     when "001001111011" => sin_data_int <= to_unsigned(7944,16); cos_data_int <= to_unsigned(15902,16);
     when "001001111100" => sin_data_int <= to_unsigned(7919,16); cos_data_int <= to_unsigned(15940,16);
     when "001001111101" => sin_data_int <= to_unsigned(7893,16); cos_data_int <= to_unsigned(15978,16);
     when "001001111110" => sin_data_int <= to_unsigned(7867,16); cos_data_int <= to_unsigned(16016,16);
     when "001001111111" => sin_data_int <= to_unsigned(7842,16); cos_data_int <= to_unsigned(16055,16);
     when "001010000000" => sin_data_int <= to_unsigned(7816,16); cos_data_int <= to_unsigned(16093,16);
     when "001010000001" => sin_data_int <= to_unsigned(7790,16); cos_data_int <= to_unsigned(16131,16);
     when "001010000010" => sin_data_int <= to_unsigned(7765,16); cos_data_int <= to_unsigned(16169,16);
     when "001010000011" => sin_data_int <= to_unsigned(7739,16); cos_data_int <= to_unsigned(16208,16);
     when "001010000100" => sin_data_int <= to_unsigned(7714,16); cos_data_int <= to_unsigned(16246,16);
     when "001010000101" => sin_data_int <= to_unsigned(7689,16); cos_data_int <= to_unsigned(16285,16);
     when "001010000110" => sin_data_int <= to_unsigned(7664,16); cos_data_int <= to_unsigned(16323,16);
     when "001010000111" => sin_data_int <= to_unsigned(7638,16); cos_data_int <= to_unsigned(16362,16);
     when "001010001000" => sin_data_int <= to_unsigned(7613,16); cos_data_int <= to_unsigned(16400,16);
     when "001010001001" => sin_data_int <= to_unsigned(7588,16); cos_data_int <= to_unsigned(16439,16);
     when "001010001010" => sin_data_int <= to_unsigned(7563,16); cos_data_int <= to_unsigned(16477,16);
     when "001010001011" => sin_data_int <= to_unsigned(7538,16); cos_data_int <= to_unsigned(16516,16);
     when "001010001100" => sin_data_int <= to_unsigned(7513,16); cos_data_int <= to_unsigned(16555,16);
     when "001010001101" => sin_data_int <= to_unsigned(7489,16); cos_data_int <= to_unsigned(16594,16);
     when "001010001110" => sin_data_int <= to_unsigned(7464,16); cos_data_int <= to_unsigned(16632,16);
     when "001010001111" => sin_data_int <= to_unsigned(7439,16); cos_data_int <= to_unsigned(16671,16);
     when "001010010000" => sin_data_int <= to_unsigned(7414,16); cos_data_int <= to_unsigned(16710,16);
     when "001010010001" => sin_data_int <= to_unsigned(7390,16); cos_data_int <= to_unsigned(16749,16);
     when "001010010010" => sin_data_int <= to_unsigned(7365,16); cos_data_int <= to_unsigned(16788,16);
     when "001010010011" => sin_data_int <= to_unsigned(7341,16); cos_data_int <= to_unsigned(16827,16);
     when "001010010100" => sin_data_int <= to_unsigned(7316,16); cos_data_int <= to_unsigned(16866,16);
     when "001010010101" => sin_data_int <= to_unsigned(7292,16); cos_data_int <= to_unsigned(16905,16);
     when "001010010110" => sin_data_int <= to_unsigned(7268,16); cos_data_int <= to_unsigned(16944,16);
     when "001010010111" => sin_data_int <= to_unsigned(7244,16); cos_data_int <= to_unsigned(16983,16);
     when "001010011000" => sin_data_int <= to_unsigned(7219,16); cos_data_int <= to_unsigned(17022,16);
     when "001010011001" => sin_data_int <= to_unsigned(7195,16); cos_data_int <= to_unsigned(17062,16);
     when "001010011010" => sin_data_int <= to_unsigned(7171,16); cos_data_int <= to_unsigned(17101,16);
     when "001010011011" => sin_data_int <= to_unsigned(7147,16); cos_data_int <= to_unsigned(17140,16);
     when "001010011100" => sin_data_int <= to_unsigned(7123,16); cos_data_int <= to_unsigned(17179,16);
     when "001010011101" => sin_data_int <= to_unsigned(7099,16); cos_data_int <= to_unsigned(17219,16);
     when "001010011110" => sin_data_int <= to_unsigned(7076,16); cos_data_int <= to_unsigned(17258,16);
     when "001010011111" => sin_data_int <= to_unsigned(7052,16); cos_data_int <= to_unsigned(17297,16);
     when "001010100000" => sin_data_int <= to_unsigned(7028,16); cos_data_int <= to_unsigned(17337,16);
     when "001010100001" => sin_data_int <= to_unsigned(7005,16); cos_data_int <= to_unsigned(17376,16);
     when "001010100010" => sin_data_int <= to_unsigned(6981,16); cos_data_int <= to_unsigned(17416,16);
     when "001010100011" => sin_data_int <= to_unsigned(6957,16); cos_data_int <= to_unsigned(17455,16);
     when "001010100100" => sin_data_int <= to_unsigned(6934,16); cos_data_int <= to_unsigned(17495,16);
     when "001010100101" => sin_data_int <= to_unsigned(6911,16); cos_data_int <= to_unsigned(17535,16);
     when "001010100110" => sin_data_int <= to_unsigned(6887,16); cos_data_int <= to_unsigned(17574,16);
     when "001010100111" => sin_data_int <= to_unsigned(6864,16); cos_data_int <= to_unsigned(17614,16);
     when "001010101000" => sin_data_int <= to_unsigned(6841,16); cos_data_int <= to_unsigned(17654,16);
     when "001010101001" => sin_data_int <= to_unsigned(6818,16); cos_data_int <= to_unsigned(17694,16);
     when "001010101010" => sin_data_int <= to_unsigned(6795,16); cos_data_int <= to_unsigned(17733,16);
     when "001010101011" => sin_data_int <= to_unsigned(6772,16); cos_data_int <= to_unsigned(17773,16);
     when "001010101100" => sin_data_int <= to_unsigned(6749,16); cos_data_int <= to_unsigned(17813,16);
     when "001010101101" => sin_data_int <= to_unsigned(6726,16); cos_data_int <= to_unsigned(17853,16);
     when "001010101110" => sin_data_int <= to_unsigned(6703,16); cos_data_int <= to_unsigned(17893,16);
     when "001010101111" => sin_data_int <= to_unsigned(6680,16); cos_data_int <= to_unsigned(17933,16);
     when "001010110000" => sin_data_int <= to_unsigned(6657,16); cos_data_int <= to_unsigned(17973,16);
     when "001010110001" => sin_data_int <= to_unsigned(6635,16); cos_data_int <= to_unsigned(18013,16);
     when "001010110010" => sin_data_int <= to_unsigned(6612,16); cos_data_int <= to_unsigned(18053,16);
     when "001010110011" => sin_data_int <= to_unsigned(6590,16); cos_data_int <= to_unsigned(18093,16);
     when "001010110100" => sin_data_int <= to_unsigned(6567,16); cos_data_int <= to_unsigned(18133,16);
     when "001010110101" => sin_data_int <= to_unsigned(6545,16); cos_data_int <= to_unsigned(18174,16);
     when "001010110110" => sin_data_int <= to_unsigned(6522,16); cos_data_int <= to_unsigned(18214,16);
     when "001010110111" => sin_data_int <= to_unsigned(6500,16); cos_data_int <= to_unsigned(18254,16);
     when "001010111000" => sin_data_int <= to_unsigned(6478,16); cos_data_int <= to_unsigned(18294,16);
     when "001010111001" => sin_data_int <= to_unsigned(6456,16); cos_data_int <= to_unsigned(18335,16);
     when "001010111010" => sin_data_int <= to_unsigned(6434,16); cos_data_int <= to_unsigned(18375,16);
     when "001010111011" => sin_data_int <= to_unsigned(6412,16); cos_data_int <= to_unsigned(18416,16);
     when "001010111100" => sin_data_int <= to_unsigned(6390,16); cos_data_int <= to_unsigned(18456,16);
     when "001010111101" => sin_data_int <= to_unsigned(6368,16); cos_data_int <= to_unsigned(18496,16);
     when "001010111110" => sin_data_int <= to_unsigned(6346,16); cos_data_int <= to_unsigned(18537,16);
     when "001010111111" => sin_data_int <= to_unsigned(6324,16); cos_data_int <= to_unsigned(18578,16);
     when "001011000000" => sin_data_int <= to_unsigned(6302,16); cos_data_int <= to_unsigned(18618,16);
     when "001011000001" => sin_data_int <= to_unsigned(6281,16); cos_data_int <= to_unsigned(18659,16);
     when "001011000010" => sin_data_int <= to_unsigned(6259,16); cos_data_int <= to_unsigned(18699,16);
     when "001011000011" => sin_data_int <= to_unsigned(6238,16); cos_data_int <= to_unsigned(18740,16);
     when "001011000100" => sin_data_int <= to_unsigned(6216,16); cos_data_int <= to_unsigned(18781,16);
     when "001011000101" => sin_data_int <= to_unsigned(6195,16); cos_data_int <= to_unsigned(18821,16);
     when "001011000110" => sin_data_int <= to_unsigned(6173,16); cos_data_int <= to_unsigned(18862,16);
     when "001011000111" => sin_data_int <= to_unsigned(6152,16); cos_data_int <= to_unsigned(18903,16);
     when "001011001000" => sin_data_int <= to_unsigned(6131,16); cos_data_int <= to_unsigned(18944,16);
     when "001011001001" => sin_data_int <= to_unsigned(6110,16); cos_data_int <= to_unsigned(18985,16);
     when "001011001010" => sin_data_int <= to_unsigned(6089,16); cos_data_int <= to_unsigned(19026,16);
     when "001011001011" => sin_data_int <= to_unsigned(6068,16); cos_data_int <= to_unsigned(19067,16);
     when "001011001100" => sin_data_int <= to_unsigned(6047,16); cos_data_int <= to_unsigned(19107,16);
     when "001011001101" => sin_data_int <= to_unsigned(6026,16); cos_data_int <= to_unsigned(19148,16);
     when "001011001110" => sin_data_int <= to_unsigned(6005,16); cos_data_int <= to_unsigned(19190,16);
     when "001011001111" => sin_data_int <= to_unsigned(5984,16); cos_data_int <= to_unsigned(19231,16);
     when "001011010000" => sin_data_int <= to_unsigned(5963,16); cos_data_int <= to_unsigned(19272,16);
     when "001011010001" => sin_data_int <= to_unsigned(5943,16); cos_data_int <= to_unsigned(19313,16);
     when "001011010010" => sin_data_int <= to_unsigned(5922,16); cos_data_int <= to_unsigned(19354,16);
     when "001011010011" => sin_data_int <= to_unsigned(5901,16); cos_data_int <= to_unsigned(19395,16);
     when "001011010100" => sin_data_int <= to_unsigned(5881,16); cos_data_int <= to_unsigned(19436,16);
     when "001011010101" => sin_data_int <= to_unsigned(5861,16); cos_data_int <= to_unsigned(19478,16);
     when "001011010110" => sin_data_int <= to_unsigned(5840,16); cos_data_int <= to_unsigned(19519,16);
     when "001011010111" => sin_data_int <= to_unsigned(5820,16); cos_data_int <= to_unsigned(19560,16);
     when "001011011000" => sin_data_int <= to_unsigned(5800,16); cos_data_int <= to_unsigned(19602,16);
     when "001011011001" => sin_data_int <= to_unsigned(5780,16); cos_data_int <= to_unsigned(19643,16);
     when "001011011010" => sin_data_int <= to_unsigned(5760,16); cos_data_int <= to_unsigned(19684,16);
     when "001011011011" => sin_data_int <= to_unsigned(5739,16); cos_data_int <= to_unsigned(19726,16);
     when "001011011100" => sin_data_int <= to_unsigned(5720,16); cos_data_int <= to_unsigned(19767,16);
     when "001011011101" => sin_data_int <= to_unsigned(5700,16); cos_data_int <= to_unsigned(19809,16);
     when "001011011110" => sin_data_int <= to_unsigned(5680,16); cos_data_int <= to_unsigned(19850,16);
     when "001011011111" => sin_data_int <= to_unsigned(5660,16); cos_data_int <= to_unsigned(19892,16);
     when "001011100000" => sin_data_int <= to_unsigned(5640,16); cos_data_int <= to_unsigned(19933,16);
     when "001011100001" => sin_data_int <= to_unsigned(5621,16); cos_data_int <= to_unsigned(19975,16);
     when "001011100010" => sin_data_int <= to_unsigned(5601,16); cos_data_int <= to_unsigned(20017,16);
     when "001011100011" => sin_data_int <= to_unsigned(5582,16); cos_data_int <= to_unsigned(20058,16);
     when "001011100100" => sin_data_int <= to_unsigned(5562,16); cos_data_int <= to_unsigned(20100,16);
     when "001011100101" => sin_data_int <= to_unsigned(5543,16); cos_data_int <= to_unsigned(20142,16);
     when "001011100110" => sin_data_int <= to_unsigned(5523,16); cos_data_int <= to_unsigned(20183,16);
     when "001011100111" => sin_data_int <= to_unsigned(5504,16); cos_data_int <= to_unsigned(20225,16);
     when "001011101000" => sin_data_int <= to_unsigned(5485,16); cos_data_int <= to_unsigned(20267,16);
     when "001011101001" => sin_data_int <= to_unsigned(5466,16); cos_data_int <= to_unsigned(20309,16);
     when "001011101010" => sin_data_int <= to_unsigned(5447,16); cos_data_int <= to_unsigned(20351,16);
     when "001011101011" => sin_data_int <= to_unsigned(5428,16); cos_data_int <= to_unsigned(20393,16);
     when "001011101100" => sin_data_int <= to_unsigned(5409,16); cos_data_int <= to_unsigned(20435,16);
     when "001011101101" => sin_data_int <= to_unsigned(5390,16); cos_data_int <= to_unsigned(20477,16);
     when "001011101110" => sin_data_int <= to_unsigned(5371,16); cos_data_int <= to_unsigned(20519,16);
     when "001011101111" => sin_data_int <= to_unsigned(5352,16); cos_data_int <= to_unsigned(20561,16);
     when "001011110000" => sin_data_int <= to_unsigned(5334,16); cos_data_int <= to_unsigned(20603,16);
     when "001011110001" => sin_data_int <= to_unsigned(5315,16); cos_data_int <= to_unsigned(20645,16);
     when "001011110010" => sin_data_int <= to_unsigned(5297,16); cos_data_int <= to_unsigned(20687,16);
     when "001011110011" => sin_data_int <= to_unsigned(5278,16); cos_data_int <= to_unsigned(20729,16);
     when "001011110100" => sin_data_int <= to_unsigned(5260,16); cos_data_int <= to_unsigned(20771,16);
     when "001011110101" => sin_data_int <= to_unsigned(5241,16); cos_data_int <= to_unsigned(20813,16);
     when "001011110110" => sin_data_int <= to_unsigned(5223,16); cos_data_int <= to_unsigned(20856,16);
     when "001011110111" => sin_data_int <= to_unsigned(5205,16); cos_data_int <= to_unsigned(20898,16);
     when "001011111000" => sin_data_int <= to_unsigned(5187,16); cos_data_int <= to_unsigned(20940,16);
     when "001011111001" => sin_data_int <= to_unsigned(5168,16); cos_data_int <= to_unsigned(20983,16);
     when "001011111010" => sin_data_int <= to_unsigned(5150,16); cos_data_int <= to_unsigned(21025,16);
     when "001011111011" => sin_data_int <= to_unsigned(5132,16); cos_data_int <= to_unsigned(21067,16);
     when "001011111100" => sin_data_int <= to_unsigned(5115,16); cos_data_int <= to_unsigned(21110,16);
     when "001011111101" => sin_data_int <= to_unsigned(5097,16); cos_data_int <= to_unsigned(21152,16);
     when "001011111110" => sin_data_int <= to_unsigned(5079,16); cos_data_int <= to_unsigned(21195,16);
     when "001011111111" => sin_data_int <= to_unsigned(5061,16); cos_data_int <= to_unsigned(21237,16);
     when "001100000000" => sin_data_int <= to_unsigned(5044,16); cos_data_int <= to_unsigned(21279,16);
     when "001100000001" => sin_data_int <= to_unsigned(5026,16); cos_data_int <= to_unsigned(21322,16);
     when "001100000010" => sin_data_int <= to_unsigned(5009,16); cos_data_int <= to_unsigned(21365,16);
     when "001100000011" => sin_data_int <= to_unsigned(4991,16); cos_data_int <= to_unsigned(21407,16);
     when "001100000100" => sin_data_int <= to_unsigned(4974,16); cos_data_int <= to_unsigned(21450,16);
     when "001100000101" => sin_data_int <= to_unsigned(4956,16); cos_data_int <= to_unsigned(21492,16);
     when "001100000110" => sin_data_int <= to_unsigned(4939,16); cos_data_int <= to_unsigned(21535,16);
     when "001100000111" => sin_data_int <= to_unsigned(4922,16); cos_data_int <= to_unsigned(21578,16);
     when "001100001000" => sin_data_int <= to_unsigned(4905,16); cos_data_int <= to_unsigned(21620,16);
     when "001100001001" => sin_data_int <= to_unsigned(4888,16); cos_data_int <= to_unsigned(21663,16);
     when "001100001010" => sin_data_int <= to_unsigned(4871,16); cos_data_int <= to_unsigned(21706,16);
     when "001100001011" => sin_data_int <= to_unsigned(4854,16); cos_data_int <= to_unsigned(21749,16);
     when "001100001100" => sin_data_int <= to_unsigned(4837,16); cos_data_int <= to_unsigned(21792,16);
     when "001100001101" => sin_data_int <= to_unsigned(4820,16); cos_data_int <= to_unsigned(21834,16);
     when "001100001110" => sin_data_int <= to_unsigned(4803,16); cos_data_int <= to_unsigned(21877,16);
     when "001100001111" => sin_data_int <= to_unsigned(4787,16); cos_data_int <= to_unsigned(21920,16);
     when "001100010000" => sin_data_int <= to_unsigned(4770,16); cos_data_int <= to_unsigned(21963,16);
     when "001100010001" => sin_data_int <= to_unsigned(4754,16); cos_data_int <= to_unsigned(22006,16);
     when "001100010010" => sin_data_int <= to_unsigned(4737,16); cos_data_int <= to_unsigned(22049,16);
     when "001100010011" => sin_data_int <= to_unsigned(4721,16); cos_data_int <= to_unsigned(22092,16);
     when "001100010100" => sin_data_int <= to_unsigned(4704,16); cos_data_int <= to_unsigned(22135,16);
     when "001100010101" => sin_data_int <= to_unsigned(4688,16); cos_data_int <= to_unsigned(22178,16);
     when "001100010110" => sin_data_int <= to_unsigned(4672,16); cos_data_int <= to_unsigned(22221,16);
     when "001100010111" => sin_data_int <= to_unsigned(4656,16); cos_data_int <= to_unsigned(22264,16);
     when "001100011000" => sin_data_int <= to_unsigned(4640,16); cos_data_int <= to_unsigned(22307,16);
     when "001100011001" => sin_data_int <= to_unsigned(4624,16); cos_data_int <= to_unsigned(22351,16);
     when "001100011010" => sin_data_int <= to_unsigned(4608,16); cos_data_int <= to_unsigned(22394,16);
     when "001100011011" => sin_data_int <= to_unsigned(4592,16); cos_data_int <= to_unsigned(22437,16);
     when "001100011100" => sin_data_int <= to_unsigned(4576,16); cos_data_int <= to_unsigned(22480,16);
     when "001100011101" => sin_data_int <= to_unsigned(4560,16); cos_data_int <= to_unsigned(22523,16);
     when "001100011110" => sin_data_int <= to_unsigned(4545,16); cos_data_int <= to_unsigned(22567,16);
     when "001100011111" => sin_data_int <= to_unsigned(4529,16); cos_data_int <= to_unsigned(22610,16);
     when "001100100000" => sin_data_int <= to_unsigned(4514,16); cos_data_int <= to_unsigned(22653,16);
     when "001100100001" => sin_data_int <= to_unsigned(4498,16); cos_data_int <= to_unsigned(22697,16);
     when "001100100010" => sin_data_int <= to_unsigned(4483,16); cos_data_int <= to_unsigned(22740,16);
     when "001100100011" => sin_data_int <= to_unsigned(4467,16); cos_data_int <= to_unsigned(22783,16);
     when "001100100100" => sin_data_int <= to_unsigned(4452,16); cos_data_int <= to_unsigned(22827,16);
     when "001100100101" => sin_data_int <= to_unsigned(4437,16); cos_data_int <= to_unsigned(22870,16);
     when "001100100110" => sin_data_int <= to_unsigned(4422,16); cos_data_int <= to_unsigned(22914,16);
     when "001100100111" => sin_data_int <= to_unsigned(4407,16); cos_data_int <= to_unsigned(22957,16);
     when "001100101000" => sin_data_int <= to_unsigned(4392,16); cos_data_int <= to_unsigned(23001,16);
     when "001100101001" => sin_data_int <= to_unsigned(4377,16); cos_data_int <= to_unsigned(23044,16);
     when "001100101010" => sin_data_int <= to_unsigned(4362,16); cos_data_int <= to_unsigned(23088,16);
     when "001100101011" => sin_data_int <= to_unsigned(4347,16); cos_data_int <= to_unsigned(23131,16);
     when "001100101100" => sin_data_int <= to_unsigned(4332,16); cos_data_int <= to_unsigned(23175,16);
     when "001100101101" => sin_data_int <= to_unsigned(4318,16); cos_data_int <= to_unsigned(23219,16);
     when "001100101110" => sin_data_int <= to_unsigned(4303,16); cos_data_int <= to_unsigned(23262,16);
     when "001100101111" => sin_data_int <= to_unsigned(4289,16); cos_data_int <= to_unsigned(23306,16);
     when "001100110000" => sin_data_int <= to_unsigned(4274,16); cos_data_int <= to_unsigned(23350,16);
     when "001100110001" => sin_data_int <= to_unsigned(4260,16); cos_data_int <= to_unsigned(23393,16);
     when "001100110010" => sin_data_int <= to_unsigned(4245,16); cos_data_int <= to_unsigned(23437,16);
     when "001100110011" => sin_data_int <= to_unsigned(4231,16); cos_data_int <= to_unsigned(23481,16);
     when "001100110100" => sin_data_int <= to_unsigned(4217,16); cos_data_int <= to_unsigned(23525,16);
     when "001100110101" => sin_data_int <= to_unsigned(4203,16); cos_data_int <= to_unsigned(23568,16);
     when "001100110110" => sin_data_int <= to_unsigned(4189,16); cos_data_int <= to_unsigned(23612,16);
     when "001100110111" => sin_data_int <= to_unsigned(4175,16); cos_data_int <= to_unsigned(23656,16);
     when "001100111000" => sin_data_int <= to_unsigned(4161,16); cos_data_int <= to_unsigned(23700,16);
     when "001100111001" => sin_data_int <= to_unsigned(4147,16); cos_data_int <= to_unsigned(23744,16);
     when "001100111010" => sin_data_int <= to_unsigned(4133,16); cos_data_int <= to_unsigned(23788,16);
     when "001100111011" => sin_data_int <= to_unsigned(4119,16); cos_data_int <= to_unsigned(23832,16);
     when "001100111100" => sin_data_int <= to_unsigned(4106,16); cos_data_int <= to_unsigned(23875,16);
     when "001100111101" => sin_data_int <= to_unsigned(4092,16); cos_data_int <= to_unsigned(23919,16);
     when "001100111110" => sin_data_int <= to_unsigned(4079,16); cos_data_int <= to_unsigned(23963,16);
     when "001100111111" => sin_data_int <= to_unsigned(4065,16); cos_data_int <= to_unsigned(24007,16);
     when "001101000000" => sin_data_int <= to_unsigned(4052,16); cos_data_int <= to_unsigned(24051,16);
     when "001101000001" => sin_data_int <= to_unsigned(4038,16); cos_data_int <= to_unsigned(24096,16);
     when "001101000010" => sin_data_int <= to_unsigned(4025,16); cos_data_int <= to_unsigned(24140,16);
     when "001101000011" => sin_data_int <= to_unsigned(4012,16); cos_data_int <= to_unsigned(24184,16);
     when "001101000100" => sin_data_int <= to_unsigned(3999,16); cos_data_int <= to_unsigned(24228,16);
     when "001101000101" => sin_data_int <= to_unsigned(3986,16); cos_data_int <= to_unsigned(24272,16);
     when "001101000110" => sin_data_int <= to_unsigned(3973,16); cos_data_int <= to_unsigned(24316,16);
     when "001101000111" => sin_data_int <= to_unsigned(3960,16); cos_data_int <= to_unsigned(24360,16);
     when "001101001000" => sin_data_int <= to_unsigned(3947,16); cos_data_int <= to_unsigned(24404,16);
     when "001101001001" => sin_data_int <= to_unsigned(3934,16); cos_data_int <= to_unsigned(24449,16);
     when "001101001010" => sin_data_int <= to_unsigned(3922,16); cos_data_int <= to_unsigned(24493,16);
     when "001101001011" => sin_data_int <= to_unsigned(3909,16); cos_data_int <= to_unsigned(24537,16);
     when "001101001100" => sin_data_int <= to_unsigned(3896,16); cos_data_int <= to_unsigned(24581,16);
     when "001101001101" => sin_data_int <= to_unsigned(3884,16); cos_data_int <= to_unsigned(24626,16);
     when "001101001110" => sin_data_int <= to_unsigned(3871,16); cos_data_int <= to_unsigned(24670,16);
     when "001101001111" => sin_data_int <= to_unsigned(3859,16); cos_data_int <= to_unsigned(24714,16);
     when "001101010000" => sin_data_int <= to_unsigned(3847,16); cos_data_int <= to_unsigned(24759,16);
     when "001101010001" => sin_data_int <= to_unsigned(3834,16); cos_data_int <= to_unsigned(24803,16);
     when "001101010010" => sin_data_int <= to_unsigned(3822,16); cos_data_int <= to_unsigned(24847,16);
     when "001101010011" => sin_data_int <= to_unsigned(3810,16); cos_data_int <= to_unsigned(24892,16);
     when "001101010100" => sin_data_int <= to_unsigned(3798,16); cos_data_int <= to_unsigned(24936,16);
     when "001101010101" => sin_data_int <= to_unsigned(3786,16); cos_data_int <= to_unsigned(24981,16);
     when "001101010110" => sin_data_int <= to_unsigned(3774,16); cos_data_int <= to_unsigned(25025,16);
     when "001101010111" => sin_data_int <= to_unsigned(3762,16); cos_data_int <= to_unsigned(25070,16);
     when "001101011000" => sin_data_int <= to_unsigned(3751,16); cos_data_int <= to_unsigned(25114,16);
     when "001101011001" => sin_data_int <= to_unsigned(3739,16); cos_data_int <= to_unsigned(25159,16);
     when "001101011010" => sin_data_int <= to_unsigned(3727,16); cos_data_int <= to_unsigned(25203,16);
     when "001101011011" => sin_data_int <= to_unsigned(3716,16); cos_data_int <= to_unsigned(25248,16);
     when "001101011100" => sin_data_int <= to_unsigned(3704,16); cos_data_int <= to_unsigned(25292,16);
     when "001101011101" => sin_data_int <= to_unsigned(3693,16); cos_data_int <= to_unsigned(25337,16);
     when "001101011110" => sin_data_int <= to_unsigned(3682,16); cos_data_int <= to_unsigned(25381,16);
     when "001101011111" => sin_data_int <= to_unsigned(3670,16); cos_data_int <= to_unsigned(25426,16);
     when "001101100000" => sin_data_int <= to_unsigned(3659,16); cos_data_int <= to_unsigned(25471,16);
     when "001101100001" => sin_data_int <= to_unsigned(3648,16); cos_data_int <= to_unsigned(25515,16);
     when "001101100010" => sin_data_int <= to_unsigned(3637,16); cos_data_int <= to_unsigned(25560,16);
     when "001101100011" => sin_data_int <= to_unsigned(3626,16); cos_data_int <= to_unsigned(25605,16);
     when "001101100100" => sin_data_int <= to_unsigned(3615,16); cos_data_int <= to_unsigned(25649,16);
     when "001101100101" => sin_data_int <= to_unsigned(3604,16); cos_data_int <= to_unsigned(25694,16);
     when "001101100110" => sin_data_int <= to_unsigned(3593,16); cos_data_int <= to_unsigned(25739,16);
     when "001101100111" => sin_data_int <= to_unsigned(3582,16); cos_data_int <= to_unsigned(25783,16);
     when "001101101000" => sin_data_int <= to_unsigned(3572,16); cos_data_int <= to_unsigned(25828,16);
     when "001101101001" => sin_data_int <= to_unsigned(3561,16); cos_data_int <= to_unsigned(25873,16);
     when "001101101010" => sin_data_int <= to_unsigned(3551,16); cos_data_int <= to_unsigned(25918,16);
     when "001101101011" => sin_data_int <= to_unsigned(3540,16); cos_data_int <= to_unsigned(25963,16);
     when "001101101100" => sin_data_int <= to_unsigned(3530,16); cos_data_int <= to_unsigned(26007,16);
     when "001101101101" => sin_data_int <= to_unsigned(3519,16); cos_data_int <= to_unsigned(26052,16);
     when "001101101110" => sin_data_int <= to_unsigned(3509,16); cos_data_int <= to_unsigned(26097,16);
     when "001101101111" => sin_data_int <= to_unsigned(3499,16); cos_data_int <= to_unsigned(26142,16);
     when "001101110000" => sin_data_int <= to_unsigned(3489,16); cos_data_int <= to_unsigned(26187,16);
     when "001101110001" => sin_data_int <= to_unsigned(3479,16); cos_data_int <= to_unsigned(26232,16);
     when "001101110010" => sin_data_int <= to_unsigned(3469,16); cos_data_int <= to_unsigned(26277,16);
     when "001101110011" => sin_data_int <= to_unsigned(3459,16); cos_data_int <= to_unsigned(26322,16);
     when "001101110100" => sin_data_int <= to_unsigned(3449,16); cos_data_int <= to_unsigned(26367,16);
     when "001101110101" => sin_data_int <= to_unsigned(3439,16); cos_data_int <= to_unsigned(26412,16);
     when "001101110110" => sin_data_int <= to_unsigned(3430,16); cos_data_int <= to_unsigned(26457,16);
     when "001101110111" => sin_data_int <= to_unsigned(3420,16); cos_data_int <= to_unsigned(26502,16);
     when "001101111000" => sin_data_int <= to_unsigned(3410,16); cos_data_int <= to_unsigned(26547,16);
     when "001101111001" => sin_data_int <= to_unsigned(3401,16); cos_data_int <= to_unsigned(26592,16);
     when "001101111010" => sin_data_int <= to_unsigned(3392,16); cos_data_int <= to_unsigned(26637,16);
     when "001101111011" => sin_data_int <= to_unsigned(3382,16); cos_data_int <= to_unsigned(26682,16);
     when "001101111100" => sin_data_int <= to_unsigned(3373,16); cos_data_int <= to_unsigned(26727,16);
     when "001101111101" => sin_data_int <= to_unsigned(3364,16); cos_data_int <= to_unsigned(26772,16);
     when "001101111110" => sin_data_int <= to_unsigned(3355,16); cos_data_int <= to_unsigned(26817,16);
     when "001101111111" => sin_data_int <= to_unsigned(3345,16); cos_data_int <= to_unsigned(26862,16);
     when "001110000000" => sin_data_int <= to_unsigned(3336,16); cos_data_int <= to_unsigned(26907,16);
     when "001110000001" => sin_data_int <= to_unsigned(3327,16); cos_data_int <= to_unsigned(26952,16);
     when "001110000010" => sin_data_int <= to_unsigned(3319,16); cos_data_int <= to_unsigned(26998,16);
     when "001110000011" => sin_data_int <= to_unsigned(3310,16); cos_data_int <= to_unsigned(27043,16);
     when "001110000100" => sin_data_int <= to_unsigned(3301,16); cos_data_int <= to_unsigned(27088,16);
     when "001110000101" => sin_data_int <= to_unsigned(3292,16); cos_data_int <= to_unsigned(27133,16);
     when "001110000110" => sin_data_int <= to_unsigned(3284,16); cos_data_int <= to_unsigned(27178,16);
     when "001110000111" => sin_data_int <= to_unsigned(3275,16); cos_data_int <= to_unsigned(27224,16);
     when "001110001000" => sin_data_int <= to_unsigned(3267,16); cos_data_int <= to_unsigned(27269,16);
     when "001110001001" => sin_data_int <= to_unsigned(3258,16); cos_data_int <= to_unsigned(27314,16);
     when "001110001010" => sin_data_int <= to_unsigned(3250,16); cos_data_int <= to_unsigned(27359,16);
     when "001110001011" => sin_data_int <= to_unsigned(3242,16); cos_data_int <= to_unsigned(27405,16);
     when "001110001100" => sin_data_int <= to_unsigned(3234,16); cos_data_int <= to_unsigned(27450,16);
     when "001110001101" => sin_data_int <= to_unsigned(3226,16); cos_data_int <= to_unsigned(27495,16);
     when "001110001110" => sin_data_int <= to_unsigned(3218,16); cos_data_int <= to_unsigned(27540,16);
     when "001110001111" => sin_data_int <= to_unsigned(3210,16); cos_data_int <= to_unsigned(27586,16);
     when "001110010000" => sin_data_int <= to_unsigned(3202,16); cos_data_int <= to_unsigned(27631,16);
     when "001110010001" => sin_data_int <= to_unsigned(3194,16); cos_data_int <= to_unsigned(27676,16);
     when "001110010010" => sin_data_int <= to_unsigned(3186,16); cos_data_int <= to_unsigned(27722,16);
     when "001110010011" => sin_data_int <= to_unsigned(3178,16); cos_data_int <= to_unsigned(27767,16);
     when "001110010100" => sin_data_int <= to_unsigned(3171,16); cos_data_int <= to_unsigned(27813,16);
     when "001110010101" => sin_data_int <= to_unsigned(3163,16); cos_data_int <= to_unsigned(27858,16);
     when "001110010110" => sin_data_int <= to_unsigned(3156,16); cos_data_int <= to_unsigned(27903,16);
     when "001110010111" => sin_data_int <= to_unsigned(3148,16); cos_data_int <= to_unsigned(27949,16);
     when "001110011000" => sin_data_int <= to_unsigned(3141,16); cos_data_int <= to_unsigned(27994,16);
     when "001110011001" => sin_data_int <= to_unsigned(3134,16); cos_data_int <= to_unsigned(28040,16);
     when "001110011010" => sin_data_int <= to_unsigned(3126,16); cos_data_int <= to_unsigned(28085,16);
     when "001110011011" => sin_data_int <= to_unsigned(3119,16); cos_data_int <= to_unsigned(28131,16);
     when "001110011100" => sin_data_int <= to_unsigned(3112,16); cos_data_int <= to_unsigned(28176,16);
     when "001110011101" => sin_data_int <= to_unsigned(3105,16); cos_data_int <= to_unsigned(28222,16);
     when "001110011110" => sin_data_int <= to_unsigned(3098,16); cos_data_int <= to_unsigned(28267,16);
     when "001110011111" => sin_data_int <= to_unsigned(3091,16); cos_data_int <= to_unsigned(28313,16);
     when "001110100000" => sin_data_int <= to_unsigned(3085,16); cos_data_int <= to_unsigned(28358,16);
     when "001110100001" => sin_data_int <= to_unsigned(3078,16); cos_data_int <= to_unsigned(28404,16);
     when "001110100010" => sin_data_int <= to_unsigned(3071,16); cos_data_int <= to_unsigned(28449,16);
     when "001110100011" => sin_data_int <= to_unsigned(3065,16); cos_data_int <= to_unsigned(28495,16);
     when "001110100100" => sin_data_int <= to_unsigned(3058,16); cos_data_int <= to_unsigned(28540,16);
     when "001110100101" => sin_data_int <= to_unsigned(3052,16); cos_data_int <= to_unsigned(28586,16);
     when "001110100110" => sin_data_int <= to_unsigned(3045,16); cos_data_int <= to_unsigned(28631,16);
     when "001110100111" => sin_data_int <= to_unsigned(3039,16); cos_data_int <= to_unsigned(28677,16);
     when "001110101000" => sin_data_int <= to_unsigned(3033,16); cos_data_int <= to_unsigned(28723,16);
     when "001110101001" => sin_data_int <= to_unsigned(3027,16); cos_data_int <= to_unsigned(28768,16);
     when "001110101010" => sin_data_int <= to_unsigned(3021,16); cos_data_int <= to_unsigned(28814,16);
     when "001110101011" => sin_data_int <= to_unsigned(3015,16); cos_data_int <= to_unsigned(28859,16);
     when "001110101100" => sin_data_int <= to_unsigned(3009,16); cos_data_int <= to_unsigned(28905,16);
     when "001110101101" => sin_data_int <= to_unsigned(3003,16); cos_data_int <= to_unsigned(28951,16);
     when "001110101110" => sin_data_int <= to_unsigned(2997,16); cos_data_int <= to_unsigned(28996,16);
     when "001110101111" => sin_data_int <= to_unsigned(2991,16); cos_data_int <= to_unsigned(29042,16);
     when "001110110000" => sin_data_int <= to_unsigned(2986,16); cos_data_int <= to_unsigned(29088,16);
     when "001110110001" => sin_data_int <= to_unsigned(2980,16); cos_data_int <= to_unsigned(29133,16);
     when "001110110010" => sin_data_int <= to_unsigned(2974,16); cos_data_int <= to_unsigned(29179,16);
     when "001110110011" => sin_data_int <= to_unsigned(2969,16); cos_data_int <= to_unsigned(29225,16);
     when "001110110100" => sin_data_int <= to_unsigned(2964,16); cos_data_int <= to_unsigned(29270,16);
     when "001110110101" => sin_data_int <= to_unsigned(2958,16); cos_data_int <= to_unsigned(29316,16);
     when "001110110110" => sin_data_int <= to_unsigned(2953,16); cos_data_int <= to_unsigned(29362,16);
     when "001110110111" => sin_data_int <= to_unsigned(2948,16); cos_data_int <= to_unsigned(29408,16);
     when "001110111000" => sin_data_int <= to_unsigned(2943,16); cos_data_int <= to_unsigned(29453,16);
     when "001110111001" => sin_data_int <= to_unsigned(2938,16); cos_data_int <= to_unsigned(29499,16);
     when "001110111010" => sin_data_int <= to_unsigned(2933,16); cos_data_int <= to_unsigned(29545,16);
     when "001110111011" => sin_data_int <= to_unsigned(2928,16); cos_data_int <= to_unsigned(29591,16);
     when "001110111100" => sin_data_int <= to_unsigned(2923,16); cos_data_int <= to_unsigned(29636,16);
     when "001110111101" => sin_data_int <= to_unsigned(2918,16); cos_data_int <= to_unsigned(29682,16);
     when "001110111110" => sin_data_int <= to_unsigned(2914,16); cos_data_int <= to_unsigned(29728,16);
     when "001110111111" => sin_data_int <= to_unsigned(2909,16); cos_data_int <= to_unsigned(29774,16);
     when "001111000000" => sin_data_int <= to_unsigned(2904,16); cos_data_int <= to_unsigned(29819,16);
     when "001111000001" => sin_data_int <= to_unsigned(2900,16); cos_data_int <= to_unsigned(29865,16);
     when "001111000010" => sin_data_int <= to_unsigned(2896,16); cos_data_int <= to_unsigned(29911,16);
     when "001111000011" => sin_data_int <= to_unsigned(2891,16); cos_data_int <= to_unsigned(29957,16);
     when "001111000100" => sin_data_int <= to_unsigned(2887,16); cos_data_int <= to_unsigned(30003,16);
     when "001111000101" => sin_data_int <= to_unsigned(2883,16); cos_data_int <= to_unsigned(30049,16);
     when "001111000110" => sin_data_int <= to_unsigned(2879,16); cos_data_int <= to_unsigned(30094,16);
     when "001111000111" => sin_data_int <= to_unsigned(2875,16); cos_data_int <= to_unsigned(30140,16);
     when "001111001000" => sin_data_int <= to_unsigned(2871,16); cos_data_int <= to_unsigned(30186,16);
     when "001111001001" => sin_data_int <= to_unsigned(2867,16); cos_data_int <= to_unsigned(30232,16);
     when "001111001010" => sin_data_int <= to_unsigned(2863,16); cos_data_int <= to_unsigned(30278,16);
     when "001111001011" => sin_data_int <= to_unsigned(2859,16); cos_data_int <= to_unsigned(30324,16);
     when "001111001100" => sin_data_int <= to_unsigned(2855,16); cos_data_int <= to_unsigned(30370,16);
     when "001111001101" => sin_data_int <= to_unsigned(2852,16); cos_data_int <= to_unsigned(30415,16);
     when "001111001110" => sin_data_int <= to_unsigned(2848,16); cos_data_int <= to_unsigned(30461,16);
     when "001111001111" => sin_data_int <= to_unsigned(2845,16); cos_data_int <= to_unsigned(30507,16);
     when "001111010000" => sin_data_int <= to_unsigned(2841,16); cos_data_int <= to_unsigned(30553,16);
     when "001111010001" => sin_data_int <= to_unsigned(2838,16); cos_data_int <= to_unsigned(30599,16);
     when "001111010010" => sin_data_int <= to_unsigned(2835,16); cos_data_int <= to_unsigned(30645,16);
     when "001111010011" => sin_data_int <= to_unsigned(2831,16); cos_data_int <= to_unsigned(30691,16);
     when "001111010100" => sin_data_int <= to_unsigned(2828,16); cos_data_int <= to_unsigned(30737,16);
     when "001111010101" => sin_data_int <= to_unsigned(2825,16); cos_data_int <= to_unsigned(30783,16);
     when "001111010110" => sin_data_int <= to_unsigned(2822,16); cos_data_int <= to_unsigned(30829,16);
     when "001111010111" => sin_data_int <= to_unsigned(2819,16); cos_data_int <= to_unsigned(30874,16);
     when "001111011000" => sin_data_int <= to_unsigned(2816,16); cos_data_int <= to_unsigned(30920,16);
     when "001111011001" => sin_data_int <= to_unsigned(2814,16); cos_data_int <= to_unsigned(30966,16);
     when "001111011010" => sin_data_int <= to_unsigned(2811,16); cos_data_int <= to_unsigned(31012,16);
     when "001111011011" => sin_data_int <= to_unsigned(2808,16); cos_data_int <= to_unsigned(31058,16);
     when "001111011100" => sin_data_int <= to_unsigned(2806,16); cos_data_int <= to_unsigned(31104,16);
     when "001111011101" => sin_data_int <= to_unsigned(2803,16); cos_data_int <= to_unsigned(31150,16);
     when "001111011110" => sin_data_int <= to_unsigned(2801,16); cos_data_int <= to_unsigned(31196,16);
     when "001111011111" => sin_data_int <= to_unsigned(2798,16); cos_data_int <= to_unsigned(31242,16);
     when "001111100000" => sin_data_int <= to_unsigned(2796,16); cos_data_int <= to_unsigned(31288,16);
     when "001111100001" => sin_data_int <= to_unsigned(2794,16); cos_data_int <= to_unsigned(31334,16);
     when "001111100010" => sin_data_int <= to_unsigned(2792,16); cos_data_int <= to_unsigned(31380,16);
     when "001111100011" => sin_data_int <= to_unsigned(2790,16); cos_data_int <= to_unsigned(31426,16);
     when "001111100100" => sin_data_int <= to_unsigned(2788,16); cos_data_int <= to_unsigned(31472,16);
     when "001111100101" => sin_data_int <= to_unsigned(2786,16); cos_data_int <= to_unsigned(31518,16);
     when "001111100110" => sin_data_int <= to_unsigned(2784,16); cos_data_int <= to_unsigned(31564,16);
     when "001111100111" => sin_data_int <= to_unsigned(2782,16); cos_data_int <= to_unsigned(31610,16);
     when "001111101000" => sin_data_int <= to_unsigned(2780,16); cos_data_int <= to_unsigned(31656,16);
     when "001111101001" => sin_data_int <= to_unsigned(2779,16); cos_data_int <= to_unsigned(31702,16);
     when "001111101010" => sin_data_int <= to_unsigned(2777,16); cos_data_int <= to_unsigned(31748,16);
     when "001111101011" => sin_data_int <= to_unsigned(2776,16); cos_data_int <= to_unsigned(31794,16);
     when "001111101100" => sin_data_int <= to_unsigned(2774,16); cos_data_int <= to_unsigned(31840,16);
     when "001111101101" => sin_data_int <= to_unsigned(2773,16); cos_data_int <= to_unsigned(31886,16);
     when "001111101110" => sin_data_int <= to_unsigned(2771,16); cos_data_int <= to_unsigned(31932,16);
     when "001111101111" => sin_data_int <= to_unsigned(2770,16); cos_data_int <= to_unsigned(31978,16);
     when "001111110000" => sin_data_int <= to_unsigned(2769,16); cos_data_int <= to_unsigned(32024,16);
     when "001111110001" => sin_data_int <= to_unsigned(2768,16); cos_data_int <= to_unsigned(32070,16);
     when "001111110010" => sin_data_int <= to_unsigned(2767,16); cos_data_int <= to_unsigned(32116,16);
     when "001111110011" => sin_data_int <= to_unsigned(2766,16); cos_data_int <= to_unsigned(32162,16);
     when "001111110100" => sin_data_int <= to_unsigned(2765,16); cos_data_int <= to_unsigned(32208,16);
     when "001111110101" => sin_data_int <= to_unsigned(2764,16); cos_data_int <= to_unsigned(32254,16);
     when "001111110110" => sin_data_int <= to_unsigned(2764,16); cos_data_int <= to_unsigned(32300,16);
     when "001111110111" => sin_data_int <= to_unsigned(2763,16); cos_data_int <= to_unsigned(32346,16);
     when "001111111000" => sin_data_int <= to_unsigned(2762,16); cos_data_int <= to_unsigned(32392,16);
     when "001111111001" => sin_data_int <= to_unsigned(2762,16); cos_data_int <= to_unsigned(32438,16);
     when "001111111010" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(32484,16);
     when "001111111011" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(32530,16);
     when "001111111100" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(32576,16);
     when "001111111101" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32622,16);
     when "001111111110" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32668,16);
     when "001111111111" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32714,16);
     when "010000000000" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32760,16);
     when "010000000001" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32806,16);
     when "010000000010" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32852,16);
     when "010000000011" => sin_data_int <= to_unsigned(2760,16); cos_data_int <= to_unsigned(32898,16);
     when "010000000100" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(32944,16);
     when "010000000101" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(32990,16);
     when "010000000110" => sin_data_int <= to_unsigned(2761,16); cos_data_int <= to_unsigned(33036,16);
     when "010000000111" => sin_data_int <= to_unsigned(2762,16); cos_data_int <= to_unsigned(33082,16);
     when "010000001000" => sin_data_int <= to_unsigned(2762,16); cos_data_int <= to_unsigned(33128,16);
     when "010000001001" => sin_data_int <= to_unsigned(2763,16); cos_data_int <= to_unsigned(33174,16);
     when "010000001010" => sin_data_int <= to_unsigned(2764,16); cos_data_int <= to_unsigned(33220,16);
     when "010000001011" => sin_data_int <= to_unsigned(2764,16); cos_data_int <= to_unsigned(33266,16);
     when "010000001100" => sin_data_int <= to_unsigned(2765,16); cos_data_int <= to_unsigned(33312,16);
     when "010000001101" => sin_data_int <= to_unsigned(2766,16); cos_data_int <= to_unsigned(33358,16);
     when "010000001110" => sin_data_int <= to_unsigned(2767,16); cos_data_int <= to_unsigned(33404,16);
     when "010000001111" => sin_data_int <= to_unsigned(2768,16); cos_data_int <= to_unsigned(33450,16);
     when "010000010000" => sin_data_int <= to_unsigned(2769,16); cos_data_int <= to_unsigned(33496,16);
     when "010000010001" => sin_data_int <= to_unsigned(2770,16); cos_data_int <= to_unsigned(33542,16);
     when "010000010010" => sin_data_int <= to_unsigned(2771,16); cos_data_int <= to_unsigned(33588,16);
     when "010000010011" => sin_data_int <= to_unsigned(2773,16); cos_data_int <= to_unsigned(33634,16);
     when "010000010100" => sin_data_int <= to_unsigned(2774,16); cos_data_int <= to_unsigned(33680,16);
     when "010000010101" => sin_data_int <= to_unsigned(2776,16); cos_data_int <= to_unsigned(33726,16);
     when "010000010110" => sin_data_int <= to_unsigned(2777,16); cos_data_int <= to_unsigned(33772,16);
     when "010000010111" => sin_data_int <= to_unsigned(2779,16); cos_data_int <= to_unsigned(33818,16);
     when "010000011000" => sin_data_int <= to_unsigned(2780,16); cos_data_int <= to_unsigned(33864,16);
     when "010000011001" => sin_data_int <= to_unsigned(2782,16); cos_data_int <= to_unsigned(33910,16);
     when "010000011010" => sin_data_int <= to_unsigned(2784,16); cos_data_int <= to_unsigned(33956,16);
     when "010000011011" => sin_data_int <= to_unsigned(2786,16); cos_data_int <= to_unsigned(34002,16);
     when "010000011100" => sin_data_int <= to_unsigned(2788,16); cos_data_int <= to_unsigned(34048,16);
     when "010000011101" => sin_data_int <= to_unsigned(2790,16); cos_data_int <= to_unsigned(34094,16);
     when "010000011110" => sin_data_int <= to_unsigned(2792,16); cos_data_int <= to_unsigned(34140,16);
     when "010000011111" => sin_data_int <= to_unsigned(2794,16); cos_data_int <= to_unsigned(34186,16);
     when "010000100000" => sin_data_int <= to_unsigned(2796,16); cos_data_int <= to_unsigned(34232,16);
     when "010000100001" => sin_data_int <= to_unsigned(2798,16); cos_data_int <= to_unsigned(34278,16);
     when "010000100010" => sin_data_int <= to_unsigned(2801,16); cos_data_int <= to_unsigned(34324,16);
     when "010000100011" => sin_data_int <= to_unsigned(2803,16); cos_data_int <= to_unsigned(34370,16);
     when "010000100100" => sin_data_int <= to_unsigned(2806,16); cos_data_int <= to_unsigned(34416,16);
     when "010000100101" => sin_data_int <= to_unsigned(2808,16); cos_data_int <= to_unsigned(34462,16);
     when "010000100110" => sin_data_int <= to_unsigned(2811,16); cos_data_int <= to_unsigned(34508,16);
     when "010000100111" => sin_data_int <= to_unsigned(2814,16); cos_data_int <= to_unsigned(34554,16);
     when "010000101000" => sin_data_int <= to_unsigned(2816,16); cos_data_int <= to_unsigned(34600,16);
     when "010000101001" => sin_data_int <= to_unsigned(2819,16); cos_data_int <= to_unsigned(34646,16);
     when "010000101010" => sin_data_int <= to_unsigned(2822,16); cos_data_int <= to_unsigned(34691,16);
     when "010000101011" => sin_data_int <= to_unsigned(2825,16); cos_data_int <= to_unsigned(34737,16);
     when "010000101100" => sin_data_int <= to_unsigned(2828,16); cos_data_int <= to_unsigned(34783,16);
     when "010000101101" => sin_data_int <= to_unsigned(2831,16); cos_data_int <= to_unsigned(34829,16);
     when "010000101110" => sin_data_int <= to_unsigned(2835,16); cos_data_int <= to_unsigned(34875,16);
     when "010000101111" => sin_data_int <= to_unsigned(2838,16); cos_data_int <= to_unsigned(34921,16);
     when "010000110000" => sin_data_int <= to_unsigned(2841,16); cos_data_int <= to_unsigned(34967,16);
     when "010000110001" => sin_data_int <= to_unsigned(2845,16); cos_data_int <= to_unsigned(35013,16);
     when "010000110010" => sin_data_int <= to_unsigned(2848,16); cos_data_int <= to_unsigned(35059,16);
     when "010000110011" => sin_data_int <= to_unsigned(2852,16); cos_data_int <= to_unsigned(35105,16);
     when "010000110100" => sin_data_int <= to_unsigned(2855,16); cos_data_int <= to_unsigned(35150,16);
     when "010000110101" => sin_data_int <= to_unsigned(2859,16); cos_data_int <= to_unsigned(35196,16);
     when "010000110110" => sin_data_int <= to_unsigned(2863,16); cos_data_int <= to_unsigned(35242,16);
     when "010000110111" => sin_data_int <= to_unsigned(2867,16); cos_data_int <= to_unsigned(35288,16);
     when "010000111000" => sin_data_int <= to_unsigned(2871,16); cos_data_int <= to_unsigned(35334,16);
     when "010000111001" => sin_data_int <= to_unsigned(2875,16); cos_data_int <= to_unsigned(35380,16);
     when "010000111010" => sin_data_int <= to_unsigned(2879,16); cos_data_int <= to_unsigned(35426,16);
     when "010000111011" => sin_data_int <= to_unsigned(2883,16); cos_data_int <= to_unsigned(35471,16);
     when "010000111100" => sin_data_int <= to_unsigned(2887,16); cos_data_int <= to_unsigned(35517,16);
     when "010000111101" => sin_data_int <= to_unsigned(2891,16); cos_data_int <= to_unsigned(35563,16);
     when "010000111110" => sin_data_int <= to_unsigned(2896,16); cos_data_int <= to_unsigned(35609,16);
     when "010000111111" => sin_data_int <= to_unsigned(2900,16); cos_data_int <= to_unsigned(35655,16);
     when "010001000000" => sin_data_int <= to_unsigned(2904,16); cos_data_int <= to_unsigned(35701,16);
     when "010001000001" => sin_data_int <= to_unsigned(2909,16); cos_data_int <= to_unsigned(35746,16);
     when "010001000010" => sin_data_int <= to_unsigned(2914,16); cos_data_int <= to_unsigned(35792,16);
     when "010001000011" => sin_data_int <= to_unsigned(2918,16); cos_data_int <= to_unsigned(35838,16);
     when "010001000100" => sin_data_int <= to_unsigned(2923,16); cos_data_int <= to_unsigned(35884,16);
     when "010001000101" => sin_data_int <= to_unsigned(2928,16); cos_data_int <= to_unsigned(35929,16);
     when "010001000110" => sin_data_int <= to_unsigned(2933,16); cos_data_int <= to_unsigned(35975,16);
     when "010001000111" => sin_data_int <= to_unsigned(2938,16); cos_data_int <= to_unsigned(36021,16);
     when "010001001000" => sin_data_int <= to_unsigned(2943,16); cos_data_int <= to_unsigned(36067,16);
     when "010001001001" => sin_data_int <= to_unsigned(2948,16); cos_data_int <= to_unsigned(36112,16);
     when "010001001010" => sin_data_int <= to_unsigned(2953,16); cos_data_int <= to_unsigned(36158,16);
     when "010001001011" => sin_data_int <= to_unsigned(2958,16); cos_data_int <= to_unsigned(36204,16);
     when "010001001100" => sin_data_int <= to_unsigned(2964,16); cos_data_int <= to_unsigned(36250,16);
     when "010001001101" => sin_data_int <= to_unsigned(2969,16); cos_data_int <= to_unsigned(36295,16);
     when "010001001110" => sin_data_int <= to_unsigned(2974,16); cos_data_int <= to_unsigned(36341,16);
     when "010001001111" => sin_data_int <= to_unsigned(2980,16); cos_data_int <= to_unsigned(36387,16);
     when "010001010000" => sin_data_int <= to_unsigned(2986,16); cos_data_int <= to_unsigned(36432,16);
     when "010001010001" => sin_data_int <= to_unsigned(2991,16); cos_data_int <= to_unsigned(36478,16);
     when "010001010010" => sin_data_int <= to_unsigned(2997,16); cos_data_int <= to_unsigned(36524,16);
     when "010001010011" => sin_data_int <= to_unsigned(3003,16); cos_data_int <= to_unsigned(36569,16);
     when "010001010100" => sin_data_int <= to_unsigned(3009,16); cos_data_int <= to_unsigned(36615,16);
     when "010001010101" => sin_data_int <= to_unsigned(3015,16); cos_data_int <= to_unsigned(36661,16);
     when "010001010110" => sin_data_int <= to_unsigned(3021,16); cos_data_int <= to_unsigned(36706,16);
     when "010001010111" => sin_data_int <= to_unsigned(3027,16); cos_data_int <= to_unsigned(36752,16);
     when "010001011000" => sin_data_int <= to_unsigned(3033,16); cos_data_int <= to_unsigned(36797,16);
     when "010001011001" => sin_data_int <= to_unsigned(3039,16); cos_data_int <= to_unsigned(36843,16);
     when "010001011010" => sin_data_int <= to_unsigned(3045,16); cos_data_int <= to_unsigned(36889,16);
     when "010001011011" => sin_data_int <= to_unsigned(3052,16); cos_data_int <= to_unsigned(36934,16);
     when "010001011100" => sin_data_int <= to_unsigned(3058,16); cos_data_int <= to_unsigned(36980,16);
     when "010001011101" => sin_data_int <= to_unsigned(3065,16); cos_data_int <= to_unsigned(37025,16);
     when "010001011110" => sin_data_int <= to_unsigned(3071,16); cos_data_int <= to_unsigned(37071,16);
     when "010001011111" => sin_data_int <= to_unsigned(3078,16); cos_data_int <= to_unsigned(37116,16);
     when "010001100000" => sin_data_int <= to_unsigned(3085,16); cos_data_int <= to_unsigned(37162,16);
     when "010001100001" => sin_data_int <= to_unsigned(3091,16); cos_data_int <= to_unsigned(37207,16);
     when "010001100010" => sin_data_int <= to_unsigned(3098,16); cos_data_int <= to_unsigned(37253,16);
     when "010001100011" => sin_data_int <= to_unsigned(3105,16); cos_data_int <= to_unsigned(37298,16);
     when "010001100100" => sin_data_int <= to_unsigned(3112,16); cos_data_int <= to_unsigned(37344,16);
     when "010001100101" => sin_data_int <= to_unsigned(3119,16); cos_data_int <= to_unsigned(37389,16);
     when "010001100110" => sin_data_int <= to_unsigned(3126,16); cos_data_int <= to_unsigned(37435,16);
     when "010001100111" => sin_data_int <= to_unsigned(3134,16); cos_data_int <= to_unsigned(37480,16);
     when "010001101000" => sin_data_int <= to_unsigned(3141,16); cos_data_int <= to_unsigned(37526,16);
     when "010001101001" => sin_data_int <= to_unsigned(3148,16); cos_data_int <= to_unsigned(37571,16);
     when "010001101010" => sin_data_int <= to_unsigned(3156,16); cos_data_int <= to_unsigned(37617,16);
     when "010001101011" => sin_data_int <= to_unsigned(3163,16); cos_data_int <= to_unsigned(37662,16);
     when "010001101100" => sin_data_int <= to_unsigned(3171,16); cos_data_int <= to_unsigned(37707,16);
     when "010001101101" => sin_data_int <= to_unsigned(3178,16); cos_data_int <= to_unsigned(37753,16);
     when "010001101110" => sin_data_int <= to_unsigned(3186,16); cos_data_int <= to_unsigned(37798,16);
     when "010001101111" => sin_data_int <= to_unsigned(3194,16); cos_data_int <= to_unsigned(37844,16);
     when "010001110000" => sin_data_int <= to_unsigned(3202,16); cos_data_int <= to_unsigned(37889,16);
     when "010001110001" => sin_data_int <= to_unsigned(3210,16); cos_data_int <= to_unsigned(37934,16);
     when "010001110010" => sin_data_int <= to_unsigned(3218,16); cos_data_int <= to_unsigned(37980,16);
     when "010001110011" => sin_data_int <= to_unsigned(3226,16); cos_data_int <= to_unsigned(38025,16);
     when "010001110100" => sin_data_int <= to_unsigned(3234,16); cos_data_int <= to_unsigned(38070,16);
     when "010001110101" => sin_data_int <= to_unsigned(3242,16); cos_data_int <= to_unsigned(38115,16);
     when "010001110110" => sin_data_int <= to_unsigned(3250,16); cos_data_int <= to_unsigned(38161,16);
     when "010001110111" => sin_data_int <= to_unsigned(3258,16); cos_data_int <= to_unsigned(38206,16);
     when "010001111000" => sin_data_int <= to_unsigned(3267,16); cos_data_int <= to_unsigned(38251,16);
     when "010001111001" => sin_data_int <= to_unsigned(3275,16); cos_data_int <= to_unsigned(38296,16);
     when "010001111010" => sin_data_int <= to_unsigned(3284,16); cos_data_int <= to_unsigned(38342,16);
     when "010001111011" => sin_data_int <= to_unsigned(3292,16); cos_data_int <= to_unsigned(38387,16);
     when "010001111100" => sin_data_int <= to_unsigned(3301,16); cos_data_int <= to_unsigned(38432,16);
     when "010001111101" => sin_data_int <= to_unsigned(3310,16); cos_data_int <= to_unsigned(38477,16);
     when "010001111110" => sin_data_int <= to_unsigned(3319,16); cos_data_int <= to_unsigned(38522,16);
     when "010001111111" => sin_data_int <= to_unsigned(3327,16); cos_data_int <= to_unsigned(38568,16);
     when "010010000000" => sin_data_int <= to_unsigned(3336,16); cos_data_int <= to_unsigned(38613,16);
     when "010010000001" => sin_data_int <= to_unsigned(3345,16); cos_data_int <= to_unsigned(38658,16);
     when "010010000010" => sin_data_int <= to_unsigned(3355,16); cos_data_int <= to_unsigned(38703,16);
     when "010010000011" => sin_data_int <= to_unsigned(3364,16); cos_data_int <= to_unsigned(38748,16);
     when "010010000100" => sin_data_int <= to_unsigned(3373,16); cos_data_int <= to_unsigned(38793,16);
     when "010010000101" => sin_data_int <= to_unsigned(3382,16); cos_data_int <= to_unsigned(38838,16);
     when "010010000110" => sin_data_int <= to_unsigned(3392,16); cos_data_int <= to_unsigned(38883,16);
     when "010010000111" => sin_data_int <= to_unsigned(3401,16); cos_data_int <= to_unsigned(38928,16);
     when "010010001000" => sin_data_int <= to_unsigned(3410,16); cos_data_int <= to_unsigned(38973,16);
     when "010010001001" => sin_data_int <= to_unsigned(3420,16); cos_data_int <= to_unsigned(39018,16);
     when "010010001010" => sin_data_int <= to_unsigned(3430,16); cos_data_int <= to_unsigned(39063,16);
     when "010010001011" => sin_data_int <= to_unsigned(3439,16); cos_data_int <= to_unsigned(39108,16);
     when "010010001100" => sin_data_int <= to_unsigned(3449,16); cos_data_int <= to_unsigned(39153,16);
     when "010010001101" => sin_data_int <= to_unsigned(3459,16); cos_data_int <= to_unsigned(39198,16);
     when "010010001110" => sin_data_int <= to_unsigned(3469,16); cos_data_int <= to_unsigned(39243,16);
     when "010010001111" => sin_data_int <= to_unsigned(3479,16); cos_data_int <= to_unsigned(39288,16);
     when "010010010000" => sin_data_int <= to_unsigned(3489,16); cos_data_int <= to_unsigned(39333,16);
     when "010010010001" => sin_data_int <= to_unsigned(3499,16); cos_data_int <= to_unsigned(39378,16);
     when "010010010010" => sin_data_int <= to_unsigned(3509,16); cos_data_int <= to_unsigned(39423,16);
     when "010010010011" => sin_data_int <= to_unsigned(3519,16); cos_data_int <= to_unsigned(39468,16);
     when "010010010100" => sin_data_int <= to_unsigned(3530,16); cos_data_int <= to_unsigned(39513,16);
     when "010010010101" => sin_data_int <= to_unsigned(3540,16); cos_data_int <= to_unsigned(39557,16);
     when "010010010110" => sin_data_int <= to_unsigned(3551,16); cos_data_int <= to_unsigned(39602,16);
     when "010010010111" => sin_data_int <= to_unsigned(3561,16); cos_data_int <= to_unsigned(39647,16);
     when "010010011000" => sin_data_int <= to_unsigned(3572,16); cos_data_int <= to_unsigned(39692,16);
     when "010010011001" => sin_data_int <= to_unsigned(3582,16); cos_data_int <= to_unsigned(39737,16);
     when "010010011010" => sin_data_int <= to_unsigned(3593,16); cos_data_int <= to_unsigned(39781,16);
     when "010010011011" => sin_data_int <= to_unsigned(3604,16); cos_data_int <= to_unsigned(39826,16);
     when "010010011100" => sin_data_int <= to_unsigned(3615,16); cos_data_int <= to_unsigned(39871,16);
     when "010010011101" => sin_data_int <= to_unsigned(3626,16); cos_data_int <= to_unsigned(39915,16);
     when "010010011110" => sin_data_int <= to_unsigned(3637,16); cos_data_int <= to_unsigned(39960,16);
     when "010010011111" => sin_data_int <= to_unsigned(3648,16); cos_data_int <= to_unsigned(40005,16);
     when "010010100000" => sin_data_int <= to_unsigned(3659,16); cos_data_int <= to_unsigned(40049,16);
     when "010010100001" => sin_data_int <= to_unsigned(3670,16); cos_data_int <= to_unsigned(40094,16);
     when "010010100010" => sin_data_int <= to_unsigned(3682,16); cos_data_int <= to_unsigned(40139,16);
     when "010010100011" => sin_data_int <= to_unsigned(3693,16); cos_data_int <= to_unsigned(40183,16);
     when "010010100100" => sin_data_int <= to_unsigned(3704,16); cos_data_int <= to_unsigned(40228,16);
     when "010010100101" => sin_data_int <= to_unsigned(3716,16); cos_data_int <= to_unsigned(40272,16);
     when "010010100110" => sin_data_int <= to_unsigned(3727,16); cos_data_int <= to_unsigned(40317,16);
     when "010010100111" => sin_data_int <= to_unsigned(3739,16); cos_data_int <= to_unsigned(40361,16);
     when "010010101000" => sin_data_int <= to_unsigned(3751,16); cos_data_int <= to_unsigned(40406,16);
     when "010010101001" => sin_data_int <= to_unsigned(3762,16); cos_data_int <= to_unsigned(40450,16);
     when "010010101010" => sin_data_int <= to_unsigned(3774,16); cos_data_int <= to_unsigned(40495,16);
     when "010010101011" => sin_data_int <= to_unsigned(3786,16); cos_data_int <= to_unsigned(40539,16);
     when "010010101100" => sin_data_int <= to_unsigned(3798,16); cos_data_int <= to_unsigned(40584,16);
     when "010010101101" => sin_data_int <= to_unsigned(3810,16); cos_data_int <= to_unsigned(40628,16);
     when "010010101110" => sin_data_int <= to_unsigned(3822,16); cos_data_int <= to_unsigned(40673,16);
     when "010010101111" => sin_data_int <= to_unsigned(3834,16); cos_data_int <= to_unsigned(40717,16);
     when "010010110000" => sin_data_int <= to_unsigned(3847,16); cos_data_int <= to_unsigned(40761,16);
     when "010010110001" => sin_data_int <= to_unsigned(3859,16); cos_data_int <= to_unsigned(40806,16);
     when "010010110010" => sin_data_int <= to_unsigned(3871,16); cos_data_int <= to_unsigned(40850,16);
     when "010010110011" => sin_data_int <= to_unsigned(3884,16); cos_data_int <= to_unsigned(40894,16);
     when "010010110100" => sin_data_int <= to_unsigned(3896,16); cos_data_int <= to_unsigned(40939,16);
     when "010010110101" => sin_data_int <= to_unsigned(3909,16); cos_data_int <= to_unsigned(40983,16);
     when "010010110110" => sin_data_int <= to_unsigned(3922,16); cos_data_int <= to_unsigned(41027,16);
     when "010010110111" => sin_data_int <= to_unsigned(3934,16); cos_data_int <= to_unsigned(41071,16);
     when "010010111000" => sin_data_int <= to_unsigned(3947,16); cos_data_int <= to_unsigned(41116,16);
     when "010010111001" => sin_data_int <= to_unsigned(3960,16); cos_data_int <= to_unsigned(41160,16);
     when "010010111010" => sin_data_int <= to_unsigned(3973,16); cos_data_int <= to_unsigned(41204,16);
     when "010010111011" => sin_data_int <= to_unsigned(3986,16); cos_data_int <= to_unsigned(41248,16);
     when "010010111100" => sin_data_int <= to_unsigned(3999,16); cos_data_int <= to_unsigned(41292,16);
     when "010010111101" => sin_data_int <= to_unsigned(4012,16); cos_data_int <= to_unsigned(41336,16);
     when "010010111110" => sin_data_int <= to_unsigned(4025,16); cos_data_int <= to_unsigned(41380,16);
     when "010010111111" => sin_data_int <= to_unsigned(4038,16); cos_data_int <= to_unsigned(41424,16);
     when "010011000000" => sin_data_int <= to_unsigned(4052,16); cos_data_int <= to_unsigned(41469,16);
     when "010011000001" => sin_data_int <= to_unsigned(4065,16); cos_data_int <= to_unsigned(41513,16);
     when "010011000010" => sin_data_int <= to_unsigned(4079,16); cos_data_int <= to_unsigned(41557,16);
     when "010011000011" => sin_data_int <= to_unsigned(4092,16); cos_data_int <= to_unsigned(41601,16);
     when "010011000100" => sin_data_int <= to_unsigned(4106,16); cos_data_int <= to_unsigned(41645,16);
     when "010011000101" => sin_data_int <= to_unsigned(4119,16); cos_data_int <= to_unsigned(41688,16);
     when "010011000110" => sin_data_int <= to_unsigned(4133,16); cos_data_int <= to_unsigned(41732,16);
     when "010011000111" => sin_data_int <= to_unsigned(4147,16); cos_data_int <= to_unsigned(41776,16);
     when "010011001000" => sin_data_int <= to_unsigned(4161,16); cos_data_int <= to_unsigned(41820,16);
     when "010011001001" => sin_data_int <= to_unsigned(4175,16); cos_data_int <= to_unsigned(41864,16);
     when "010011001010" => sin_data_int <= to_unsigned(4189,16); cos_data_int <= to_unsigned(41908,16);
     when "010011001011" => sin_data_int <= to_unsigned(4203,16); cos_data_int <= to_unsigned(41952,16);
     when "010011001100" => sin_data_int <= to_unsigned(4217,16); cos_data_int <= to_unsigned(41995,16);
     when "010011001101" => sin_data_int <= to_unsigned(4231,16); cos_data_int <= to_unsigned(42039,16);
     when "010011001110" => sin_data_int <= to_unsigned(4245,16); cos_data_int <= to_unsigned(42083,16);
     when "010011001111" => sin_data_int <= to_unsigned(4260,16); cos_data_int <= to_unsigned(42127,16);
     when "010011010000" => sin_data_int <= to_unsigned(4274,16); cos_data_int <= to_unsigned(42170,16);
     when "010011010001" => sin_data_int <= to_unsigned(4289,16); cos_data_int <= to_unsigned(42214,16);
     when "010011010010" => sin_data_int <= to_unsigned(4303,16); cos_data_int <= to_unsigned(42258,16);
     when "010011010011" => sin_data_int <= to_unsigned(4318,16); cos_data_int <= to_unsigned(42301,16);
     when "010011010100" => sin_data_int <= to_unsigned(4332,16); cos_data_int <= to_unsigned(42345,16);
     when "010011010101" => sin_data_int <= to_unsigned(4347,16); cos_data_int <= to_unsigned(42389,16);
     when "010011010110" => sin_data_int <= to_unsigned(4362,16); cos_data_int <= to_unsigned(42432,16);
     when "010011010111" => sin_data_int <= to_unsigned(4377,16); cos_data_int <= to_unsigned(42476,16);
     when "010011011000" => sin_data_int <= to_unsigned(4392,16); cos_data_int <= to_unsigned(42519,16);
     when "010011011001" => sin_data_int <= to_unsigned(4407,16); cos_data_int <= to_unsigned(42563,16);
     when "010011011010" => sin_data_int <= to_unsigned(4422,16); cos_data_int <= to_unsigned(42606,16);
     when "010011011011" => sin_data_int <= to_unsigned(4437,16); cos_data_int <= to_unsigned(42650,16);
     when "010011011100" => sin_data_int <= to_unsigned(4452,16); cos_data_int <= to_unsigned(42693,16);
     when "010011011101" => sin_data_int <= to_unsigned(4467,16); cos_data_int <= to_unsigned(42737,16);
     when "010011011110" => sin_data_int <= to_unsigned(4483,16); cos_data_int <= to_unsigned(42780,16);
     when "010011011111" => sin_data_int <= to_unsigned(4498,16); cos_data_int <= to_unsigned(42823,16);
     when "010011100000" => sin_data_int <= to_unsigned(4514,16); cos_data_int <= to_unsigned(42867,16);
     when "010011100001" => sin_data_int <= to_unsigned(4529,16); cos_data_int <= to_unsigned(42910,16);
     when "010011100010" => sin_data_int <= to_unsigned(4545,16); cos_data_int <= to_unsigned(42953,16);
     when "010011100011" => sin_data_int <= to_unsigned(4560,16); cos_data_int <= to_unsigned(42997,16);
     when "010011100100" => sin_data_int <= to_unsigned(4576,16); cos_data_int <= to_unsigned(43040,16);
     when "010011100101" => sin_data_int <= to_unsigned(4592,16); cos_data_int <= to_unsigned(43083,16);
     when "010011100110" => sin_data_int <= to_unsigned(4608,16); cos_data_int <= to_unsigned(43126,16);
     when "010011100111" => sin_data_int <= to_unsigned(4624,16); cos_data_int <= to_unsigned(43169,16);
     when "010011101000" => sin_data_int <= to_unsigned(4640,16); cos_data_int <= to_unsigned(43213,16);
     when "010011101001" => sin_data_int <= to_unsigned(4656,16); cos_data_int <= to_unsigned(43256,16);
     when "010011101010" => sin_data_int <= to_unsigned(4672,16); cos_data_int <= to_unsigned(43299,16);
     when "010011101011" => sin_data_int <= to_unsigned(4688,16); cos_data_int <= to_unsigned(43342,16);
     when "010011101100" => sin_data_int <= to_unsigned(4704,16); cos_data_int <= to_unsigned(43385,16);
     when "010011101101" => sin_data_int <= to_unsigned(4721,16); cos_data_int <= to_unsigned(43428,16);
     when "010011101110" => sin_data_int <= to_unsigned(4737,16); cos_data_int <= to_unsigned(43471,16);
     when "010011101111" => sin_data_int <= to_unsigned(4754,16); cos_data_int <= to_unsigned(43514,16);
     when "010011110000" => sin_data_int <= to_unsigned(4770,16); cos_data_int <= to_unsigned(43557,16);
     when "010011110001" => sin_data_int <= to_unsigned(4787,16); cos_data_int <= to_unsigned(43600,16);
     when "010011110010" => sin_data_int <= to_unsigned(4803,16); cos_data_int <= to_unsigned(43643,16);
     when "010011110011" => sin_data_int <= to_unsigned(4820,16); cos_data_int <= to_unsigned(43686,16);
     when "010011110100" => sin_data_int <= to_unsigned(4837,16); cos_data_int <= to_unsigned(43728,16);
     when "010011110101" => sin_data_int <= to_unsigned(4854,16); cos_data_int <= to_unsigned(43771,16);
     when "010011110110" => sin_data_int <= to_unsigned(4871,16); cos_data_int <= to_unsigned(43814,16);
     when "010011110111" => sin_data_int <= to_unsigned(4888,16); cos_data_int <= to_unsigned(43857,16);
     when "010011111000" => sin_data_int <= to_unsigned(4905,16); cos_data_int <= to_unsigned(43900,16);
     when "010011111001" => sin_data_int <= to_unsigned(4922,16); cos_data_int <= to_unsigned(43942,16);
     when "010011111010" => sin_data_int <= to_unsigned(4939,16); cos_data_int <= to_unsigned(43985,16);
     when "010011111011" => sin_data_int <= to_unsigned(4956,16); cos_data_int <= to_unsigned(44028,16);
     when "010011111100" => sin_data_int <= to_unsigned(4974,16); cos_data_int <= to_unsigned(44070,16);
     when "010011111101" => sin_data_int <= to_unsigned(4991,16); cos_data_int <= to_unsigned(44113,16);
     when "010011111110" => sin_data_int <= to_unsigned(5009,16); cos_data_int <= to_unsigned(44155,16);
     when "010011111111" => sin_data_int <= to_unsigned(5026,16); cos_data_int <= to_unsigned(44198,16);
     when "010100000000" => sin_data_int <= to_unsigned(5044,16); cos_data_int <= to_unsigned(44241,16);
     when "010100000001" => sin_data_int <= to_unsigned(5061,16); cos_data_int <= to_unsigned(44283,16);
     when "010100000010" => sin_data_int <= to_unsigned(5079,16); cos_data_int <= to_unsigned(44325,16);
     when "010100000011" => sin_data_int <= to_unsigned(5097,16); cos_data_int <= to_unsigned(44368,16);
     when "010100000100" => sin_data_int <= to_unsigned(5115,16); cos_data_int <= to_unsigned(44410,16);
     when "010100000101" => sin_data_int <= to_unsigned(5132,16); cos_data_int <= to_unsigned(44453,16);
     when "010100000110" => sin_data_int <= to_unsigned(5150,16); cos_data_int <= to_unsigned(44495,16);
     when "010100000111" => sin_data_int <= to_unsigned(5168,16); cos_data_int <= to_unsigned(44537,16);
     when "010100001000" => sin_data_int <= to_unsigned(5187,16); cos_data_int <= to_unsigned(44580,16);
     when "010100001001" => sin_data_int <= to_unsigned(5205,16); cos_data_int <= to_unsigned(44622,16);
     when "010100001010" => sin_data_int <= to_unsigned(5223,16); cos_data_int <= to_unsigned(44664,16);
     when "010100001011" => sin_data_int <= to_unsigned(5241,16); cos_data_int <= to_unsigned(44707,16);
     when "010100001100" => sin_data_int <= to_unsigned(5260,16); cos_data_int <= to_unsigned(44749,16);
     when "010100001101" => sin_data_int <= to_unsigned(5278,16); cos_data_int <= to_unsigned(44791,16);
     when "010100001110" => sin_data_int <= to_unsigned(5297,16); cos_data_int <= to_unsigned(44833,16);
     when "010100001111" => sin_data_int <= to_unsigned(5315,16); cos_data_int <= to_unsigned(44875,16);
     when "010100010000" => sin_data_int <= to_unsigned(5334,16); cos_data_int <= to_unsigned(44917,16);
     when "010100010001" => sin_data_int <= to_unsigned(5352,16); cos_data_int <= to_unsigned(44959,16);
     when "010100010010" => sin_data_int <= to_unsigned(5371,16); cos_data_int <= to_unsigned(45001,16);
     when "010100010011" => sin_data_int <= to_unsigned(5390,16); cos_data_int <= to_unsigned(45043,16);
     when "010100010100" => sin_data_int <= to_unsigned(5409,16); cos_data_int <= to_unsigned(45085,16);
     when "010100010101" => sin_data_int <= to_unsigned(5428,16); cos_data_int <= to_unsigned(45127,16);
     when "010100010110" => sin_data_int <= to_unsigned(5447,16); cos_data_int <= to_unsigned(45169,16);
     when "010100010111" => sin_data_int <= to_unsigned(5466,16); cos_data_int <= to_unsigned(45211,16);
     when "010100011000" => sin_data_int <= to_unsigned(5485,16); cos_data_int <= to_unsigned(45253,16);
     when "010100011001" => sin_data_int <= to_unsigned(5504,16); cos_data_int <= to_unsigned(45295,16);
     when "010100011010" => sin_data_int <= to_unsigned(5523,16); cos_data_int <= to_unsigned(45337,16);
     when "010100011011" => sin_data_int <= to_unsigned(5543,16); cos_data_int <= to_unsigned(45378,16);
     when "010100011100" => sin_data_int <= to_unsigned(5562,16); cos_data_int <= to_unsigned(45420,16);
     when "010100011101" => sin_data_int <= to_unsigned(5582,16); cos_data_int <= to_unsigned(45462,16);
     when "010100011110" => sin_data_int <= to_unsigned(5601,16); cos_data_int <= to_unsigned(45503,16);
     when "010100011111" => sin_data_int <= to_unsigned(5621,16); cos_data_int <= to_unsigned(45545,16);
     when "010100100000" => sin_data_int <= to_unsigned(5640,16); cos_data_int <= to_unsigned(45587,16);
     when "010100100001" => sin_data_int <= to_unsigned(5660,16); cos_data_int <= to_unsigned(45628,16);
     when "010100100010" => sin_data_int <= to_unsigned(5680,16); cos_data_int <= to_unsigned(45670,16);
     when "010100100011" => sin_data_int <= to_unsigned(5700,16); cos_data_int <= to_unsigned(45711,16);
     when "010100100100" => sin_data_int <= to_unsigned(5720,16); cos_data_int <= to_unsigned(45753,16);
     when "010100100101" => sin_data_int <= to_unsigned(5739,16); cos_data_int <= to_unsigned(45794,16);
     when "010100100110" => sin_data_int <= to_unsigned(5760,16); cos_data_int <= to_unsigned(45836,16);
     when "010100100111" => sin_data_int <= to_unsigned(5780,16); cos_data_int <= to_unsigned(45877,16);
     when "010100101000" => sin_data_int <= to_unsigned(5800,16); cos_data_int <= to_unsigned(45918,16);
     when "010100101001" => sin_data_int <= to_unsigned(5820,16); cos_data_int <= to_unsigned(45960,16);
     when "010100101010" => sin_data_int <= to_unsigned(5840,16); cos_data_int <= to_unsigned(46001,16);
     when "010100101011" => sin_data_int <= to_unsigned(5861,16); cos_data_int <= to_unsigned(46042,16);
     when "010100101100" => sin_data_int <= to_unsigned(5881,16); cos_data_int <= to_unsigned(46084,16);
     when "010100101101" => sin_data_int <= to_unsigned(5901,16); cos_data_int <= to_unsigned(46125,16);
     when "010100101110" => sin_data_int <= to_unsigned(5922,16); cos_data_int <= to_unsigned(46166,16);
     when "010100101111" => sin_data_int <= to_unsigned(5943,16); cos_data_int <= to_unsigned(46207,16);
     when "010100110000" => sin_data_int <= to_unsigned(5963,16); cos_data_int <= to_unsigned(46248,16);
     when "010100110001" => sin_data_int <= to_unsigned(5984,16); cos_data_int <= to_unsigned(46289,16);
     when "010100110010" => sin_data_int <= to_unsigned(6005,16); cos_data_int <= to_unsigned(46330,16);
     when "010100110011" => sin_data_int <= to_unsigned(6026,16); cos_data_int <= to_unsigned(46372,16);
     when "010100110100" => sin_data_int <= to_unsigned(6047,16); cos_data_int <= to_unsigned(46413,16);
     when "010100110101" => sin_data_int <= to_unsigned(6068,16); cos_data_int <= to_unsigned(46453,16);
     when "010100110110" => sin_data_int <= to_unsigned(6089,16); cos_data_int <= to_unsigned(46494,16);
     when "010100110111" => sin_data_int <= to_unsigned(6110,16); cos_data_int <= to_unsigned(46535,16);
     when "010100111000" => sin_data_int <= to_unsigned(6131,16); cos_data_int <= to_unsigned(46576,16);
     when "010100111001" => sin_data_int <= to_unsigned(6152,16); cos_data_int <= to_unsigned(46617,16);
     when "010100111010" => sin_data_int <= to_unsigned(6173,16); cos_data_int <= to_unsigned(46658,16);
     when "010100111011" => sin_data_int <= to_unsigned(6195,16); cos_data_int <= to_unsigned(46699,16);
     when "010100111100" => sin_data_int <= to_unsigned(6216,16); cos_data_int <= to_unsigned(46739,16);
     when "010100111101" => sin_data_int <= to_unsigned(6238,16); cos_data_int <= to_unsigned(46780,16);
     when "010100111110" => sin_data_int <= to_unsigned(6259,16); cos_data_int <= to_unsigned(46821,16);
     when "010100111111" => sin_data_int <= to_unsigned(6281,16); cos_data_int <= to_unsigned(46861,16);
     when "010101000000" => sin_data_int <= to_unsigned(6302,16); cos_data_int <= to_unsigned(46902,16);
     when "010101000001" => sin_data_int <= to_unsigned(6324,16); cos_data_int <= to_unsigned(46942,16);
     when "010101000010" => sin_data_int <= to_unsigned(6346,16); cos_data_int <= to_unsigned(46983,16);
     when "010101000011" => sin_data_int <= to_unsigned(6368,16); cos_data_int <= to_unsigned(47024,16);
     when "010101000100" => sin_data_int <= to_unsigned(6390,16); cos_data_int <= to_unsigned(47064,16);
     when "010101000101" => sin_data_int <= to_unsigned(6412,16); cos_data_int <= to_unsigned(47104,16);
     when "010101000110" => sin_data_int <= to_unsigned(6434,16); cos_data_int <= to_unsigned(47145,16);
     when "010101000111" => sin_data_int <= to_unsigned(6456,16); cos_data_int <= to_unsigned(47185,16);
     when "010101001000" => sin_data_int <= to_unsigned(6478,16); cos_data_int <= to_unsigned(47226,16);
     when "010101001001" => sin_data_int <= to_unsigned(6500,16); cos_data_int <= to_unsigned(47266,16);
     when "010101001010" => sin_data_int <= to_unsigned(6522,16); cos_data_int <= to_unsigned(47306,16);
     when "010101001011" => sin_data_int <= to_unsigned(6545,16); cos_data_int <= to_unsigned(47346,16);
     when "010101001100" => sin_data_int <= to_unsigned(6567,16); cos_data_int <= to_unsigned(47387,16);
     when "010101001101" => sin_data_int <= to_unsigned(6590,16); cos_data_int <= to_unsigned(47427,16);
     when "010101001110" => sin_data_int <= to_unsigned(6612,16); cos_data_int <= to_unsigned(47467,16);
     when "010101001111" => sin_data_int <= to_unsigned(6635,16); cos_data_int <= to_unsigned(47507,16);
     when "010101010000" => sin_data_int <= to_unsigned(6657,16); cos_data_int <= to_unsigned(47547,16);
     when "010101010001" => sin_data_int <= to_unsigned(6680,16); cos_data_int <= to_unsigned(47587,16);
     when "010101010010" => sin_data_int <= to_unsigned(6703,16); cos_data_int <= to_unsigned(47627,16);
     when "010101010011" => sin_data_int <= to_unsigned(6726,16); cos_data_int <= to_unsigned(47667,16);
     when "010101010100" => sin_data_int <= to_unsigned(6749,16); cos_data_int <= to_unsigned(47707,16);
     when "010101010101" => sin_data_int <= to_unsigned(6772,16); cos_data_int <= to_unsigned(47747,16);
     when "010101010110" => sin_data_int <= to_unsigned(6795,16); cos_data_int <= to_unsigned(47787,16);
     when "010101010111" => sin_data_int <= to_unsigned(6818,16); cos_data_int <= to_unsigned(47826,16);
     when "010101011000" => sin_data_int <= to_unsigned(6841,16); cos_data_int <= to_unsigned(47866,16);
     when "010101011001" => sin_data_int <= to_unsigned(6864,16); cos_data_int <= to_unsigned(47906,16);
     when "010101011010" => sin_data_int <= to_unsigned(6887,16); cos_data_int <= to_unsigned(47946,16);
     when "010101011011" => sin_data_int <= to_unsigned(6911,16); cos_data_int <= to_unsigned(47985,16);
     when "010101011100" => sin_data_int <= to_unsigned(6934,16); cos_data_int <= to_unsigned(48025,16);
     when "010101011101" => sin_data_int <= to_unsigned(6957,16); cos_data_int <= to_unsigned(48065,16);
     when "010101011110" => sin_data_int <= to_unsigned(6981,16); cos_data_int <= to_unsigned(48104,16);
     when "010101011111" => sin_data_int <= to_unsigned(7005,16); cos_data_int <= to_unsigned(48144,16);
     when "010101100000" => sin_data_int <= to_unsigned(7028,16); cos_data_int <= to_unsigned(48183,16);
     when "010101100001" => sin_data_int <= to_unsigned(7052,16); cos_data_int <= to_unsigned(48223,16);
     when "010101100010" => sin_data_int <= to_unsigned(7076,16); cos_data_int <= to_unsigned(48262,16);
     when "010101100011" => sin_data_int <= to_unsigned(7099,16); cos_data_int <= to_unsigned(48301,16);
     when "010101100100" => sin_data_int <= to_unsigned(7123,16); cos_data_int <= to_unsigned(48341,16);
     when "010101100101" => sin_data_int <= to_unsigned(7147,16); cos_data_int <= to_unsigned(48380,16);
     when "010101100110" => sin_data_int <= to_unsigned(7171,16); cos_data_int <= to_unsigned(48419,16);
     when "010101100111" => sin_data_int <= to_unsigned(7195,16); cos_data_int <= to_unsigned(48458,16);
     when "010101101000" => sin_data_int <= to_unsigned(7219,16); cos_data_int <= to_unsigned(48498,16);
     when "010101101001" => sin_data_int <= to_unsigned(7244,16); cos_data_int <= to_unsigned(48537,16);
     when "010101101010" => sin_data_int <= to_unsigned(7268,16); cos_data_int <= to_unsigned(48576,16);
     when "010101101011" => sin_data_int <= to_unsigned(7292,16); cos_data_int <= to_unsigned(48615,16);
     when "010101101100" => sin_data_int <= to_unsigned(7316,16); cos_data_int <= to_unsigned(48654,16);
     when "010101101101" => sin_data_int <= to_unsigned(7341,16); cos_data_int <= to_unsigned(48693,16);
     when "010101101110" => sin_data_int <= to_unsigned(7365,16); cos_data_int <= to_unsigned(48732,16);
     when "010101101111" => sin_data_int <= to_unsigned(7390,16); cos_data_int <= to_unsigned(48771,16);
     when "010101110000" => sin_data_int <= to_unsigned(7414,16); cos_data_int <= to_unsigned(48810,16);
     when "010101110001" => sin_data_int <= to_unsigned(7439,16); cos_data_int <= to_unsigned(48849,16);
     when "010101110010" => sin_data_int <= to_unsigned(7464,16); cos_data_int <= to_unsigned(48888,16);
     when "010101110011" => sin_data_int <= to_unsigned(7489,16); cos_data_int <= to_unsigned(48926,16);
     when "010101110100" => sin_data_int <= to_unsigned(7513,16); cos_data_int <= to_unsigned(48965,16);
     when "010101110101" => sin_data_int <= to_unsigned(7538,16); cos_data_int <= to_unsigned(49004,16);
     when "010101110110" => sin_data_int <= to_unsigned(7563,16); cos_data_int <= to_unsigned(49043,16);
     when "010101110111" => sin_data_int <= to_unsigned(7588,16); cos_data_int <= to_unsigned(49081,16);
     when "010101111000" => sin_data_int <= to_unsigned(7613,16); cos_data_int <= to_unsigned(49120,16);
     when "010101111001" => sin_data_int <= to_unsigned(7638,16); cos_data_int <= to_unsigned(49158,16);
     when "010101111010" => sin_data_int <= to_unsigned(7664,16); cos_data_int <= to_unsigned(49197,16);
     when "010101111011" => sin_data_int <= to_unsigned(7689,16); cos_data_int <= to_unsigned(49235,16);
     when "010101111100" => sin_data_int <= to_unsigned(7714,16); cos_data_int <= to_unsigned(49274,16);
     when "010101111101" => sin_data_int <= to_unsigned(7739,16); cos_data_int <= to_unsigned(49312,16);
     when "010101111110" => sin_data_int <= to_unsigned(7765,16); cos_data_int <= to_unsigned(49351,16);
     when "010101111111" => sin_data_int <= to_unsigned(7790,16); cos_data_int <= to_unsigned(49389,16);
     when "010110000000" => sin_data_int <= to_unsigned(7816,16); cos_data_int <= to_unsigned(49427,16);
     when "010110000001" => sin_data_int <= to_unsigned(7842,16); cos_data_int <= to_unsigned(49465,16);
     when "010110000010" => sin_data_int <= to_unsigned(7867,16); cos_data_int <= to_unsigned(49504,16);
     when "010110000011" => sin_data_int <= to_unsigned(7893,16); cos_data_int <= to_unsigned(49542,16);
     when "010110000100" => sin_data_int <= to_unsigned(7919,16); cos_data_int <= to_unsigned(49580,16);
     when "010110000101" => sin_data_int <= to_unsigned(7944,16); cos_data_int <= to_unsigned(49618,16);
     when "010110000110" => sin_data_int <= to_unsigned(7970,16); cos_data_int <= to_unsigned(49656,16);
     when "010110000111" => sin_data_int <= to_unsigned(7996,16); cos_data_int <= to_unsigned(49694,16);
     when "010110001000" => sin_data_int <= to_unsigned(8022,16); cos_data_int <= to_unsigned(49732,16);
     when "010110001001" => sin_data_int <= to_unsigned(8048,16); cos_data_int <= to_unsigned(49770,16);
     when "010110001010" => sin_data_int <= to_unsigned(8075,16); cos_data_int <= to_unsigned(49808,16);
     when "010110001011" => sin_data_int <= to_unsigned(8101,16); cos_data_int <= to_unsigned(49846,16);
     when "010110001100" => sin_data_int <= to_unsigned(8127,16); cos_data_int <= to_unsigned(49883,16);
     when "010110001101" => sin_data_int <= to_unsigned(8153,16); cos_data_int <= to_unsigned(49921,16);
     when "010110001110" => sin_data_int <= to_unsigned(8180,16); cos_data_int <= to_unsigned(49959,16);
     when "010110001111" => sin_data_int <= to_unsigned(8206,16); cos_data_int <= to_unsigned(49997,16);
     when "010110010000" => sin_data_int <= to_unsigned(8232,16); cos_data_int <= to_unsigned(50034,16);
     when "010110010001" => sin_data_int <= to_unsigned(8259,16); cos_data_int <= to_unsigned(50072,16);
     when "010110010010" => sin_data_int <= to_unsigned(8286,16); cos_data_int <= to_unsigned(50109,16);
     when "010110010011" => sin_data_int <= to_unsigned(8312,16); cos_data_int <= to_unsigned(50147,16);
     when "010110010100" => sin_data_int <= to_unsigned(8339,16); cos_data_int <= to_unsigned(50184,16);
     when "010110010101" => sin_data_int <= to_unsigned(8366,16); cos_data_int <= to_unsigned(50222,16);
     when "010110010110" => sin_data_int <= to_unsigned(8392,16); cos_data_int <= to_unsigned(50259,16);
     when "010110010111" => sin_data_int <= to_unsigned(8419,16); cos_data_int <= to_unsigned(50297,16);
     when "010110011000" => sin_data_int <= to_unsigned(8446,16); cos_data_int <= to_unsigned(50334,16);
     when "010110011001" => sin_data_int <= to_unsigned(8473,16); cos_data_int <= to_unsigned(50371,16);
     when "010110011010" => sin_data_int <= to_unsigned(8500,16); cos_data_int <= to_unsigned(50408,16);
     when "010110011011" => sin_data_int <= to_unsigned(8527,16); cos_data_int <= to_unsigned(50446,16);
     when "010110011100" => sin_data_int <= to_unsigned(8555,16); cos_data_int <= to_unsigned(50483,16);
     when "010110011101" => sin_data_int <= to_unsigned(8582,16); cos_data_int <= to_unsigned(50520,16);
     when "010110011110" => sin_data_int <= to_unsigned(8609,16); cos_data_int <= to_unsigned(50557,16);
     when "010110011111" => sin_data_int <= to_unsigned(8636,16); cos_data_int <= to_unsigned(50594,16);
     when "010110100000" => sin_data_int <= to_unsigned(8664,16); cos_data_int <= to_unsigned(50631,16);
     when "010110100001" => sin_data_int <= to_unsigned(8691,16); cos_data_int <= to_unsigned(50668,16);
     when "010110100010" => sin_data_int <= to_unsigned(8719,16); cos_data_int <= to_unsigned(50705,16);
     when "010110100011" => sin_data_int <= to_unsigned(8746,16); cos_data_int <= to_unsigned(50742,16);
     when "010110100100" => sin_data_int <= to_unsigned(8774,16); cos_data_int <= to_unsigned(50778,16);
     when "010110100101" => sin_data_int <= to_unsigned(8802,16); cos_data_int <= to_unsigned(50815,16);
     when "010110100110" => sin_data_int <= to_unsigned(8829,16); cos_data_int <= to_unsigned(50852,16);
     when "010110100111" => sin_data_int <= to_unsigned(8857,16); cos_data_int <= to_unsigned(50889,16);
     when "010110101000" => sin_data_int <= to_unsigned(8885,16); cos_data_int <= to_unsigned(50925,16);
     when "010110101001" => sin_data_int <= to_unsigned(8913,16); cos_data_int <= to_unsigned(50962,16);
     when "010110101010" => sin_data_int <= to_unsigned(8941,16); cos_data_int <= to_unsigned(50998,16);
     when "010110101011" => sin_data_int <= to_unsigned(8969,16); cos_data_int <= to_unsigned(51035,16);
     when "010110101100" => sin_data_int <= to_unsigned(8997,16); cos_data_int <= to_unsigned(51071,16);
     when "010110101101" => sin_data_int <= to_unsigned(9025,16); cos_data_int <= to_unsigned(51108,16);
     when "010110101110" => sin_data_int <= to_unsigned(9053,16); cos_data_int <= to_unsigned(51144,16);
     when "010110101111" => sin_data_int <= to_unsigned(9081,16); cos_data_int <= to_unsigned(51181,16);
     when "010110110000" => sin_data_int <= to_unsigned(9110,16); cos_data_int <= to_unsigned(51217,16);
     when "010110110001" => sin_data_int <= to_unsigned(9138,16); cos_data_int <= to_unsigned(51253,16);
     when "010110110010" => sin_data_int <= to_unsigned(9166,16); cos_data_int <= to_unsigned(51289,16);
     when "010110110011" => sin_data_int <= to_unsigned(9195,16); cos_data_int <= to_unsigned(51326,16);
     when "010110110100" => sin_data_int <= to_unsigned(9223,16); cos_data_int <= to_unsigned(51362,16);
     when "010110110101" => sin_data_int <= to_unsigned(9252,16); cos_data_int <= to_unsigned(51398,16);
     when "010110110110" => sin_data_int <= to_unsigned(9280,16); cos_data_int <= to_unsigned(51434,16);
     when "010110110111" => sin_data_int <= to_unsigned(9309,16); cos_data_int <= to_unsigned(51470,16);
     when "010110111000" => sin_data_int <= to_unsigned(9338,16); cos_data_int <= to_unsigned(51506,16);
     when "010110111001" => sin_data_int <= to_unsigned(9367,16); cos_data_int <= to_unsigned(51542,16);
     when "010110111010" => sin_data_int <= to_unsigned(9396,16); cos_data_int <= to_unsigned(51578,16);
     when "010110111011" => sin_data_int <= to_unsigned(9424,16); cos_data_int <= to_unsigned(51613,16);
     when "010110111100" => sin_data_int <= to_unsigned(9453,16); cos_data_int <= to_unsigned(51649,16);
     when "010110111101" => sin_data_int <= to_unsigned(9482,16); cos_data_int <= to_unsigned(51685,16);
     when "010110111110" => sin_data_int <= to_unsigned(9511,16); cos_data_int <= to_unsigned(51721,16);
     when "010110111111" => sin_data_int <= to_unsigned(9541,16); cos_data_int <= to_unsigned(51756,16);
     when "010111000000" => sin_data_int <= to_unsigned(9570,16); cos_data_int <= to_unsigned(51792,16);
     when "010111000001" => sin_data_int <= to_unsigned(9599,16); cos_data_int <= to_unsigned(51827,16);
     when "010111000010" => sin_data_int <= to_unsigned(9628,16); cos_data_int <= to_unsigned(51863,16);
     when "010111000011" => sin_data_int <= to_unsigned(9658,16); cos_data_int <= to_unsigned(51898,16);
     when "010111000100" => sin_data_int <= to_unsigned(9687,16); cos_data_int <= to_unsigned(51934,16);
     when "010111000101" => sin_data_int <= to_unsigned(9716,16); cos_data_int <= to_unsigned(51969,16);
     when "010111000110" => sin_data_int <= to_unsigned(9746,16); cos_data_int <= to_unsigned(52004,16);
     when "010111000111" => sin_data_int <= to_unsigned(9775,16); cos_data_int <= to_unsigned(52040,16);
     when "010111001000" => sin_data_int <= to_unsigned(9805,16); cos_data_int <= to_unsigned(52075,16);
     when "010111001001" => sin_data_int <= to_unsigned(9835,16); cos_data_int <= to_unsigned(52110,16);
     when "010111001010" => sin_data_int <= to_unsigned(9864,16); cos_data_int <= to_unsigned(52145,16);
     when "010111001011" => sin_data_int <= to_unsigned(9894,16); cos_data_int <= to_unsigned(52180,16);
     when "010111001100" => sin_data_int <= to_unsigned(9924,16); cos_data_int <= to_unsigned(52215,16);
     when "010111001101" => sin_data_int <= to_unsigned(9954,16); cos_data_int <= to_unsigned(52250,16);
     when "010111001110" => sin_data_int <= to_unsigned(9984,16); cos_data_int <= to_unsigned(52285,16);
     when "010111001111" => sin_data_int <= to_unsigned(10014,16); cos_data_int <= to_unsigned(52320,16);
     when "010111010000" => sin_data_int <= to_unsigned(10044,16); cos_data_int <= to_unsigned(52355,16);
     when "010111010001" => sin_data_int <= to_unsigned(10074,16); cos_data_int <= to_unsigned(52390,16);
     when "010111010010" => sin_data_int <= to_unsigned(10104,16); cos_data_int <= to_unsigned(52425,16);
     when "010111010011" => sin_data_int <= to_unsigned(10134,16); cos_data_int <= to_unsigned(52460,16);
     when "010111010100" => sin_data_int <= to_unsigned(10164,16); cos_data_int <= to_unsigned(52494,16);
     when "010111010101" => sin_data_int <= to_unsigned(10195,16); cos_data_int <= to_unsigned(52529,16);
     when "010111010110" => sin_data_int <= to_unsigned(10225,16); cos_data_int <= to_unsigned(52563,16);
     when "010111010111" => sin_data_int <= to_unsigned(10255,16); cos_data_int <= to_unsigned(52598,16);
     when "010111011000" => sin_data_int <= to_unsigned(10286,16); cos_data_int <= to_unsigned(52632,16);
     when "010111011001" => sin_data_int <= to_unsigned(10316,16); cos_data_int <= to_unsigned(52667,16);
     when "010111011010" => sin_data_int <= to_unsigned(10347,16); cos_data_int <= to_unsigned(52701,16);
     when "010111011011" => sin_data_int <= to_unsigned(10378,16); cos_data_int <= to_unsigned(52736,16);
     when "010111011100" => sin_data_int <= to_unsigned(10408,16); cos_data_int <= to_unsigned(52770,16);
     when "010111011101" => sin_data_int <= to_unsigned(10439,16); cos_data_int <= to_unsigned(52804,16);
     when "010111011110" => sin_data_int <= to_unsigned(10470,16); cos_data_int <= to_unsigned(52838,16);
     when "010111011111" => sin_data_int <= to_unsigned(10501,16); cos_data_int <= to_unsigned(52873,16);
     when "010111100000" => sin_data_int <= to_unsigned(10531,16); cos_data_int <= to_unsigned(52907,16);
     when "010111100001" => sin_data_int <= to_unsigned(10562,16); cos_data_int <= to_unsigned(52941,16);
     when "010111100010" => sin_data_int <= to_unsigned(10593,16); cos_data_int <= to_unsigned(52975,16);
     when "010111100011" => sin_data_int <= to_unsigned(10624,16); cos_data_int <= to_unsigned(53009,16);
     when "010111100100" => sin_data_int <= to_unsigned(10656,16); cos_data_int <= to_unsigned(53043,16);
     when "010111100101" => sin_data_int <= to_unsigned(10687,16); cos_data_int <= to_unsigned(53077,16);
     when "010111100110" => sin_data_int <= to_unsigned(10718,16); cos_data_int <= to_unsigned(53111,16);
     when "010111100111" => sin_data_int <= to_unsigned(10749,16); cos_data_int <= to_unsigned(53144,16);
     when "010111101000" => sin_data_int <= to_unsigned(10780,16); cos_data_int <= to_unsigned(53178,16);
     when "010111101001" => sin_data_int <= to_unsigned(10812,16); cos_data_int <= to_unsigned(53212,16);
     when "010111101010" => sin_data_int <= to_unsigned(10843,16); cos_data_int <= to_unsigned(53245,16);
     when "010111101011" => sin_data_int <= to_unsigned(10875,16); cos_data_int <= to_unsigned(53279,16);
     when "010111101100" => sin_data_int <= to_unsigned(10906,16); cos_data_int <= to_unsigned(53313,16);
     when "010111101101" => sin_data_int <= to_unsigned(10938,16); cos_data_int <= to_unsigned(53346,16);
     when "010111101110" => sin_data_int <= to_unsigned(10969,16); cos_data_int <= to_unsigned(53379,16);
     when "010111101111" => sin_data_int <= to_unsigned(11001,16); cos_data_int <= to_unsigned(53413,16);
     when "010111110000" => sin_data_int <= to_unsigned(11033,16); cos_data_int <= to_unsigned(53446,16);
     when "010111110001" => sin_data_int <= to_unsigned(11064,16); cos_data_int <= to_unsigned(53480,16);
     when "010111110010" => sin_data_int <= to_unsigned(11096,16); cos_data_int <= to_unsigned(53513,16);
     when "010111110011" => sin_data_int <= to_unsigned(11128,16); cos_data_int <= to_unsigned(53546,16);
     when "010111110100" => sin_data_int <= to_unsigned(11160,16); cos_data_int <= to_unsigned(53579,16);
     when "010111110101" => sin_data_int <= to_unsigned(11192,16); cos_data_int <= to_unsigned(53612,16);
     when "010111110110" => sin_data_int <= to_unsigned(11224,16); cos_data_int <= to_unsigned(53645,16);
     when "010111110111" => sin_data_int <= to_unsigned(11256,16); cos_data_int <= to_unsigned(53678,16);
     when "010111111000" => sin_data_int <= to_unsigned(11288,16); cos_data_int <= to_unsigned(53711,16);
     when "010111111001" => sin_data_int <= to_unsigned(11320,16); cos_data_int <= to_unsigned(53744,16);
     when "010111111010" => sin_data_int <= to_unsigned(11352,16); cos_data_int <= to_unsigned(53777,16);
     when "010111111011" => sin_data_int <= to_unsigned(11385,16); cos_data_int <= to_unsigned(53810,16);
     when "010111111100" => sin_data_int <= to_unsigned(11417,16); cos_data_int <= to_unsigned(53843,16);
     when "010111111101" => sin_data_int <= to_unsigned(11449,16); cos_data_int <= to_unsigned(53875,16);
     when "010111111110" => sin_data_int <= to_unsigned(11482,16); cos_data_int <= to_unsigned(53908,16);
     when "010111111111" => sin_data_int <= to_unsigned(11514,16); cos_data_int <= to_unsigned(53941,16);
     when "011000000000" => sin_data_int <= to_unsigned(11547,16); cos_data_int <= to_unsigned(53973,16);
     when "011000000001" => sin_data_int <= to_unsigned(11579,16); cos_data_int <= to_unsigned(54006,16);
     when "011000000010" => sin_data_int <= to_unsigned(11612,16); cos_data_int <= to_unsigned(54038,16);
     when "011000000011" => sin_data_int <= to_unsigned(11645,16); cos_data_int <= to_unsigned(54071,16);
     when "011000000100" => sin_data_int <= to_unsigned(11677,16); cos_data_int <= to_unsigned(54103,16);
     when "011000000101" => sin_data_int <= to_unsigned(11710,16); cos_data_int <= to_unsigned(54135,16);
     when "011000000110" => sin_data_int <= to_unsigned(11743,16); cos_data_int <= to_unsigned(54168,16);
     when "011000000111" => sin_data_int <= to_unsigned(11776,16); cos_data_int <= to_unsigned(54200,16);
     when "011000001000" => sin_data_int <= to_unsigned(11809,16); cos_data_int <= to_unsigned(54232,16);
     when "011000001001" => sin_data_int <= to_unsigned(11842,16); cos_data_int <= to_unsigned(54264,16);
     when "011000001010" => sin_data_int <= to_unsigned(11875,16); cos_data_int <= to_unsigned(54296,16);
     when "011000001011" => sin_data_int <= to_unsigned(11908,16); cos_data_int <= to_unsigned(54328,16);
     when "011000001100" => sin_data_int <= to_unsigned(11941,16); cos_data_int <= to_unsigned(54360,16);
     when "011000001101" => sin_data_int <= to_unsigned(11974,16); cos_data_int <= to_unsigned(54392,16);
     when "011000001110" => sin_data_int <= to_unsigned(12007,16); cos_data_int <= to_unsigned(54424,16);
     when "011000001111" => sin_data_int <= to_unsigned(12040,16); cos_data_int <= to_unsigned(54456,16);
     when "011000010000" => sin_data_int <= to_unsigned(12074,16); cos_data_int <= to_unsigned(54487,16);
     when "011000010001" => sin_data_int <= to_unsigned(12107,16); cos_data_int <= to_unsigned(54519,16);
     when "011000010010" => sin_data_int <= to_unsigned(12141,16); cos_data_int <= to_unsigned(54551,16);
     when "011000010011" => sin_data_int <= to_unsigned(12174,16); cos_data_int <= to_unsigned(54582,16);
     when "011000010100" => sin_data_int <= to_unsigned(12207,16); cos_data_int <= to_unsigned(54614,16);
     when "011000010101" => sin_data_int <= to_unsigned(12241,16); cos_data_int <= to_unsigned(54645,16);
     when "011000010110" => sin_data_int <= to_unsigned(12275,16); cos_data_int <= to_unsigned(54677,16);
     when "011000010111" => sin_data_int <= to_unsigned(12308,16); cos_data_int <= to_unsigned(54708,16);
     when "011000011000" => sin_data_int <= to_unsigned(12342,16); cos_data_int <= to_unsigned(54740,16);
     when "011000011001" => sin_data_int <= to_unsigned(12376,16); cos_data_int <= to_unsigned(54771,16);
     when "011000011010" => sin_data_int <= to_unsigned(12409,16); cos_data_int <= to_unsigned(54802,16);
     when "011000011011" => sin_data_int <= to_unsigned(12443,16); cos_data_int <= to_unsigned(54833,16);
     when "011000011100" => sin_data_int <= to_unsigned(12477,16); cos_data_int <= to_unsigned(54864,16);
     when "011000011101" => sin_data_int <= to_unsigned(12511,16); cos_data_int <= to_unsigned(54896,16);
     when "011000011110" => sin_data_int <= to_unsigned(12545,16); cos_data_int <= to_unsigned(54927,16);
     when "011000011111" => sin_data_int <= to_unsigned(12579,16); cos_data_int <= to_unsigned(54958,16);
     when "011000100000" => sin_data_int <= to_unsigned(12613,16); cos_data_int <= to_unsigned(54989,16);
     when "011000100001" => sin_data_int <= to_unsigned(12647,16); cos_data_int <= to_unsigned(55019,16);
     when "011000100010" => sin_data_int <= to_unsigned(12682,16); cos_data_int <= to_unsigned(55050,16);
     when "011000100011" => sin_data_int <= to_unsigned(12716,16); cos_data_int <= to_unsigned(55081,16);
     when "011000100100" => sin_data_int <= to_unsigned(12750,16); cos_data_int <= to_unsigned(55112,16);
     when "011000100101" => sin_data_int <= to_unsigned(12784,16); cos_data_int <= to_unsigned(55142,16);
     when "011000100110" => sin_data_int <= to_unsigned(12819,16); cos_data_int <= to_unsigned(55173,16);
     when "011000100111" => sin_data_int <= to_unsigned(12853,16); cos_data_int <= to_unsigned(55204,16);
     when "011000101000" => sin_data_int <= to_unsigned(12888,16); cos_data_int <= to_unsigned(55234,16);
     when "011000101001" => sin_data_int <= to_unsigned(12922,16); cos_data_int <= to_unsigned(55265,16);
     when "011000101010" => sin_data_int <= to_unsigned(12957,16); cos_data_int <= to_unsigned(55295,16);
     when "011000101011" => sin_data_int <= to_unsigned(12991,16); cos_data_int <= to_unsigned(55325,16);
     when "011000101100" => sin_data_int <= to_unsigned(13026,16); cos_data_int <= to_unsigned(55356,16);
     when "011000101101" => sin_data_int <= to_unsigned(13060,16); cos_data_int <= to_unsigned(55386,16);
     when "011000101110" => sin_data_int <= to_unsigned(13095,16); cos_data_int <= to_unsigned(55416,16);
     when "011000101111" => sin_data_int <= to_unsigned(13130,16); cos_data_int <= to_unsigned(55446,16);
     when "011000110000" => sin_data_int <= to_unsigned(13165,16); cos_data_int <= to_unsigned(55476,16);
     when "011000110001" => sin_data_int <= to_unsigned(13200,16); cos_data_int <= to_unsigned(55506,16);
     when "011000110010" => sin_data_int <= to_unsigned(13235,16); cos_data_int <= to_unsigned(55536,16);
     when "011000110011" => sin_data_int <= to_unsigned(13270,16); cos_data_int <= to_unsigned(55566,16);
     when "011000110100" => sin_data_int <= to_unsigned(13305,16); cos_data_int <= to_unsigned(55596,16);
     when "011000110101" => sin_data_int <= to_unsigned(13340,16); cos_data_int <= to_unsigned(55626,16);
     when "011000110110" => sin_data_int <= to_unsigned(13375,16); cos_data_int <= to_unsigned(55656,16);
     when "011000110111" => sin_data_int <= to_unsigned(13410,16); cos_data_int <= to_unsigned(55685,16);
     when "011000111000" => sin_data_int <= to_unsigned(13445,16); cos_data_int <= to_unsigned(55715,16);
     when "011000111001" => sin_data_int <= to_unsigned(13480,16); cos_data_int <= to_unsigned(55745,16);
     when "011000111010" => sin_data_int <= to_unsigned(13516,16); cos_data_int <= to_unsigned(55774,16);
     when "011000111011" => sin_data_int <= to_unsigned(13551,16); cos_data_int <= to_unsigned(55804,16);
     when "011000111100" => sin_data_int <= to_unsigned(13586,16); cos_data_int <= to_unsigned(55833,16);
     when "011000111101" => sin_data_int <= to_unsigned(13622,16); cos_data_int <= to_unsigned(55862,16);
     when "011000111110" => sin_data_int <= to_unsigned(13657,16); cos_data_int <= to_unsigned(55892,16);
     when "011000111111" => sin_data_int <= to_unsigned(13693,16); cos_data_int <= to_unsigned(55921,16);
     when "011001000000" => sin_data_int <= to_unsigned(13728,16); cos_data_int <= to_unsigned(55950,16);
     when "011001000001" => sin_data_int <= to_unsigned(13764,16); cos_data_int <= to_unsigned(55979,16);
     when "011001000010" => sin_data_int <= to_unsigned(13799,16); cos_data_int <= to_unsigned(56009,16);
     when "011001000011" => sin_data_int <= to_unsigned(13835,16); cos_data_int <= to_unsigned(56038,16);
     when "011001000100" => sin_data_int <= to_unsigned(13871,16); cos_data_int <= to_unsigned(56067,16);
     when "011001000101" => sin_data_int <= to_unsigned(13907,16); cos_data_int <= to_unsigned(56096,16);
     when "011001000110" => sin_data_int <= to_unsigned(13942,16); cos_data_int <= to_unsigned(56124,16);
     when "011001000111" => sin_data_int <= to_unsigned(13978,16); cos_data_int <= to_unsigned(56153,16);
     when "011001001000" => sin_data_int <= to_unsigned(14014,16); cos_data_int <= to_unsigned(56182,16);
     when "011001001001" => sin_data_int <= to_unsigned(14050,16); cos_data_int <= to_unsigned(56211,16);
     when "011001001010" => sin_data_int <= to_unsigned(14086,16); cos_data_int <= to_unsigned(56240,16);
     when "011001001011" => sin_data_int <= to_unsigned(14122,16); cos_data_int <= to_unsigned(56268,16);
     when "011001001100" => sin_data_int <= to_unsigned(14158,16); cos_data_int <= to_unsigned(56297,16);
     when "011001001101" => sin_data_int <= to_unsigned(14194,16); cos_data_int <= to_unsigned(56325,16);
     when "011001001110" => sin_data_int <= to_unsigned(14231,16); cos_data_int <= to_unsigned(56354,16);
     when "011001001111" => sin_data_int <= to_unsigned(14267,16); cos_data_int <= to_unsigned(56382,16);
     when "011001010000" => sin_data_int <= to_unsigned(14303,16); cos_data_int <= to_unsigned(56410,16);
     when "011001010001" => sin_data_int <= to_unsigned(14339,16); cos_data_int <= to_unsigned(56439,16);
     when "011001010010" => sin_data_int <= to_unsigned(14376,16); cos_data_int <= to_unsigned(56467,16);
     when "011001010011" => sin_data_int <= to_unsigned(14412,16); cos_data_int <= to_unsigned(56495,16);
     when "011001010100" => sin_data_int <= to_unsigned(14449,16); cos_data_int <= to_unsigned(56523,16);
     when "011001010101" => sin_data_int <= to_unsigned(14485,16); cos_data_int <= to_unsigned(56551,16);
     when "011001010110" => sin_data_int <= to_unsigned(14522,16); cos_data_int <= to_unsigned(56579,16);
     when "011001010111" => sin_data_int <= to_unsigned(14558,16); cos_data_int <= to_unsigned(56607,16);
     when "011001011000" => sin_data_int <= to_unsigned(14595,16); cos_data_int <= to_unsigned(56635,16);
     when "011001011001" => sin_data_int <= to_unsigned(14631,16); cos_data_int <= to_unsigned(56663,16);
     when "011001011010" => sin_data_int <= to_unsigned(14668,16); cos_data_int <= to_unsigned(56691,16);
     when "011001011011" => sin_data_int <= to_unsigned(14705,16); cos_data_int <= to_unsigned(56718,16);
     when "011001011100" => sin_data_int <= to_unsigned(14742,16); cos_data_int <= to_unsigned(56746,16);
     when "011001011101" => sin_data_int <= to_unsigned(14778,16); cos_data_int <= to_unsigned(56774,16);
     when "011001011110" => sin_data_int <= to_unsigned(14815,16); cos_data_int <= to_unsigned(56801,16);
     when "011001011111" => sin_data_int <= to_unsigned(14852,16); cos_data_int <= to_unsigned(56829,16);
     when "011001100000" => sin_data_int <= to_unsigned(14889,16); cos_data_int <= to_unsigned(56856,16);
     when "011001100001" => sin_data_int <= to_unsigned(14926,16); cos_data_int <= to_unsigned(56884,16);
     when "011001100010" => sin_data_int <= to_unsigned(14963,16); cos_data_int <= to_unsigned(56911,16);
     when "011001100011" => sin_data_int <= to_unsigned(15000,16); cos_data_int <= to_unsigned(56938,16);
     when "011001100100" => sin_data_int <= to_unsigned(15037,16); cos_data_int <= to_unsigned(56965,16);
     when "011001100101" => sin_data_int <= to_unsigned(15074,16); cos_data_int <= to_unsigned(56993,16);
     when "011001100110" => sin_data_int <= to_unsigned(15112,16); cos_data_int <= to_unsigned(57020,16);
     when "011001100111" => sin_data_int <= to_unsigned(15149,16); cos_data_int <= to_unsigned(57047,16);
     when "011001101000" => sin_data_int <= to_unsigned(15186,16); cos_data_int <= to_unsigned(57074,16);
     when "011001101001" => sin_data_int <= to_unsigned(15223,16); cos_data_int <= to_unsigned(57101,16);
     when "011001101010" => sin_data_int <= to_unsigned(15261,16); cos_data_int <= to_unsigned(57128,16);
     when "011001101011" => sin_data_int <= to_unsigned(15298,16); cos_data_int <= to_unsigned(57154,16);
     when "011001101100" => sin_data_int <= to_unsigned(15336,16); cos_data_int <= to_unsigned(57181,16);
     when "011001101101" => sin_data_int <= to_unsigned(15373,16); cos_data_int <= to_unsigned(57208,16);
     when "011001101110" => sin_data_int <= to_unsigned(15411,16); cos_data_int <= to_unsigned(57234,16);
     when "011001101111" => sin_data_int <= to_unsigned(15448,16); cos_data_int <= to_unsigned(57261,16);
     when "011001110000" => sin_data_int <= to_unsigned(15486,16); cos_data_int <= to_unsigned(57288,16);
     when "011001110001" => sin_data_int <= to_unsigned(15523,16); cos_data_int <= to_unsigned(57314,16);
     when "011001110010" => sin_data_int <= to_unsigned(15561,16); cos_data_int <= to_unsigned(57340,16);
     when "011001110011" => sin_data_int <= to_unsigned(15599,16); cos_data_int <= to_unsigned(57367,16);
     when "011001110100" => sin_data_int <= to_unsigned(15637,16); cos_data_int <= to_unsigned(57393,16);
     when "011001110101" => sin_data_int <= to_unsigned(15674,16); cos_data_int <= to_unsigned(57419,16);
     when "011001110110" => sin_data_int <= to_unsigned(15712,16); cos_data_int <= to_unsigned(57445,16);
     when "011001110111" => sin_data_int <= to_unsigned(15750,16); cos_data_int <= to_unsigned(57472,16);
     when "011001111000" => sin_data_int <= to_unsigned(15788,16); cos_data_int <= to_unsigned(57498,16);
     when "011001111001" => sin_data_int <= to_unsigned(15826,16); cos_data_int <= to_unsigned(57524,16);
     when "011001111010" => sin_data_int <= to_unsigned(15864,16); cos_data_int <= to_unsigned(57550,16);
     when "011001111011" => sin_data_int <= to_unsigned(15902,16); cos_data_int <= to_unsigned(57576,16);
     when "011001111100" => sin_data_int <= to_unsigned(15940,16); cos_data_int <= to_unsigned(57601,16);
     when "011001111101" => sin_data_int <= to_unsigned(15978,16); cos_data_int <= to_unsigned(57627,16);
     when "011001111110" => sin_data_int <= to_unsigned(16016,16); cos_data_int <= to_unsigned(57653,16);
     when "011001111111" => sin_data_int <= to_unsigned(16055,16); cos_data_int <= to_unsigned(57678,16);
     when "011010000000" => sin_data_int <= to_unsigned(16093,16); cos_data_int <= to_unsigned(57704,16);
     when "011010000001" => sin_data_int <= to_unsigned(16131,16); cos_data_int <= to_unsigned(57730,16);
     when "011010000010" => sin_data_int <= to_unsigned(16169,16); cos_data_int <= to_unsigned(57755,16);
     when "011010000011" => sin_data_int <= to_unsigned(16208,16); cos_data_int <= to_unsigned(57781,16);
     when "011010000100" => sin_data_int <= to_unsigned(16246,16); cos_data_int <= to_unsigned(57806,16);
     when "011010000101" => sin_data_int <= to_unsigned(16285,16); cos_data_int <= to_unsigned(57831,16);
     when "011010000110" => sin_data_int <= to_unsigned(16323,16); cos_data_int <= to_unsigned(57856,16);
     when "011010000111" => sin_data_int <= to_unsigned(16362,16); cos_data_int <= to_unsigned(57882,16);
     when "011010001000" => sin_data_int <= to_unsigned(16400,16); cos_data_int <= to_unsigned(57907,16);
     when "011010001001" => sin_data_int <= to_unsigned(16439,16); cos_data_int <= to_unsigned(57932,16);
     when "011010001010" => sin_data_int <= to_unsigned(16477,16); cos_data_int <= to_unsigned(57957,16);
     when "011010001011" => sin_data_int <= to_unsigned(16516,16); cos_data_int <= to_unsigned(57982,16);
     when "011010001100" => sin_data_int <= to_unsigned(16555,16); cos_data_int <= to_unsigned(58007,16);
     when "011010001101" => sin_data_int <= to_unsigned(16594,16); cos_data_int <= to_unsigned(58031,16);
     when "011010001110" => sin_data_int <= to_unsigned(16632,16); cos_data_int <= to_unsigned(58056,16);
     when "011010001111" => sin_data_int <= to_unsigned(16671,16); cos_data_int <= to_unsigned(58081,16);
     when "011010010000" => sin_data_int <= to_unsigned(16710,16); cos_data_int <= to_unsigned(58106,16);
     when "011010010001" => sin_data_int <= to_unsigned(16749,16); cos_data_int <= to_unsigned(58130,16);
     when "011010010010" => sin_data_int <= to_unsigned(16788,16); cos_data_int <= to_unsigned(58155,16);
     when "011010010011" => sin_data_int <= to_unsigned(16827,16); cos_data_int <= to_unsigned(58179,16);
     when "011010010100" => sin_data_int <= to_unsigned(16866,16); cos_data_int <= to_unsigned(58204,16);
     when "011010010101" => sin_data_int <= to_unsigned(16905,16); cos_data_int <= to_unsigned(58228,16);
     when "011010010110" => sin_data_int <= to_unsigned(16944,16); cos_data_int <= to_unsigned(58252,16);
     when "011010010111" => sin_data_int <= to_unsigned(16983,16); cos_data_int <= to_unsigned(58276,16);
     when "011010011000" => sin_data_int <= to_unsigned(17022,16); cos_data_int <= to_unsigned(58301,16);
     when "011010011001" => sin_data_int <= to_unsigned(17062,16); cos_data_int <= to_unsigned(58325,16);
     when "011010011010" => sin_data_int <= to_unsigned(17101,16); cos_data_int <= to_unsigned(58349,16);
     when "011010011011" => sin_data_int <= to_unsigned(17140,16); cos_data_int <= to_unsigned(58373,16);
     when "011010011100" => sin_data_int <= to_unsigned(17179,16); cos_data_int <= to_unsigned(58397,16);
     when "011010011101" => sin_data_int <= to_unsigned(17219,16); cos_data_int <= to_unsigned(58421,16);
     when "011010011110" => sin_data_int <= to_unsigned(17258,16); cos_data_int <= to_unsigned(58444,16);
     when "011010011111" => sin_data_int <= to_unsigned(17297,16); cos_data_int <= to_unsigned(58468,16);
     when "011010100000" => sin_data_int <= to_unsigned(17337,16); cos_data_int <= to_unsigned(58492,16);
     when "011010100001" => sin_data_int <= to_unsigned(17376,16); cos_data_int <= to_unsigned(58515,16);
     when "011010100010" => sin_data_int <= to_unsigned(17416,16); cos_data_int <= to_unsigned(58539,16);
     when "011010100011" => sin_data_int <= to_unsigned(17455,16); cos_data_int <= to_unsigned(58563,16);
     when "011010100100" => sin_data_int <= to_unsigned(17495,16); cos_data_int <= to_unsigned(58586,16);
     when "011010100101" => sin_data_int <= to_unsigned(17535,16); cos_data_int <= to_unsigned(58609,16);
     when "011010100110" => sin_data_int <= to_unsigned(17574,16); cos_data_int <= to_unsigned(58633,16);
     when "011010100111" => sin_data_int <= to_unsigned(17614,16); cos_data_int <= to_unsigned(58656,16);
     when "011010101000" => sin_data_int <= to_unsigned(17654,16); cos_data_int <= to_unsigned(58679,16);
     when "011010101001" => sin_data_int <= to_unsigned(17694,16); cos_data_int <= to_unsigned(58702,16);
     when "011010101010" => sin_data_int <= to_unsigned(17733,16); cos_data_int <= to_unsigned(58725,16);
     when "011010101011" => sin_data_int <= to_unsigned(17773,16); cos_data_int <= to_unsigned(58748,16);
     when "011010101100" => sin_data_int <= to_unsigned(17813,16); cos_data_int <= to_unsigned(58771,16);
     when "011010101101" => sin_data_int <= to_unsigned(17853,16); cos_data_int <= to_unsigned(58794,16);
     when "011010101110" => sin_data_int <= to_unsigned(17893,16); cos_data_int <= to_unsigned(58817,16);
     when "011010101111" => sin_data_int <= to_unsigned(17933,16); cos_data_int <= to_unsigned(58840,16);
     when "011010110000" => sin_data_int <= to_unsigned(17973,16); cos_data_int <= to_unsigned(58863,16);
     when "011010110001" => sin_data_int <= to_unsigned(18013,16); cos_data_int <= to_unsigned(58885,16);
     when "011010110010" => sin_data_int <= to_unsigned(18053,16); cos_data_int <= to_unsigned(58908,16);
     when "011010110011" => sin_data_int <= to_unsigned(18093,16); cos_data_int <= to_unsigned(58930,16);
     when "011010110100" => sin_data_int <= to_unsigned(18133,16); cos_data_int <= to_unsigned(58953,16);
     when "011010110101" => sin_data_int <= to_unsigned(18174,16); cos_data_int <= to_unsigned(58975,16);
     when "011010110110" => sin_data_int <= to_unsigned(18214,16); cos_data_int <= to_unsigned(58998,16);
     when "011010110111" => sin_data_int <= to_unsigned(18254,16); cos_data_int <= to_unsigned(59020,16);
     when "011010111000" => sin_data_int <= to_unsigned(18294,16); cos_data_int <= to_unsigned(59042,16);
     when "011010111001" => sin_data_int <= to_unsigned(18335,16); cos_data_int <= to_unsigned(59064,16);
     when "011010111010" => sin_data_int <= to_unsigned(18375,16); cos_data_int <= to_unsigned(59086,16);
     when "011010111011" => sin_data_int <= to_unsigned(18416,16); cos_data_int <= to_unsigned(59108,16);
     when "011010111100" => sin_data_int <= to_unsigned(18456,16); cos_data_int <= to_unsigned(59130,16);
     when "011010111101" => sin_data_int <= to_unsigned(18496,16); cos_data_int <= to_unsigned(59152,16);
     when "011010111110" => sin_data_int <= to_unsigned(18537,16); cos_data_int <= to_unsigned(59174,16);
     when "011010111111" => sin_data_int <= to_unsigned(18578,16); cos_data_int <= to_unsigned(59196,16);
     when "011011000000" => sin_data_int <= to_unsigned(18618,16); cos_data_int <= to_unsigned(59218,16);
     when "011011000001" => sin_data_int <= to_unsigned(18659,16); cos_data_int <= to_unsigned(59239,16);
     when "011011000010" => sin_data_int <= to_unsigned(18699,16); cos_data_int <= to_unsigned(59261,16);
     when "011011000011" => sin_data_int <= to_unsigned(18740,16); cos_data_int <= to_unsigned(59282,16);
     when "011011000100" => sin_data_int <= to_unsigned(18781,16); cos_data_int <= to_unsigned(59304,16);
     when "011011000101" => sin_data_int <= to_unsigned(18821,16); cos_data_int <= to_unsigned(59325,16);
     when "011011000110" => sin_data_int <= to_unsigned(18862,16); cos_data_int <= to_unsigned(59347,16);
     when "011011000111" => sin_data_int <= to_unsigned(18903,16); cos_data_int <= to_unsigned(59368,16);
     when "011011001000" => sin_data_int <= to_unsigned(18944,16); cos_data_int <= to_unsigned(59389,16);
     when "011011001001" => sin_data_int <= to_unsigned(18985,16); cos_data_int <= to_unsigned(59410,16);
     when "011011001010" => sin_data_int <= to_unsigned(19026,16); cos_data_int <= to_unsigned(59431,16);
     when "011011001011" => sin_data_int <= to_unsigned(19067,16); cos_data_int <= to_unsigned(59452,16);
     when "011011001100" => sin_data_int <= to_unsigned(19107,16); cos_data_int <= to_unsigned(59473,16);
     when "011011001101" => sin_data_int <= to_unsigned(19148,16); cos_data_int <= to_unsigned(59494,16);
     when "011011001110" => sin_data_int <= to_unsigned(19190,16); cos_data_int <= to_unsigned(59515,16);
     when "011011001111" => sin_data_int <= to_unsigned(19231,16); cos_data_int <= to_unsigned(59536,16);
     when "011011010000" => sin_data_int <= to_unsigned(19272,16); cos_data_int <= to_unsigned(59557,16);
     when "011011010001" => sin_data_int <= to_unsigned(19313,16); cos_data_int <= to_unsigned(59577,16);
     when "011011010010" => sin_data_int <= to_unsigned(19354,16); cos_data_int <= to_unsigned(59598,16);
     when "011011010011" => sin_data_int <= to_unsigned(19395,16); cos_data_int <= to_unsigned(59619,16);
     when "011011010100" => sin_data_int <= to_unsigned(19436,16); cos_data_int <= to_unsigned(59639,16);
     when "011011010101" => sin_data_int <= to_unsigned(19478,16); cos_data_int <= to_unsigned(59659,16);
     when "011011010110" => sin_data_int <= to_unsigned(19519,16); cos_data_int <= to_unsigned(59680,16);
     when "011011010111" => sin_data_int <= to_unsigned(19560,16); cos_data_int <= to_unsigned(59700,16);
     when "011011011000" => sin_data_int <= to_unsigned(19602,16); cos_data_int <= to_unsigned(59720,16);
     when "011011011001" => sin_data_int <= to_unsigned(19643,16); cos_data_int <= to_unsigned(59740,16);
     when "011011011010" => sin_data_int <= to_unsigned(19684,16); cos_data_int <= to_unsigned(59760,16);
     when "011011011011" => sin_data_int <= to_unsigned(19726,16); cos_data_int <= to_unsigned(59781,16);
     when "011011011100" => sin_data_int <= to_unsigned(19767,16); cos_data_int <= to_unsigned(59800,16);
     when "011011011101" => sin_data_int <= to_unsigned(19809,16); cos_data_int <= to_unsigned(59820,16);
     when "011011011110" => sin_data_int <= to_unsigned(19850,16); cos_data_int <= to_unsigned(59840,16);
     when "011011011111" => sin_data_int <= to_unsigned(19892,16); cos_data_int <= to_unsigned(59860,16);
     when "011011100000" => sin_data_int <= to_unsigned(19933,16); cos_data_int <= to_unsigned(59880,16);
     when "011011100001" => sin_data_int <= to_unsigned(19975,16); cos_data_int <= to_unsigned(59899,16);
     when "011011100010" => sin_data_int <= to_unsigned(20017,16); cos_data_int <= to_unsigned(59919,16);
     when "011011100011" => sin_data_int <= to_unsigned(20058,16); cos_data_int <= to_unsigned(59938,16);
     when "011011100100" => sin_data_int <= to_unsigned(20100,16); cos_data_int <= to_unsigned(59958,16);
     when "011011100101" => sin_data_int <= to_unsigned(20142,16); cos_data_int <= to_unsigned(59977,16);
     when "011011100110" => sin_data_int <= to_unsigned(20183,16); cos_data_int <= to_unsigned(59997,16);
     when "011011100111" => sin_data_int <= to_unsigned(20225,16); cos_data_int <= to_unsigned(60016,16);
     when "011011101000" => sin_data_int <= to_unsigned(20267,16); cos_data_int <= to_unsigned(60035,16);
     when "011011101001" => sin_data_int <= to_unsigned(20309,16); cos_data_int <= to_unsigned(60054,16);
     when "011011101010" => sin_data_int <= to_unsigned(20351,16); cos_data_int <= to_unsigned(60073,16);
     when "011011101011" => sin_data_int <= to_unsigned(20393,16); cos_data_int <= to_unsigned(60092,16);
     when "011011101100" => sin_data_int <= to_unsigned(20435,16); cos_data_int <= to_unsigned(60111,16);
     when "011011101101" => sin_data_int <= to_unsigned(20477,16); cos_data_int <= to_unsigned(60130,16);
     when "011011101110" => sin_data_int <= to_unsigned(20519,16); cos_data_int <= to_unsigned(60149,16);
     when "011011101111" => sin_data_int <= to_unsigned(20561,16); cos_data_int <= to_unsigned(60168,16);
     when "011011110000" => sin_data_int <= to_unsigned(20603,16); cos_data_int <= to_unsigned(60186,16);
     when "011011110001" => sin_data_int <= to_unsigned(20645,16); cos_data_int <= to_unsigned(60205,16);
     when "011011110010" => sin_data_int <= to_unsigned(20687,16); cos_data_int <= to_unsigned(60223,16);
     when "011011110011" => sin_data_int <= to_unsigned(20729,16); cos_data_int <= to_unsigned(60242,16);
     when "011011110100" => sin_data_int <= to_unsigned(20771,16); cos_data_int <= to_unsigned(60260,16);
     when "011011110101" => sin_data_int <= to_unsigned(20813,16); cos_data_int <= to_unsigned(60279,16);
     when "011011110110" => sin_data_int <= to_unsigned(20856,16); cos_data_int <= to_unsigned(60297,16);
     when "011011110111" => sin_data_int <= to_unsigned(20898,16); cos_data_int <= to_unsigned(60315,16);
     when "011011111000" => sin_data_int <= to_unsigned(20940,16); cos_data_int <= to_unsigned(60333,16);
     when "011011111001" => sin_data_int <= to_unsigned(20983,16); cos_data_int <= to_unsigned(60352,16);
     when "011011111010" => sin_data_int <= to_unsigned(21025,16); cos_data_int <= to_unsigned(60370,16);
     when "011011111011" => sin_data_int <= to_unsigned(21067,16); cos_data_int <= to_unsigned(60388,16);
     when "011011111100" => sin_data_int <= to_unsigned(21110,16); cos_data_int <= to_unsigned(60405,16);
     when "011011111101" => sin_data_int <= to_unsigned(21152,16); cos_data_int <= to_unsigned(60423,16);
     when "011011111110" => sin_data_int <= to_unsigned(21195,16); cos_data_int <= to_unsigned(60441,16);
     when "011011111111" => sin_data_int <= to_unsigned(21237,16); cos_data_int <= to_unsigned(60459,16);
     when "011100000000" => sin_data_int <= to_unsigned(21279,16); cos_data_int <= to_unsigned(60476,16);
     when "011100000001" => sin_data_int <= to_unsigned(21322,16); cos_data_int <= to_unsigned(60494,16);
     when "011100000010" => sin_data_int <= to_unsigned(21365,16); cos_data_int <= to_unsigned(60511,16);
     when "011100000011" => sin_data_int <= to_unsigned(21407,16); cos_data_int <= to_unsigned(60529,16);
     when "011100000100" => sin_data_int <= to_unsigned(21450,16); cos_data_int <= to_unsigned(60546,16);
     when "011100000101" => sin_data_int <= to_unsigned(21492,16); cos_data_int <= to_unsigned(60564,16);
     when "011100000110" => sin_data_int <= to_unsigned(21535,16); cos_data_int <= to_unsigned(60581,16);
     when "011100000111" => sin_data_int <= to_unsigned(21578,16); cos_data_int <= to_unsigned(60598,16);
     when "011100001000" => sin_data_int <= to_unsigned(21620,16); cos_data_int <= to_unsigned(60615,16);
     when "011100001001" => sin_data_int <= to_unsigned(21663,16); cos_data_int <= to_unsigned(60632,16);
     when "011100001010" => sin_data_int <= to_unsigned(21706,16); cos_data_int <= to_unsigned(60649,16);
     when "011100001011" => sin_data_int <= to_unsigned(21749,16); cos_data_int <= to_unsigned(60666,16);
     when "011100001100" => sin_data_int <= to_unsigned(21792,16); cos_data_int <= to_unsigned(60683,16);
     when "011100001101" => sin_data_int <= to_unsigned(21834,16); cos_data_int <= to_unsigned(60700,16);
     when "011100001110" => sin_data_int <= to_unsigned(21877,16); cos_data_int <= to_unsigned(60717,16);
     when "011100001111" => sin_data_int <= to_unsigned(21920,16); cos_data_int <= to_unsigned(60733,16);
     when "011100010000" => sin_data_int <= to_unsigned(21963,16); cos_data_int <= to_unsigned(60750,16);
     when "011100010001" => sin_data_int <= to_unsigned(22006,16); cos_data_int <= to_unsigned(60766,16);
     when "011100010010" => sin_data_int <= to_unsigned(22049,16); cos_data_int <= to_unsigned(60783,16);
     when "011100010011" => sin_data_int <= to_unsigned(22092,16); cos_data_int <= to_unsigned(60799,16);
     when "011100010100" => sin_data_int <= to_unsigned(22135,16); cos_data_int <= to_unsigned(60816,16);
     when "011100010101" => sin_data_int <= to_unsigned(22178,16); cos_data_int <= to_unsigned(60832,16);
     when "011100010110" => sin_data_int <= to_unsigned(22221,16); cos_data_int <= to_unsigned(60848,16);
     when "011100010111" => sin_data_int <= to_unsigned(22264,16); cos_data_int <= to_unsigned(60864,16);
     when "011100011000" => sin_data_int <= to_unsigned(22307,16); cos_data_int <= to_unsigned(60880,16);
     when "011100011001" => sin_data_int <= to_unsigned(22351,16); cos_data_int <= to_unsigned(60896,16);
     when "011100011010" => sin_data_int <= to_unsigned(22394,16); cos_data_int <= to_unsigned(60912,16);
     when "011100011011" => sin_data_int <= to_unsigned(22437,16); cos_data_int <= to_unsigned(60928,16);
     when "011100011100" => sin_data_int <= to_unsigned(22480,16); cos_data_int <= to_unsigned(60944,16);
     when "011100011101" => sin_data_int <= to_unsigned(22523,16); cos_data_int <= to_unsigned(60960,16);
     when "011100011110" => sin_data_int <= to_unsigned(22567,16); cos_data_int <= to_unsigned(60975,16);
     when "011100011111" => sin_data_int <= to_unsigned(22610,16); cos_data_int <= to_unsigned(60991,16);
     when "011100100000" => sin_data_int <= to_unsigned(22653,16); cos_data_int <= to_unsigned(61006,16);
     when "011100100001" => sin_data_int <= to_unsigned(22697,16); cos_data_int <= to_unsigned(61022,16);
     when "011100100010" => sin_data_int <= to_unsigned(22740,16); cos_data_int <= to_unsigned(61037,16);
     when "011100100011" => sin_data_int <= to_unsigned(22783,16); cos_data_int <= to_unsigned(61053,16);
     when "011100100100" => sin_data_int <= to_unsigned(22827,16); cos_data_int <= to_unsigned(61068,16);
     when "011100100101" => sin_data_int <= to_unsigned(22870,16); cos_data_int <= to_unsigned(61083,16);
     when "011100100110" => sin_data_int <= to_unsigned(22914,16); cos_data_int <= to_unsigned(61098,16);
     when "011100100111" => sin_data_int <= to_unsigned(22957,16); cos_data_int <= to_unsigned(61113,16);
     when "011100101000" => sin_data_int <= to_unsigned(23001,16); cos_data_int <= to_unsigned(61128,16);
     when "011100101001" => sin_data_int <= to_unsigned(23044,16); cos_data_int <= to_unsigned(61143,16);
     when "011100101010" => sin_data_int <= to_unsigned(23088,16); cos_data_int <= to_unsigned(61158,16);
     when "011100101011" => sin_data_int <= to_unsigned(23131,16); cos_data_int <= to_unsigned(61173,16);
     when "011100101100" => sin_data_int <= to_unsigned(23175,16); cos_data_int <= to_unsigned(61188,16);
     when "011100101101" => sin_data_int <= to_unsigned(23219,16); cos_data_int <= to_unsigned(61202,16);
     when "011100101110" => sin_data_int <= to_unsigned(23262,16); cos_data_int <= to_unsigned(61217,16);
     when "011100101111" => sin_data_int <= to_unsigned(23306,16); cos_data_int <= to_unsigned(61231,16);
     when "011100110000" => sin_data_int <= to_unsigned(23350,16); cos_data_int <= to_unsigned(61246,16);
     when "011100110001" => sin_data_int <= to_unsigned(23393,16); cos_data_int <= to_unsigned(61260,16);
     when "011100110010" => sin_data_int <= to_unsigned(23437,16); cos_data_int <= to_unsigned(61275,16);
     when "011100110011" => sin_data_int <= to_unsigned(23481,16); cos_data_int <= to_unsigned(61289,16);
     when "011100110100" => sin_data_int <= to_unsigned(23525,16); cos_data_int <= to_unsigned(61303,16);
     when "011100110101" => sin_data_int <= to_unsigned(23568,16); cos_data_int <= to_unsigned(61317,16);
     when "011100110110" => sin_data_int <= to_unsigned(23612,16); cos_data_int <= to_unsigned(61331,16);
     when "011100110111" => sin_data_int <= to_unsigned(23656,16); cos_data_int <= to_unsigned(61345,16);
     when "011100111000" => sin_data_int <= to_unsigned(23700,16); cos_data_int <= to_unsigned(61359,16);
     when "011100111001" => sin_data_int <= to_unsigned(23744,16); cos_data_int <= to_unsigned(61373,16);
     when "011100111010" => sin_data_int <= to_unsigned(23788,16); cos_data_int <= to_unsigned(61387,16);
     when "011100111011" => sin_data_int <= to_unsigned(23832,16); cos_data_int <= to_unsigned(61401,16);
     when "011100111100" => sin_data_int <= to_unsigned(23875,16); cos_data_int <= to_unsigned(61414,16);
     when "011100111101" => sin_data_int <= to_unsigned(23919,16); cos_data_int <= to_unsigned(61428,16);
     when "011100111110" => sin_data_int <= to_unsigned(23963,16); cos_data_int <= to_unsigned(61441,16);
     when "011100111111" => sin_data_int <= to_unsigned(24007,16); cos_data_int <= to_unsigned(61455,16);
     when "011101000000" => sin_data_int <= to_unsigned(24051,16); cos_data_int <= to_unsigned(61468,16);
     when "011101000001" => sin_data_int <= to_unsigned(24096,16); cos_data_int <= to_unsigned(61482,16);
     when "011101000010" => sin_data_int <= to_unsigned(24140,16); cos_data_int <= to_unsigned(61495,16);
     when "011101000011" => sin_data_int <= to_unsigned(24184,16); cos_data_int <= to_unsigned(61508,16);
     when "011101000100" => sin_data_int <= to_unsigned(24228,16); cos_data_int <= to_unsigned(61521,16);
     when "011101000101" => sin_data_int <= to_unsigned(24272,16); cos_data_int <= to_unsigned(61534,16);
     when "011101000110" => sin_data_int <= to_unsigned(24316,16); cos_data_int <= to_unsigned(61547,16);
     when "011101000111" => sin_data_int <= to_unsigned(24360,16); cos_data_int <= to_unsigned(61560,16);
     when "011101001000" => sin_data_int <= to_unsigned(24404,16); cos_data_int <= to_unsigned(61573,16);
     when "011101001001" => sin_data_int <= to_unsigned(24449,16); cos_data_int <= to_unsigned(61586,16);
     when "011101001010" => sin_data_int <= to_unsigned(24493,16); cos_data_int <= to_unsigned(61598,16);
     when "011101001011" => sin_data_int <= to_unsigned(24537,16); cos_data_int <= to_unsigned(61611,16);
     when "011101001100" => sin_data_int <= to_unsigned(24581,16); cos_data_int <= to_unsigned(61624,16);
     when "011101001101" => sin_data_int <= to_unsigned(24626,16); cos_data_int <= to_unsigned(61636,16);
     when "011101001110" => sin_data_int <= to_unsigned(24670,16); cos_data_int <= to_unsigned(61649,16);
     when "011101001111" => sin_data_int <= to_unsigned(24714,16); cos_data_int <= to_unsigned(61661,16);
     when "011101010000" => sin_data_int <= to_unsigned(24759,16); cos_data_int <= to_unsigned(61673,16);
     when "011101010001" => sin_data_int <= to_unsigned(24803,16); cos_data_int <= to_unsigned(61686,16);
     when "011101010010" => sin_data_int <= to_unsigned(24847,16); cos_data_int <= to_unsigned(61698,16);
     when "011101010011" => sin_data_int <= to_unsigned(24892,16); cos_data_int <= to_unsigned(61710,16);
     when "011101010100" => sin_data_int <= to_unsigned(24936,16); cos_data_int <= to_unsigned(61722,16);
     when "011101010101" => sin_data_int <= to_unsigned(24981,16); cos_data_int <= to_unsigned(61734,16);
     when "011101010110" => sin_data_int <= to_unsigned(25025,16); cos_data_int <= to_unsigned(61746,16);
     when "011101010111" => sin_data_int <= to_unsigned(25070,16); cos_data_int <= to_unsigned(61758,16);
     when "011101011000" => sin_data_int <= to_unsigned(25114,16); cos_data_int <= to_unsigned(61769,16);
     when "011101011001" => sin_data_int <= to_unsigned(25159,16); cos_data_int <= to_unsigned(61781,16);
     when "011101011010" => sin_data_int <= to_unsigned(25203,16); cos_data_int <= to_unsigned(61793,16);
     when "011101011011" => sin_data_int <= to_unsigned(25248,16); cos_data_int <= to_unsigned(61804,16);
     when "011101011100" => sin_data_int <= to_unsigned(25292,16); cos_data_int <= to_unsigned(61816,16);
     when "011101011101" => sin_data_int <= to_unsigned(25337,16); cos_data_int <= to_unsigned(61827,16);
     when "011101011110" => sin_data_int <= to_unsigned(25381,16); cos_data_int <= to_unsigned(61838,16);
     when "011101011111" => sin_data_int <= to_unsigned(25426,16); cos_data_int <= to_unsigned(61850,16);
     when "011101100000" => sin_data_int <= to_unsigned(25471,16); cos_data_int <= to_unsigned(61861,16);
     when "011101100001" => sin_data_int <= to_unsigned(25515,16); cos_data_int <= to_unsigned(61872,16);
     when "011101100010" => sin_data_int <= to_unsigned(25560,16); cos_data_int <= to_unsigned(61883,16);
     when "011101100011" => sin_data_int <= to_unsigned(25605,16); cos_data_int <= to_unsigned(61894,16);
     when "011101100100" => sin_data_int <= to_unsigned(25649,16); cos_data_int <= to_unsigned(61905,16);
     when "011101100101" => sin_data_int <= to_unsigned(25694,16); cos_data_int <= to_unsigned(61916,16);
     when "011101100110" => sin_data_int <= to_unsigned(25739,16); cos_data_int <= to_unsigned(61927,16);
     when "011101100111" => sin_data_int <= to_unsigned(25783,16); cos_data_int <= to_unsigned(61938,16);
     when "011101101000" => sin_data_int <= to_unsigned(25828,16); cos_data_int <= to_unsigned(61948,16);
     when "011101101001" => sin_data_int <= to_unsigned(25873,16); cos_data_int <= to_unsigned(61959,16);
     when "011101101010" => sin_data_int <= to_unsigned(25918,16); cos_data_int <= to_unsigned(61969,16);
     when "011101101011" => sin_data_int <= to_unsigned(25963,16); cos_data_int <= to_unsigned(61980,16);
     when "011101101100" => sin_data_int <= to_unsigned(26007,16); cos_data_int <= to_unsigned(61990,16);
     when "011101101101" => sin_data_int <= to_unsigned(26052,16); cos_data_int <= to_unsigned(62001,16);
     when "011101101110" => sin_data_int <= to_unsigned(26097,16); cos_data_int <= to_unsigned(62011,16);
     when "011101101111" => sin_data_int <= to_unsigned(26142,16); cos_data_int <= to_unsigned(62021,16);
     when "011101110000" => sin_data_int <= to_unsigned(26187,16); cos_data_int <= to_unsigned(62031,16);
     when "011101110001" => sin_data_int <= to_unsigned(26232,16); cos_data_int <= to_unsigned(62041,16);
     when "011101110010" => sin_data_int <= to_unsigned(26277,16); cos_data_int <= to_unsigned(62051,16);
     when "011101110011" => sin_data_int <= to_unsigned(26322,16); cos_data_int <= to_unsigned(62061,16);
     when "011101110100" => sin_data_int <= to_unsigned(26367,16); cos_data_int <= to_unsigned(62071,16);
     when "011101110101" => sin_data_int <= to_unsigned(26412,16); cos_data_int <= to_unsigned(62081,16);
     when "011101110110" => sin_data_int <= to_unsigned(26457,16); cos_data_int <= to_unsigned(62090,16);
     when "011101110111" => sin_data_int <= to_unsigned(26502,16); cos_data_int <= to_unsigned(62100,16);
     when "011101111000" => sin_data_int <= to_unsigned(26547,16); cos_data_int <= to_unsigned(62110,16);
     when "011101111001" => sin_data_int <= to_unsigned(26592,16); cos_data_int <= to_unsigned(62119,16);
     when "011101111010" => sin_data_int <= to_unsigned(26637,16); cos_data_int <= to_unsigned(62128,16);
     when "011101111011" => sin_data_int <= to_unsigned(26682,16); cos_data_int <= to_unsigned(62138,16);
     when "011101111100" => sin_data_int <= to_unsigned(26727,16); cos_data_int <= to_unsigned(62147,16);
     when "011101111101" => sin_data_int <= to_unsigned(26772,16); cos_data_int <= to_unsigned(62156,16);
     when "011101111110" => sin_data_int <= to_unsigned(26817,16); cos_data_int <= to_unsigned(62165,16);
     when "011101111111" => sin_data_int <= to_unsigned(26862,16); cos_data_int <= to_unsigned(62175,16);
     when "011110000000" => sin_data_int <= to_unsigned(26907,16); cos_data_int <= to_unsigned(62184,16);
     when "011110000001" => sin_data_int <= to_unsigned(26952,16); cos_data_int <= to_unsigned(62193,16);
     when "011110000010" => sin_data_int <= to_unsigned(26998,16); cos_data_int <= to_unsigned(62201,16);
     when "011110000011" => sin_data_int <= to_unsigned(27043,16); cos_data_int <= to_unsigned(62210,16);
     when "011110000100" => sin_data_int <= to_unsigned(27088,16); cos_data_int <= to_unsigned(62219,16);
     when "011110000101" => sin_data_int <= to_unsigned(27133,16); cos_data_int <= to_unsigned(62228,16);
     when "011110000110" => sin_data_int <= to_unsigned(27178,16); cos_data_int <= to_unsigned(62236,16);
     when "011110000111" => sin_data_int <= to_unsigned(27224,16); cos_data_int <= to_unsigned(62245,16);
     when "011110001000" => sin_data_int <= to_unsigned(27269,16); cos_data_int <= to_unsigned(62253,16);
     when "011110001001" => sin_data_int <= to_unsigned(27314,16); cos_data_int <= to_unsigned(62262,16);
     when "011110001010" => sin_data_int <= to_unsigned(27359,16); cos_data_int <= to_unsigned(62270,16);
     when "011110001011" => sin_data_int <= to_unsigned(27405,16); cos_data_int <= to_unsigned(62278,16);
     when "011110001100" => sin_data_int <= to_unsigned(27450,16); cos_data_int <= to_unsigned(62286,16);
     when "011110001101" => sin_data_int <= to_unsigned(27495,16); cos_data_int <= to_unsigned(62294,16);
     when "011110001110" => sin_data_int <= to_unsigned(27540,16); cos_data_int <= to_unsigned(62302,16);
     when "011110001111" => sin_data_int <= to_unsigned(27586,16); cos_data_int <= to_unsigned(62310,16);
     when "011110010000" => sin_data_int <= to_unsigned(27631,16); cos_data_int <= to_unsigned(62318,16);
     when "011110010001" => sin_data_int <= to_unsigned(27676,16); cos_data_int <= to_unsigned(62326,16);
     when "011110010010" => sin_data_int <= to_unsigned(27722,16); cos_data_int <= to_unsigned(62334,16);
     when "011110010011" => sin_data_int <= to_unsigned(27767,16); cos_data_int <= to_unsigned(62342,16);
     when "011110010100" => sin_data_int <= to_unsigned(27813,16); cos_data_int <= to_unsigned(62349,16);
     when "011110010101" => sin_data_int <= to_unsigned(27858,16); cos_data_int <= to_unsigned(62357,16);
     when "011110010110" => sin_data_int <= to_unsigned(27903,16); cos_data_int <= to_unsigned(62364,16);
     when "011110010111" => sin_data_int <= to_unsigned(27949,16); cos_data_int <= to_unsigned(62372,16);
     when "011110011000" => sin_data_int <= to_unsigned(27994,16); cos_data_int <= to_unsigned(62379,16);
     when "011110011001" => sin_data_int <= to_unsigned(28040,16); cos_data_int <= to_unsigned(62386,16);
     when "011110011010" => sin_data_int <= to_unsigned(28085,16); cos_data_int <= to_unsigned(62394,16);
     when "011110011011" => sin_data_int <= to_unsigned(28131,16); cos_data_int <= to_unsigned(62401,16);
     when "011110011100" => sin_data_int <= to_unsigned(28176,16); cos_data_int <= to_unsigned(62408,16);
     when "011110011101" => sin_data_int <= to_unsigned(28222,16); cos_data_int <= to_unsigned(62415,16);
     when "011110011110" => sin_data_int <= to_unsigned(28267,16); cos_data_int <= to_unsigned(62422,16);
     when "011110011111" => sin_data_int <= to_unsigned(28313,16); cos_data_int <= to_unsigned(62429,16);
     when "011110100000" => sin_data_int <= to_unsigned(28358,16); cos_data_int <= to_unsigned(62435,16);
     when "011110100001" => sin_data_int <= to_unsigned(28404,16); cos_data_int <= to_unsigned(62442,16);
     when "011110100010" => sin_data_int <= to_unsigned(28449,16); cos_data_int <= to_unsigned(62449,16);
     when "011110100011" => sin_data_int <= to_unsigned(28495,16); cos_data_int <= to_unsigned(62455,16);
     when "011110100100" => sin_data_int <= to_unsigned(28540,16); cos_data_int <= to_unsigned(62462,16);
     when "011110100101" => sin_data_int <= to_unsigned(28586,16); cos_data_int <= to_unsigned(62468,16);
     when "011110100110" => sin_data_int <= to_unsigned(28631,16); cos_data_int <= to_unsigned(62475,16);
     when "011110100111" => sin_data_int <= to_unsigned(28677,16); cos_data_int <= to_unsigned(62481,16);
     when "011110101000" => sin_data_int <= to_unsigned(28723,16); cos_data_int <= to_unsigned(62487,16);
     when "011110101001" => sin_data_int <= to_unsigned(28768,16); cos_data_int <= to_unsigned(62493,16);
     when "011110101010" => sin_data_int <= to_unsigned(28814,16); cos_data_int <= to_unsigned(62499,16);
     when "011110101011" => sin_data_int <= to_unsigned(28859,16); cos_data_int <= to_unsigned(62505,16);
     when "011110101100" => sin_data_int <= to_unsigned(28905,16); cos_data_int <= to_unsigned(62511,16);
     when "011110101101" => sin_data_int <= to_unsigned(28951,16); cos_data_int <= to_unsigned(62517,16);
     when "011110101110" => sin_data_int <= to_unsigned(28996,16); cos_data_int <= to_unsigned(62523,16);
     when "011110101111" => sin_data_int <= to_unsigned(29042,16); cos_data_int <= to_unsigned(62529,16);
     when "011110110000" => sin_data_int <= to_unsigned(29088,16); cos_data_int <= to_unsigned(62534,16);
     when "011110110001" => sin_data_int <= to_unsigned(29133,16); cos_data_int <= to_unsigned(62540,16);
     when "011110110010" => sin_data_int <= to_unsigned(29179,16); cos_data_int <= to_unsigned(62546,16);
     when "011110110011" => sin_data_int <= to_unsigned(29225,16); cos_data_int <= to_unsigned(62551,16);
     when "011110110100" => sin_data_int <= to_unsigned(29270,16); cos_data_int <= to_unsigned(62556,16);
     when "011110110101" => sin_data_int <= to_unsigned(29316,16); cos_data_int <= to_unsigned(62562,16);
     when "011110110110" => sin_data_int <= to_unsigned(29362,16); cos_data_int <= to_unsigned(62567,16);
     when "011110110111" => sin_data_int <= to_unsigned(29408,16); cos_data_int <= to_unsigned(62572,16);
     when "011110111000" => sin_data_int <= to_unsigned(29453,16); cos_data_int <= to_unsigned(62577,16);
     when "011110111001" => sin_data_int <= to_unsigned(29499,16); cos_data_int <= to_unsigned(62582,16);
     when "011110111010" => sin_data_int <= to_unsigned(29545,16); cos_data_int <= to_unsigned(62587,16);
     when "011110111011" => sin_data_int <= to_unsigned(29591,16); cos_data_int <= to_unsigned(62592,16);
     when "011110111100" => sin_data_int <= to_unsigned(29636,16); cos_data_int <= to_unsigned(62597,16);
     when "011110111101" => sin_data_int <= to_unsigned(29682,16); cos_data_int <= to_unsigned(62602,16);
     when "011110111110" => sin_data_int <= to_unsigned(29728,16); cos_data_int <= to_unsigned(62606,16);
     when "011110111111" => sin_data_int <= to_unsigned(29774,16); cos_data_int <= to_unsigned(62611,16);
     when "011111000000" => sin_data_int <= to_unsigned(29819,16); cos_data_int <= to_unsigned(62616,16);
     when "011111000001" => sin_data_int <= to_unsigned(29865,16); cos_data_int <= to_unsigned(62620,16);
     when "011111000010" => sin_data_int <= to_unsigned(29911,16); cos_data_int <= to_unsigned(62624,16);
     when "011111000011" => sin_data_int <= to_unsigned(29957,16); cos_data_int <= to_unsigned(62629,16);
     when "011111000100" => sin_data_int <= to_unsigned(30003,16); cos_data_int <= to_unsigned(62633,16);
     when "011111000101" => sin_data_int <= to_unsigned(30049,16); cos_data_int <= to_unsigned(62637,16);
     when "011111000110" => sin_data_int <= to_unsigned(30094,16); cos_data_int <= to_unsigned(62641,16);
     when "011111000111" => sin_data_int <= to_unsigned(30140,16); cos_data_int <= to_unsigned(62645,16);
     when "011111001000" => sin_data_int <= to_unsigned(30186,16); cos_data_int <= to_unsigned(62649,16);
     when "011111001001" => sin_data_int <= to_unsigned(30232,16); cos_data_int <= to_unsigned(62653,16);
     when "011111001010" => sin_data_int <= to_unsigned(30278,16); cos_data_int <= to_unsigned(62657,16);
     when "011111001011" => sin_data_int <= to_unsigned(30324,16); cos_data_int <= to_unsigned(62661,16);
     when "011111001100" => sin_data_int <= to_unsigned(30370,16); cos_data_int <= to_unsigned(62665,16);
     when "011111001101" => sin_data_int <= to_unsigned(30415,16); cos_data_int <= to_unsigned(62668,16);
     when "011111001110" => sin_data_int <= to_unsigned(30461,16); cos_data_int <= to_unsigned(62672,16);
     when "011111001111" => sin_data_int <= to_unsigned(30507,16); cos_data_int <= to_unsigned(62675,16);
     when "011111010000" => sin_data_int <= to_unsigned(30553,16); cos_data_int <= to_unsigned(62679,16);
     when "011111010001" => sin_data_int <= to_unsigned(30599,16); cos_data_int <= to_unsigned(62682,16);
     when "011111010010" => sin_data_int <= to_unsigned(30645,16); cos_data_int <= to_unsigned(62685,16);
     when "011111010011" => sin_data_int <= to_unsigned(30691,16); cos_data_int <= to_unsigned(62689,16);
     when "011111010100" => sin_data_int <= to_unsigned(30737,16); cos_data_int <= to_unsigned(62692,16);
     when "011111010101" => sin_data_int <= to_unsigned(30783,16); cos_data_int <= to_unsigned(62695,16);
     when "011111010110" => sin_data_int <= to_unsigned(30829,16); cos_data_int <= to_unsigned(62698,16);
     when "011111010111" => sin_data_int <= to_unsigned(30874,16); cos_data_int <= to_unsigned(62701,16);
     when "011111011000" => sin_data_int <= to_unsigned(30920,16); cos_data_int <= to_unsigned(62704,16);
     when "011111011001" => sin_data_int <= to_unsigned(30966,16); cos_data_int <= to_unsigned(62706,16);
     when "011111011010" => sin_data_int <= to_unsigned(31012,16); cos_data_int <= to_unsigned(62709,16);
     when "011111011011" => sin_data_int <= to_unsigned(31058,16); cos_data_int <= to_unsigned(62712,16);
     when "011111011100" => sin_data_int <= to_unsigned(31104,16); cos_data_int <= to_unsigned(62714,16);
     when "011111011101" => sin_data_int <= to_unsigned(31150,16); cos_data_int <= to_unsigned(62717,16);
     when "011111011110" => sin_data_int <= to_unsigned(31196,16); cos_data_int <= to_unsigned(62719,16);
     when "011111011111" => sin_data_int <= to_unsigned(31242,16); cos_data_int <= to_unsigned(62722,16);
     when "011111100000" => sin_data_int <= to_unsigned(31288,16); cos_data_int <= to_unsigned(62724,16);
     when "011111100001" => sin_data_int <= to_unsigned(31334,16); cos_data_int <= to_unsigned(62726,16);
     when "011111100010" => sin_data_int <= to_unsigned(31380,16); cos_data_int <= to_unsigned(62728,16);
     when "011111100011" => sin_data_int <= to_unsigned(31426,16); cos_data_int <= to_unsigned(62730,16);
     when "011111100100" => sin_data_int <= to_unsigned(31472,16); cos_data_int <= to_unsigned(62732,16);
     when "011111100101" => sin_data_int <= to_unsigned(31518,16); cos_data_int <= to_unsigned(62734,16);
     when "011111100110" => sin_data_int <= to_unsigned(31564,16); cos_data_int <= to_unsigned(62736,16);
     when "011111100111" => sin_data_int <= to_unsigned(31610,16); cos_data_int <= to_unsigned(62738,16);
     when "011111101000" => sin_data_int <= to_unsigned(31656,16); cos_data_int <= to_unsigned(62740,16);
     when "011111101001" => sin_data_int <= to_unsigned(31702,16); cos_data_int <= to_unsigned(62741,16);
     when "011111101010" => sin_data_int <= to_unsigned(31748,16); cos_data_int <= to_unsigned(62743,16);
     when "011111101011" => sin_data_int <= to_unsigned(31794,16); cos_data_int <= to_unsigned(62744,16);
     when "011111101100" => sin_data_int <= to_unsigned(31840,16); cos_data_int <= to_unsigned(62746,16);
     when "011111101101" => sin_data_int <= to_unsigned(31886,16); cos_data_int <= to_unsigned(62747,16);
     when "011111101110" => sin_data_int <= to_unsigned(31932,16); cos_data_int <= to_unsigned(62749,16);
     when "011111101111" => sin_data_int <= to_unsigned(31978,16); cos_data_int <= to_unsigned(62750,16);
     when "011111110000" => sin_data_int <= to_unsigned(32024,16); cos_data_int <= to_unsigned(62751,16);
     when "011111110001" => sin_data_int <= to_unsigned(32070,16); cos_data_int <= to_unsigned(62752,16);
     when "011111110010" => sin_data_int <= to_unsigned(32116,16); cos_data_int <= to_unsigned(62753,16);
     when "011111110011" => sin_data_int <= to_unsigned(32162,16); cos_data_int <= to_unsigned(62754,16);
     when "011111110100" => sin_data_int <= to_unsigned(32208,16); cos_data_int <= to_unsigned(62755,16);
     when "011111110101" => sin_data_int <= to_unsigned(32254,16); cos_data_int <= to_unsigned(62756,16);
     when "011111110110" => sin_data_int <= to_unsigned(32300,16); cos_data_int <= to_unsigned(62756,16);
     when "011111110111" => sin_data_int <= to_unsigned(32346,16); cos_data_int <= to_unsigned(62757,16);
     when "011111111000" => sin_data_int <= to_unsigned(32392,16); cos_data_int <= to_unsigned(62758,16);
     when "011111111001" => sin_data_int <= to_unsigned(32438,16); cos_data_int <= to_unsigned(62758,16);
     when "011111111010" => sin_data_int <= to_unsigned(32484,16); cos_data_int <= to_unsigned(62759,16);
     when "011111111011" => sin_data_int <= to_unsigned(32530,16); cos_data_int <= to_unsigned(62759,16);
     when "011111111100" => sin_data_int <= to_unsigned(32576,16); cos_data_int <= to_unsigned(62759,16);
     when "011111111101" => sin_data_int <= to_unsigned(32622,16); cos_data_int <= to_unsigned(62760,16);
     when "011111111110" => sin_data_int <= to_unsigned(32668,16); cos_data_int <= to_unsigned(62760,16);
     when "011111111111" => sin_data_int <= to_unsigned(32714,16); cos_data_int <= to_unsigned(62760,16);
     when "100000000000" => sin_data_int <= to_unsigned(32760,16); cos_data_int <= to_unsigned(62760,16);
     when "100000000001" => sin_data_int <= to_unsigned(32806,16); cos_data_int <= to_unsigned(62760,16);
     when "100000000010" => sin_data_int <= to_unsigned(32852,16); cos_data_int <= to_unsigned(62760,16);
     when "100000000011" => sin_data_int <= to_unsigned(32898,16); cos_data_int <= to_unsigned(62760,16);
     when "100000000100" => sin_data_int <= to_unsigned(32944,16); cos_data_int <= to_unsigned(62759,16);
     when "100000000101" => sin_data_int <= to_unsigned(32990,16); cos_data_int <= to_unsigned(62759,16);
     when "100000000110" => sin_data_int <= to_unsigned(33036,16); cos_data_int <= to_unsigned(62759,16);
     when "100000000111" => sin_data_int <= to_unsigned(33082,16); cos_data_int <= to_unsigned(62758,16);
     when "100000001000" => sin_data_int <= to_unsigned(33128,16); cos_data_int <= to_unsigned(62758,16);
     when "100000001001" => sin_data_int <= to_unsigned(33174,16); cos_data_int <= to_unsigned(62757,16);
     when "100000001010" => sin_data_int <= to_unsigned(33220,16); cos_data_int <= to_unsigned(62756,16);
     when "100000001011" => sin_data_int <= to_unsigned(33266,16); cos_data_int <= to_unsigned(62756,16);
     when "100000001100" => sin_data_int <= to_unsigned(33312,16); cos_data_int <= to_unsigned(62755,16);
     when "100000001101" => sin_data_int <= to_unsigned(33358,16); cos_data_int <= to_unsigned(62754,16);
     when "100000001110" => sin_data_int <= to_unsigned(33404,16); cos_data_int <= to_unsigned(62753,16);
     when "100000001111" => sin_data_int <= to_unsigned(33450,16); cos_data_int <= to_unsigned(62752,16);
     when "100000010000" => sin_data_int <= to_unsigned(33496,16); cos_data_int <= to_unsigned(62751,16);
     when "100000010001" => sin_data_int <= to_unsigned(33542,16); cos_data_int <= to_unsigned(62750,16);
     when "100000010010" => sin_data_int <= to_unsigned(33588,16); cos_data_int <= to_unsigned(62749,16);
     when "100000010011" => sin_data_int <= to_unsigned(33634,16); cos_data_int <= to_unsigned(62747,16);
     when "100000010100" => sin_data_int <= to_unsigned(33680,16); cos_data_int <= to_unsigned(62746,16);
     when "100000010101" => sin_data_int <= to_unsigned(33726,16); cos_data_int <= to_unsigned(62744,16);
     when "100000010110" => sin_data_int <= to_unsigned(33772,16); cos_data_int <= to_unsigned(62743,16);
     when "100000010111" => sin_data_int <= to_unsigned(33818,16); cos_data_int <= to_unsigned(62741,16);
     when "100000011000" => sin_data_int <= to_unsigned(33864,16); cos_data_int <= to_unsigned(62740,16);
     when "100000011001" => sin_data_int <= to_unsigned(33910,16); cos_data_int <= to_unsigned(62738,16);
     when "100000011010" => sin_data_int <= to_unsigned(33956,16); cos_data_int <= to_unsigned(62736,16);
     when "100000011011" => sin_data_int <= to_unsigned(34002,16); cos_data_int <= to_unsigned(62734,16);
     when "100000011100" => sin_data_int <= to_unsigned(34048,16); cos_data_int <= to_unsigned(62732,16);
     when "100000011101" => sin_data_int <= to_unsigned(34094,16); cos_data_int <= to_unsigned(62730,16);
     when "100000011110" => sin_data_int <= to_unsigned(34140,16); cos_data_int <= to_unsigned(62728,16);
     when "100000011111" => sin_data_int <= to_unsigned(34186,16); cos_data_int <= to_unsigned(62726,16);
     when "100000100000" => sin_data_int <= to_unsigned(34232,16); cos_data_int <= to_unsigned(62724,16);
     when "100000100001" => sin_data_int <= to_unsigned(34278,16); cos_data_int <= to_unsigned(62722,16);
     when "100000100010" => sin_data_int <= to_unsigned(34324,16); cos_data_int <= to_unsigned(62719,16);
     when "100000100011" => sin_data_int <= to_unsigned(34370,16); cos_data_int <= to_unsigned(62717,16);
     when "100000100100" => sin_data_int <= to_unsigned(34416,16); cos_data_int <= to_unsigned(62714,16);
     when "100000100101" => sin_data_int <= to_unsigned(34462,16); cos_data_int <= to_unsigned(62712,16);
     when "100000100110" => sin_data_int <= to_unsigned(34508,16); cos_data_int <= to_unsigned(62709,16);
     when "100000100111" => sin_data_int <= to_unsigned(34554,16); cos_data_int <= to_unsigned(62706,16);
     when "100000101000" => sin_data_int <= to_unsigned(34600,16); cos_data_int <= to_unsigned(62704,16);
     when "100000101001" => sin_data_int <= to_unsigned(34646,16); cos_data_int <= to_unsigned(62701,16);
     when "100000101010" => sin_data_int <= to_unsigned(34691,16); cos_data_int <= to_unsigned(62698,16);
     when "100000101011" => sin_data_int <= to_unsigned(34737,16); cos_data_int <= to_unsigned(62695,16);
     when "100000101100" => sin_data_int <= to_unsigned(34783,16); cos_data_int <= to_unsigned(62692,16);
     when "100000101101" => sin_data_int <= to_unsigned(34829,16); cos_data_int <= to_unsigned(62689,16);
     when "100000101110" => sin_data_int <= to_unsigned(34875,16); cos_data_int <= to_unsigned(62685,16);
     when "100000101111" => sin_data_int <= to_unsigned(34921,16); cos_data_int <= to_unsigned(62682,16);
     when "100000110000" => sin_data_int <= to_unsigned(34967,16); cos_data_int <= to_unsigned(62679,16);
     when "100000110001" => sin_data_int <= to_unsigned(35013,16); cos_data_int <= to_unsigned(62675,16);
     when "100000110010" => sin_data_int <= to_unsigned(35059,16); cos_data_int <= to_unsigned(62672,16);
     when "100000110011" => sin_data_int <= to_unsigned(35105,16); cos_data_int <= to_unsigned(62668,16);
     when "100000110100" => sin_data_int <= to_unsigned(35150,16); cos_data_int <= to_unsigned(62665,16);
     when "100000110101" => sin_data_int <= to_unsigned(35196,16); cos_data_int <= to_unsigned(62661,16);
     when "100000110110" => sin_data_int <= to_unsigned(35242,16); cos_data_int <= to_unsigned(62657,16);
     when "100000110111" => sin_data_int <= to_unsigned(35288,16); cos_data_int <= to_unsigned(62653,16);
     when "100000111000" => sin_data_int <= to_unsigned(35334,16); cos_data_int <= to_unsigned(62649,16);
     when "100000111001" => sin_data_int <= to_unsigned(35380,16); cos_data_int <= to_unsigned(62645,16);
     when "100000111010" => sin_data_int <= to_unsigned(35426,16); cos_data_int <= to_unsigned(62641,16);
     when "100000111011" => sin_data_int <= to_unsigned(35471,16); cos_data_int <= to_unsigned(62637,16);
     when "100000111100" => sin_data_int <= to_unsigned(35517,16); cos_data_int <= to_unsigned(62633,16);
     when "100000111101" => sin_data_int <= to_unsigned(35563,16); cos_data_int <= to_unsigned(62629,16);
     when "100000111110" => sin_data_int <= to_unsigned(35609,16); cos_data_int <= to_unsigned(62624,16);
     when "100000111111" => sin_data_int <= to_unsigned(35655,16); cos_data_int <= to_unsigned(62620,16);
     when "100001000000" => sin_data_int <= to_unsigned(35701,16); cos_data_int <= to_unsigned(62616,16);
     when "100001000001" => sin_data_int <= to_unsigned(35746,16); cos_data_int <= to_unsigned(62611,16);
     when "100001000010" => sin_data_int <= to_unsigned(35792,16); cos_data_int <= to_unsigned(62606,16);
     when "100001000011" => sin_data_int <= to_unsigned(35838,16); cos_data_int <= to_unsigned(62602,16);
     when "100001000100" => sin_data_int <= to_unsigned(35884,16); cos_data_int <= to_unsigned(62597,16);
     when "100001000101" => sin_data_int <= to_unsigned(35929,16); cos_data_int <= to_unsigned(62592,16);
     when "100001000110" => sin_data_int <= to_unsigned(35975,16); cos_data_int <= to_unsigned(62587,16);
     when "100001000111" => sin_data_int <= to_unsigned(36021,16); cos_data_int <= to_unsigned(62582,16);
     when "100001001000" => sin_data_int <= to_unsigned(36067,16); cos_data_int <= to_unsigned(62577,16);
     when "100001001001" => sin_data_int <= to_unsigned(36112,16); cos_data_int <= to_unsigned(62572,16);
     when "100001001010" => sin_data_int <= to_unsigned(36158,16); cos_data_int <= to_unsigned(62567,16);
     when "100001001011" => sin_data_int <= to_unsigned(36204,16); cos_data_int <= to_unsigned(62562,16);
     when "100001001100" => sin_data_int <= to_unsigned(36250,16); cos_data_int <= to_unsigned(62556,16);
     when "100001001101" => sin_data_int <= to_unsigned(36295,16); cos_data_int <= to_unsigned(62551,16);
     when "100001001110" => sin_data_int <= to_unsigned(36341,16); cos_data_int <= to_unsigned(62546,16);
     when "100001001111" => sin_data_int <= to_unsigned(36387,16); cos_data_int <= to_unsigned(62540,16);
     when "100001010000" => sin_data_int <= to_unsigned(36432,16); cos_data_int <= to_unsigned(62534,16);
     when "100001010001" => sin_data_int <= to_unsigned(36478,16); cos_data_int <= to_unsigned(62529,16);
     when "100001010010" => sin_data_int <= to_unsigned(36524,16); cos_data_int <= to_unsigned(62523,16);
     when "100001010011" => sin_data_int <= to_unsigned(36569,16); cos_data_int <= to_unsigned(62517,16);
     when "100001010100" => sin_data_int <= to_unsigned(36615,16); cos_data_int <= to_unsigned(62511,16);
     when "100001010101" => sin_data_int <= to_unsigned(36661,16); cos_data_int <= to_unsigned(62505,16);
     when "100001010110" => sin_data_int <= to_unsigned(36706,16); cos_data_int <= to_unsigned(62499,16);
     when "100001010111" => sin_data_int <= to_unsigned(36752,16); cos_data_int <= to_unsigned(62493,16);
     when "100001011000" => sin_data_int <= to_unsigned(36797,16); cos_data_int <= to_unsigned(62487,16);
     when "100001011001" => sin_data_int <= to_unsigned(36843,16); cos_data_int <= to_unsigned(62481,16);
     when "100001011010" => sin_data_int <= to_unsigned(36889,16); cos_data_int <= to_unsigned(62475,16);
     when "100001011011" => sin_data_int <= to_unsigned(36934,16); cos_data_int <= to_unsigned(62468,16);
     when "100001011100" => sin_data_int <= to_unsigned(36980,16); cos_data_int <= to_unsigned(62462,16);
     when "100001011101" => sin_data_int <= to_unsigned(37025,16); cos_data_int <= to_unsigned(62455,16);
     when "100001011110" => sin_data_int <= to_unsigned(37071,16); cos_data_int <= to_unsigned(62449,16);
     when "100001011111" => sin_data_int <= to_unsigned(37116,16); cos_data_int <= to_unsigned(62442,16);
     when "100001100000" => sin_data_int <= to_unsigned(37162,16); cos_data_int <= to_unsigned(62435,16);
     when "100001100001" => sin_data_int <= to_unsigned(37207,16); cos_data_int <= to_unsigned(62429,16);
     when "100001100010" => sin_data_int <= to_unsigned(37253,16); cos_data_int <= to_unsigned(62422,16);
     when "100001100011" => sin_data_int <= to_unsigned(37298,16); cos_data_int <= to_unsigned(62415,16);
     when "100001100100" => sin_data_int <= to_unsigned(37344,16); cos_data_int <= to_unsigned(62408,16);
     when "100001100101" => sin_data_int <= to_unsigned(37389,16); cos_data_int <= to_unsigned(62401,16);
     when "100001100110" => sin_data_int <= to_unsigned(37435,16); cos_data_int <= to_unsigned(62394,16);
     when "100001100111" => sin_data_int <= to_unsigned(37480,16); cos_data_int <= to_unsigned(62386,16);
     when "100001101000" => sin_data_int <= to_unsigned(37526,16); cos_data_int <= to_unsigned(62379,16);
     when "100001101001" => sin_data_int <= to_unsigned(37571,16); cos_data_int <= to_unsigned(62372,16);
     when "100001101010" => sin_data_int <= to_unsigned(37617,16); cos_data_int <= to_unsigned(62364,16);
     when "100001101011" => sin_data_int <= to_unsigned(37662,16); cos_data_int <= to_unsigned(62357,16);
     when "100001101100" => sin_data_int <= to_unsigned(37707,16); cos_data_int <= to_unsigned(62349,16);
     when "100001101101" => sin_data_int <= to_unsigned(37753,16); cos_data_int <= to_unsigned(62342,16);
     when "100001101110" => sin_data_int <= to_unsigned(37798,16); cos_data_int <= to_unsigned(62334,16);
     when "100001101111" => sin_data_int <= to_unsigned(37844,16); cos_data_int <= to_unsigned(62326,16);
     when "100001110000" => sin_data_int <= to_unsigned(37889,16); cos_data_int <= to_unsigned(62318,16);
     when "100001110001" => sin_data_int <= to_unsigned(37934,16); cos_data_int <= to_unsigned(62310,16);
     when "100001110010" => sin_data_int <= to_unsigned(37980,16); cos_data_int <= to_unsigned(62302,16);
     when "100001110011" => sin_data_int <= to_unsigned(38025,16); cos_data_int <= to_unsigned(62294,16);
     when "100001110100" => sin_data_int <= to_unsigned(38070,16); cos_data_int <= to_unsigned(62286,16);
     when "100001110101" => sin_data_int <= to_unsigned(38115,16); cos_data_int <= to_unsigned(62278,16);
     when "100001110110" => sin_data_int <= to_unsigned(38161,16); cos_data_int <= to_unsigned(62270,16);
     when "100001110111" => sin_data_int <= to_unsigned(38206,16); cos_data_int <= to_unsigned(62262,16);
     when "100001111000" => sin_data_int <= to_unsigned(38251,16); cos_data_int <= to_unsigned(62253,16);
     when "100001111001" => sin_data_int <= to_unsigned(38296,16); cos_data_int <= to_unsigned(62245,16);
     when "100001111010" => sin_data_int <= to_unsigned(38342,16); cos_data_int <= to_unsigned(62236,16);
     when "100001111011" => sin_data_int <= to_unsigned(38387,16); cos_data_int <= to_unsigned(62228,16);
     when "100001111100" => sin_data_int <= to_unsigned(38432,16); cos_data_int <= to_unsigned(62219,16);
     when "100001111101" => sin_data_int <= to_unsigned(38477,16); cos_data_int <= to_unsigned(62210,16);
     when "100001111110" => sin_data_int <= to_unsigned(38522,16); cos_data_int <= to_unsigned(62201,16);
     when "100001111111" => sin_data_int <= to_unsigned(38568,16); cos_data_int <= to_unsigned(62193,16);
     when "100010000000" => sin_data_int <= to_unsigned(38613,16); cos_data_int <= to_unsigned(62184,16);
     when "100010000001" => sin_data_int <= to_unsigned(38658,16); cos_data_int <= to_unsigned(62175,16);
     when "100010000010" => sin_data_int <= to_unsigned(38703,16); cos_data_int <= to_unsigned(62165,16);
     when "100010000011" => sin_data_int <= to_unsigned(38748,16); cos_data_int <= to_unsigned(62156,16);
     when "100010000100" => sin_data_int <= to_unsigned(38793,16); cos_data_int <= to_unsigned(62147,16);
     when "100010000101" => sin_data_int <= to_unsigned(38838,16); cos_data_int <= to_unsigned(62138,16);
     when "100010000110" => sin_data_int <= to_unsigned(38883,16); cos_data_int <= to_unsigned(62128,16);
     when "100010000111" => sin_data_int <= to_unsigned(38928,16); cos_data_int <= to_unsigned(62119,16);
     when "100010001000" => sin_data_int <= to_unsigned(38973,16); cos_data_int <= to_unsigned(62110,16);
     when "100010001001" => sin_data_int <= to_unsigned(39018,16); cos_data_int <= to_unsigned(62100,16);
     when "100010001010" => sin_data_int <= to_unsigned(39063,16); cos_data_int <= to_unsigned(62090,16);
     when "100010001011" => sin_data_int <= to_unsigned(39108,16); cos_data_int <= to_unsigned(62081,16);
     when "100010001100" => sin_data_int <= to_unsigned(39153,16); cos_data_int <= to_unsigned(62071,16);
     when "100010001101" => sin_data_int <= to_unsigned(39198,16); cos_data_int <= to_unsigned(62061,16);
     when "100010001110" => sin_data_int <= to_unsigned(39243,16); cos_data_int <= to_unsigned(62051,16);
     when "100010001111" => sin_data_int <= to_unsigned(39288,16); cos_data_int <= to_unsigned(62041,16);
     when "100010010000" => sin_data_int <= to_unsigned(39333,16); cos_data_int <= to_unsigned(62031,16);
     when "100010010001" => sin_data_int <= to_unsigned(39378,16); cos_data_int <= to_unsigned(62021,16);
     when "100010010010" => sin_data_int <= to_unsigned(39423,16); cos_data_int <= to_unsigned(62011,16);
     when "100010010011" => sin_data_int <= to_unsigned(39468,16); cos_data_int <= to_unsigned(62001,16);
     when "100010010100" => sin_data_int <= to_unsigned(39513,16); cos_data_int <= to_unsigned(61990,16);
     when "100010010101" => sin_data_int <= to_unsigned(39557,16); cos_data_int <= to_unsigned(61980,16);
     when "100010010110" => sin_data_int <= to_unsigned(39602,16); cos_data_int <= to_unsigned(61969,16);
     when "100010010111" => sin_data_int <= to_unsigned(39647,16); cos_data_int <= to_unsigned(61959,16);
     when "100010011000" => sin_data_int <= to_unsigned(39692,16); cos_data_int <= to_unsigned(61948,16);
     when "100010011001" => sin_data_int <= to_unsigned(39737,16); cos_data_int <= to_unsigned(61938,16);
     when "100010011010" => sin_data_int <= to_unsigned(39781,16); cos_data_int <= to_unsigned(61927,16);
     when "100010011011" => sin_data_int <= to_unsigned(39826,16); cos_data_int <= to_unsigned(61916,16);
     when "100010011100" => sin_data_int <= to_unsigned(39871,16); cos_data_int <= to_unsigned(61905,16);
     when "100010011101" => sin_data_int <= to_unsigned(39915,16); cos_data_int <= to_unsigned(61894,16);
     when "100010011110" => sin_data_int <= to_unsigned(39960,16); cos_data_int <= to_unsigned(61883,16);
     when "100010011111" => sin_data_int <= to_unsigned(40005,16); cos_data_int <= to_unsigned(61872,16);
     when "100010100000" => sin_data_int <= to_unsigned(40049,16); cos_data_int <= to_unsigned(61861,16);
     when "100010100001" => sin_data_int <= to_unsigned(40094,16); cos_data_int <= to_unsigned(61850,16);
     when "100010100010" => sin_data_int <= to_unsigned(40139,16); cos_data_int <= to_unsigned(61838,16);
     when "100010100011" => sin_data_int <= to_unsigned(40183,16); cos_data_int <= to_unsigned(61827,16);
     when "100010100100" => sin_data_int <= to_unsigned(40228,16); cos_data_int <= to_unsigned(61816,16);
     when "100010100101" => sin_data_int <= to_unsigned(40272,16); cos_data_int <= to_unsigned(61804,16);
     when "100010100110" => sin_data_int <= to_unsigned(40317,16); cos_data_int <= to_unsigned(61793,16);
     when "100010100111" => sin_data_int <= to_unsigned(40361,16); cos_data_int <= to_unsigned(61781,16);
     when "100010101000" => sin_data_int <= to_unsigned(40406,16); cos_data_int <= to_unsigned(61769,16);
     when "100010101001" => sin_data_int <= to_unsigned(40450,16); cos_data_int <= to_unsigned(61758,16);
     when "100010101010" => sin_data_int <= to_unsigned(40495,16); cos_data_int <= to_unsigned(61746,16);
     when "100010101011" => sin_data_int <= to_unsigned(40539,16); cos_data_int <= to_unsigned(61734,16);
     when "100010101100" => sin_data_int <= to_unsigned(40584,16); cos_data_int <= to_unsigned(61722,16);
     when "100010101101" => sin_data_int <= to_unsigned(40628,16); cos_data_int <= to_unsigned(61710,16);
     when "100010101110" => sin_data_int <= to_unsigned(40673,16); cos_data_int <= to_unsigned(61698,16);
     when "100010101111" => sin_data_int <= to_unsigned(40717,16); cos_data_int <= to_unsigned(61686,16);
     when "100010110000" => sin_data_int <= to_unsigned(40761,16); cos_data_int <= to_unsigned(61673,16);
     when "100010110001" => sin_data_int <= to_unsigned(40806,16); cos_data_int <= to_unsigned(61661,16);
     when "100010110010" => sin_data_int <= to_unsigned(40850,16); cos_data_int <= to_unsigned(61649,16);
     when "100010110011" => sin_data_int <= to_unsigned(40894,16); cos_data_int <= to_unsigned(61636,16);
     when "100010110100" => sin_data_int <= to_unsigned(40939,16); cos_data_int <= to_unsigned(61624,16);
     when "100010110101" => sin_data_int <= to_unsigned(40983,16); cos_data_int <= to_unsigned(61611,16);
     when "100010110110" => sin_data_int <= to_unsigned(41027,16); cos_data_int <= to_unsigned(61598,16);
     when "100010110111" => sin_data_int <= to_unsigned(41071,16); cos_data_int <= to_unsigned(61586,16);
     when "100010111000" => sin_data_int <= to_unsigned(41116,16); cos_data_int <= to_unsigned(61573,16);
     when "100010111001" => sin_data_int <= to_unsigned(41160,16); cos_data_int <= to_unsigned(61560,16);
     when "100010111010" => sin_data_int <= to_unsigned(41204,16); cos_data_int <= to_unsigned(61547,16);
     when "100010111011" => sin_data_int <= to_unsigned(41248,16); cos_data_int <= to_unsigned(61534,16);
     when "100010111100" => sin_data_int <= to_unsigned(41292,16); cos_data_int <= to_unsigned(61521,16);
     when "100010111101" => sin_data_int <= to_unsigned(41336,16); cos_data_int <= to_unsigned(61508,16);
     when "100010111110" => sin_data_int <= to_unsigned(41380,16); cos_data_int <= to_unsigned(61495,16);
     when "100010111111" => sin_data_int <= to_unsigned(41424,16); cos_data_int <= to_unsigned(61482,16);
     when "100011000000" => sin_data_int <= to_unsigned(41469,16); cos_data_int <= to_unsigned(61468,16);
     when "100011000001" => sin_data_int <= to_unsigned(41513,16); cos_data_int <= to_unsigned(61455,16);
     when "100011000010" => sin_data_int <= to_unsigned(41557,16); cos_data_int <= to_unsigned(61441,16);
     when "100011000011" => sin_data_int <= to_unsigned(41601,16); cos_data_int <= to_unsigned(61428,16);
     when "100011000100" => sin_data_int <= to_unsigned(41645,16); cos_data_int <= to_unsigned(61414,16);
     when "100011000101" => sin_data_int <= to_unsigned(41688,16); cos_data_int <= to_unsigned(61401,16);
     when "100011000110" => sin_data_int <= to_unsigned(41732,16); cos_data_int <= to_unsigned(61387,16);
     when "100011000111" => sin_data_int <= to_unsigned(41776,16); cos_data_int <= to_unsigned(61373,16);
     when "100011001000" => sin_data_int <= to_unsigned(41820,16); cos_data_int <= to_unsigned(61359,16);
     when "100011001001" => sin_data_int <= to_unsigned(41864,16); cos_data_int <= to_unsigned(61345,16);
     when "100011001010" => sin_data_int <= to_unsigned(41908,16); cos_data_int <= to_unsigned(61331,16);
     when "100011001011" => sin_data_int <= to_unsigned(41952,16); cos_data_int <= to_unsigned(61317,16);
     when "100011001100" => sin_data_int <= to_unsigned(41995,16); cos_data_int <= to_unsigned(61303,16);
     when "100011001101" => sin_data_int <= to_unsigned(42039,16); cos_data_int <= to_unsigned(61289,16);
     when "100011001110" => sin_data_int <= to_unsigned(42083,16); cos_data_int <= to_unsigned(61275,16);
     when "100011001111" => sin_data_int <= to_unsigned(42127,16); cos_data_int <= to_unsigned(61260,16);
     when "100011010000" => sin_data_int <= to_unsigned(42170,16); cos_data_int <= to_unsigned(61246,16);
     when "100011010001" => sin_data_int <= to_unsigned(42214,16); cos_data_int <= to_unsigned(61231,16);
     when "100011010010" => sin_data_int <= to_unsigned(42258,16); cos_data_int <= to_unsigned(61217,16);
     when "100011010011" => sin_data_int <= to_unsigned(42301,16); cos_data_int <= to_unsigned(61202,16);
     when "100011010100" => sin_data_int <= to_unsigned(42345,16); cos_data_int <= to_unsigned(61188,16);
     when "100011010101" => sin_data_int <= to_unsigned(42389,16); cos_data_int <= to_unsigned(61173,16);
     when "100011010110" => sin_data_int <= to_unsigned(42432,16); cos_data_int <= to_unsigned(61158,16);
     when "100011010111" => sin_data_int <= to_unsigned(42476,16); cos_data_int <= to_unsigned(61143,16);
     when "100011011000" => sin_data_int <= to_unsigned(42519,16); cos_data_int <= to_unsigned(61128,16);
     when "100011011001" => sin_data_int <= to_unsigned(42563,16); cos_data_int <= to_unsigned(61113,16);
     when "100011011010" => sin_data_int <= to_unsigned(42606,16); cos_data_int <= to_unsigned(61098,16);
     when "100011011011" => sin_data_int <= to_unsigned(42650,16); cos_data_int <= to_unsigned(61083,16);
     when "100011011100" => sin_data_int <= to_unsigned(42693,16); cos_data_int <= to_unsigned(61068,16);
     when "100011011101" => sin_data_int <= to_unsigned(42737,16); cos_data_int <= to_unsigned(61053,16);
     when "100011011110" => sin_data_int <= to_unsigned(42780,16); cos_data_int <= to_unsigned(61037,16);
     when "100011011111" => sin_data_int <= to_unsigned(42823,16); cos_data_int <= to_unsigned(61022,16);
     when "100011100000" => sin_data_int <= to_unsigned(42867,16); cos_data_int <= to_unsigned(61006,16);
     when "100011100001" => sin_data_int <= to_unsigned(42910,16); cos_data_int <= to_unsigned(60991,16);
     when "100011100010" => sin_data_int <= to_unsigned(42953,16); cos_data_int <= to_unsigned(60975,16);
     when "100011100011" => sin_data_int <= to_unsigned(42997,16); cos_data_int <= to_unsigned(60960,16);
     when "100011100100" => sin_data_int <= to_unsigned(43040,16); cos_data_int <= to_unsigned(60944,16);
     when "100011100101" => sin_data_int <= to_unsigned(43083,16); cos_data_int <= to_unsigned(60928,16);
     when "100011100110" => sin_data_int <= to_unsigned(43126,16); cos_data_int <= to_unsigned(60912,16);
     when "100011100111" => sin_data_int <= to_unsigned(43169,16); cos_data_int <= to_unsigned(60896,16);
     when "100011101000" => sin_data_int <= to_unsigned(43213,16); cos_data_int <= to_unsigned(60880,16);
     when "100011101001" => sin_data_int <= to_unsigned(43256,16); cos_data_int <= to_unsigned(60864,16);
     when "100011101010" => sin_data_int <= to_unsigned(43299,16); cos_data_int <= to_unsigned(60848,16);
     when "100011101011" => sin_data_int <= to_unsigned(43342,16); cos_data_int <= to_unsigned(60832,16);
     when "100011101100" => sin_data_int <= to_unsigned(43385,16); cos_data_int <= to_unsigned(60816,16);
     when "100011101101" => sin_data_int <= to_unsigned(43428,16); cos_data_int <= to_unsigned(60799,16);
     when "100011101110" => sin_data_int <= to_unsigned(43471,16); cos_data_int <= to_unsigned(60783,16);
     when "100011101111" => sin_data_int <= to_unsigned(43514,16); cos_data_int <= to_unsigned(60766,16);
     when "100011110000" => sin_data_int <= to_unsigned(43557,16); cos_data_int <= to_unsigned(60750,16);
     when "100011110001" => sin_data_int <= to_unsigned(43600,16); cos_data_int <= to_unsigned(60733,16);
     when "100011110010" => sin_data_int <= to_unsigned(43643,16); cos_data_int <= to_unsigned(60717,16);
     when "100011110011" => sin_data_int <= to_unsigned(43686,16); cos_data_int <= to_unsigned(60700,16);
     when "100011110100" => sin_data_int <= to_unsigned(43728,16); cos_data_int <= to_unsigned(60683,16);
     when "100011110101" => sin_data_int <= to_unsigned(43771,16); cos_data_int <= to_unsigned(60666,16);
     when "100011110110" => sin_data_int <= to_unsigned(43814,16); cos_data_int <= to_unsigned(60649,16);
     when "100011110111" => sin_data_int <= to_unsigned(43857,16); cos_data_int <= to_unsigned(60632,16);
     when "100011111000" => sin_data_int <= to_unsigned(43900,16); cos_data_int <= to_unsigned(60615,16);
     when "100011111001" => sin_data_int <= to_unsigned(43942,16); cos_data_int <= to_unsigned(60598,16);
     when "100011111010" => sin_data_int <= to_unsigned(43985,16); cos_data_int <= to_unsigned(60581,16);
     when "100011111011" => sin_data_int <= to_unsigned(44028,16); cos_data_int <= to_unsigned(60564,16);
     when "100011111100" => sin_data_int <= to_unsigned(44070,16); cos_data_int <= to_unsigned(60546,16);
     when "100011111101" => sin_data_int <= to_unsigned(44113,16); cos_data_int <= to_unsigned(60529,16);
     when "100011111110" => sin_data_int <= to_unsigned(44155,16); cos_data_int <= to_unsigned(60511,16);
     when "100011111111" => sin_data_int <= to_unsigned(44198,16); cos_data_int <= to_unsigned(60494,16);
     when "100100000000" => sin_data_int <= to_unsigned(44241,16); cos_data_int <= to_unsigned(60476,16);
     when "100100000001" => sin_data_int <= to_unsigned(44283,16); cos_data_int <= to_unsigned(60459,16);
     when "100100000010" => sin_data_int <= to_unsigned(44325,16); cos_data_int <= to_unsigned(60441,16);
     when "100100000011" => sin_data_int <= to_unsigned(44368,16); cos_data_int <= to_unsigned(60423,16);
     when "100100000100" => sin_data_int <= to_unsigned(44410,16); cos_data_int <= to_unsigned(60405,16);
     when "100100000101" => sin_data_int <= to_unsigned(44453,16); cos_data_int <= to_unsigned(60388,16);
     when "100100000110" => sin_data_int <= to_unsigned(44495,16); cos_data_int <= to_unsigned(60370,16);
     when "100100000111" => sin_data_int <= to_unsigned(44537,16); cos_data_int <= to_unsigned(60352,16);
     when "100100001000" => sin_data_int <= to_unsigned(44580,16); cos_data_int <= to_unsigned(60333,16);
     when "100100001001" => sin_data_int <= to_unsigned(44622,16); cos_data_int <= to_unsigned(60315,16);
     when "100100001010" => sin_data_int <= to_unsigned(44664,16); cos_data_int <= to_unsigned(60297,16);
     when "100100001011" => sin_data_int <= to_unsigned(44707,16); cos_data_int <= to_unsigned(60279,16);
     when "100100001100" => sin_data_int <= to_unsigned(44749,16); cos_data_int <= to_unsigned(60260,16);
     when "100100001101" => sin_data_int <= to_unsigned(44791,16); cos_data_int <= to_unsigned(60242,16);
     when "100100001110" => sin_data_int <= to_unsigned(44833,16); cos_data_int <= to_unsigned(60223,16);
     when "100100001111" => sin_data_int <= to_unsigned(44875,16); cos_data_int <= to_unsigned(60205,16);
     when "100100010000" => sin_data_int <= to_unsigned(44917,16); cos_data_int <= to_unsigned(60186,16);
     when "100100010001" => sin_data_int <= to_unsigned(44959,16); cos_data_int <= to_unsigned(60168,16);
     when "100100010010" => sin_data_int <= to_unsigned(45001,16); cos_data_int <= to_unsigned(60149,16);
     when "100100010011" => sin_data_int <= to_unsigned(45043,16); cos_data_int <= to_unsigned(60130,16);
     when "100100010100" => sin_data_int <= to_unsigned(45085,16); cos_data_int <= to_unsigned(60111,16);
     when "100100010101" => sin_data_int <= to_unsigned(45127,16); cos_data_int <= to_unsigned(60092,16);
     when "100100010110" => sin_data_int <= to_unsigned(45169,16); cos_data_int <= to_unsigned(60073,16);
     when "100100010111" => sin_data_int <= to_unsigned(45211,16); cos_data_int <= to_unsigned(60054,16);
     when "100100011000" => sin_data_int <= to_unsigned(45253,16); cos_data_int <= to_unsigned(60035,16);
     when "100100011001" => sin_data_int <= to_unsigned(45295,16); cos_data_int <= to_unsigned(60016,16);
     when "100100011010" => sin_data_int <= to_unsigned(45337,16); cos_data_int <= to_unsigned(59997,16);
     when "100100011011" => sin_data_int <= to_unsigned(45378,16); cos_data_int <= to_unsigned(59977,16);
     when "100100011100" => sin_data_int <= to_unsigned(45420,16); cos_data_int <= to_unsigned(59958,16);
     when "100100011101" => sin_data_int <= to_unsigned(45462,16); cos_data_int <= to_unsigned(59938,16);
     when "100100011110" => sin_data_int <= to_unsigned(45503,16); cos_data_int <= to_unsigned(59919,16);
     when "100100011111" => sin_data_int <= to_unsigned(45545,16); cos_data_int <= to_unsigned(59899,16);
     when "100100100000" => sin_data_int <= to_unsigned(45587,16); cos_data_int <= to_unsigned(59880,16);
     when "100100100001" => sin_data_int <= to_unsigned(45628,16); cos_data_int <= to_unsigned(59860,16);
     when "100100100010" => sin_data_int <= to_unsigned(45670,16); cos_data_int <= to_unsigned(59840,16);
     when "100100100011" => sin_data_int <= to_unsigned(45711,16); cos_data_int <= to_unsigned(59820,16);
     when "100100100100" => sin_data_int <= to_unsigned(45753,16); cos_data_int <= to_unsigned(59800,16);
     when "100100100101" => sin_data_int <= to_unsigned(45794,16); cos_data_int <= to_unsigned(59781,16);
     when "100100100110" => sin_data_int <= to_unsigned(45836,16); cos_data_int <= to_unsigned(59760,16);
     when "100100100111" => sin_data_int <= to_unsigned(45877,16); cos_data_int <= to_unsigned(59740,16);
     when "100100101000" => sin_data_int <= to_unsigned(45918,16); cos_data_int <= to_unsigned(59720,16);
     when "100100101001" => sin_data_int <= to_unsigned(45960,16); cos_data_int <= to_unsigned(59700,16);
     when "100100101010" => sin_data_int <= to_unsigned(46001,16); cos_data_int <= to_unsigned(59680,16);
     when "100100101011" => sin_data_int <= to_unsigned(46042,16); cos_data_int <= to_unsigned(59659,16);
     when "100100101100" => sin_data_int <= to_unsigned(46084,16); cos_data_int <= to_unsigned(59639,16);
     when "100100101101" => sin_data_int <= to_unsigned(46125,16); cos_data_int <= to_unsigned(59619,16);
     when "100100101110" => sin_data_int <= to_unsigned(46166,16); cos_data_int <= to_unsigned(59598,16);
     when "100100101111" => sin_data_int <= to_unsigned(46207,16); cos_data_int <= to_unsigned(59577,16);
     when "100100110000" => sin_data_int <= to_unsigned(46248,16); cos_data_int <= to_unsigned(59557,16);
     when "100100110001" => sin_data_int <= to_unsigned(46289,16); cos_data_int <= to_unsigned(59536,16);
     when "100100110010" => sin_data_int <= to_unsigned(46330,16); cos_data_int <= to_unsigned(59515,16);
     when "100100110011" => sin_data_int <= to_unsigned(46372,16); cos_data_int <= to_unsigned(59494,16);
     when "100100110100" => sin_data_int <= to_unsigned(46413,16); cos_data_int <= to_unsigned(59473,16);
     when "100100110101" => sin_data_int <= to_unsigned(46453,16); cos_data_int <= to_unsigned(59452,16);
     when "100100110110" => sin_data_int <= to_unsigned(46494,16); cos_data_int <= to_unsigned(59431,16);
     when "100100110111" => sin_data_int <= to_unsigned(46535,16); cos_data_int <= to_unsigned(59410,16);
     when "100100111000" => sin_data_int <= to_unsigned(46576,16); cos_data_int <= to_unsigned(59389,16);
     when "100100111001" => sin_data_int <= to_unsigned(46617,16); cos_data_int <= to_unsigned(59368,16);
     when "100100111010" => sin_data_int <= to_unsigned(46658,16); cos_data_int <= to_unsigned(59347,16);
     when "100100111011" => sin_data_int <= to_unsigned(46699,16); cos_data_int <= to_unsigned(59325,16);
     when "100100111100" => sin_data_int <= to_unsigned(46739,16); cos_data_int <= to_unsigned(59304,16);
     when "100100111101" => sin_data_int <= to_unsigned(46780,16); cos_data_int <= to_unsigned(59282,16);
     when "100100111110" => sin_data_int <= to_unsigned(46821,16); cos_data_int <= to_unsigned(59261,16);
     when "100100111111" => sin_data_int <= to_unsigned(46861,16); cos_data_int <= to_unsigned(59239,16);
     when "100101000000" => sin_data_int <= to_unsigned(46902,16); cos_data_int <= to_unsigned(59218,16);
     when "100101000001" => sin_data_int <= to_unsigned(46942,16); cos_data_int <= to_unsigned(59196,16);
     when "100101000010" => sin_data_int <= to_unsigned(46983,16); cos_data_int <= to_unsigned(59174,16);
     when "100101000011" => sin_data_int <= to_unsigned(47024,16); cos_data_int <= to_unsigned(59152,16);
     when "100101000100" => sin_data_int <= to_unsigned(47064,16); cos_data_int <= to_unsigned(59130,16);
     when "100101000101" => sin_data_int <= to_unsigned(47104,16); cos_data_int <= to_unsigned(59108,16);
     when "100101000110" => sin_data_int <= to_unsigned(47145,16); cos_data_int <= to_unsigned(59086,16);
     when "100101000111" => sin_data_int <= to_unsigned(47185,16); cos_data_int <= to_unsigned(59064,16);
     when "100101001000" => sin_data_int <= to_unsigned(47226,16); cos_data_int <= to_unsigned(59042,16);
     when "100101001001" => sin_data_int <= to_unsigned(47266,16); cos_data_int <= to_unsigned(59020,16);
     when "100101001010" => sin_data_int <= to_unsigned(47306,16); cos_data_int <= to_unsigned(58998,16);
     when "100101001011" => sin_data_int <= to_unsigned(47346,16); cos_data_int <= to_unsigned(58975,16);
     when "100101001100" => sin_data_int <= to_unsigned(47387,16); cos_data_int <= to_unsigned(58953,16);
     when "100101001101" => sin_data_int <= to_unsigned(47427,16); cos_data_int <= to_unsigned(58930,16);
     when "100101001110" => sin_data_int <= to_unsigned(47467,16); cos_data_int <= to_unsigned(58908,16);
     when "100101001111" => sin_data_int <= to_unsigned(47507,16); cos_data_int <= to_unsigned(58885,16);
     when "100101010000" => sin_data_int <= to_unsigned(47547,16); cos_data_int <= to_unsigned(58863,16);
     when "100101010001" => sin_data_int <= to_unsigned(47587,16); cos_data_int <= to_unsigned(58840,16);
     when "100101010010" => sin_data_int <= to_unsigned(47627,16); cos_data_int <= to_unsigned(58817,16);
     when "100101010011" => sin_data_int <= to_unsigned(47667,16); cos_data_int <= to_unsigned(58794,16);
     when "100101010100" => sin_data_int <= to_unsigned(47707,16); cos_data_int <= to_unsigned(58771,16);
     when "100101010101" => sin_data_int <= to_unsigned(47747,16); cos_data_int <= to_unsigned(58748,16);
     when "100101010110" => sin_data_int <= to_unsigned(47787,16); cos_data_int <= to_unsigned(58725,16);
     when "100101010111" => sin_data_int <= to_unsigned(47826,16); cos_data_int <= to_unsigned(58702,16);
     when "100101011000" => sin_data_int <= to_unsigned(47866,16); cos_data_int <= to_unsigned(58679,16);
     when "100101011001" => sin_data_int <= to_unsigned(47906,16); cos_data_int <= to_unsigned(58656,16);
     when "100101011010" => sin_data_int <= to_unsigned(47946,16); cos_data_int <= to_unsigned(58633,16);
     when "100101011011" => sin_data_int <= to_unsigned(47985,16); cos_data_int <= to_unsigned(58609,16);
     when "100101011100" => sin_data_int <= to_unsigned(48025,16); cos_data_int <= to_unsigned(58586,16);
     when "100101011101" => sin_data_int <= to_unsigned(48065,16); cos_data_int <= to_unsigned(58563,16);
     when "100101011110" => sin_data_int <= to_unsigned(48104,16); cos_data_int <= to_unsigned(58539,16);
     when "100101011111" => sin_data_int <= to_unsigned(48144,16); cos_data_int <= to_unsigned(58515,16);
     when "100101100000" => sin_data_int <= to_unsigned(48183,16); cos_data_int <= to_unsigned(58492,16);
     when "100101100001" => sin_data_int <= to_unsigned(48223,16); cos_data_int <= to_unsigned(58468,16);
     when "100101100010" => sin_data_int <= to_unsigned(48262,16); cos_data_int <= to_unsigned(58444,16);
     when "100101100011" => sin_data_int <= to_unsigned(48301,16); cos_data_int <= to_unsigned(58421,16);
     when "100101100100" => sin_data_int <= to_unsigned(48341,16); cos_data_int <= to_unsigned(58397,16);
     when "100101100101" => sin_data_int <= to_unsigned(48380,16); cos_data_int <= to_unsigned(58373,16);
     when "100101100110" => sin_data_int <= to_unsigned(48419,16); cos_data_int <= to_unsigned(58349,16);
     when "100101100111" => sin_data_int <= to_unsigned(48458,16); cos_data_int <= to_unsigned(58325,16);
     when "100101101000" => sin_data_int <= to_unsigned(48498,16); cos_data_int <= to_unsigned(58301,16);
     when "100101101001" => sin_data_int <= to_unsigned(48537,16); cos_data_int <= to_unsigned(58276,16);
     when "100101101010" => sin_data_int <= to_unsigned(48576,16); cos_data_int <= to_unsigned(58252,16);
     when "100101101011" => sin_data_int <= to_unsigned(48615,16); cos_data_int <= to_unsigned(58228,16);
     when "100101101100" => sin_data_int <= to_unsigned(48654,16); cos_data_int <= to_unsigned(58204,16);
     when "100101101101" => sin_data_int <= to_unsigned(48693,16); cos_data_int <= to_unsigned(58179,16);
     when "100101101110" => sin_data_int <= to_unsigned(48732,16); cos_data_int <= to_unsigned(58155,16);
     when "100101101111" => sin_data_int <= to_unsigned(48771,16); cos_data_int <= to_unsigned(58130,16);
     when "100101110000" => sin_data_int <= to_unsigned(48810,16); cos_data_int <= to_unsigned(58106,16);
     when "100101110001" => sin_data_int <= to_unsigned(48849,16); cos_data_int <= to_unsigned(58081,16);
     when "100101110010" => sin_data_int <= to_unsigned(48888,16); cos_data_int <= to_unsigned(58056,16);
     when "100101110011" => sin_data_int <= to_unsigned(48926,16); cos_data_int <= to_unsigned(58031,16);
     when "100101110100" => sin_data_int <= to_unsigned(48965,16); cos_data_int <= to_unsigned(58007,16);
     when "100101110101" => sin_data_int <= to_unsigned(49004,16); cos_data_int <= to_unsigned(57982,16);
     when "100101110110" => sin_data_int <= to_unsigned(49043,16); cos_data_int <= to_unsigned(57957,16);
     when "100101110111" => sin_data_int <= to_unsigned(49081,16); cos_data_int <= to_unsigned(57932,16);
     when "100101111000" => sin_data_int <= to_unsigned(49120,16); cos_data_int <= to_unsigned(57907,16);
     when "100101111001" => sin_data_int <= to_unsigned(49158,16); cos_data_int <= to_unsigned(57882,16);
     when "100101111010" => sin_data_int <= to_unsigned(49197,16); cos_data_int <= to_unsigned(57856,16);
     when "100101111011" => sin_data_int <= to_unsigned(49235,16); cos_data_int <= to_unsigned(57831,16);
     when "100101111100" => sin_data_int <= to_unsigned(49274,16); cos_data_int <= to_unsigned(57806,16);
     when "100101111101" => sin_data_int <= to_unsigned(49312,16); cos_data_int <= to_unsigned(57781,16);
     when "100101111110" => sin_data_int <= to_unsigned(49351,16); cos_data_int <= to_unsigned(57755,16);
     when "100101111111" => sin_data_int <= to_unsigned(49389,16); cos_data_int <= to_unsigned(57730,16);
     when "100110000000" => sin_data_int <= to_unsigned(49427,16); cos_data_int <= to_unsigned(57704,16);
     when "100110000001" => sin_data_int <= to_unsigned(49465,16); cos_data_int <= to_unsigned(57678,16);
     when "100110000010" => sin_data_int <= to_unsigned(49504,16); cos_data_int <= to_unsigned(57653,16);
     when "100110000011" => sin_data_int <= to_unsigned(49542,16); cos_data_int <= to_unsigned(57627,16);
     when "100110000100" => sin_data_int <= to_unsigned(49580,16); cos_data_int <= to_unsigned(57601,16);
     when "100110000101" => sin_data_int <= to_unsigned(49618,16); cos_data_int <= to_unsigned(57576,16);
     when "100110000110" => sin_data_int <= to_unsigned(49656,16); cos_data_int <= to_unsigned(57550,16);
     when "100110000111" => sin_data_int <= to_unsigned(49694,16); cos_data_int <= to_unsigned(57524,16);
     when "100110001000" => sin_data_int <= to_unsigned(49732,16); cos_data_int <= to_unsigned(57498,16);
     when "100110001001" => sin_data_int <= to_unsigned(49770,16); cos_data_int <= to_unsigned(57472,16);
     when "100110001010" => sin_data_int <= to_unsigned(49808,16); cos_data_int <= to_unsigned(57445,16);
     when "100110001011" => sin_data_int <= to_unsigned(49846,16); cos_data_int <= to_unsigned(57419,16);
     when "100110001100" => sin_data_int <= to_unsigned(49883,16); cos_data_int <= to_unsigned(57393,16);
     when "100110001101" => sin_data_int <= to_unsigned(49921,16); cos_data_int <= to_unsigned(57367,16);
     when "100110001110" => sin_data_int <= to_unsigned(49959,16); cos_data_int <= to_unsigned(57340,16);
     when "100110001111" => sin_data_int <= to_unsigned(49997,16); cos_data_int <= to_unsigned(57314,16);
     when "100110010000" => sin_data_int <= to_unsigned(50034,16); cos_data_int <= to_unsigned(57288,16);
     when "100110010001" => sin_data_int <= to_unsigned(50072,16); cos_data_int <= to_unsigned(57261,16);
     when "100110010010" => sin_data_int <= to_unsigned(50109,16); cos_data_int <= to_unsigned(57234,16);
     when "100110010011" => sin_data_int <= to_unsigned(50147,16); cos_data_int <= to_unsigned(57208,16);
     when "100110010100" => sin_data_int <= to_unsigned(50184,16); cos_data_int <= to_unsigned(57181,16);
     when "100110010101" => sin_data_int <= to_unsigned(50222,16); cos_data_int <= to_unsigned(57154,16);
     when "100110010110" => sin_data_int <= to_unsigned(50259,16); cos_data_int <= to_unsigned(57128,16);
     when "100110010111" => sin_data_int <= to_unsigned(50297,16); cos_data_int <= to_unsigned(57101,16);
     when "100110011000" => sin_data_int <= to_unsigned(50334,16); cos_data_int <= to_unsigned(57074,16);
     when "100110011001" => sin_data_int <= to_unsigned(50371,16); cos_data_int <= to_unsigned(57047,16);
     when "100110011010" => sin_data_int <= to_unsigned(50408,16); cos_data_int <= to_unsigned(57020,16);
     when "100110011011" => sin_data_int <= to_unsigned(50446,16); cos_data_int <= to_unsigned(56993,16);
     when "100110011100" => sin_data_int <= to_unsigned(50483,16); cos_data_int <= to_unsigned(56965,16);
     when "100110011101" => sin_data_int <= to_unsigned(50520,16); cos_data_int <= to_unsigned(56938,16);
     when "100110011110" => sin_data_int <= to_unsigned(50557,16); cos_data_int <= to_unsigned(56911,16);
     when "100110011111" => sin_data_int <= to_unsigned(50594,16); cos_data_int <= to_unsigned(56884,16);
     when "100110100000" => sin_data_int <= to_unsigned(50631,16); cos_data_int <= to_unsigned(56856,16);
     when "100110100001" => sin_data_int <= to_unsigned(50668,16); cos_data_int <= to_unsigned(56829,16);
     when "100110100010" => sin_data_int <= to_unsigned(50705,16); cos_data_int <= to_unsigned(56801,16);
     when "100110100011" => sin_data_int <= to_unsigned(50742,16); cos_data_int <= to_unsigned(56774,16);
     when "100110100100" => sin_data_int <= to_unsigned(50778,16); cos_data_int <= to_unsigned(56746,16);
     when "100110100101" => sin_data_int <= to_unsigned(50815,16); cos_data_int <= to_unsigned(56718,16);
     when "100110100110" => sin_data_int <= to_unsigned(50852,16); cos_data_int <= to_unsigned(56691,16);
     when "100110100111" => sin_data_int <= to_unsigned(50889,16); cos_data_int <= to_unsigned(56663,16);
     when "100110101000" => sin_data_int <= to_unsigned(50925,16); cos_data_int <= to_unsigned(56635,16);
     when "100110101001" => sin_data_int <= to_unsigned(50962,16); cos_data_int <= to_unsigned(56607,16);
     when "100110101010" => sin_data_int <= to_unsigned(50998,16); cos_data_int <= to_unsigned(56579,16);
     when "100110101011" => sin_data_int <= to_unsigned(51035,16); cos_data_int <= to_unsigned(56551,16);
     when "100110101100" => sin_data_int <= to_unsigned(51071,16); cos_data_int <= to_unsigned(56523,16);
     when "100110101101" => sin_data_int <= to_unsigned(51108,16); cos_data_int <= to_unsigned(56495,16);
     when "100110101110" => sin_data_int <= to_unsigned(51144,16); cos_data_int <= to_unsigned(56467,16);
     when "100110101111" => sin_data_int <= to_unsigned(51181,16); cos_data_int <= to_unsigned(56439,16);
     when "100110110000" => sin_data_int <= to_unsigned(51217,16); cos_data_int <= to_unsigned(56410,16);
     when "100110110001" => sin_data_int <= to_unsigned(51253,16); cos_data_int <= to_unsigned(56382,16);
     when "100110110010" => sin_data_int <= to_unsigned(51289,16); cos_data_int <= to_unsigned(56354,16);
     when "100110110011" => sin_data_int <= to_unsigned(51326,16); cos_data_int <= to_unsigned(56325,16);
     when "100110110100" => sin_data_int <= to_unsigned(51362,16); cos_data_int <= to_unsigned(56297,16);
     when "100110110101" => sin_data_int <= to_unsigned(51398,16); cos_data_int <= to_unsigned(56268,16);
     when "100110110110" => sin_data_int <= to_unsigned(51434,16); cos_data_int <= to_unsigned(56240,16);
     when "100110110111" => sin_data_int <= to_unsigned(51470,16); cos_data_int <= to_unsigned(56211,16);
     when "100110111000" => sin_data_int <= to_unsigned(51506,16); cos_data_int <= to_unsigned(56182,16);
     when "100110111001" => sin_data_int <= to_unsigned(51542,16); cos_data_int <= to_unsigned(56153,16);
     when "100110111010" => sin_data_int <= to_unsigned(51578,16); cos_data_int <= to_unsigned(56124,16);
     when "100110111011" => sin_data_int <= to_unsigned(51613,16); cos_data_int <= to_unsigned(56096,16);
     when "100110111100" => sin_data_int <= to_unsigned(51649,16); cos_data_int <= to_unsigned(56067,16);
     when "100110111101" => sin_data_int <= to_unsigned(51685,16); cos_data_int <= to_unsigned(56038,16);
     when "100110111110" => sin_data_int <= to_unsigned(51721,16); cos_data_int <= to_unsigned(56009,16);
     when "100110111111" => sin_data_int <= to_unsigned(51756,16); cos_data_int <= to_unsigned(55979,16);
     when "100111000000" => sin_data_int <= to_unsigned(51792,16); cos_data_int <= to_unsigned(55950,16);
     when "100111000001" => sin_data_int <= to_unsigned(51827,16); cos_data_int <= to_unsigned(55921,16);
     when "100111000010" => sin_data_int <= to_unsigned(51863,16); cos_data_int <= to_unsigned(55892,16);
     when "100111000011" => sin_data_int <= to_unsigned(51898,16); cos_data_int <= to_unsigned(55862,16);
     when "100111000100" => sin_data_int <= to_unsigned(51934,16); cos_data_int <= to_unsigned(55833,16);
     when "100111000101" => sin_data_int <= to_unsigned(51969,16); cos_data_int <= to_unsigned(55804,16);
     when "100111000110" => sin_data_int <= to_unsigned(52004,16); cos_data_int <= to_unsigned(55774,16);
     when "100111000111" => sin_data_int <= to_unsigned(52040,16); cos_data_int <= to_unsigned(55745,16);
     when "100111001000" => sin_data_int <= to_unsigned(52075,16); cos_data_int <= to_unsigned(55715,16);
     when "100111001001" => sin_data_int <= to_unsigned(52110,16); cos_data_int <= to_unsigned(55685,16);
     when "100111001010" => sin_data_int <= to_unsigned(52145,16); cos_data_int <= to_unsigned(55656,16);
     when "100111001011" => sin_data_int <= to_unsigned(52180,16); cos_data_int <= to_unsigned(55626,16);
     when "100111001100" => sin_data_int <= to_unsigned(52215,16); cos_data_int <= to_unsigned(55596,16);
     when "100111001101" => sin_data_int <= to_unsigned(52250,16); cos_data_int <= to_unsigned(55566,16);
     when "100111001110" => sin_data_int <= to_unsigned(52285,16); cos_data_int <= to_unsigned(55536,16);
     when "100111001111" => sin_data_int <= to_unsigned(52320,16); cos_data_int <= to_unsigned(55506,16);
     when "100111010000" => sin_data_int <= to_unsigned(52355,16); cos_data_int <= to_unsigned(55476,16);
     when "100111010001" => sin_data_int <= to_unsigned(52390,16); cos_data_int <= to_unsigned(55446,16);
     when "100111010010" => sin_data_int <= to_unsigned(52425,16); cos_data_int <= to_unsigned(55416,16);
     when "100111010011" => sin_data_int <= to_unsigned(52460,16); cos_data_int <= to_unsigned(55386,16);
     when "100111010100" => sin_data_int <= to_unsigned(52494,16); cos_data_int <= to_unsigned(55356,16);
     when "100111010101" => sin_data_int <= to_unsigned(52529,16); cos_data_int <= to_unsigned(55325,16);
     when "100111010110" => sin_data_int <= to_unsigned(52563,16); cos_data_int <= to_unsigned(55295,16);
     when "100111010111" => sin_data_int <= to_unsigned(52598,16); cos_data_int <= to_unsigned(55265,16);
     when "100111011000" => sin_data_int <= to_unsigned(52632,16); cos_data_int <= to_unsigned(55234,16);
     when "100111011001" => sin_data_int <= to_unsigned(52667,16); cos_data_int <= to_unsigned(55204,16);
     when "100111011010" => sin_data_int <= to_unsigned(52701,16); cos_data_int <= to_unsigned(55173,16);
     when "100111011011" => sin_data_int <= to_unsigned(52736,16); cos_data_int <= to_unsigned(55142,16);
     when "100111011100" => sin_data_int <= to_unsigned(52770,16); cos_data_int <= to_unsigned(55112,16);
     when "100111011101" => sin_data_int <= to_unsigned(52804,16); cos_data_int <= to_unsigned(55081,16);
     when "100111011110" => sin_data_int <= to_unsigned(52838,16); cos_data_int <= to_unsigned(55050,16);
     when "100111011111" => sin_data_int <= to_unsigned(52873,16); cos_data_int <= to_unsigned(55019,16);
     when "100111100000" => sin_data_int <= to_unsigned(52907,16); cos_data_int <= to_unsigned(54989,16);
     when "100111100001" => sin_data_int <= to_unsigned(52941,16); cos_data_int <= to_unsigned(54958,16);
     when "100111100010" => sin_data_int <= to_unsigned(52975,16); cos_data_int <= to_unsigned(54927,16);
     when "100111100011" => sin_data_int <= to_unsigned(53009,16); cos_data_int <= to_unsigned(54896,16);
     when "100111100100" => sin_data_int <= to_unsigned(53043,16); cos_data_int <= to_unsigned(54864,16);
     when "100111100101" => sin_data_int <= to_unsigned(53077,16); cos_data_int <= to_unsigned(54833,16);
     when "100111100110" => sin_data_int <= to_unsigned(53111,16); cos_data_int <= to_unsigned(54802,16);
     when "100111100111" => sin_data_int <= to_unsigned(53144,16); cos_data_int <= to_unsigned(54771,16);
     when "100111101000" => sin_data_int <= to_unsigned(53178,16); cos_data_int <= to_unsigned(54740,16);
     when "100111101001" => sin_data_int <= to_unsigned(53212,16); cos_data_int <= to_unsigned(54708,16);
     when "100111101010" => sin_data_int <= to_unsigned(53245,16); cos_data_int <= to_unsigned(54677,16);
     when "100111101011" => sin_data_int <= to_unsigned(53279,16); cos_data_int <= to_unsigned(54645,16);
     when "100111101100" => sin_data_int <= to_unsigned(53313,16); cos_data_int <= to_unsigned(54614,16);
     when "100111101101" => sin_data_int <= to_unsigned(53346,16); cos_data_int <= to_unsigned(54582,16);
     when "100111101110" => sin_data_int <= to_unsigned(53379,16); cos_data_int <= to_unsigned(54551,16);
     when "100111101111" => sin_data_int <= to_unsigned(53413,16); cos_data_int <= to_unsigned(54519,16);
     when "100111110000" => sin_data_int <= to_unsigned(53446,16); cos_data_int <= to_unsigned(54487,16);
     when "100111110001" => sin_data_int <= to_unsigned(53480,16); cos_data_int <= to_unsigned(54456,16);
     when "100111110010" => sin_data_int <= to_unsigned(53513,16); cos_data_int <= to_unsigned(54424,16);
     when "100111110011" => sin_data_int <= to_unsigned(53546,16); cos_data_int <= to_unsigned(54392,16);
     when "100111110100" => sin_data_int <= to_unsigned(53579,16); cos_data_int <= to_unsigned(54360,16);
     when "100111110101" => sin_data_int <= to_unsigned(53612,16); cos_data_int <= to_unsigned(54328,16);
     when "100111110110" => sin_data_int <= to_unsigned(53645,16); cos_data_int <= to_unsigned(54296,16);
     when "100111110111" => sin_data_int <= to_unsigned(53678,16); cos_data_int <= to_unsigned(54264,16);
     when "100111111000" => sin_data_int <= to_unsigned(53711,16); cos_data_int <= to_unsigned(54232,16);
     when "100111111001" => sin_data_int <= to_unsigned(53744,16); cos_data_int <= to_unsigned(54200,16);
     when "100111111010" => sin_data_int <= to_unsigned(53777,16); cos_data_int <= to_unsigned(54168,16);
     when "100111111011" => sin_data_int <= to_unsigned(53810,16); cos_data_int <= to_unsigned(54135,16);
     when "100111111100" => sin_data_int <= to_unsigned(53843,16); cos_data_int <= to_unsigned(54103,16);
     when "100111111101" => sin_data_int <= to_unsigned(53875,16); cos_data_int <= to_unsigned(54071,16);
     when "100111111110" => sin_data_int <= to_unsigned(53908,16); cos_data_int <= to_unsigned(54038,16);
     when "100111111111" => sin_data_int <= to_unsigned(53941,16); cos_data_int <= to_unsigned(54006,16);
     when "101000000000" => sin_data_int <= to_unsigned(53973,16); cos_data_int <= to_unsigned(53973,16);
     when "101000000001" => sin_data_int <= to_unsigned(54006,16); cos_data_int <= to_unsigned(53941,16);
     when "101000000010" => sin_data_int <= to_unsigned(54038,16); cos_data_int <= to_unsigned(53908,16);
     when "101000000011" => sin_data_int <= to_unsigned(54071,16); cos_data_int <= to_unsigned(53875,16);
     when "101000000100" => sin_data_int <= to_unsigned(54103,16); cos_data_int <= to_unsigned(53843,16);
     when "101000000101" => sin_data_int <= to_unsigned(54135,16); cos_data_int <= to_unsigned(53810,16);
     when "101000000110" => sin_data_int <= to_unsigned(54168,16); cos_data_int <= to_unsigned(53777,16);
     when "101000000111" => sin_data_int <= to_unsigned(54200,16); cos_data_int <= to_unsigned(53744,16);
     when "101000001000" => sin_data_int <= to_unsigned(54232,16); cos_data_int <= to_unsigned(53711,16);
     when "101000001001" => sin_data_int <= to_unsigned(54264,16); cos_data_int <= to_unsigned(53678,16);
     when "101000001010" => sin_data_int <= to_unsigned(54296,16); cos_data_int <= to_unsigned(53645,16);
     when "101000001011" => sin_data_int <= to_unsigned(54328,16); cos_data_int <= to_unsigned(53612,16);
     when "101000001100" => sin_data_int <= to_unsigned(54360,16); cos_data_int <= to_unsigned(53579,16);
     when "101000001101" => sin_data_int <= to_unsigned(54392,16); cos_data_int <= to_unsigned(53546,16);
     when "101000001110" => sin_data_int <= to_unsigned(54424,16); cos_data_int <= to_unsigned(53513,16);
     when "101000001111" => sin_data_int <= to_unsigned(54456,16); cos_data_int <= to_unsigned(53480,16);
     when "101000010000" => sin_data_int <= to_unsigned(54487,16); cos_data_int <= to_unsigned(53446,16);
     when "101000010001" => sin_data_int <= to_unsigned(54519,16); cos_data_int <= to_unsigned(53413,16);
     when "101000010010" => sin_data_int <= to_unsigned(54551,16); cos_data_int <= to_unsigned(53379,16);
     when "101000010011" => sin_data_int <= to_unsigned(54582,16); cos_data_int <= to_unsigned(53346,16);
     when "101000010100" => sin_data_int <= to_unsigned(54614,16); cos_data_int <= to_unsigned(53313,16);
     when "101000010101" => sin_data_int <= to_unsigned(54645,16); cos_data_int <= to_unsigned(53279,16);
     when "101000010110" => sin_data_int <= to_unsigned(54677,16); cos_data_int <= to_unsigned(53245,16);
     when "101000010111" => sin_data_int <= to_unsigned(54708,16); cos_data_int <= to_unsigned(53212,16);
     when "101000011000" => sin_data_int <= to_unsigned(54740,16); cos_data_int <= to_unsigned(53178,16);
     when "101000011001" => sin_data_int <= to_unsigned(54771,16); cos_data_int <= to_unsigned(53144,16);
     when "101000011010" => sin_data_int <= to_unsigned(54802,16); cos_data_int <= to_unsigned(53111,16);
     when "101000011011" => sin_data_int <= to_unsigned(54833,16); cos_data_int <= to_unsigned(53077,16);
     when "101000011100" => sin_data_int <= to_unsigned(54864,16); cos_data_int <= to_unsigned(53043,16);
     when "101000011101" => sin_data_int <= to_unsigned(54896,16); cos_data_int <= to_unsigned(53009,16);
     when "101000011110" => sin_data_int <= to_unsigned(54927,16); cos_data_int <= to_unsigned(52975,16);
     when "101000011111" => sin_data_int <= to_unsigned(54958,16); cos_data_int <= to_unsigned(52941,16);
     when "101000100000" => sin_data_int <= to_unsigned(54989,16); cos_data_int <= to_unsigned(52907,16);
     when "101000100001" => sin_data_int <= to_unsigned(55019,16); cos_data_int <= to_unsigned(52873,16);
     when "101000100010" => sin_data_int <= to_unsigned(55050,16); cos_data_int <= to_unsigned(52838,16);
     when "101000100011" => sin_data_int <= to_unsigned(55081,16); cos_data_int <= to_unsigned(52804,16);
     when "101000100100" => sin_data_int <= to_unsigned(55112,16); cos_data_int <= to_unsigned(52770,16);
     when "101000100101" => sin_data_int <= to_unsigned(55142,16); cos_data_int <= to_unsigned(52736,16);
     when "101000100110" => sin_data_int <= to_unsigned(55173,16); cos_data_int <= to_unsigned(52701,16);
     when "101000100111" => sin_data_int <= to_unsigned(55204,16); cos_data_int <= to_unsigned(52667,16);
     when "101000101000" => sin_data_int <= to_unsigned(55234,16); cos_data_int <= to_unsigned(52632,16);
     when "101000101001" => sin_data_int <= to_unsigned(55265,16); cos_data_int <= to_unsigned(52598,16);
     when "101000101010" => sin_data_int <= to_unsigned(55295,16); cos_data_int <= to_unsigned(52563,16);
     when "101000101011" => sin_data_int <= to_unsigned(55325,16); cos_data_int <= to_unsigned(52529,16);
     when "101000101100" => sin_data_int <= to_unsigned(55356,16); cos_data_int <= to_unsigned(52494,16);
     when "101000101101" => sin_data_int <= to_unsigned(55386,16); cos_data_int <= to_unsigned(52460,16);
     when "101000101110" => sin_data_int <= to_unsigned(55416,16); cos_data_int <= to_unsigned(52425,16);
     when "101000101111" => sin_data_int <= to_unsigned(55446,16); cos_data_int <= to_unsigned(52390,16);
     when "101000110000" => sin_data_int <= to_unsigned(55476,16); cos_data_int <= to_unsigned(52355,16);
     when "101000110001" => sin_data_int <= to_unsigned(55506,16); cos_data_int <= to_unsigned(52320,16);
     when "101000110010" => sin_data_int <= to_unsigned(55536,16); cos_data_int <= to_unsigned(52285,16);
     when "101000110011" => sin_data_int <= to_unsigned(55566,16); cos_data_int <= to_unsigned(52250,16);
     when "101000110100" => sin_data_int <= to_unsigned(55596,16); cos_data_int <= to_unsigned(52215,16);
     when "101000110101" => sin_data_int <= to_unsigned(55626,16); cos_data_int <= to_unsigned(52180,16);
     when "101000110110" => sin_data_int <= to_unsigned(55656,16); cos_data_int <= to_unsigned(52145,16);
     when "101000110111" => sin_data_int <= to_unsigned(55685,16); cos_data_int <= to_unsigned(52110,16);
     when "101000111000" => sin_data_int <= to_unsigned(55715,16); cos_data_int <= to_unsigned(52075,16);
     when "101000111001" => sin_data_int <= to_unsigned(55745,16); cos_data_int <= to_unsigned(52040,16);
     when "101000111010" => sin_data_int <= to_unsigned(55774,16); cos_data_int <= to_unsigned(52004,16);
     when "101000111011" => sin_data_int <= to_unsigned(55804,16); cos_data_int <= to_unsigned(51969,16);
     when "101000111100" => sin_data_int <= to_unsigned(55833,16); cos_data_int <= to_unsigned(51934,16);
     when "101000111101" => sin_data_int <= to_unsigned(55862,16); cos_data_int <= to_unsigned(51898,16);
     when "101000111110" => sin_data_int <= to_unsigned(55892,16); cos_data_int <= to_unsigned(51863,16);
     when "101000111111" => sin_data_int <= to_unsigned(55921,16); cos_data_int <= to_unsigned(51827,16);
     when "101001000000" => sin_data_int <= to_unsigned(55950,16); cos_data_int <= to_unsigned(51792,16);
     when "101001000001" => sin_data_int <= to_unsigned(55979,16); cos_data_int <= to_unsigned(51756,16);
     when "101001000010" => sin_data_int <= to_unsigned(56009,16); cos_data_int <= to_unsigned(51721,16);
     when "101001000011" => sin_data_int <= to_unsigned(56038,16); cos_data_int <= to_unsigned(51685,16);
     when "101001000100" => sin_data_int <= to_unsigned(56067,16); cos_data_int <= to_unsigned(51649,16);
     when "101001000101" => sin_data_int <= to_unsigned(56096,16); cos_data_int <= to_unsigned(51613,16);
     when "101001000110" => sin_data_int <= to_unsigned(56124,16); cos_data_int <= to_unsigned(51578,16);
     when "101001000111" => sin_data_int <= to_unsigned(56153,16); cos_data_int <= to_unsigned(51542,16);
     when "101001001000" => sin_data_int <= to_unsigned(56182,16); cos_data_int <= to_unsigned(51506,16);
     when "101001001001" => sin_data_int <= to_unsigned(56211,16); cos_data_int <= to_unsigned(51470,16);
     when "101001001010" => sin_data_int <= to_unsigned(56240,16); cos_data_int <= to_unsigned(51434,16);
     when "101001001011" => sin_data_int <= to_unsigned(56268,16); cos_data_int <= to_unsigned(51398,16);
     when "101001001100" => sin_data_int <= to_unsigned(56297,16); cos_data_int <= to_unsigned(51362,16);
     when "101001001101" => sin_data_int <= to_unsigned(56325,16); cos_data_int <= to_unsigned(51326,16);
     when "101001001110" => sin_data_int <= to_unsigned(56354,16); cos_data_int <= to_unsigned(51289,16);
     when "101001001111" => sin_data_int <= to_unsigned(56382,16); cos_data_int <= to_unsigned(51253,16);
     when "101001010000" => sin_data_int <= to_unsigned(56410,16); cos_data_int <= to_unsigned(51217,16);
     when "101001010001" => sin_data_int <= to_unsigned(56439,16); cos_data_int <= to_unsigned(51181,16);
     when "101001010010" => sin_data_int <= to_unsigned(56467,16); cos_data_int <= to_unsigned(51144,16);
     when "101001010011" => sin_data_int <= to_unsigned(56495,16); cos_data_int <= to_unsigned(51108,16);
     when "101001010100" => sin_data_int <= to_unsigned(56523,16); cos_data_int <= to_unsigned(51071,16);
     when "101001010101" => sin_data_int <= to_unsigned(56551,16); cos_data_int <= to_unsigned(51035,16);
     when "101001010110" => sin_data_int <= to_unsigned(56579,16); cos_data_int <= to_unsigned(50998,16);
     when "101001010111" => sin_data_int <= to_unsigned(56607,16); cos_data_int <= to_unsigned(50962,16);
     when "101001011000" => sin_data_int <= to_unsigned(56635,16); cos_data_int <= to_unsigned(50925,16);
     when "101001011001" => sin_data_int <= to_unsigned(56663,16); cos_data_int <= to_unsigned(50889,16);
     when "101001011010" => sin_data_int <= to_unsigned(56691,16); cos_data_int <= to_unsigned(50852,16);
     when "101001011011" => sin_data_int <= to_unsigned(56718,16); cos_data_int <= to_unsigned(50815,16);
     when "101001011100" => sin_data_int <= to_unsigned(56746,16); cos_data_int <= to_unsigned(50778,16);
     when "101001011101" => sin_data_int <= to_unsigned(56774,16); cos_data_int <= to_unsigned(50742,16);
     when "101001011110" => sin_data_int <= to_unsigned(56801,16); cos_data_int <= to_unsigned(50705,16);
     when "101001011111" => sin_data_int <= to_unsigned(56829,16); cos_data_int <= to_unsigned(50668,16);
     when "101001100000" => sin_data_int <= to_unsigned(56856,16); cos_data_int <= to_unsigned(50631,16);
     when "101001100001" => sin_data_int <= to_unsigned(56884,16); cos_data_int <= to_unsigned(50594,16);
     when "101001100010" => sin_data_int <= to_unsigned(56911,16); cos_data_int <= to_unsigned(50557,16);
     when "101001100011" => sin_data_int <= to_unsigned(56938,16); cos_data_int <= to_unsigned(50520,16);
     when "101001100100" => sin_data_int <= to_unsigned(56965,16); cos_data_int <= to_unsigned(50483,16);
     when "101001100101" => sin_data_int <= to_unsigned(56993,16); cos_data_int <= to_unsigned(50446,16);
     when "101001100110" => sin_data_int <= to_unsigned(57020,16); cos_data_int <= to_unsigned(50408,16);
     when "101001100111" => sin_data_int <= to_unsigned(57047,16); cos_data_int <= to_unsigned(50371,16);
     when "101001101000" => sin_data_int <= to_unsigned(57074,16); cos_data_int <= to_unsigned(50334,16);
     when "101001101001" => sin_data_int <= to_unsigned(57101,16); cos_data_int <= to_unsigned(50297,16);
     when "101001101010" => sin_data_int <= to_unsigned(57128,16); cos_data_int <= to_unsigned(50259,16);
     when "101001101011" => sin_data_int <= to_unsigned(57154,16); cos_data_int <= to_unsigned(50222,16);
     when "101001101100" => sin_data_int <= to_unsigned(57181,16); cos_data_int <= to_unsigned(50184,16);
     when "101001101101" => sin_data_int <= to_unsigned(57208,16); cos_data_int <= to_unsigned(50147,16);
     when "101001101110" => sin_data_int <= to_unsigned(57234,16); cos_data_int <= to_unsigned(50109,16);
     when "101001101111" => sin_data_int <= to_unsigned(57261,16); cos_data_int <= to_unsigned(50072,16);
     when "101001110000" => sin_data_int <= to_unsigned(57288,16); cos_data_int <= to_unsigned(50034,16);
     when "101001110001" => sin_data_int <= to_unsigned(57314,16); cos_data_int <= to_unsigned(49997,16);
     when "101001110010" => sin_data_int <= to_unsigned(57340,16); cos_data_int <= to_unsigned(49959,16);
     when "101001110011" => sin_data_int <= to_unsigned(57367,16); cos_data_int <= to_unsigned(49921,16);
     when "101001110100" => sin_data_int <= to_unsigned(57393,16); cos_data_int <= to_unsigned(49883,16);
     when "101001110101" => sin_data_int <= to_unsigned(57419,16); cos_data_int <= to_unsigned(49846,16);
     when "101001110110" => sin_data_int <= to_unsigned(57445,16); cos_data_int <= to_unsigned(49808,16);
     when "101001110111" => sin_data_int <= to_unsigned(57472,16); cos_data_int <= to_unsigned(49770,16);
     when "101001111000" => sin_data_int <= to_unsigned(57498,16); cos_data_int <= to_unsigned(49732,16);
     when "101001111001" => sin_data_int <= to_unsigned(57524,16); cos_data_int <= to_unsigned(49694,16);
     when "101001111010" => sin_data_int <= to_unsigned(57550,16); cos_data_int <= to_unsigned(49656,16);
     when "101001111011" => sin_data_int <= to_unsigned(57576,16); cos_data_int <= to_unsigned(49618,16);
     when "101001111100" => sin_data_int <= to_unsigned(57601,16); cos_data_int <= to_unsigned(49580,16);
     when "101001111101" => sin_data_int <= to_unsigned(57627,16); cos_data_int <= to_unsigned(49542,16);
     when "101001111110" => sin_data_int <= to_unsigned(57653,16); cos_data_int <= to_unsigned(49504,16);
     when "101001111111" => sin_data_int <= to_unsigned(57678,16); cos_data_int <= to_unsigned(49465,16);
     when "101010000000" => sin_data_int <= to_unsigned(57704,16); cos_data_int <= to_unsigned(49427,16);
     when "101010000001" => sin_data_int <= to_unsigned(57730,16); cos_data_int <= to_unsigned(49389,16);
     when "101010000010" => sin_data_int <= to_unsigned(57755,16); cos_data_int <= to_unsigned(49351,16);
     when "101010000011" => sin_data_int <= to_unsigned(57781,16); cos_data_int <= to_unsigned(49312,16);
     when "101010000100" => sin_data_int <= to_unsigned(57806,16); cos_data_int <= to_unsigned(49274,16);
     when "101010000101" => sin_data_int <= to_unsigned(57831,16); cos_data_int <= to_unsigned(49235,16);
     when "101010000110" => sin_data_int <= to_unsigned(57856,16); cos_data_int <= to_unsigned(49197,16);
     when "101010000111" => sin_data_int <= to_unsigned(57882,16); cos_data_int <= to_unsigned(49158,16);
     when "101010001000" => sin_data_int <= to_unsigned(57907,16); cos_data_int <= to_unsigned(49120,16);
     when "101010001001" => sin_data_int <= to_unsigned(57932,16); cos_data_int <= to_unsigned(49081,16);
     when "101010001010" => sin_data_int <= to_unsigned(57957,16); cos_data_int <= to_unsigned(49043,16);
     when "101010001011" => sin_data_int <= to_unsigned(57982,16); cos_data_int <= to_unsigned(49004,16);
     when "101010001100" => sin_data_int <= to_unsigned(58007,16); cos_data_int <= to_unsigned(48965,16);
     when "101010001101" => sin_data_int <= to_unsigned(58031,16); cos_data_int <= to_unsigned(48926,16);
     when "101010001110" => sin_data_int <= to_unsigned(58056,16); cos_data_int <= to_unsigned(48888,16);
     when "101010001111" => sin_data_int <= to_unsigned(58081,16); cos_data_int <= to_unsigned(48849,16);
     when "101010010000" => sin_data_int <= to_unsigned(58106,16); cos_data_int <= to_unsigned(48810,16);
     when "101010010001" => sin_data_int <= to_unsigned(58130,16); cos_data_int <= to_unsigned(48771,16);
     when "101010010010" => sin_data_int <= to_unsigned(58155,16); cos_data_int <= to_unsigned(48732,16);
     when "101010010011" => sin_data_int <= to_unsigned(58179,16); cos_data_int <= to_unsigned(48693,16);
     when "101010010100" => sin_data_int <= to_unsigned(58204,16); cos_data_int <= to_unsigned(48654,16);
     when "101010010101" => sin_data_int <= to_unsigned(58228,16); cos_data_int <= to_unsigned(48615,16);
     when "101010010110" => sin_data_int <= to_unsigned(58252,16); cos_data_int <= to_unsigned(48576,16);
     when "101010010111" => sin_data_int <= to_unsigned(58276,16); cos_data_int <= to_unsigned(48537,16);
     when "101010011000" => sin_data_int <= to_unsigned(58301,16); cos_data_int <= to_unsigned(48498,16);
     when "101010011001" => sin_data_int <= to_unsigned(58325,16); cos_data_int <= to_unsigned(48458,16);
     when "101010011010" => sin_data_int <= to_unsigned(58349,16); cos_data_int <= to_unsigned(48419,16);
     when "101010011011" => sin_data_int <= to_unsigned(58373,16); cos_data_int <= to_unsigned(48380,16);
     when "101010011100" => sin_data_int <= to_unsigned(58397,16); cos_data_int <= to_unsigned(48341,16);
     when "101010011101" => sin_data_int <= to_unsigned(58421,16); cos_data_int <= to_unsigned(48301,16);
     when "101010011110" => sin_data_int <= to_unsigned(58444,16); cos_data_int <= to_unsigned(48262,16);
     when "101010011111" => sin_data_int <= to_unsigned(58468,16); cos_data_int <= to_unsigned(48223,16);
     when "101010100000" => sin_data_int <= to_unsigned(58492,16); cos_data_int <= to_unsigned(48183,16);
     when "101010100001" => sin_data_int <= to_unsigned(58515,16); cos_data_int <= to_unsigned(48144,16);
     when "101010100010" => sin_data_int <= to_unsigned(58539,16); cos_data_int <= to_unsigned(48104,16);
     when "101010100011" => sin_data_int <= to_unsigned(58563,16); cos_data_int <= to_unsigned(48065,16);
     when "101010100100" => sin_data_int <= to_unsigned(58586,16); cos_data_int <= to_unsigned(48025,16);
     when "101010100101" => sin_data_int <= to_unsigned(58609,16); cos_data_int <= to_unsigned(47985,16);
     when "101010100110" => sin_data_int <= to_unsigned(58633,16); cos_data_int <= to_unsigned(47946,16);
     when "101010100111" => sin_data_int <= to_unsigned(58656,16); cos_data_int <= to_unsigned(47906,16);
     when "101010101000" => sin_data_int <= to_unsigned(58679,16); cos_data_int <= to_unsigned(47866,16);
     when "101010101001" => sin_data_int <= to_unsigned(58702,16); cos_data_int <= to_unsigned(47826,16);
     when "101010101010" => sin_data_int <= to_unsigned(58725,16); cos_data_int <= to_unsigned(47787,16);
     when "101010101011" => sin_data_int <= to_unsigned(58748,16); cos_data_int <= to_unsigned(47747,16);
     when "101010101100" => sin_data_int <= to_unsigned(58771,16); cos_data_int <= to_unsigned(47707,16);
     when "101010101101" => sin_data_int <= to_unsigned(58794,16); cos_data_int <= to_unsigned(47667,16);
     when "101010101110" => sin_data_int <= to_unsigned(58817,16); cos_data_int <= to_unsigned(47627,16);
     when "101010101111" => sin_data_int <= to_unsigned(58840,16); cos_data_int <= to_unsigned(47587,16);
     when "101010110000" => sin_data_int <= to_unsigned(58863,16); cos_data_int <= to_unsigned(47547,16);
     when "101010110001" => sin_data_int <= to_unsigned(58885,16); cos_data_int <= to_unsigned(47507,16);
     when "101010110010" => sin_data_int <= to_unsigned(58908,16); cos_data_int <= to_unsigned(47467,16);
     when "101010110011" => sin_data_int <= to_unsigned(58930,16); cos_data_int <= to_unsigned(47427,16);
     when "101010110100" => sin_data_int <= to_unsigned(58953,16); cos_data_int <= to_unsigned(47387,16);
     when "101010110101" => sin_data_int <= to_unsigned(58975,16); cos_data_int <= to_unsigned(47346,16);
     when "101010110110" => sin_data_int <= to_unsigned(58998,16); cos_data_int <= to_unsigned(47306,16);
     when "101010110111" => sin_data_int <= to_unsigned(59020,16); cos_data_int <= to_unsigned(47266,16);
     when "101010111000" => sin_data_int <= to_unsigned(59042,16); cos_data_int <= to_unsigned(47226,16);
     when "101010111001" => sin_data_int <= to_unsigned(59064,16); cos_data_int <= to_unsigned(47185,16);
     when "101010111010" => sin_data_int <= to_unsigned(59086,16); cos_data_int <= to_unsigned(47145,16);
     when "101010111011" => sin_data_int <= to_unsigned(59108,16); cos_data_int <= to_unsigned(47104,16);
     when "101010111100" => sin_data_int <= to_unsigned(59130,16); cos_data_int <= to_unsigned(47064,16);
     when "101010111101" => sin_data_int <= to_unsigned(59152,16); cos_data_int <= to_unsigned(47024,16);
     when "101010111110" => sin_data_int <= to_unsigned(59174,16); cos_data_int <= to_unsigned(46983,16);
     when "101010111111" => sin_data_int <= to_unsigned(59196,16); cos_data_int <= to_unsigned(46942,16);
     when "101011000000" => sin_data_int <= to_unsigned(59218,16); cos_data_int <= to_unsigned(46902,16);
     when "101011000001" => sin_data_int <= to_unsigned(59239,16); cos_data_int <= to_unsigned(46861,16);
     when "101011000010" => sin_data_int <= to_unsigned(59261,16); cos_data_int <= to_unsigned(46821,16);
     when "101011000011" => sin_data_int <= to_unsigned(59282,16); cos_data_int <= to_unsigned(46780,16);
     when "101011000100" => sin_data_int <= to_unsigned(59304,16); cos_data_int <= to_unsigned(46739,16);
     when "101011000101" => sin_data_int <= to_unsigned(59325,16); cos_data_int <= to_unsigned(46699,16);
     when "101011000110" => sin_data_int <= to_unsigned(59347,16); cos_data_int <= to_unsigned(46658,16);
     when "101011000111" => sin_data_int <= to_unsigned(59368,16); cos_data_int <= to_unsigned(46617,16);
     when "101011001000" => sin_data_int <= to_unsigned(59389,16); cos_data_int <= to_unsigned(46576,16);
     when "101011001001" => sin_data_int <= to_unsigned(59410,16); cos_data_int <= to_unsigned(46535,16);
     when "101011001010" => sin_data_int <= to_unsigned(59431,16); cos_data_int <= to_unsigned(46494,16);
     when "101011001011" => sin_data_int <= to_unsigned(59452,16); cos_data_int <= to_unsigned(46453,16);
     when "101011001100" => sin_data_int <= to_unsigned(59473,16); cos_data_int <= to_unsigned(46413,16);
     when "101011001101" => sin_data_int <= to_unsigned(59494,16); cos_data_int <= to_unsigned(46372,16);
     when "101011001110" => sin_data_int <= to_unsigned(59515,16); cos_data_int <= to_unsigned(46330,16);
     when "101011001111" => sin_data_int <= to_unsigned(59536,16); cos_data_int <= to_unsigned(46289,16);
     when "101011010000" => sin_data_int <= to_unsigned(59557,16); cos_data_int <= to_unsigned(46248,16);
     when "101011010001" => sin_data_int <= to_unsigned(59577,16); cos_data_int <= to_unsigned(46207,16);
     when "101011010010" => sin_data_int <= to_unsigned(59598,16); cos_data_int <= to_unsigned(46166,16);
     when "101011010011" => sin_data_int <= to_unsigned(59619,16); cos_data_int <= to_unsigned(46125,16);
     when "101011010100" => sin_data_int <= to_unsigned(59639,16); cos_data_int <= to_unsigned(46084,16);
     when "101011010101" => sin_data_int <= to_unsigned(59659,16); cos_data_int <= to_unsigned(46042,16);
     when "101011010110" => sin_data_int <= to_unsigned(59680,16); cos_data_int <= to_unsigned(46001,16);
     when "101011010111" => sin_data_int <= to_unsigned(59700,16); cos_data_int <= to_unsigned(45960,16);
     when "101011011000" => sin_data_int <= to_unsigned(59720,16); cos_data_int <= to_unsigned(45918,16);
     when "101011011001" => sin_data_int <= to_unsigned(59740,16); cos_data_int <= to_unsigned(45877,16);
     when "101011011010" => sin_data_int <= to_unsigned(59760,16); cos_data_int <= to_unsigned(45836,16);
     when "101011011011" => sin_data_int <= to_unsigned(59781,16); cos_data_int <= to_unsigned(45794,16);
     when "101011011100" => sin_data_int <= to_unsigned(59800,16); cos_data_int <= to_unsigned(45753,16);
     when "101011011101" => sin_data_int <= to_unsigned(59820,16); cos_data_int <= to_unsigned(45711,16);
     when "101011011110" => sin_data_int <= to_unsigned(59840,16); cos_data_int <= to_unsigned(45670,16);
     when "101011011111" => sin_data_int <= to_unsigned(59860,16); cos_data_int <= to_unsigned(45628,16);
     when "101011100000" => sin_data_int <= to_unsigned(59880,16); cos_data_int <= to_unsigned(45587,16);
     when "101011100001" => sin_data_int <= to_unsigned(59899,16); cos_data_int <= to_unsigned(45545,16);
     when "101011100010" => sin_data_int <= to_unsigned(59919,16); cos_data_int <= to_unsigned(45503,16);
     when "101011100011" => sin_data_int <= to_unsigned(59938,16); cos_data_int <= to_unsigned(45462,16);
     when "101011100100" => sin_data_int <= to_unsigned(59958,16); cos_data_int <= to_unsigned(45420,16);
     when "101011100101" => sin_data_int <= to_unsigned(59977,16); cos_data_int <= to_unsigned(45378,16);
     when "101011100110" => sin_data_int <= to_unsigned(59997,16); cos_data_int <= to_unsigned(45337,16);
     when "101011100111" => sin_data_int <= to_unsigned(60016,16); cos_data_int <= to_unsigned(45295,16);
     when "101011101000" => sin_data_int <= to_unsigned(60035,16); cos_data_int <= to_unsigned(45253,16);
     when "101011101001" => sin_data_int <= to_unsigned(60054,16); cos_data_int <= to_unsigned(45211,16);
     when "101011101010" => sin_data_int <= to_unsigned(60073,16); cos_data_int <= to_unsigned(45169,16);
     when "101011101011" => sin_data_int <= to_unsigned(60092,16); cos_data_int <= to_unsigned(45127,16);
     when "101011101100" => sin_data_int <= to_unsigned(60111,16); cos_data_int <= to_unsigned(45085,16);
     when "101011101101" => sin_data_int <= to_unsigned(60130,16); cos_data_int <= to_unsigned(45043,16);
     when "101011101110" => sin_data_int <= to_unsigned(60149,16); cos_data_int <= to_unsigned(45001,16);
     when "101011101111" => sin_data_int <= to_unsigned(60168,16); cos_data_int <= to_unsigned(44959,16);
     when "101011110000" => sin_data_int <= to_unsigned(60186,16); cos_data_int <= to_unsigned(44917,16);
     when "101011110001" => sin_data_int <= to_unsigned(60205,16); cos_data_int <= to_unsigned(44875,16);
     when "101011110010" => sin_data_int <= to_unsigned(60223,16); cos_data_int <= to_unsigned(44833,16);
     when "101011110011" => sin_data_int <= to_unsigned(60242,16); cos_data_int <= to_unsigned(44791,16);
     when "101011110100" => sin_data_int <= to_unsigned(60260,16); cos_data_int <= to_unsigned(44749,16);
     when "101011110101" => sin_data_int <= to_unsigned(60279,16); cos_data_int <= to_unsigned(44707,16);
     when "101011110110" => sin_data_int <= to_unsigned(60297,16); cos_data_int <= to_unsigned(44664,16);
     when "101011110111" => sin_data_int <= to_unsigned(60315,16); cos_data_int <= to_unsigned(44622,16);
     when "101011111000" => sin_data_int <= to_unsigned(60333,16); cos_data_int <= to_unsigned(44580,16);
     when "101011111001" => sin_data_int <= to_unsigned(60352,16); cos_data_int <= to_unsigned(44537,16);
     when "101011111010" => sin_data_int <= to_unsigned(60370,16); cos_data_int <= to_unsigned(44495,16);
     when "101011111011" => sin_data_int <= to_unsigned(60388,16); cos_data_int <= to_unsigned(44453,16);
     when "101011111100" => sin_data_int <= to_unsigned(60405,16); cos_data_int <= to_unsigned(44410,16);
     when "101011111101" => sin_data_int <= to_unsigned(60423,16); cos_data_int <= to_unsigned(44368,16);
     when "101011111110" => sin_data_int <= to_unsigned(60441,16); cos_data_int <= to_unsigned(44325,16);
     when "101011111111" => sin_data_int <= to_unsigned(60459,16); cos_data_int <= to_unsigned(44283,16);
     when "101100000000" => sin_data_int <= to_unsigned(60476,16); cos_data_int <= to_unsigned(44241,16);
     when "101100000001" => sin_data_int <= to_unsigned(60494,16); cos_data_int <= to_unsigned(44198,16);
     when "101100000010" => sin_data_int <= to_unsigned(60511,16); cos_data_int <= to_unsigned(44155,16);
     when "101100000011" => sin_data_int <= to_unsigned(60529,16); cos_data_int <= to_unsigned(44113,16);
     when "101100000100" => sin_data_int <= to_unsigned(60546,16); cos_data_int <= to_unsigned(44070,16);
     when "101100000101" => sin_data_int <= to_unsigned(60564,16); cos_data_int <= to_unsigned(44028,16);
     when "101100000110" => sin_data_int <= to_unsigned(60581,16); cos_data_int <= to_unsigned(43985,16);
     when "101100000111" => sin_data_int <= to_unsigned(60598,16); cos_data_int <= to_unsigned(43942,16);
     when "101100001000" => sin_data_int <= to_unsigned(60615,16); cos_data_int <= to_unsigned(43900,16);
     when "101100001001" => sin_data_int <= to_unsigned(60632,16); cos_data_int <= to_unsigned(43857,16);
     when "101100001010" => sin_data_int <= to_unsigned(60649,16); cos_data_int <= to_unsigned(43814,16);
     when "101100001011" => sin_data_int <= to_unsigned(60666,16); cos_data_int <= to_unsigned(43771,16);
     when "101100001100" => sin_data_int <= to_unsigned(60683,16); cos_data_int <= to_unsigned(43728,16);
     when "101100001101" => sin_data_int <= to_unsigned(60700,16); cos_data_int <= to_unsigned(43686,16);
     when "101100001110" => sin_data_int <= to_unsigned(60717,16); cos_data_int <= to_unsigned(43643,16);
     when "101100001111" => sin_data_int <= to_unsigned(60733,16); cos_data_int <= to_unsigned(43600,16);
     when "101100010000" => sin_data_int <= to_unsigned(60750,16); cos_data_int <= to_unsigned(43557,16);
     when "101100010001" => sin_data_int <= to_unsigned(60766,16); cos_data_int <= to_unsigned(43514,16);
     when "101100010010" => sin_data_int <= to_unsigned(60783,16); cos_data_int <= to_unsigned(43471,16);
     when "101100010011" => sin_data_int <= to_unsigned(60799,16); cos_data_int <= to_unsigned(43428,16);
     when "101100010100" => sin_data_int <= to_unsigned(60816,16); cos_data_int <= to_unsigned(43385,16);
     when "101100010101" => sin_data_int <= to_unsigned(60832,16); cos_data_int <= to_unsigned(43342,16);
     when "101100010110" => sin_data_int <= to_unsigned(60848,16); cos_data_int <= to_unsigned(43299,16);
     when "101100010111" => sin_data_int <= to_unsigned(60864,16); cos_data_int <= to_unsigned(43256,16);
     when "101100011000" => sin_data_int <= to_unsigned(60880,16); cos_data_int <= to_unsigned(43213,16);
     when "101100011001" => sin_data_int <= to_unsigned(60896,16); cos_data_int <= to_unsigned(43169,16);
     when "101100011010" => sin_data_int <= to_unsigned(60912,16); cos_data_int <= to_unsigned(43126,16);
     when "101100011011" => sin_data_int <= to_unsigned(60928,16); cos_data_int <= to_unsigned(43083,16);
     when "101100011100" => sin_data_int <= to_unsigned(60944,16); cos_data_int <= to_unsigned(43040,16);
     when "101100011101" => sin_data_int <= to_unsigned(60960,16); cos_data_int <= to_unsigned(42997,16);
     when "101100011110" => sin_data_int <= to_unsigned(60975,16); cos_data_int <= to_unsigned(42953,16);
     when "101100011111" => sin_data_int <= to_unsigned(60991,16); cos_data_int <= to_unsigned(42910,16);
     when "101100100000" => sin_data_int <= to_unsigned(61006,16); cos_data_int <= to_unsigned(42867,16);
     when "101100100001" => sin_data_int <= to_unsigned(61022,16); cos_data_int <= to_unsigned(42823,16);
     when "101100100010" => sin_data_int <= to_unsigned(61037,16); cos_data_int <= to_unsigned(42780,16);
     when "101100100011" => sin_data_int <= to_unsigned(61053,16); cos_data_int <= to_unsigned(42737,16);
     when "101100100100" => sin_data_int <= to_unsigned(61068,16); cos_data_int <= to_unsigned(42693,16);
     when "101100100101" => sin_data_int <= to_unsigned(61083,16); cos_data_int <= to_unsigned(42650,16);
     when "101100100110" => sin_data_int <= to_unsigned(61098,16); cos_data_int <= to_unsigned(42606,16);
     when "101100100111" => sin_data_int <= to_unsigned(61113,16); cos_data_int <= to_unsigned(42563,16);
     when "101100101000" => sin_data_int <= to_unsigned(61128,16); cos_data_int <= to_unsigned(42519,16);
     when "101100101001" => sin_data_int <= to_unsigned(61143,16); cos_data_int <= to_unsigned(42476,16);
     when "101100101010" => sin_data_int <= to_unsigned(61158,16); cos_data_int <= to_unsigned(42432,16);
     when "101100101011" => sin_data_int <= to_unsigned(61173,16); cos_data_int <= to_unsigned(42389,16);
     when "101100101100" => sin_data_int <= to_unsigned(61188,16); cos_data_int <= to_unsigned(42345,16);
     when "101100101101" => sin_data_int <= to_unsigned(61202,16); cos_data_int <= to_unsigned(42301,16);
     when "101100101110" => sin_data_int <= to_unsigned(61217,16); cos_data_int <= to_unsigned(42258,16);
     when "101100101111" => sin_data_int <= to_unsigned(61231,16); cos_data_int <= to_unsigned(42214,16);
     when "101100110000" => sin_data_int <= to_unsigned(61246,16); cos_data_int <= to_unsigned(42170,16);
     when "101100110001" => sin_data_int <= to_unsigned(61260,16); cos_data_int <= to_unsigned(42127,16);
     when "101100110010" => sin_data_int <= to_unsigned(61275,16); cos_data_int <= to_unsigned(42083,16);
     when "101100110011" => sin_data_int <= to_unsigned(61289,16); cos_data_int <= to_unsigned(42039,16);
     when "101100110100" => sin_data_int <= to_unsigned(61303,16); cos_data_int <= to_unsigned(41995,16);
     when "101100110101" => sin_data_int <= to_unsigned(61317,16); cos_data_int <= to_unsigned(41952,16);
     when "101100110110" => sin_data_int <= to_unsigned(61331,16); cos_data_int <= to_unsigned(41908,16);
     when "101100110111" => sin_data_int <= to_unsigned(61345,16); cos_data_int <= to_unsigned(41864,16);
     when "101100111000" => sin_data_int <= to_unsigned(61359,16); cos_data_int <= to_unsigned(41820,16);
     when "101100111001" => sin_data_int <= to_unsigned(61373,16); cos_data_int <= to_unsigned(41776,16);
     when "101100111010" => sin_data_int <= to_unsigned(61387,16); cos_data_int <= to_unsigned(41732,16);
     when "101100111011" => sin_data_int <= to_unsigned(61401,16); cos_data_int <= to_unsigned(41688,16);
     when "101100111100" => sin_data_int <= to_unsigned(61414,16); cos_data_int <= to_unsigned(41645,16);
     when "101100111101" => sin_data_int <= to_unsigned(61428,16); cos_data_int <= to_unsigned(41601,16);
     when "101100111110" => sin_data_int <= to_unsigned(61441,16); cos_data_int <= to_unsigned(41557,16);
     when "101100111111" => sin_data_int <= to_unsigned(61455,16); cos_data_int <= to_unsigned(41513,16);
     when "101101000000" => sin_data_int <= to_unsigned(61468,16); cos_data_int <= to_unsigned(41469,16);
     when "101101000001" => sin_data_int <= to_unsigned(61482,16); cos_data_int <= to_unsigned(41424,16);
     when "101101000010" => sin_data_int <= to_unsigned(61495,16); cos_data_int <= to_unsigned(41380,16);
     when "101101000011" => sin_data_int <= to_unsigned(61508,16); cos_data_int <= to_unsigned(41336,16);
     when "101101000100" => sin_data_int <= to_unsigned(61521,16); cos_data_int <= to_unsigned(41292,16);
     when "101101000101" => sin_data_int <= to_unsigned(61534,16); cos_data_int <= to_unsigned(41248,16);
     when "101101000110" => sin_data_int <= to_unsigned(61547,16); cos_data_int <= to_unsigned(41204,16);
     when "101101000111" => sin_data_int <= to_unsigned(61560,16); cos_data_int <= to_unsigned(41160,16);
     when "101101001000" => sin_data_int <= to_unsigned(61573,16); cos_data_int <= to_unsigned(41116,16);
     when "101101001001" => sin_data_int <= to_unsigned(61586,16); cos_data_int <= to_unsigned(41071,16);
     when "101101001010" => sin_data_int <= to_unsigned(61598,16); cos_data_int <= to_unsigned(41027,16);
     when "101101001011" => sin_data_int <= to_unsigned(61611,16); cos_data_int <= to_unsigned(40983,16);
     when "101101001100" => sin_data_int <= to_unsigned(61624,16); cos_data_int <= to_unsigned(40939,16);
     when "101101001101" => sin_data_int <= to_unsigned(61636,16); cos_data_int <= to_unsigned(40894,16);
     when "101101001110" => sin_data_int <= to_unsigned(61649,16); cos_data_int <= to_unsigned(40850,16);
     when "101101001111" => sin_data_int <= to_unsigned(61661,16); cos_data_int <= to_unsigned(40806,16);
     when "101101010000" => sin_data_int <= to_unsigned(61673,16); cos_data_int <= to_unsigned(40761,16);
     when "101101010001" => sin_data_int <= to_unsigned(61686,16); cos_data_int <= to_unsigned(40717,16);
     when "101101010010" => sin_data_int <= to_unsigned(61698,16); cos_data_int <= to_unsigned(40673,16);
     when "101101010011" => sin_data_int <= to_unsigned(61710,16); cos_data_int <= to_unsigned(40628,16);
     when "101101010100" => sin_data_int <= to_unsigned(61722,16); cos_data_int <= to_unsigned(40584,16);
     when "101101010101" => sin_data_int <= to_unsigned(61734,16); cos_data_int <= to_unsigned(40539,16);
     when "101101010110" => sin_data_int <= to_unsigned(61746,16); cos_data_int <= to_unsigned(40495,16);
     when "101101010111" => sin_data_int <= to_unsigned(61758,16); cos_data_int <= to_unsigned(40450,16);
     when "101101011000" => sin_data_int <= to_unsigned(61769,16); cos_data_int <= to_unsigned(40406,16);
     when "101101011001" => sin_data_int <= to_unsigned(61781,16); cos_data_int <= to_unsigned(40361,16);
     when "101101011010" => sin_data_int <= to_unsigned(61793,16); cos_data_int <= to_unsigned(40317,16);
     when "101101011011" => sin_data_int <= to_unsigned(61804,16); cos_data_int <= to_unsigned(40272,16);
     when "101101011100" => sin_data_int <= to_unsigned(61816,16); cos_data_int <= to_unsigned(40228,16);
     when "101101011101" => sin_data_int <= to_unsigned(61827,16); cos_data_int <= to_unsigned(40183,16);
     when "101101011110" => sin_data_int <= to_unsigned(61838,16); cos_data_int <= to_unsigned(40139,16);
     when "101101011111" => sin_data_int <= to_unsigned(61850,16); cos_data_int <= to_unsigned(40094,16);
     when "101101100000" => sin_data_int <= to_unsigned(61861,16); cos_data_int <= to_unsigned(40049,16);
     when "101101100001" => sin_data_int <= to_unsigned(61872,16); cos_data_int <= to_unsigned(40005,16);
     when "101101100010" => sin_data_int <= to_unsigned(61883,16); cos_data_int <= to_unsigned(39960,16);
     when "101101100011" => sin_data_int <= to_unsigned(61894,16); cos_data_int <= to_unsigned(39915,16);
     when "101101100100" => sin_data_int <= to_unsigned(61905,16); cos_data_int <= to_unsigned(39871,16);
     when "101101100101" => sin_data_int <= to_unsigned(61916,16); cos_data_int <= to_unsigned(39826,16);
     when "101101100110" => sin_data_int <= to_unsigned(61927,16); cos_data_int <= to_unsigned(39781,16);
     when "101101100111" => sin_data_int <= to_unsigned(61938,16); cos_data_int <= to_unsigned(39737,16);
     when "101101101000" => sin_data_int <= to_unsigned(61948,16); cos_data_int <= to_unsigned(39692,16);
     when "101101101001" => sin_data_int <= to_unsigned(61959,16); cos_data_int <= to_unsigned(39647,16);
     when "101101101010" => sin_data_int <= to_unsigned(61969,16); cos_data_int <= to_unsigned(39602,16);
     when "101101101011" => sin_data_int <= to_unsigned(61980,16); cos_data_int <= to_unsigned(39557,16);
     when "101101101100" => sin_data_int <= to_unsigned(61990,16); cos_data_int <= to_unsigned(39513,16);
     when "101101101101" => sin_data_int <= to_unsigned(62001,16); cos_data_int <= to_unsigned(39468,16);
     when "101101101110" => sin_data_int <= to_unsigned(62011,16); cos_data_int <= to_unsigned(39423,16);
     when "101101101111" => sin_data_int <= to_unsigned(62021,16); cos_data_int <= to_unsigned(39378,16);
     when "101101110000" => sin_data_int <= to_unsigned(62031,16); cos_data_int <= to_unsigned(39333,16);
     when "101101110001" => sin_data_int <= to_unsigned(62041,16); cos_data_int <= to_unsigned(39288,16);
     when "101101110010" => sin_data_int <= to_unsigned(62051,16); cos_data_int <= to_unsigned(39243,16);
     when "101101110011" => sin_data_int <= to_unsigned(62061,16); cos_data_int <= to_unsigned(39198,16);
     when "101101110100" => sin_data_int <= to_unsigned(62071,16); cos_data_int <= to_unsigned(39153,16);
     when "101101110101" => sin_data_int <= to_unsigned(62081,16); cos_data_int <= to_unsigned(39108,16);
     when "101101110110" => sin_data_int <= to_unsigned(62090,16); cos_data_int <= to_unsigned(39063,16);
     when "101101110111" => sin_data_int <= to_unsigned(62100,16); cos_data_int <= to_unsigned(39018,16);
     when "101101111000" => sin_data_int <= to_unsigned(62110,16); cos_data_int <= to_unsigned(38973,16);
     when "101101111001" => sin_data_int <= to_unsigned(62119,16); cos_data_int <= to_unsigned(38928,16);
     when "101101111010" => sin_data_int <= to_unsigned(62128,16); cos_data_int <= to_unsigned(38883,16);
     when "101101111011" => sin_data_int <= to_unsigned(62138,16); cos_data_int <= to_unsigned(38838,16);
     when "101101111100" => sin_data_int <= to_unsigned(62147,16); cos_data_int <= to_unsigned(38793,16);
     when "101101111101" => sin_data_int <= to_unsigned(62156,16); cos_data_int <= to_unsigned(38748,16);
     when "101101111110" => sin_data_int <= to_unsigned(62165,16); cos_data_int <= to_unsigned(38703,16);
     when "101101111111" => sin_data_int <= to_unsigned(62175,16); cos_data_int <= to_unsigned(38658,16);
     when "101110000000" => sin_data_int <= to_unsigned(62184,16); cos_data_int <= to_unsigned(38613,16);
     when "101110000001" => sin_data_int <= to_unsigned(62193,16); cos_data_int <= to_unsigned(38568,16);
     when "101110000010" => sin_data_int <= to_unsigned(62201,16); cos_data_int <= to_unsigned(38522,16);
     when "101110000011" => sin_data_int <= to_unsigned(62210,16); cos_data_int <= to_unsigned(38477,16);
     when "101110000100" => sin_data_int <= to_unsigned(62219,16); cos_data_int <= to_unsigned(38432,16);
     when "101110000101" => sin_data_int <= to_unsigned(62228,16); cos_data_int <= to_unsigned(38387,16);
     when "101110000110" => sin_data_int <= to_unsigned(62236,16); cos_data_int <= to_unsigned(38342,16);
     when "101110000111" => sin_data_int <= to_unsigned(62245,16); cos_data_int <= to_unsigned(38296,16);
     when "101110001000" => sin_data_int <= to_unsigned(62253,16); cos_data_int <= to_unsigned(38251,16);
     when "101110001001" => sin_data_int <= to_unsigned(62262,16); cos_data_int <= to_unsigned(38206,16);
     when "101110001010" => sin_data_int <= to_unsigned(62270,16); cos_data_int <= to_unsigned(38161,16);
     when "101110001011" => sin_data_int <= to_unsigned(62278,16); cos_data_int <= to_unsigned(38115,16);
     when "101110001100" => sin_data_int <= to_unsigned(62286,16); cos_data_int <= to_unsigned(38070,16);
     when "101110001101" => sin_data_int <= to_unsigned(62294,16); cos_data_int <= to_unsigned(38025,16);
     when "101110001110" => sin_data_int <= to_unsigned(62302,16); cos_data_int <= to_unsigned(37980,16);
     when "101110001111" => sin_data_int <= to_unsigned(62310,16); cos_data_int <= to_unsigned(37934,16);
     when "101110010000" => sin_data_int <= to_unsigned(62318,16); cos_data_int <= to_unsigned(37889,16);
     when "101110010001" => sin_data_int <= to_unsigned(62326,16); cos_data_int <= to_unsigned(37844,16);
     when "101110010010" => sin_data_int <= to_unsigned(62334,16); cos_data_int <= to_unsigned(37798,16);
     when "101110010011" => sin_data_int <= to_unsigned(62342,16); cos_data_int <= to_unsigned(37753,16);
     when "101110010100" => sin_data_int <= to_unsigned(62349,16); cos_data_int <= to_unsigned(37707,16);
     when "101110010101" => sin_data_int <= to_unsigned(62357,16); cos_data_int <= to_unsigned(37662,16);
     when "101110010110" => sin_data_int <= to_unsigned(62364,16); cos_data_int <= to_unsigned(37617,16);
     when "101110010111" => sin_data_int <= to_unsigned(62372,16); cos_data_int <= to_unsigned(37571,16);
     when "101110011000" => sin_data_int <= to_unsigned(62379,16); cos_data_int <= to_unsigned(37526,16);
     when "101110011001" => sin_data_int <= to_unsigned(62386,16); cos_data_int <= to_unsigned(37480,16);
     when "101110011010" => sin_data_int <= to_unsigned(62394,16); cos_data_int <= to_unsigned(37435,16);
     when "101110011011" => sin_data_int <= to_unsigned(62401,16); cos_data_int <= to_unsigned(37389,16);
     when "101110011100" => sin_data_int <= to_unsigned(62408,16); cos_data_int <= to_unsigned(37344,16);
     when "101110011101" => sin_data_int <= to_unsigned(62415,16); cos_data_int <= to_unsigned(37298,16);
     when "101110011110" => sin_data_int <= to_unsigned(62422,16); cos_data_int <= to_unsigned(37253,16);
     when "101110011111" => sin_data_int <= to_unsigned(62429,16); cos_data_int <= to_unsigned(37207,16);
     when "101110100000" => sin_data_int <= to_unsigned(62435,16); cos_data_int <= to_unsigned(37162,16);
     when "101110100001" => sin_data_int <= to_unsigned(62442,16); cos_data_int <= to_unsigned(37116,16);
     when "101110100010" => sin_data_int <= to_unsigned(62449,16); cos_data_int <= to_unsigned(37071,16);
     when "101110100011" => sin_data_int <= to_unsigned(62455,16); cos_data_int <= to_unsigned(37025,16);
     when "101110100100" => sin_data_int <= to_unsigned(62462,16); cos_data_int <= to_unsigned(36980,16);
     when "101110100101" => sin_data_int <= to_unsigned(62468,16); cos_data_int <= to_unsigned(36934,16);
     when "101110100110" => sin_data_int <= to_unsigned(62475,16); cos_data_int <= to_unsigned(36889,16);
     when "101110100111" => sin_data_int <= to_unsigned(62481,16); cos_data_int <= to_unsigned(36843,16);
     when "101110101000" => sin_data_int <= to_unsigned(62487,16); cos_data_int <= to_unsigned(36797,16);
     when "101110101001" => sin_data_int <= to_unsigned(62493,16); cos_data_int <= to_unsigned(36752,16);
     when "101110101010" => sin_data_int <= to_unsigned(62499,16); cos_data_int <= to_unsigned(36706,16);
     when "101110101011" => sin_data_int <= to_unsigned(62505,16); cos_data_int <= to_unsigned(36661,16);
     when "101110101100" => sin_data_int <= to_unsigned(62511,16); cos_data_int <= to_unsigned(36615,16);
     when "101110101101" => sin_data_int <= to_unsigned(62517,16); cos_data_int <= to_unsigned(36569,16);
     when "101110101110" => sin_data_int <= to_unsigned(62523,16); cos_data_int <= to_unsigned(36524,16);
     when "101110101111" => sin_data_int <= to_unsigned(62529,16); cos_data_int <= to_unsigned(36478,16);
     when "101110110000" => sin_data_int <= to_unsigned(62534,16); cos_data_int <= to_unsigned(36432,16);
     when "101110110001" => sin_data_int <= to_unsigned(62540,16); cos_data_int <= to_unsigned(36387,16);
     when "101110110010" => sin_data_int <= to_unsigned(62546,16); cos_data_int <= to_unsigned(36341,16);
     when "101110110011" => sin_data_int <= to_unsigned(62551,16); cos_data_int <= to_unsigned(36295,16);
     when "101110110100" => sin_data_int <= to_unsigned(62556,16); cos_data_int <= to_unsigned(36250,16);
     when "101110110101" => sin_data_int <= to_unsigned(62562,16); cos_data_int <= to_unsigned(36204,16);
     when "101110110110" => sin_data_int <= to_unsigned(62567,16); cos_data_int <= to_unsigned(36158,16);
     when "101110110111" => sin_data_int <= to_unsigned(62572,16); cos_data_int <= to_unsigned(36112,16);
     when "101110111000" => sin_data_int <= to_unsigned(62577,16); cos_data_int <= to_unsigned(36067,16);
     when "101110111001" => sin_data_int <= to_unsigned(62582,16); cos_data_int <= to_unsigned(36021,16);
     when "101110111010" => sin_data_int <= to_unsigned(62587,16); cos_data_int <= to_unsigned(35975,16);
     when "101110111011" => sin_data_int <= to_unsigned(62592,16); cos_data_int <= to_unsigned(35929,16);
     when "101110111100" => sin_data_int <= to_unsigned(62597,16); cos_data_int <= to_unsigned(35884,16);
     when "101110111101" => sin_data_int <= to_unsigned(62602,16); cos_data_int <= to_unsigned(35838,16);
     when "101110111110" => sin_data_int <= to_unsigned(62606,16); cos_data_int <= to_unsigned(35792,16);
     when "101110111111" => sin_data_int <= to_unsigned(62611,16); cos_data_int <= to_unsigned(35746,16);
     when "101111000000" => sin_data_int <= to_unsigned(62616,16); cos_data_int <= to_unsigned(35701,16);
     when "101111000001" => sin_data_int <= to_unsigned(62620,16); cos_data_int <= to_unsigned(35655,16);
     when "101111000010" => sin_data_int <= to_unsigned(62624,16); cos_data_int <= to_unsigned(35609,16);
     when "101111000011" => sin_data_int <= to_unsigned(62629,16); cos_data_int <= to_unsigned(35563,16);
     when "101111000100" => sin_data_int <= to_unsigned(62633,16); cos_data_int <= to_unsigned(35517,16);
     when "101111000101" => sin_data_int <= to_unsigned(62637,16); cos_data_int <= to_unsigned(35471,16);
     when "101111000110" => sin_data_int <= to_unsigned(62641,16); cos_data_int <= to_unsigned(35426,16);
     when "101111000111" => sin_data_int <= to_unsigned(62645,16); cos_data_int <= to_unsigned(35380,16);
     when "101111001000" => sin_data_int <= to_unsigned(62649,16); cos_data_int <= to_unsigned(35334,16);
     when "101111001001" => sin_data_int <= to_unsigned(62653,16); cos_data_int <= to_unsigned(35288,16);
     when "101111001010" => sin_data_int <= to_unsigned(62657,16); cos_data_int <= to_unsigned(35242,16);
     when "101111001011" => sin_data_int <= to_unsigned(62661,16); cos_data_int <= to_unsigned(35196,16);
     when "101111001100" => sin_data_int <= to_unsigned(62665,16); cos_data_int <= to_unsigned(35150,16);
     when "101111001101" => sin_data_int <= to_unsigned(62668,16); cos_data_int <= to_unsigned(35105,16);
     when "101111001110" => sin_data_int <= to_unsigned(62672,16); cos_data_int <= to_unsigned(35059,16);
     when "101111001111" => sin_data_int <= to_unsigned(62675,16); cos_data_int <= to_unsigned(35013,16);
     when "101111010000" => sin_data_int <= to_unsigned(62679,16); cos_data_int <= to_unsigned(34967,16);
     when "101111010001" => sin_data_int <= to_unsigned(62682,16); cos_data_int <= to_unsigned(34921,16);
     when "101111010010" => sin_data_int <= to_unsigned(62685,16); cos_data_int <= to_unsigned(34875,16);
     when "101111010011" => sin_data_int <= to_unsigned(62689,16); cos_data_int <= to_unsigned(34829,16);
     when "101111010100" => sin_data_int <= to_unsigned(62692,16); cos_data_int <= to_unsigned(34783,16);
     when "101111010101" => sin_data_int <= to_unsigned(62695,16); cos_data_int <= to_unsigned(34737,16);
     when "101111010110" => sin_data_int <= to_unsigned(62698,16); cos_data_int <= to_unsigned(34691,16);
     when "101111010111" => sin_data_int <= to_unsigned(62701,16); cos_data_int <= to_unsigned(34646,16);
     when "101111011000" => sin_data_int <= to_unsigned(62704,16); cos_data_int <= to_unsigned(34600,16);
     when "101111011001" => sin_data_int <= to_unsigned(62706,16); cos_data_int <= to_unsigned(34554,16);
     when "101111011010" => sin_data_int <= to_unsigned(62709,16); cos_data_int <= to_unsigned(34508,16);
     when "101111011011" => sin_data_int <= to_unsigned(62712,16); cos_data_int <= to_unsigned(34462,16);
     when "101111011100" => sin_data_int <= to_unsigned(62714,16); cos_data_int <= to_unsigned(34416,16);
     when "101111011101" => sin_data_int <= to_unsigned(62717,16); cos_data_int <= to_unsigned(34370,16);
     when "101111011110" => sin_data_int <= to_unsigned(62719,16); cos_data_int <= to_unsigned(34324,16);
     when "101111011111" => sin_data_int <= to_unsigned(62722,16); cos_data_int <= to_unsigned(34278,16);
     when "101111100000" => sin_data_int <= to_unsigned(62724,16); cos_data_int <= to_unsigned(34232,16);
     when "101111100001" => sin_data_int <= to_unsigned(62726,16); cos_data_int <= to_unsigned(34186,16);
     when "101111100010" => sin_data_int <= to_unsigned(62728,16); cos_data_int <= to_unsigned(34140,16);
     when "101111100011" => sin_data_int <= to_unsigned(62730,16); cos_data_int <= to_unsigned(34094,16);
     when "101111100100" => sin_data_int <= to_unsigned(62732,16); cos_data_int <= to_unsigned(34048,16);
     when "101111100101" => sin_data_int <= to_unsigned(62734,16); cos_data_int <= to_unsigned(34002,16);
     when "101111100110" => sin_data_int <= to_unsigned(62736,16); cos_data_int <= to_unsigned(33956,16);
     when "101111100111" => sin_data_int <= to_unsigned(62738,16); cos_data_int <= to_unsigned(33910,16);
     when "101111101000" => sin_data_int <= to_unsigned(62740,16); cos_data_int <= to_unsigned(33864,16);
     when "101111101001" => sin_data_int <= to_unsigned(62741,16); cos_data_int <= to_unsigned(33818,16);
     when "101111101010" => sin_data_int <= to_unsigned(62743,16); cos_data_int <= to_unsigned(33772,16);
     when "101111101011" => sin_data_int <= to_unsigned(62744,16); cos_data_int <= to_unsigned(33726,16);
     when "101111101100" => sin_data_int <= to_unsigned(62746,16); cos_data_int <= to_unsigned(33680,16);
     when "101111101101" => sin_data_int <= to_unsigned(62747,16); cos_data_int <= to_unsigned(33634,16);
     when "101111101110" => sin_data_int <= to_unsigned(62749,16); cos_data_int <= to_unsigned(33588,16);
     when "101111101111" => sin_data_int <= to_unsigned(62750,16); cos_data_int <= to_unsigned(33542,16);
     when "101111110000" => sin_data_int <= to_unsigned(62751,16); cos_data_int <= to_unsigned(33496,16);
     when "101111110001" => sin_data_int <= to_unsigned(62752,16); cos_data_int <= to_unsigned(33450,16);
     when "101111110010" => sin_data_int <= to_unsigned(62753,16); cos_data_int <= to_unsigned(33404,16);
     when "101111110011" => sin_data_int <= to_unsigned(62754,16); cos_data_int <= to_unsigned(33358,16);
     when "101111110100" => sin_data_int <= to_unsigned(62755,16); cos_data_int <= to_unsigned(33312,16);
     when "101111110101" => sin_data_int <= to_unsigned(62756,16); cos_data_int <= to_unsigned(33266,16);
     when "101111110110" => sin_data_int <= to_unsigned(62756,16); cos_data_int <= to_unsigned(33220,16);
     when "101111110111" => sin_data_int <= to_unsigned(62757,16); cos_data_int <= to_unsigned(33174,16);
     when "101111111000" => sin_data_int <= to_unsigned(62758,16); cos_data_int <= to_unsigned(33128,16);
     when "101111111001" => sin_data_int <= to_unsigned(62758,16); cos_data_int <= to_unsigned(33082,16);
     when "101111111010" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(33036,16);
     when "101111111011" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(32990,16);
     when "101111111100" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(32944,16);
     when "101111111101" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32898,16);
     when "101111111110" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32852,16);
     when "101111111111" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32806,16);
     when "110000000000" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32760,16);
     when "110000000001" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32714,16);
     when "110000000010" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32668,16);
     when "110000000011" => sin_data_int <= to_unsigned(62760,16); cos_data_int <= to_unsigned(32622,16);
     when "110000000100" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(32576,16);
     when "110000000101" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(32530,16);
     when "110000000110" => sin_data_int <= to_unsigned(62759,16); cos_data_int <= to_unsigned(32484,16);
     when "110000000111" => sin_data_int <= to_unsigned(62758,16); cos_data_int <= to_unsigned(32438,16);
     when "110000001000" => sin_data_int <= to_unsigned(62758,16); cos_data_int <= to_unsigned(32392,16);
     when "110000001001" => sin_data_int <= to_unsigned(62757,16); cos_data_int <= to_unsigned(32346,16);
     when "110000001010" => sin_data_int <= to_unsigned(62756,16); cos_data_int <= to_unsigned(32300,16);
     when "110000001011" => sin_data_int <= to_unsigned(62756,16); cos_data_int <= to_unsigned(32254,16);
     when "110000001100" => sin_data_int <= to_unsigned(62755,16); cos_data_int <= to_unsigned(32208,16);
     when "110000001101" => sin_data_int <= to_unsigned(62754,16); cos_data_int <= to_unsigned(32162,16);
     when "110000001110" => sin_data_int <= to_unsigned(62753,16); cos_data_int <= to_unsigned(32116,16);
     when "110000001111" => sin_data_int <= to_unsigned(62752,16); cos_data_int <= to_unsigned(32070,16);
     when "110000010000" => sin_data_int <= to_unsigned(62751,16); cos_data_int <= to_unsigned(32024,16);
     when "110000010001" => sin_data_int <= to_unsigned(62750,16); cos_data_int <= to_unsigned(31978,16);
     when "110000010010" => sin_data_int <= to_unsigned(62749,16); cos_data_int <= to_unsigned(31932,16);
     when "110000010011" => sin_data_int <= to_unsigned(62747,16); cos_data_int <= to_unsigned(31886,16);
     when "110000010100" => sin_data_int <= to_unsigned(62746,16); cos_data_int <= to_unsigned(31840,16);
     when "110000010101" => sin_data_int <= to_unsigned(62744,16); cos_data_int <= to_unsigned(31794,16);
     when "110000010110" => sin_data_int <= to_unsigned(62743,16); cos_data_int <= to_unsigned(31748,16);
     when "110000010111" => sin_data_int <= to_unsigned(62741,16); cos_data_int <= to_unsigned(31702,16);
     when "110000011000" => sin_data_int <= to_unsigned(62740,16); cos_data_int <= to_unsigned(31656,16);
     when "110000011001" => sin_data_int <= to_unsigned(62738,16); cos_data_int <= to_unsigned(31610,16);
     when "110000011010" => sin_data_int <= to_unsigned(62736,16); cos_data_int <= to_unsigned(31564,16);
     when "110000011011" => sin_data_int <= to_unsigned(62734,16); cos_data_int <= to_unsigned(31518,16);
     when "110000011100" => sin_data_int <= to_unsigned(62732,16); cos_data_int <= to_unsigned(31472,16);
     when "110000011101" => sin_data_int <= to_unsigned(62730,16); cos_data_int <= to_unsigned(31426,16);
     when "110000011110" => sin_data_int <= to_unsigned(62728,16); cos_data_int <= to_unsigned(31380,16);
     when "110000011111" => sin_data_int <= to_unsigned(62726,16); cos_data_int <= to_unsigned(31334,16);
     when "110000100000" => sin_data_int <= to_unsigned(62724,16); cos_data_int <= to_unsigned(31288,16);
     when "110000100001" => sin_data_int <= to_unsigned(62722,16); cos_data_int <= to_unsigned(31242,16);
     when "110000100010" => sin_data_int <= to_unsigned(62719,16); cos_data_int <= to_unsigned(31196,16);
     when "110000100011" => sin_data_int <= to_unsigned(62717,16); cos_data_int <= to_unsigned(31150,16);
     when "110000100100" => sin_data_int <= to_unsigned(62714,16); cos_data_int <= to_unsigned(31104,16);
     when "110000100101" => sin_data_int <= to_unsigned(62712,16); cos_data_int <= to_unsigned(31058,16);
     when "110000100110" => sin_data_int <= to_unsigned(62709,16); cos_data_int <= to_unsigned(31012,16);
     when "110000100111" => sin_data_int <= to_unsigned(62706,16); cos_data_int <= to_unsigned(30966,16);
     when "110000101000" => sin_data_int <= to_unsigned(62704,16); cos_data_int <= to_unsigned(30920,16);
     when "110000101001" => sin_data_int <= to_unsigned(62701,16); cos_data_int <= to_unsigned(30874,16);
     when "110000101010" => sin_data_int <= to_unsigned(62698,16); cos_data_int <= to_unsigned(30829,16);
     when "110000101011" => sin_data_int <= to_unsigned(62695,16); cos_data_int <= to_unsigned(30783,16);
     when "110000101100" => sin_data_int <= to_unsigned(62692,16); cos_data_int <= to_unsigned(30737,16);
     when "110000101101" => sin_data_int <= to_unsigned(62689,16); cos_data_int <= to_unsigned(30691,16);
     when "110000101110" => sin_data_int <= to_unsigned(62685,16); cos_data_int <= to_unsigned(30645,16);
     when "110000101111" => sin_data_int <= to_unsigned(62682,16); cos_data_int <= to_unsigned(30599,16);
     when "110000110000" => sin_data_int <= to_unsigned(62679,16); cos_data_int <= to_unsigned(30553,16);
     when "110000110001" => sin_data_int <= to_unsigned(62675,16); cos_data_int <= to_unsigned(30507,16);
     when "110000110010" => sin_data_int <= to_unsigned(62672,16); cos_data_int <= to_unsigned(30461,16);
     when "110000110011" => sin_data_int <= to_unsigned(62668,16); cos_data_int <= to_unsigned(30415,16);
     when "110000110100" => sin_data_int <= to_unsigned(62665,16); cos_data_int <= to_unsigned(30370,16);
     when "110000110101" => sin_data_int <= to_unsigned(62661,16); cos_data_int <= to_unsigned(30324,16);
     when "110000110110" => sin_data_int <= to_unsigned(62657,16); cos_data_int <= to_unsigned(30278,16);
     when "110000110111" => sin_data_int <= to_unsigned(62653,16); cos_data_int <= to_unsigned(30232,16);
     when "110000111000" => sin_data_int <= to_unsigned(62649,16); cos_data_int <= to_unsigned(30186,16);
     when "110000111001" => sin_data_int <= to_unsigned(62645,16); cos_data_int <= to_unsigned(30140,16);
     when "110000111010" => sin_data_int <= to_unsigned(62641,16); cos_data_int <= to_unsigned(30094,16);
     when "110000111011" => sin_data_int <= to_unsigned(62637,16); cos_data_int <= to_unsigned(30049,16);
     when "110000111100" => sin_data_int <= to_unsigned(62633,16); cos_data_int <= to_unsigned(30003,16);
     when "110000111101" => sin_data_int <= to_unsigned(62629,16); cos_data_int <= to_unsigned(29957,16);
     when "110000111110" => sin_data_int <= to_unsigned(62624,16); cos_data_int <= to_unsigned(29911,16);
     when "110000111111" => sin_data_int <= to_unsigned(62620,16); cos_data_int <= to_unsigned(29865,16);
     when "110001000000" => sin_data_int <= to_unsigned(62616,16); cos_data_int <= to_unsigned(29819,16);
     when "110001000001" => sin_data_int <= to_unsigned(62611,16); cos_data_int <= to_unsigned(29774,16);
     when "110001000010" => sin_data_int <= to_unsigned(62606,16); cos_data_int <= to_unsigned(29728,16);
     when "110001000011" => sin_data_int <= to_unsigned(62602,16); cos_data_int <= to_unsigned(29682,16);
     when "110001000100" => sin_data_int <= to_unsigned(62597,16); cos_data_int <= to_unsigned(29636,16);
     when "110001000101" => sin_data_int <= to_unsigned(62592,16); cos_data_int <= to_unsigned(29591,16);
     when "110001000110" => sin_data_int <= to_unsigned(62587,16); cos_data_int <= to_unsigned(29545,16);
     when "110001000111" => sin_data_int <= to_unsigned(62582,16); cos_data_int <= to_unsigned(29499,16);
     when "110001001000" => sin_data_int <= to_unsigned(62577,16); cos_data_int <= to_unsigned(29453,16);
     when "110001001001" => sin_data_int <= to_unsigned(62572,16); cos_data_int <= to_unsigned(29408,16);
     when "110001001010" => sin_data_int <= to_unsigned(62567,16); cos_data_int <= to_unsigned(29362,16);
     when "110001001011" => sin_data_int <= to_unsigned(62562,16); cos_data_int <= to_unsigned(29316,16);
     when "110001001100" => sin_data_int <= to_unsigned(62556,16); cos_data_int <= to_unsigned(29270,16);
     when "110001001101" => sin_data_int <= to_unsigned(62551,16); cos_data_int <= to_unsigned(29225,16);
     when "110001001110" => sin_data_int <= to_unsigned(62546,16); cos_data_int <= to_unsigned(29179,16);
     when "110001001111" => sin_data_int <= to_unsigned(62540,16); cos_data_int <= to_unsigned(29133,16);
     when "110001010000" => sin_data_int <= to_unsigned(62534,16); cos_data_int <= to_unsigned(29088,16);
     when "110001010001" => sin_data_int <= to_unsigned(62529,16); cos_data_int <= to_unsigned(29042,16);
     when "110001010010" => sin_data_int <= to_unsigned(62523,16); cos_data_int <= to_unsigned(28996,16);
     when "110001010011" => sin_data_int <= to_unsigned(62517,16); cos_data_int <= to_unsigned(28951,16);
     when "110001010100" => sin_data_int <= to_unsigned(62511,16); cos_data_int <= to_unsigned(28905,16);
     when "110001010101" => sin_data_int <= to_unsigned(62505,16); cos_data_int <= to_unsigned(28859,16);
     when "110001010110" => sin_data_int <= to_unsigned(62499,16); cos_data_int <= to_unsigned(28814,16);
     when "110001010111" => sin_data_int <= to_unsigned(62493,16); cos_data_int <= to_unsigned(28768,16);
     when "110001011000" => sin_data_int <= to_unsigned(62487,16); cos_data_int <= to_unsigned(28723,16);
     when "110001011001" => sin_data_int <= to_unsigned(62481,16); cos_data_int <= to_unsigned(28677,16);
     when "110001011010" => sin_data_int <= to_unsigned(62475,16); cos_data_int <= to_unsigned(28631,16);
     when "110001011011" => sin_data_int <= to_unsigned(62468,16); cos_data_int <= to_unsigned(28586,16);
     when "110001011100" => sin_data_int <= to_unsigned(62462,16); cos_data_int <= to_unsigned(28540,16);
     when "110001011101" => sin_data_int <= to_unsigned(62455,16); cos_data_int <= to_unsigned(28495,16);
     when "110001011110" => sin_data_int <= to_unsigned(62449,16); cos_data_int <= to_unsigned(28449,16);
     when "110001011111" => sin_data_int <= to_unsigned(62442,16); cos_data_int <= to_unsigned(28404,16);
     when "110001100000" => sin_data_int <= to_unsigned(62435,16); cos_data_int <= to_unsigned(28358,16);
     when "110001100001" => sin_data_int <= to_unsigned(62429,16); cos_data_int <= to_unsigned(28313,16);
     when "110001100010" => sin_data_int <= to_unsigned(62422,16); cos_data_int <= to_unsigned(28267,16);
     when "110001100011" => sin_data_int <= to_unsigned(62415,16); cos_data_int <= to_unsigned(28222,16);
     when "110001100100" => sin_data_int <= to_unsigned(62408,16); cos_data_int <= to_unsigned(28176,16);
     when "110001100101" => sin_data_int <= to_unsigned(62401,16); cos_data_int <= to_unsigned(28131,16);
     when "110001100110" => sin_data_int <= to_unsigned(62394,16); cos_data_int <= to_unsigned(28085,16);
     when "110001100111" => sin_data_int <= to_unsigned(62386,16); cos_data_int <= to_unsigned(28040,16);
     when "110001101000" => sin_data_int <= to_unsigned(62379,16); cos_data_int <= to_unsigned(27994,16);
     when "110001101001" => sin_data_int <= to_unsigned(62372,16); cos_data_int <= to_unsigned(27949,16);
     when "110001101010" => sin_data_int <= to_unsigned(62364,16); cos_data_int <= to_unsigned(27903,16);
     when "110001101011" => sin_data_int <= to_unsigned(62357,16); cos_data_int <= to_unsigned(27858,16);
     when "110001101100" => sin_data_int <= to_unsigned(62349,16); cos_data_int <= to_unsigned(27813,16);
     when "110001101101" => sin_data_int <= to_unsigned(62342,16); cos_data_int <= to_unsigned(27767,16);
     when "110001101110" => sin_data_int <= to_unsigned(62334,16); cos_data_int <= to_unsigned(27722,16);
     when "110001101111" => sin_data_int <= to_unsigned(62326,16); cos_data_int <= to_unsigned(27676,16);
     when "110001110000" => sin_data_int <= to_unsigned(62318,16); cos_data_int <= to_unsigned(27631,16);
     when "110001110001" => sin_data_int <= to_unsigned(62310,16); cos_data_int <= to_unsigned(27586,16);
     when "110001110010" => sin_data_int <= to_unsigned(62302,16); cos_data_int <= to_unsigned(27540,16);
     when "110001110011" => sin_data_int <= to_unsigned(62294,16); cos_data_int <= to_unsigned(27495,16);
     when "110001110100" => sin_data_int <= to_unsigned(62286,16); cos_data_int <= to_unsigned(27450,16);
     when "110001110101" => sin_data_int <= to_unsigned(62278,16); cos_data_int <= to_unsigned(27405,16);
     when "110001110110" => sin_data_int <= to_unsigned(62270,16); cos_data_int <= to_unsigned(27359,16);
     when "110001110111" => sin_data_int <= to_unsigned(62262,16); cos_data_int <= to_unsigned(27314,16);
     when "110001111000" => sin_data_int <= to_unsigned(62253,16); cos_data_int <= to_unsigned(27269,16);
     when "110001111001" => sin_data_int <= to_unsigned(62245,16); cos_data_int <= to_unsigned(27224,16);
     when "110001111010" => sin_data_int <= to_unsigned(62236,16); cos_data_int <= to_unsigned(27178,16);
     when "110001111011" => sin_data_int <= to_unsigned(62228,16); cos_data_int <= to_unsigned(27133,16);
     when "110001111100" => sin_data_int <= to_unsigned(62219,16); cos_data_int <= to_unsigned(27088,16);
     when "110001111101" => sin_data_int <= to_unsigned(62210,16); cos_data_int <= to_unsigned(27043,16);
     when "110001111110" => sin_data_int <= to_unsigned(62201,16); cos_data_int <= to_unsigned(26998,16);
     when "110001111111" => sin_data_int <= to_unsigned(62193,16); cos_data_int <= to_unsigned(26952,16);
     when "110010000000" => sin_data_int <= to_unsigned(62184,16); cos_data_int <= to_unsigned(26907,16);
     when "110010000001" => sin_data_int <= to_unsigned(62175,16); cos_data_int <= to_unsigned(26862,16);
     when "110010000010" => sin_data_int <= to_unsigned(62165,16); cos_data_int <= to_unsigned(26817,16);
     when "110010000011" => sin_data_int <= to_unsigned(62156,16); cos_data_int <= to_unsigned(26772,16);
     when "110010000100" => sin_data_int <= to_unsigned(62147,16); cos_data_int <= to_unsigned(26727,16);
     when "110010000101" => sin_data_int <= to_unsigned(62138,16); cos_data_int <= to_unsigned(26682,16);
     when "110010000110" => sin_data_int <= to_unsigned(62128,16); cos_data_int <= to_unsigned(26637,16);
     when "110010000111" => sin_data_int <= to_unsigned(62119,16); cos_data_int <= to_unsigned(26592,16);
     when "110010001000" => sin_data_int <= to_unsigned(62110,16); cos_data_int <= to_unsigned(26547,16);
     when "110010001001" => sin_data_int <= to_unsigned(62100,16); cos_data_int <= to_unsigned(26502,16);
     when "110010001010" => sin_data_int <= to_unsigned(62090,16); cos_data_int <= to_unsigned(26457,16);
     when "110010001011" => sin_data_int <= to_unsigned(62081,16); cos_data_int <= to_unsigned(26412,16);
     when "110010001100" => sin_data_int <= to_unsigned(62071,16); cos_data_int <= to_unsigned(26367,16);
     when "110010001101" => sin_data_int <= to_unsigned(62061,16); cos_data_int <= to_unsigned(26322,16);
     when "110010001110" => sin_data_int <= to_unsigned(62051,16); cos_data_int <= to_unsigned(26277,16);
     when "110010001111" => sin_data_int <= to_unsigned(62041,16); cos_data_int <= to_unsigned(26232,16);
     when "110010010000" => sin_data_int <= to_unsigned(62031,16); cos_data_int <= to_unsigned(26187,16);
     when "110010010001" => sin_data_int <= to_unsigned(62021,16); cos_data_int <= to_unsigned(26142,16);
     when "110010010010" => sin_data_int <= to_unsigned(62011,16); cos_data_int <= to_unsigned(26097,16);
     when "110010010011" => sin_data_int <= to_unsigned(62001,16); cos_data_int <= to_unsigned(26052,16);
     when "110010010100" => sin_data_int <= to_unsigned(61990,16); cos_data_int <= to_unsigned(26007,16);
     when "110010010101" => sin_data_int <= to_unsigned(61980,16); cos_data_int <= to_unsigned(25963,16);
     when "110010010110" => sin_data_int <= to_unsigned(61969,16); cos_data_int <= to_unsigned(25918,16);
     when "110010010111" => sin_data_int <= to_unsigned(61959,16); cos_data_int <= to_unsigned(25873,16);
     when "110010011000" => sin_data_int <= to_unsigned(61948,16); cos_data_int <= to_unsigned(25828,16);
     when "110010011001" => sin_data_int <= to_unsigned(61938,16); cos_data_int <= to_unsigned(25783,16);
     when "110010011010" => sin_data_int <= to_unsigned(61927,16); cos_data_int <= to_unsigned(25739,16);
     when "110010011011" => sin_data_int <= to_unsigned(61916,16); cos_data_int <= to_unsigned(25694,16);
     when "110010011100" => sin_data_int <= to_unsigned(61905,16); cos_data_int <= to_unsigned(25649,16);
     when "110010011101" => sin_data_int <= to_unsigned(61894,16); cos_data_int <= to_unsigned(25605,16);
     when "110010011110" => sin_data_int <= to_unsigned(61883,16); cos_data_int <= to_unsigned(25560,16);
     when "110010011111" => sin_data_int <= to_unsigned(61872,16); cos_data_int <= to_unsigned(25515,16);
     when "110010100000" => sin_data_int <= to_unsigned(61861,16); cos_data_int <= to_unsigned(25471,16);
     when "110010100001" => sin_data_int <= to_unsigned(61850,16); cos_data_int <= to_unsigned(25426,16);
     when "110010100010" => sin_data_int <= to_unsigned(61838,16); cos_data_int <= to_unsigned(25381,16);
     when "110010100011" => sin_data_int <= to_unsigned(61827,16); cos_data_int <= to_unsigned(25337,16);
     when "110010100100" => sin_data_int <= to_unsigned(61816,16); cos_data_int <= to_unsigned(25292,16);
     when "110010100101" => sin_data_int <= to_unsigned(61804,16); cos_data_int <= to_unsigned(25248,16);
     when "110010100110" => sin_data_int <= to_unsigned(61793,16); cos_data_int <= to_unsigned(25203,16);
     when "110010100111" => sin_data_int <= to_unsigned(61781,16); cos_data_int <= to_unsigned(25159,16);
     when "110010101000" => sin_data_int <= to_unsigned(61769,16); cos_data_int <= to_unsigned(25114,16);
     when "110010101001" => sin_data_int <= to_unsigned(61758,16); cos_data_int <= to_unsigned(25070,16);
     when "110010101010" => sin_data_int <= to_unsigned(61746,16); cos_data_int <= to_unsigned(25025,16);
     when "110010101011" => sin_data_int <= to_unsigned(61734,16); cos_data_int <= to_unsigned(24981,16);
     when "110010101100" => sin_data_int <= to_unsigned(61722,16); cos_data_int <= to_unsigned(24936,16);
     when "110010101101" => sin_data_int <= to_unsigned(61710,16); cos_data_int <= to_unsigned(24892,16);
     when "110010101110" => sin_data_int <= to_unsigned(61698,16); cos_data_int <= to_unsigned(24847,16);
     when "110010101111" => sin_data_int <= to_unsigned(61686,16); cos_data_int <= to_unsigned(24803,16);
     when "110010110000" => sin_data_int <= to_unsigned(61673,16); cos_data_int <= to_unsigned(24759,16);
     when "110010110001" => sin_data_int <= to_unsigned(61661,16); cos_data_int <= to_unsigned(24714,16);
     when "110010110010" => sin_data_int <= to_unsigned(61649,16); cos_data_int <= to_unsigned(24670,16);
     when "110010110011" => sin_data_int <= to_unsigned(61636,16); cos_data_int <= to_unsigned(24626,16);
     when "110010110100" => sin_data_int <= to_unsigned(61624,16); cos_data_int <= to_unsigned(24581,16);
     when "110010110101" => sin_data_int <= to_unsigned(61611,16); cos_data_int <= to_unsigned(24537,16);
     when "110010110110" => sin_data_int <= to_unsigned(61598,16); cos_data_int <= to_unsigned(24493,16);
     when "110010110111" => sin_data_int <= to_unsigned(61586,16); cos_data_int <= to_unsigned(24449,16);
     when "110010111000" => sin_data_int <= to_unsigned(61573,16); cos_data_int <= to_unsigned(24404,16);
     when "110010111001" => sin_data_int <= to_unsigned(61560,16); cos_data_int <= to_unsigned(24360,16);
     when "110010111010" => sin_data_int <= to_unsigned(61547,16); cos_data_int <= to_unsigned(24316,16);
     when "110010111011" => sin_data_int <= to_unsigned(61534,16); cos_data_int <= to_unsigned(24272,16);
     when "110010111100" => sin_data_int <= to_unsigned(61521,16); cos_data_int <= to_unsigned(24228,16);
     when "110010111101" => sin_data_int <= to_unsigned(61508,16); cos_data_int <= to_unsigned(24184,16);
     when "110010111110" => sin_data_int <= to_unsigned(61495,16); cos_data_int <= to_unsigned(24140,16);
     when "110010111111" => sin_data_int <= to_unsigned(61482,16); cos_data_int <= to_unsigned(24096,16);
     when "110011000000" => sin_data_int <= to_unsigned(61468,16); cos_data_int <= to_unsigned(24051,16);
     when "110011000001" => sin_data_int <= to_unsigned(61455,16); cos_data_int <= to_unsigned(24007,16);
     when "110011000010" => sin_data_int <= to_unsigned(61441,16); cos_data_int <= to_unsigned(23963,16);
     when "110011000011" => sin_data_int <= to_unsigned(61428,16); cos_data_int <= to_unsigned(23919,16);
     when "110011000100" => sin_data_int <= to_unsigned(61414,16); cos_data_int <= to_unsigned(23875,16);
     when "110011000101" => sin_data_int <= to_unsigned(61401,16); cos_data_int <= to_unsigned(23832,16);
     when "110011000110" => sin_data_int <= to_unsigned(61387,16); cos_data_int <= to_unsigned(23788,16);
     when "110011000111" => sin_data_int <= to_unsigned(61373,16); cos_data_int <= to_unsigned(23744,16);
     when "110011001000" => sin_data_int <= to_unsigned(61359,16); cos_data_int <= to_unsigned(23700,16);
     when "110011001001" => sin_data_int <= to_unsigned(61345,16); cos_data_int <= to_unsigned(23656,16);
     when "110011001010" => sin_data_int <= to_unsigned(61331,16); cos_data_int <= to_unsigned(23612,16);
     when "110011001011" => sin_data_int <= to_unsigned(61317,16); cos_data_int <= to_unsigned(23568,16);
     when "110011001100" => sin_data_int <= to_unsigned(61303,16); cos_data_int <= to_unsigned(23525,16);
     when "110011001101" => sin_data_int <= to_unsigned(61289,16); cos_data_int <= to_unsigned(23481,16);
     when "110011001110" => sin_data_int <= to_unsigned(61275,16); cos_data_int <= to_unsigned(23437,16);
     when "110011001111" => sin_data_int <= to_unsigned(61260,16); cos_data_int <= to_unsigned(23393,16);
     when "110011010000" => sin_data_int <= to_unsigned(61246,16); cos_data_int <= to_unsigned(23350,16);
     when "110011010001" => sin_data_int <= to_unsigned(61231,16); cos_data_int <= to_unsigned(23306,16);
     when "110011010010" => sin_data_int <= to_unsigned(61217,16); cos_data_int <= to_unsigned(23262,16);
     when "110011010011" => sin_data_int <= to_unsigned(61202,16); cos_data_int <= to_unsigned(23219,16);
     when "110011010100" => sin_data_int <= to_unsigned(61188,16); cos_data_int <= to_unsigned(23175,16);
     when "110011010101" => sin_data_int <= to_unsigned(61173,16); cos_data_int <= to_unsigned(23131,16);
     when "110011010110" => sin_data_int <= to_unsigned(61158,16); cos_data_int <= to_unsigned(23088,16);
     when "110011010111" => sin_data_int <= to_unsigned(61143,16); cos_data_int <= to_unsigned(23044,16);
     when "110011011000" => sin_data_int <= to_unsigned(61128,16); cos_data_int <= to_unsigned(23001,16);
     when "110011011001" => sin_data_int <= to_unsigned(61113,16); cos_data_int <= to_unsigned(22957,16);
     when "110011011010" => sin_data_int <= to_unsigned(61098,16); cos_data_int <= to_unsigned(22914,16);
     when "110011011011" => sin_data_int <= to_unsigned(61083,16); cos_data_int <= to_unsigned(22870,16);
     when "110011011100" => sin_data_int <= to_unsigned(61068,16); cos_data_int <= to_unsigned(22827,16);
     when "110011011101" => sin_data_int <= to_unsigned(61053,16); cos_data_int <= to_unsigned(22783,16);
     when "110011011110" => sin_data_int <= to_unsigned(61037,16); cos_data_int <= to_unsigned(22740,16);
     when "110011011111" => sin_data_int <= to_unsigned(61022,16); cos_data_int <= to_unsigned(22697,16);
     when "110011100000" => sin_data_int <= to_unsigned(61006,16); cos_data_int <= to_unsigned(22653,16);
     when "110011100001" => sin_data_int <= to_unsigned(60991,16); cos_data_int <= to_unsigned(22610,16);
     when "110011100010" => sin_data_int <= to_unsigned(60975,16); cos_data_int <= to_unsigned(22567,16);
     when "110011100011" => sin_data_int <= to_unsigned(60960,16); cos_data_int <= to_unsigned(22523,16);
     when "110011100100" => sin_data_int <= to_unsigned(60944,16); cos_data_int <= to_unsigned(22480,16);
     when "110011100101" => sin_data_int <= to_unsigned(60928,16); cos_data_int <= to_unsigned(22437,16);
     when "110011100110" => sin_data_int <= to_unsigned(60912,16); cos_data_int <= to_unsigned(22394,16);
     when "110011100111" => sin_data_int <= to_unsigned(60896,16); cos_data_int <= to_unsigned(22351,16);
     when "110011101000" => sin_data_int <= to_unsigned(60880,16); cos_data_int <= to_unsigned(22307,16);
     when "110011101001" => sin_data_int <= to_unsigned(60864,16); cos_data_int <= to_unsigned(22264,16);
     when "110011101010" => sin_data_int <= to_unsigned(60848,16); cos_data_int <= to_unsigned(22221,16);
     when "110011101011" => sin_data_int <= to_unsigned(60832,16); cos_data_int <= to_unsigned(22178,16);
     when "110011101100" => sin_data_int <= to_unsigned(60816,16); cos_data_int <= to_unsigned(22135,16);
     when "110011101101" => sin_data_int <= to_unsigned(60799,16); cos_data_int <= to_unsigned(22092,16);
     when "110011101110" => sin_data_int <= to_unsigned(60783,16); cos_data_int <= to_unsigned(22049,16);
     when "110011101111" => sin_data_int <= to_unsigned(60766,16); cos_data_int <= to_unsigned(22006,16);
     when "110011110000" => sin_data_int <= to_unsigned(60750,16); cos_data_int <= to_unsigned(21963,16);
     when "110011110001" => sin_data_int <= to_unsigned(60733,16); cos_data_int <= to_unsigned(21920,16);
     when "110011110010" => sin_data_int <= to_unsigned(60717,16); cos_data_int <= to_unsigned(21877,16);
     when "110011110011" => sin_data_int <= to_unsigned(60700,16); cos_data_int <= to_unsigned(21834,16);
     when "110011110100" => sin_data_int <= to_unsigned(60683,16); cos_data_int <= to_unsigned(21792,16);
     when "110011110101" => sin_data_int <= to_unsigned(60666,16); cos_data_int <= to_unsigned(21749,16);
     when "110011110110" => sin_data_int <= to_unsigned(60649,16); cos_data_int <= to_unsigned(21706,16);
     when "110011110111" => sin_data_int <= to_unsigned(60632,16); cos_data_int <= to_unsigned(21663,16);
     when "110011111000" => sin_data_int <= to_unsigned(60615,16); cos_data_int <= to_unsigned(21620,16);
     when "110011111001" => sin_data_int <= to_unsigned(60598,16); cos_data_int <= to_unsigned(21578,16);
     when "110011111010" => sin_data_int <= to_unsigned(60581,16); cos_data_int <= to_unsigned(21535,16);
     when "110011111011" => sin_data_int <= to_unsigned(60564,16); cos_data_int <= to_unsigned(21492,16);
     when "110011111100" => sin_data_int <= to_unsigned(60546,16); cos_data_int <= to_unsigned(21450,16);
     when "110011111101" => sin_data_int <= to_unsigned(60529,16); cos_data_int <= to_unsigned(21407,16);
     when "110011111110" => sin_data_int <= to_unsigned(60511,16); cos_data_int <= to_unsigned(21365,16);
     when "110011111111" => sin_data_int <= to_unsigned(60494,16); cos_data_int <= to_unsigned(21322,16);
     when "110100000000" => sin_data_int <= to_unsigned(60476,16); cos_data_int <= to_unsigned(21279,16);
     when "110100000001" => sin_data_int <= to_unsigned(60459,16); cos_data_int <= to_unsigned(21237,16);
     when "110100000010" => sin_data_int <= to_unsigned(60441,16); cos_data_int <= to_unsigned(21195,16);
     when "110100000011" => sin_data_int <= to_unsigned(60423,16); cos_data_int <= to_unsigned(21152,16);
     when "110100000100" => sin_data_int <= to_unsigned(60405,16); cos_data_int <= to_unsigned(21110,16);
     when "110100000101" => sin_data_int <= to_unsigned(60388,16); cos_data_int <= to_unsigned(21067,16);
     when "110100000110" => sin_data_int <= to_unsigned(60370,16); cos_data_int <= to_unsigned(21025,16);
     when "110100000111" => sin_data_int <= to_unsigned(60352,16); cos_data_int <= to_unsigned(20983,16);
     when "110100001000" => sin_data_int <= to_unsigned(60333,16); cos_data_int <= to_unsigned(20940,16);
     when "110100001001" => sin_data_int <= to_unsigned(60315,16); cos_data_int <= to_unsigned(20898,16);
     when "110100001010" => sin_data_int <= to_unsigned(60297,16); cos_data_int <= to_unsigned(20856,16);
     when "110100001011" => sin_data_int <= to_unsigned(60279,16); cos_data_int <= to_unsigned(20813,16);
     when "110100001100" => sin_data_int <= to_unsigned(60260,16); cos_data_int <= to_unsigned(20771,16);
     when "110100001101" => sin_data_int <= to_unsigned(60242,16); cos_data_int <= to_unsigned(20729,16);
     when "110100001110" => sin_data_int <= to_unsigned(60223,16); cos_data_int <= to_unsigned(20687,16);
     when "110100001111" => sin_data_int <= to_unsigned(60205,16); cos_data_int <= to_unsigned(20645,16);
     when "110100010000" => sin_data_int <= to_unsigned(60186,16); cos_data_int <= to_unsigned(20603,16);
     when "110100010001" => sin_data_int <= to_unsigned(60168,16); cos_data_int <= to_unsigned(20561,16);
     when "110100010010" => sin_data_int <= to_unsigned(60149,16); cos_data_int <= to_unsigned(20519,16);
     when "110100010011" => sin_data_int <= to_unsigned(60130,16); cos_data_int <= to_unsigned(20477,16);
     when "110100010100" => sin_data_int <= to_unsigned(60111,16); cos_data_int <= to_unsigned(20435,16);
     when "110100010101" => sin_data_int <= to_unsigned(60092,16); cos_data_int <= to_unsigned(20393,16);
     when "110100010110" => sin_data_int <= to_unsigned(60073,16); cos_data_int <= to_unsigned(20351,16);
     when "110100010111" => sin_data_int <= to_unsigned(60054,16); cos_data_int <= to_unsigned(20309,16);
     when "110100011000" => sin_data_int <= to_unsigned(60035,16); cos_data_int <= to_unsigned(20267,16);
     when "110100011001" => sin_data_int <= to_unsigned(60016,16); cos_data_int <= to_unsigned(20225,16);
     when "110100011010" => sin_data_int <= to_unsigned(59997,16); cos_data_int <= to_unsigned(20183,16);
     when "110100011011" => sin_data_int <= to_unsigned(59977,16); cos_data_int <= to_unsigned(20142,16);
     when "110100011100" => sin_data_int <= to_unsigned(59958,16); cos_data_int <= to_unsigned(20100,16);
     when "110100011101" => sin_data_int <= to_unsigned(59938,16); cos_data_int <= to_unsigned(20058,16);
     when "110100011110" => sin_data_int <= to_unsigned(59919,16); cos_data_int <= to_unsigned(20017,16);
     when "110100011111" => sin_data_int <= to_unsigned(59899,16); cos_data_int <= to_unsigned(19975,16);
     when "110100100000" => sin_data_int <= to_unsigned(59880,16); cos_data_int <= to_unsigned(19933,16);
     when "110100100001" => sin_data_int <= to_unsigned(59860,16); cos_data_int <= to_unsigned(19892,16);
     when "110100100010" => sin_data_int <= to_unsigned(59840,16); cos_data_int <= to_unsigned(19850,16);
     when "110100100011" => sin_data_int <= to_unsigned(59820,16); cos_data_int <= to_unsigned(19809,16);
     when "110100100100" => sin_data_int <= to_unsigned(59800,16); cos_data_int <= to_unsigned(19767,16);
     when "110100100101" => sin_data_int <= to_unsigned(59781,16); cos_data_int <= to_unsigned(19726,16);
     when "110100100110" => sin_data_int <= to_unsigned(59760,16); cos_data_int <= to_unsigned(19684,16);
     when "110100100111" => sin_data_int <= to_unsigned(59740,16); cos_data_int <= to_unsigned(19643,16);
     when "110100101000" => sin_data_int <= to_unsigned(59720,16); cos_data_int <= to_unsigned(19602,16);
     when "110100101001" => sin_data_int <= to_unsigned(59700,16); cos_data_int <= to_unsigned(19560,16);
     when "110100101010" => sin_data_int <= to_unsigned(59680,16); cos_data_int <= to_unsigned(19519,16);
     when "110100101011" => sin_data_int <= to_unsigned(59659,16); cos_data_int <= to_unsigned(19478,16);
     when "110100101100" => sin_data_int <= to_unsigned(59639,16); cos_data_int <= to_unsigned(19436,16);
     when "110100101101" => sin_data_int <= to_unsigned(59619,16); cos_data_int <= to_unsigned(19395,16);
     when "110100101110" => sin_data_int <= to_unsigned(59598,16); cos_data_int <= to_unsigned(19354,16);
     when "110100101111" => sin_data_int <= to_unsigned(59577,16); cos_data_int <= to_unsigned(19313,16);
     when "110100110000" => sin_data_int <= to_unsigned(59557,16); cos_data_int <= to_unsigned(19272,16);
     when "110100110001" => sin_data_int <= to_unsigned(59536,16); cos_data_int <= to_unsigned(19231,16);
     when "110100110010" => sin_data_int <= to_unsigned(59515,16); cos_data_int <= to_unsigned(19190,16);
     when "110100110011" => sin_data_int <= to_unsigned(59494,16); cos_data_int <= to_unsigned(19148,16);
     when "110100110100" => sin_data_int <= to_unsigned(59473,16); cos_data_int <= to_unsigned(19107,16);
     when "110100110101" => sin_data_int <= to_unsigned(59452,16); cos_data_int <= to_unsigned(19067,16);
     when "110100110110" => sin_data_int <= to_unsigned(59431,16); cos_data_int <= to_unsigned(19026,16);
     when "110100110111" => sin_data_int <= to_unsigned(59410,16); cos_data_int <= to_unsigned(18985,16);
     when "110100111000" => sin_data_int <= to_unsigned(59389,16); cos_data_int <= to_unsigned(18944,16);
     when "110100111001" => sin_data_int <= to_unsigned(59368,16); cos_data_int <= to_unsigned(18903,16);
     when "110100111010" => sin_data_int <= to_unsigned(59347,16); cos_data_int <= to_unsigned(18862,16);
     when "110100111011" => sin_data_int <= to_unsigned(59325,16); cos_data_int <= to_unsigned(18821,16);
     when "110100111100" => sin_data_int <= to_unsigned(59304,16); cos_data_int <= to_unsigned(18781,16);
     when "110100111101" => sin_data_int <= to_unsigned(59282,16); cos_data_int <= to_unsigned(18740,16);
     when "110100111110" => sin_data_int <= to_unsigned(59261,16); cos_data_int <= to_unsigned(18699,16);
     when "110100111111" => sin_data_int <= to_unsigned(59239,16); cos_data_int <= to_unsigned(18659,16);
     when "110101000000" => sin_data_int <= to_unsigned(59218,16); cos_data_int <= to_unsigned(18618,16);
     when "110101000001" => sin_data_int <= to_unsigned(59196,16); cos_data_int <= to_unsigned(18578,16);
     when "110101000010" => sin_data_int <= to_unsigned(59174,16); cos_data_int <= to_unsigned(18537,16);
     when "110101000011" => sin_data_int <= to_unsigned(59152,16); cos_data_int <= to_unsigned(18496,16);
     when "110101000100" => sin_data_int <= to_unsigned(59130,16); cos_data_int <= to_unsigned(18456,16);
     when "110101000101" => sin_data_int <= to_unsigned(59108,16); cos_data_int <= to_unsigned(18416,16);
     when "110101000110" => sin_data_int <= to_unsigned(59086,16); cos_data_int <= to_unsigned(18375,16);
     when "110101000111" => sin_data_int <= to_unsigned(59064,16); cos_data_int <= to_unsigned(18335,16);
     when "110101001000" => sin_data_int <= to_unsigned(59042,16); cos_data_int <= to_unsigned(18294,16);
     when "110101001001" => sin_data_int <= to_unsigned(59020,16); cos_data_int <= to_unsigned(18254,16);
     when "110101001010" => sin_data_int <= to_unsigned(58998,16); cos_data_int <= to_unsigned(18214,16);
     when "110101001011" => sin_data_int <= to_unsigned(58975,16); cos_data_int <= to_unsigned(18174,16);
     when "110101001100" => sin_data_int <= to_unsigned(58953,16); cos_data_int <= to_unsigned(18133,16);
     when "110101001101" => sin_data_int <= to_unsigned(58930,16); cos_data_int <= to_unsigned(18093,16);
     when "110101001110" => sin_data_int <= to_unsigned(58908,16); cos_data_int <= to_unsigned(18053,16);
     when "110101001111" => sin_data_int <= to_unsigned(58885,16); cos_data_int <= to_unsigned(18013,16);
     when "110101010000" => sin_data_int <= to_unsigned(58863,16); cos_data_int <= to_unsigned(17973,16);
     when "110101010001" => sin_data_int <= to_unsigned(58840,16); cos_data_int <= to_unsigned(17933,16);
     when "110101010010" => sin_data_int <= to_unsigned(58817,16); cos_data_int <= to_unsigned(17893,16);
     when "110101010011" => sin_data_int <= to_unsigned(58794,16); cos_data_int <= to_unsigned(17853,16);
     when "110101010100" => sin_data_int <= to_unsigned(58771,16); cos_data_int <= to_unsigned(17813,16);
     when "110101010101" => sin_data_int <= to_unsigned(58748,16); cos_data_int <= to_unsigned(17773,16);
     when "110101010110" => sin_data_int <= to_unsigned(58725,16); cos_data_int <= to_unsigned(17733,16);
     when "110101010111" => sin_data_int <= to_unsigned(58702,16); cos_data_int <= to_unsigned(17694,16);
     when "110101011000" => sin_data_int <= to_unsigned(58679,16); cos_data_int <= to_unsigned(17654,16);
     when "110101011001" => sin_data_int <= to_unsigned(58656,16); cos_data_int <= to_unsigned(17614,16);
     when "110101011010" => sin_data_int <= to_unsigned(58633,16); cos_data_int <= to_unsigned(17574,16);
     when "110101011011" => sin_data_int <= to_unsigned(58609,16); cos_data_int <= to_unsigned(17535,16);
     when "110101011100" => sin_data_int <= to_unsigned(58586,16); cos_data_int <= to_unsigned(17495,16);
     when "110101011101" => sin_data_int <= to_unsigned(58563,16); cos_data_int <= to_unsigned(17455,16);
     when "110101011110" => sin_data_int <= to_unsigned(58539,16); cos_data_int <= to_unsigned(17416,16);
     when "110101011111" => sin_data_int <= to_unsigned(58515,16); cos_data_int <= to_unsigned(17376,16);
     when "110101100000" => sin_data_int <= to_unsigned(58492,16); cos_data_int <= to_unsigned(17337,16);
     when "110101100001" => sin_data_int <= to_unsigned(58468,16); cos_data_int <= to_unsigned(17297,16);
     when "110101100010" => sin_data_int <= to_unsigned(58444,16); cos_data_int <= to_unsigned(17258,16);
     when "110101100011" => sin_data_int <= to_unsigned(58421,16); cos_data_int <= to_unsigned(17219,16);
     when "110101100100" => sin_data_int <= to_unsigned(58397,16); cos_data_int <= to_unsigned(17179,16);
     when "110101100101" => sin_data_int <= to_unsigned(58373,16); cos_data_int <= to_unsigned(17140,16);
     when "110101100110" => sin_data_int <= to_unsigned(58349,16); cos_data_int <= to_unsigned(17101,16);
     when "110101100111" => sin_data_int <= to_unsigned(58325,16); cos_data_int <= to_unsigned(17062,16);
     when "110101101000" => sin_data_int <= to_unsigned(58301,16); cos_data_int <= to_unsigned(17022,16);
     when "110101101001" => sin_data_int <= to_unsigned(58276,16); cos_data_int <= to_unsigned(16983,16);
     when "110101101010" => sin_data_int <= to_unsigned(58252,16); cos_data_int <= to_unsigned(16944,16);
     when "110101101011" => sin_data_int <= to_unsigned(58228,16); cos_data_int <= to_unsigned(16905,16);
     when "110101101100" => sin_data_int <= to_unsigned(58204,16); cos_data_int <= to_unsigned(16866,16);
     when "110101101101" => sin_data_int <= to_unsigned(58179,16); cos_data_int <= to_unsigned(16827,16);
     when "110101101110" => sin_data_int <= to_unsigned(58155,16); cos_data_int <= to_unsigned(16788,16);
     when "110101101111" => sin_data_int <= to_unsigned(58130,16); cos_data_int <= to_unsigned(16749,16);
     when "110101110000" => sin_data_int <= to_unsigned(58106,16); cos_data_int <= to_unsigned(16710,16);
     when "110101110001" => sin_data_int <= to_unsigned(58081,16); cos_data_int <= to_unsigned(16671,16);
     when "110101110010" => sin_data_int <= to_unsigned(58056,16); cos_data_int <= to_unsigned(16632,16);
     when "110101110011" => sin_data_int <= to_unsigned(58031,16); cos_data_int <= to_unsigned(16594,16);
     when "110101110100" => sin_data_int <= to_unsigned(58007,16); cos_data_int <= to_unsigned(16555,16);
     when "110101110101" => sin_data_int <= to_unsigned(57982,16); cos_data_int <= to_unsigned(16516,16);
     when "110101110110" => sin_data_int <= to_unsigned(57957,16); cos_data_int <= to_unsigned(16477,16);
     when "110101110111" => sin_data_int <= to_unsigned(57932,16); cos_data_int <= to_unsigned(16439,16);
     when "110101111000" => sin_data_int <= to_unsigned(57907,16); cos_data_int <= to_unsigned(16400,16);
     when "110101111001" => sin_data_int <= to_unsigned(57882,16); cos_data_int <= to_unsigned(16362,16);
     when "110101111010" => sin_data_int <= to_unsigned(57856,16); cos_data_int <= to_unsigned(16323,16);
     when "110101111011" => sin_data_int <= to_unsigned(57831,16); cos_data_int <= to_unsigned(16285,16);
     when "110101111100" => sin_data_int <= to_unsigned(57806,16); cos_data_int <= to_unsigned(16246,16);
     when "110101111101" => sin_data_int <= to_unsigned(57781,16); cos_data_int <= to_unsigned(16208,16);
     when "110101111110" => sin_data_int <= to_unsigned(57755,16); cos_data_int <= to_unsigned(16169,16);
     when "110101111111" => sin_data_int <= to_unsigned(57730,16); cos_data_int <= to_unsigned(16131,16);
     when "110110000000" => sin_data_int <= to_unsigned(57704,16); cos_data_int <= to_unsigned(16093,16);
     when "110110000001" => sin_data_int <= to_unsigned(57678,16); cos_data_int <= to_unsigned(16055,16);
     when "110110000010" => sin_data_int <= to_unsigned(57653,16); cos_data_int <= to_unsigned(16016,16);
     when "110110000011" => sin_data_int <= to_unsigned(57627,16); cos_data_int <= to_unsigned(15978,16);
     when "110110000100" => sin_data_int <= to_unsigned(57601,16); cos_data_int <= to_unsigned(15940,16);
     when "110110000101" => sin_data_int <= to_unsigned(57576,16); cos_data_int <= to_unsigned(15902,16);
     when "110110000110" => sin_data_int <= to_unsigned(57550,16); cos_data_int <= to_unsigned(15864,16);
     when "110110000111" => sin_data_int <= to_unsigned(57524,16); cos_data_int <= to_unsigned(15826,16);
     when "110110001000" => sin_data_int <= to_unsigned(57498,16); cos_data_int <= to_unsigned(15788,16);
     when "110110001001" => sin_data_int <= to_unsigned(57472,16); cos_data_int <= to_unsigned(15750,16);
     when "110110001010" => sin_data_int <= to_unsigned(57445,16); cos_data_int <= to_unsigned(15712,16);
     when "110110001011" => sin_data_int <= to_unsigned(57419,16); cos_data_int <= to_unsigned(15674,16);
     when "110110001100" => sin_data_int <= to_unsigned(57393,16); cos_data_int <= to_unsigned(15637,16);
     when "110110001101" => sin_data_int <= to_unsigned(57367,16); cos_data_int <= to_unsigned(15599,16);
     when "110110001110" => sin_data_int <= to_unsigned(57340,16); cos_data_int <= to_unsigned(15561,16);
     when "110110001111" => sin_data_int <= to_unsigned(57314,16); cos_data_int <= to_unsigned(15523,16);
     when "110110010000" => sin_data_int <= to_unsigned(57288,16); cos_data_int <= to_unsigned(15486,16);
     when "110110010001" => sin_data_int <= to_unsigned(57261,16); cos_data_int <= to_unsigned(15448,16);
     when "110110010010" => sin_data_int <= to_unsigned(57234,16); cos_data_int <= to_unsigned(15411,16);
     when "110110010011" => sin_data_int <= to_unsigned(57208,16); cos_data_int <= to_unsigned(15373,16);
     when "110110010100" => sin_data_int <= to_unsigned(57181,16); cos_data_int <= to_unsigned(15336,16);
     when "110110010101" => sin_data_int <= to_unsigned(57154,16); cos_data_int <= to_unsigned(15298,16);
     when "110110010110" => sin_data_int <= to_unsigned(57128,16); cos_data_int <= to_unsigned(15261,16);
     when "110110010111" => sin_data_int <= to_unsigned(57101,16); cos_data_int <= to_unsigned(15223,16);
     when "110110011000" => sin_data_int <= to_unsigned(57074,16); cos_data_int <= to_unsigned(15186,16);
     when "110110011001" => sin_data_int <= to_unsigned(57047,16); cos_data_int <= to_unsigned(15149,16);
     when "110110011010" => sin_data_int <= to_unsigned(57020,16); cos_data_int <= to_unsigned(15112,16);
     when "110110011011" => sin_data_int <= to_unsigned(56993,16); cos_data_int <= to_unsigned(15074,16);
     when "110110011100" => sin_data_int <= to_unsigned(56965,16); cos_data_int <= to_unsigned(15037,16);
     when "110110011101" => sin_data_int <= to_unsigned(56938,16); cos_data_int <= to_unsigned(15000,16);
     when "110110011110" => sin_data_int <= to_unsigned(56911,16); cos_data_int <= to_unsigned(14963,16);
     when "110110011111" => sin_data_int <= to_unsigned(56884,16); cos_data_int <= to_unsigned(14926,16);
     when "110110100000" => sin_data_int <= to_unsigned(56856,16); cos_data_int <= to_unsigned(14889,16);
     when "110110100001" => sin_data_int <= to_unsigned(56829,16); cos_data_int <= to_unsigned(14852,16);
     when "110110100010" => sin_data_int <= to_unsigned(56801,16); cos_data_int <= to_unsigned(14815,16);
     when "110110100011" => sin_data_int <= to_unsigned(56774,16); cos_data_int <= to_unsigned(14778,16);
     when "110110100100" => sin_data_int <= to_unsigned(56746,16); cos_data_int <= to_unsigned(14742,16);
     when "110110100101" => sin_data_int <= to_unsigned(56718,16); cos_data_int <= to_unsigned(14705,16);
     when "110110100110" => sin_data_int <= to_unsigned(56691,16); cos_data_int <= to_unsigned(14668,16);
     when "110110100111" => sin_data_int <= to_unsigned(56663,16); cos_data_int <= to_unsigned(14631,16);
     when "110110101000" => sin_data_int <= to_unsigned(56635,16); cos_data_int <= to_unsigned(14595,16);
     when "110110101001" => sin_data_int <= to_unsigned(56607,16); cos_data_int <= to_unsigned(14558,16);
     when "110110101010" => sin_data_int <= to_unsigned(56579,16); cos_data_int <= to_unsigned(14522,16);
     when "110110101011" => sin_data_int <= to_unsigned(56551,16); cos_data_int <= to_unsigned(14485,16);
     when "110110101100" => sin_data_int <= to_unsigned(56523,16); cos_data_int <= to_unsigned(14449,16);
     when "110110101101" => sin_data_int <= to_unsigned(56495,16); cos_data_int <= to_unsigned(14412,16);
     when "110110101110" => sin_data_int <= to_unsigned(56467,16); cos_data_int <= to_unsigned(14376,16);
     when "110110101111" => sin_data_int <= to_unsigned(56439,16); cos_data_int <= to_unsigned(14339,16);
     when "110110110000" => sin_data_int <= to_unsigned(56410,16); cos_data_int <= to_unsigned(14303,16);
     when "110110110001" => sin_data_int <= to_unsigned(56382,16); cos_data_int <= to_unsigned(14267,16);
     when "110110110010" => sin_data_int <= to_unsigned(56354,16); cos_data_int <= to_unsigned(14231,16);
     when "110110110011" => sin_data_int <= to_unsigned(56325,16); cos_data_int <= to_unsigned(14194,16);
     when "110110110100" => sin_data_int <= to_unsigned(56297,16); cos_data_int <= to_unsigned(14158,16);
     when "110110110101" => sin_data_int <= to_unsigned(56268,16); cos_data_int <= to_unsigned(14122,16);
     when "110110110110" => sin_data_int <= to_unsigned(56240,16); cos_data_int <= to_unsigned(14086,16);
     when "110110110111" => sin_data_int <= to_unsigned(56211,16); cos_data_int <= to_unsigned(14050,16);
     when "110110111000" => sin_data_int <= to_unsigned(56182,16); cos_data_int <= to_unsigned(14014,16);
     when "110110111001" => sin_data_int <= to_unsigned(56153,16); cos_data_int <= to_unsigned(13978,16);
     when "110110111010" => sin_data_int <= to_unsigned(56124,16); cos_data_int <= to_unsigned(13942,16);
     when "110110111011" => sin_data_int <= to_unsigned(56096,16); cos_data_int <= to_unsigned(13907,16);
     when "110110111100" => sin_data_int <= to_unsigned(56067,16); cos_data_int <= to_unsigned(13871,16);
     when "110110111101" => sin_data_int <= to_unsigned(56038,16); cos_data_int <= to_unsigned(13835,16);
     when "110110111110" => sin_data_int <= to_unsigned(56009,16); cos_data_int <= to_unsigned(13799,16);
     when "110110111111" => sin_data_int <= to_unsigned(55979,16); cos_data_int <= to_unsigned(13764,16);
     when "110111000000" => sin_data_int <= to_unsigned(55950,16); cos_data_int <= to_unsigned(13728,16);
     when "110111000001" => sin_data_int <= to_unsigned(55921,16); cos_data_int <= to_unsigned(13693,16);
     when "110111000010" => sin_data_int <= to_unsigned(55892,16); cos_data_int <= to_unsigned(13657,16);
     when "110111000011" => sin_data_int <= to_unsigned(55862,16); cos_data_int <= to_unsigned(13622,16);
     when "110111000100" => sin_data_int <= to_unsigned(55833,16); cos_data_int <= to_unsigned(13586,16);
     when "110111000101" => sin_data_int <= to_unsigned(55804,16); cos_data_int <= to_unsigned(13551,16);
     when "110111000110" => sin_data_int <= to_unsigned(55774,16); cos_data_int <= to_unsigned(13516,16);
     when "110111000111" => sin_data_int <= to_unsigned(55745,16); cos_data_int <= to_unsigned(13480,16);
     when "110111001000" => sin_data_int <= to_unsigned(55715,16); cos_data_int <= to_unsigned(13445,16);
     when "110111001001" => sin_data_int <= to_unsigned(55685,16); cos_data_int <= to_unsigned(13410,16);
     when "110111001010" => sin_data_int <= to_unsigned(55656,16); cos_data_int <= to_unsigned(13375,16);
     when "110111001011" => sin_data_int <= to_unsigned(55626,16); cos_data_int <= to_unsigned(13340,16);
     when "110111001100" => sin_data_int <= to_unsigned(55596,16); cos_data_int <= to_unsigned(13305,16);
     when "110111001101" => sin_data_int <= to_unsigned(55566,16); cos_data_int <= to_unsigned(13270,16);
     when "110111001110" => sin_data_int <= to_unsigned(55536,16); cos_data_int <= to_unsigned(13235,16);
     when "110111001111" => sin_data_int <= to_unsigned(55506,16); cos_data_int <= to_unsigned(13200,16);
     when "110111010000" => sin_data_int <= to_unsigned(55476,16); cos_data_int <= to_unsigned(13165,16);
     when "110111010001" => sin_data_int <= to_unsigned(55446,16); cos_data_int <= to_unsigned(13130,16);
     when "110111010010" => sin_data_int <= to_unsigned(55416,16); cos_data_int <= to_unsigned(13095,16);
     when "110111010011" => sin_data_int <= to_unsigned(55386,16); cos_data_int <= to_unsigned(13060,16);
     when "110111010100" => sin_data_int <= to_unsigned(55356,16); cos_data_int <= to_unsigned(13026,16);
     when "110111010101" => sin_data_int <= to_unsigned(55325,16); cos_data_int <= to_unsigned(12991,16);
     when "110111010110" => sin_data_int <= to_unsigned(55295,16); cos_data_int <= to_unsigned(12957,16);
     when "110111010111" => sin_data_int <= to_unsigned(55265,16); cos_data_int <= to_unsigned(12922,16);
     when "110111011000" => sin_data_int <= to_unsigned(55234,16); cos_data_int <= to_unsigned(12888,16);
     when "110111011001" => sin_data_int <= to_unsigned(55204,16); cos_data_int <= to_unsigned(12853,16);
     when "110111011010" => sin_data_int <= to_unsigned(55173,16); cos_data_int <= to_unsigned(12819,16);
     when "110111011011" => sin_data_int <= to_unsigned(55142,16); cos_data_int <= to_unsigned(12784,16);
     when "110111011100" => sin_data_int <= to_unsigned(55112,16); cos_data_int <= to_unsigned(12750,16);
     when "110111011101" => sin_data_int <= to_unsigned(55081,16); cos_data_int <= to_unsigned(12716,16);
     when "110111011110" => sin_data_int <= to_unsigned(55050,16); cos_data_int <= to_unsigned(12682,16);
     when "110111011111" => sin_data_int <= to_unsigned(55019,16); cos_data_int <= to_unsigned(12647,16);
     when "110111100000" => sin_data_int <= to_unsigned(54989,16); cos_data_int <= to_unsigned(12613,16);
     when "110111100001" => sin_data_int <= to_unsigned(54958,16); cos_data_int <= to_unsigned(12579,16);
     when "110111100010" => sin_data_int <= to_unsigned(54927,16); cos_data_int <= to_unsigned(12545,16);
     when "110111100011" => sin_data_int <= to_unsigned(54896,16); cos_data_int <= to_unsigned(12511,16);
     when "110111100100" => sin_data_int <= to_unsigned(54864,16); cos_data_int <= to_unsigned(12477,16);
     when "110111100101" => sin_data_int <= to_unsigned(54833,16); cos_data_int <= to_unsigned(12443,16);
     when "110111100110" => sin_data_int <= to_unsigned(54802,16); cos_data_int <= to_unsigned(12409,16);
     when "110111100111" => sin_data_int <= to_unsigned(54771,16); cos_data_int <= to_unsigned(12376,16);
     when "110111101000" => sin_data_int <= to_unsigned(54740,16); cos_data_int <= to_unsigned(12342,16);
     when "110111101001" => sin_data_int <= to_unsigned(54708,16); cos_data_int <= to_unsigned(12308,16);
     when "110111101010" => sin_data_int <= to_unsigned(54677,16); cos_data_int <= to_unsigned(12275,16);
     when "110111101011" => sin_data_int <= to_unsigned(54645,16); cos_data_int <= to_unsigned(12241,16);
     when "110111101100" => sin_data_int <= to_unsigned(54614,16); cos_data_int <= to_unsigned(12207,16);
     when "110111101101" => sin_data_int <= to_unsigned(54582,16); cos_data_int <= to_unsigned(12174,16);
     when "110111101110" => sin_data_int <= to_unsigned(54551,16); cos_data_int <= to_unsigned(12141,16);
     when "110111101111" => sin_data_int <= to_unsigned(54519,16); cos_data_int <= to_unsigned(12107,16);
     when "110111110000" => sin_data_int <= to_unsigned(54487,16); cos_data_int <= to_unsigned(12074,16);
     when "110111110001" => sin_data_int <= to_unsigned(54456,16); cos_data_int <= to_unsigned(12040,16);
     when "110111110010" => sin_data_int <= to_unsigned(54424,16); cos_data_int <= to_unsigned(12007,16);
     when "110111110011" => sin_data_int <= to_unsigned(54392,16); cos_data_int <= to_unsigned(11974,16);
     when "110111110100" => sin_data_int <= to_unsigned(54360,16); cos_data_int <= to_unsigned(11941,16);
     when "110111110101" => sin_data_int <= to_unsigned(54328,16); cos_data_int <= to_unsigned(11908,16);
     when "110111110110" => sin_data_int <= to_unsigned(54296,16); cos_data_int <= to_unsigned(11875,16);
     when "110111110111" => sin_data_int <= to_unsigned(54264,16); cos_data_int <= to_unsigned(11842,16);
     when "110111111000" => sin_data_int <= to_unsigned(54232,16); cos_data_int <= to_unsigned(11809,16);
     when "110111111001" => sin_data_int <= to_unsigned(54200,16); cos_data_int <= to_unsigned(11776,16);
     when "110111111010" => sin_data_int <= to_unsigned(54168,16); cos_data_int <= to_unsigned(11743,16);
     when "110111111011" => sin_data_int <= to_unsigned(54135,16); cos_data_int <= to_unsigned(11710,16);
     when "110111111100" => sin_data_int <= to_unsigned(54103,16); cos_data_int <= to_unsigned(11677,16);
     when "110111111101" => sin_data_int <= to_unsigned(54071,16); cos_data_int <= to_unsigned(11645,16);
     when "110111111110" => sin_data_int <= to_unsigned(54038,16); cos_data_int <= to_unsigned(11612,16);
     when "110111111111" => sin_data_int <= to_unsigned(54006,16); cos_data_int <= to_unsigned(11579,16);
     when "111000000000" => sin_data_int <= to_unsigned(53973,16); cos_data_int <= to_unsigned(11547,16);
     when "111000000001" => sin_data_int <= to_unsigned(53941,16); cos_data_int <= to_unsigned(11514,16);
     when "111000000010" => sin_data_int <= to_unsigned(53908,16); cos_data_int <= to_unsigned(11482,16);
     when "111000000011" => sin_data_int <= to_unsigned(53875,16); cos_data_int <= to_unsigned(11449,16);
     when "111000000100" => sin_data_int <= to_unsigned(53843,16); cos_data_int <= to_unsigned(11417,16);
     when "111000000101" => sin_data_int <= to_unsigned(53810,16); cos_data_int <= to_unsigned(11385,16);
     when "111000000110" => sin_data_int <= to_unsigned(53777,16); cos_data_int <= to_unsigned(11352,16);
     when "111000000111" => sin_data_int <= to_unsigned(53744,16); cos_data_int <= to_unsigned(11320,16);
     when "111000001000" => sin_data_int <= to_unsigned(53711,16); cos_data_int <= to_unsigned(11288,16);
     when "111000001001" => sin_data_int <= to_unsigned(53678,16); cos_data_int <= to_unsigned(11256,16);
     when "111000001010" => sin_data_int <= to_unsigned(53645,16); cos_data_int <= to_unsigned(11224,16);
     when "111000001011" => sin_data_int <= to_unsigned(53612,16); cos_data_int <= to_unsigned(11192,16);
     when "111000001100" => sin_data_int <= to_unsigned(53579,16); cos_data_int <= to_unsigned(11160,16);
     when "111000001101" => sin_data_int <= to_unsigned(53546,16); cos_data_int <= to_unsigned(11128,16);
     when "111000001110" => sin_data_int <= to_unsigned(53513,16); cos_data_int <= to_unsigned(11096,16);
     when "111000001111" => sin_data_int <= to_unsigned(53480,16); cos_data_int <= to_unsigned(11064,16);
     when "111000010000" => sin_data_int <= to_unsigned(53446,16); cos_data_int <= to_unsigned(11033,16);
     when "111000010001" => sin_data_int <= to_unsigned(53413,16); cos_data_int <= to_unsigned(11001,16);
     when "111000010010" => sin_data_int <= to_unsigned(53379,16); cos_data_int <= to_unsigned(10969,16);
     when "111000010011" => sin_data_int <= to_unsigned(53346,16); cos_data_int <= to_unsigned(10938,16);
     when "111000010100" => sin_data_int <= to_unsigned(53313,16); cos_data_int <= to_unsigned(10906,16);
     when "111000010101" => sin_data_int <= to_unsigned(53279,16); cos_data_int <= to_unsigned(10875,16);
     when "111000010110" => sin_data_int <= to_unsigned(53245,16); cos_data_int <= to_unsigned(10843,16);
     when "111000010111" => sin_data_int <= to_unsigned(53212,16); cos_data_int <= to_unsigned(10812,16);
     when "111000011000" => sin_data_int <= to_unsigned(53178,16); cos_data_int <= to_unsigned(10780,16);
     when "111000011001" => sin_data_int <= to_unsigned(53144,16); cos_data_int <= to_unsigned(10749,16);
     when "111000011010" => sin_data_int <= to_unsigned(53111,16); cos_data_int <= to_unsigned(10718,16);
     when "111000011011" => sin_data_int <= to_unsigned(53077,16); cos_data_int <= to_unsigned(10687,16);
     when "111000011100" => sin_data_int <= to_unsigned(53043,16); cos_data_int <= to_unsigned(10656,16);
     when "111000011101" => sin_data_int <= to_unsigned(53009,16); cos_data_int <= to_unsigned(10624,16);
     when "111000011110" => sin_data_int <= to_unsigned(52975,16); cos_data_int <= to_unsigned(10593,16);
     when "111000011111" => sin_data_int <= to_unsigned(52941,16); cos_data_int <= to_unsigned(10562,16);
     when "111000100000" => sin_data_int <= to_unsigned(52907,16); cos_data_int <= to_unsigned(10531,16);
     when "111000100001" => sin_data_int <= to_unsigned(52873,16); cos_data_int <= to_unsigned(10501,16);
     when "111000100010" => sin_data_int <= to_unsigned(52838,16); cos_data_int <= to_unsigned(10470,16);
     when "111000100011" => sin_data_int <= to_unsigned(52804,16); cos_data_int <= to_unsigned(10439,16);
     when "111000100100" => sin_data_int <= to_unsigned(52770,16); cos_data_int <= to_unsigned(10408,16);
     when "111000100101" => sin_data_int <= to_unsigned(52736,16); cos_data_int <= to_unsigned(10378,16);
     when "111000100110" => sin_data_int <= to_unsigned(52701,16); cos_data_int <= to_unsigned(10347,16);
     when "111000100111" => sin_data_int <= to_unsigned(52667,16); cos_data_int <= to_unsigned(10316,16);
     when "111000101000" => sin_data_int <= to_unsigned(52632,16); cos_data_int <= to_unsigned(10286,16);
     when "111000101001" => sin_data_int <= to_unsigned(52598,16); cos_data_int <= to_unsigned(10255,16);
     when "111000101010" => sin_data_int <= to_unsigned(52563,16); cos_data_int <= to_unsigned(10225,16);
     when "111000101011" => sin_data_int <= to_unsigned(52529,16); cos_data_int <= to_unsigned(10195,16);
     when "111000101100" => sin_data_int <= to_unsigned(52494,16); cos_data_int <= to_unsigned(10164,16);
     when "111000101101" => sin_data_int <= to_unsigned(52460,16); cos_data_int <= to_unsigned(10134,16);
     when "111000101110" => sin_data_int <= to_unsigned(52425,16); cos_data_int <= to_unsigned(10104,16);
     when "111000101111" => sin_data_int <= to_unsigned(52390,16); cos_data_int <= to_unsigned(10074,16);
     when "111000110000" => sin_data_int <= to_unsigned(52355,16); cos_data_int <= to_unsigned(10044,16);
     when "111000110001" => sin_data_int <= to_unsigned(52320,16); cos_data_int <= to_unsigned(10014,16);
     when "111000110010" => sin_data_int <= to_unsigned(52285,16); cos_data_int <= to_unsigned(9984,16);
     when "111000110011" => sin_data_int <= to_unsigned(52250,16); cos_data_int <= to_unsigned(9954,16);
     when "111000110100" => sin_data_int <= to_unsigned(52215,16); cos_data_int <= to_unsigned(9924,16);
     when "111000110101" => sin_data_int <= to_unsigned(52180,16); cos_data_int <= to_unsigned(9894,16);
     when "111000110110" => sin_data_int <= to_unsigned(52145,16); cos_data_int <= to_unsigned(9864,16);
     when "111000110111" => sin_data_int <= to_unsigned(52110,16); cos_data_int <= to_unsigned(9835,16);
     when "111000111000" => sin_data_int <= to_unsigned(52075,16); cos_data_int <= to_unsigned(9805,16);
     when "111000111001" => sin_data_int <= to_unsigned(52040,16); cos_data_int <= to_unsigned(9775,16);
     when "111000111010" => sin_data_int <= to_unsigned(52004,16); cos_data_int <= to_unsigned(9746,16);
     when "111000111011" => sin_data_int <= to_unsigned(51969,16); cos_data_int <= to_unsigned(9716,16);
     when "111000111100" => sin_data_int <= to_unsigned(51934,16); cos_data_int <= to_unsigned(9687,16);
     when "111000111101" => sin_data_int <= to_unsigned(51898,16); cos_data_int <= to_unsigned(9658,16);
     when "111000111110" => sin_data_int <= to_unsigned(51863,16); cos_data_int <= to_unsigned(9628,16);
     when "111000111111" => sin_data_int <= to_unsigned(51827,16); cos_data_int <= to_unsigned(9599,16);
     when "111001000000" => sin_data_int <= to_unsigned(51792,16); cos_data_int <= to_unsigned(9570,16);
     when "111001000001" => sin_data_int <= to_unsigned(51756,16); cos_data_int <= to_unsigned(9541,16);
     when "111001000010" => sin_data_int <= to_unsigned(51721,16); cos_data_int <= to_unsigned(9511,16);
     when "111001000011" => sin_data_int <= to_unsigned(51685,16); cos_data_int <= to_unsigned(9482,16);
     when "111001000100" => sin_data_int <= to_unsigned(51649,16); cos_data_int <= to_unsigned(9453,16);
     when "111001000101" => sin_data_int <= to_unsigned(51613,16); cos_data_int <= to_unsigned(9424,16);
     when "111001000110" => sin_data_int <= to_unsigned(51578,16); cos_data_int <= to_unsigned(9396,16);
     when "111001000111" => sin_data_int <= to_unsigned(51542,16); cos_data_int <= to_unsigned(9367,16);
     when "111001001000" => sin_data_int <= to_unsigned(51506,16); cos_data_int <= to_unsigned(9338,16);
     when "111001001001" => sin_data_int <= to_unsigned(51470,16); cos_data_int <= to_unsigned(9309,16);
     when "111001001010" => sin_data_int <= to_unsigned(51434,16); cos_data_int <= to_unsigned(9280,16);
     when "111001001011" => sin_data_int <= to_unsigned(51398,16); cos_data_int <= to_unsigned(9252,16);
     when "111001001100" => sin_data_int <= to_unsigned(51362,16); cos_data_int <= to_unsigned(9223,16);
     when "111001001101" => sin_data_int <= to_unsigned(51326,16); cos_data_int <= to_unsigned(9195,16);
     when "111001001110" => sin_data_int <= to_unsigned(51289,16); cos_data_int <= to_unsigned(9166,16);
     when "111001001111" => sin_data_int <= to_unsigned(51253,16); cos_data_int <= to_unsigned(9138,16);
     when "111001010000" => sin_data_int <= to_unsigned(51217,16); cos_data_int <= to_unsigned(9110,16);
     when "111001010001" => sin_data_int <= to_unsigned(51181,16); cos_data_int <= to_unsigned(9081,16);
     when "111001010010" => sin_data_int <= to_unsigned(51144,16); cos_data_int <= to_unsigned(9053,16);
     when "111001010011" => sin_data_int <= to_unsigned(51108,16); cos_data_int <= to_unsigned(9025,16);
     when "111001010100" => sin_data_int <= to_unsigned(51071,16); cos_data_int <= to_unsigned(8997,16);
     when "111001010101" => sin_data_int <= to_unsigned(51035,16); cos_data_int <= to_unsigned(8969,16);
     when "111001010110" => sin_data_int <= to_unsigned(50998,16); cos_data_int <= to_unsigned(8941,16);
     when "111001010111" => sin_data_int <= to_unsigned(50962,16); cos_data_int <= to_unsigned(8913,16);
     when "111001011000" => sin_data_int <= to_unsigned(50925,16); cos_data_int <= to_unsigned(8885,16);
     when "111001011001" => sin_data_int <= to_unsigned(50889,16); cos_data_int <= to_unsigned(8857,16);
     when "111001011010" => sin_data_int <= to_unsigned(50852,16); cos_data_int <= to_unsigned(8829,16);
     when "111001011011" => sin_data_int <= to_unsigned(50815,16); cos_data_int <= to_unsigned(8802,16);
     when "111001011100" => sin_data_int <= to_unsigned(50778,16); cos_data_int <= to_unsigned(8774,16);
     when "111001011101" => sin_data_int <= to_unsigned(50742,16); cos_data_int <= to_unsigned(8746,16);
     when "111001011110" => sin_data_int <= to_unsigned(50705,16); cos_data_int <= to_unsigned(8719,16);
     when "111001011111" => sin_data_int <= to_unsigned(50668,16); cos_data_int <= to_unsigned(8691,16);
     when "111001100000" => sin_data_int <= to_unsigned(50631,16); cos_data_int <= to_unsigned(8664,16);
     when "111001100001" => sin_data_int <= to_unsigned(50594,16); cos_data_int <= to_unsigned(8636,16);
     when "111001100010" => sin_data_int <= to_unsigned(50557,16); cos_data_int <= to_unsigned(8609,16);
     when "111001100011" => sin_data_int <= to_unsigned(50520,16); cos_data_int <= to_unsigned(8582,16);
     when "111001100100" => sin_data_int <= to_unsigned(50483,16); cos_data_int <= to_unsigned(8555,16);
     when "111001100101" => sin_data_int <= to_unsigned(50446,16); cos_data_int <= to_unsigned(8527,16);
     when "111001100110" => sin_data_int <= to_unsigned(50408,16); cos_data_int <= to_unsigned(8500,16);
     when "111001100111" => sin_data_int <= to_unsigned(50371,16); cos_data_int <= to_unsigned(8473,16);
     when "111001101000" => sin_data_int <= to_unsigned(50334,16); cos_data_int <= to_unsigned(8446,16);
     when "111001101001" => sin_data_int <= to_unsigned(50297,16); cos_data_int <= to_unsigned(8419,16);
     when "111001101010" => sin_data_int <= to_unsigned(50259,16); cos_data_int <= to_unsigned(8392,16);
     when "111001101011" => sin_data_int <= to_unsigned(50222,16); cos_data_int <= to_unsigned(8366,16);
     when "111001101100" => sin_data_int <= to_unsigned(50184,16); cos_data_int <= to_unsigned(8339,16);
     when "111001101101" => sin_data_int <= to_unsigned(50147,16); cos_data_int <= to_unsigned(8312,16);
     when "111001101110" => sin_data_int <= to_unsigned(50109,16); cos_data_int <= to_unsigned(8286,16);
     when "111001101111" => sin_data_int <= to_unsigned(50072,16); cos_data_int <= to_unsigned(8259,16);
     when "111001110000" => sin_data_int <= to_unsigned(50034,16); cos_data_int <= to_unsigned(8232,16);
     when "111001110001" => sin_data_int <= to_unsigned(49997,16); cos_data_int <= to_unsigned(8206,16);
     when "111001110010" => sin_data_int <= to_unsigned(49959,16); cos_data_int <= to_unsigned(8180,16);
     when "111001110011" => sin_data_int <= to_unsigned(49921,16); cos_data_int <= to_unsigned(8153,16);
     when "111001110100" => sin_data_int <= to_unsigned(49883,16); cos_data_int <= to_unsigned(8127,16);
     when "111001110101" => sin_data_int <= to_unsigned(49846,16); cos_data_int <= to_unsigned(8101,16);
     when "111001110110" => sin_data_int <= to_unsigned(49808,16); cos_data_int <= to_unsigned(8075,16);
     when "111001110111" => sin_data_int <= to_unsigned(49770,16); cos_data_int <= to_unsigned(8048,16);
     when "111001111000" => sin_data_int <= to_unsigned(49732,16); cos_data_int <= to_unsigned(8022,16);
     when "111001111001" => sin_data_int <= to_unsigned(49694,16); cos_data_int <= to_unsigned(7996,16);
     when "111001111010" => sin_data_int <= to_unsigned(49656,16); cos_data_int <= to_unsigned(7970,16);
     when "111001111011" => sin_data_int <= to_unsigned(49618,16); cos_data_int <= to_unsigned(7944,16);
     when "111001111100" => sin_data_int <= to_unsigned(49580,16); cos_data_int <= to_unsigned(7919,16);
     when "111001111101" => sin_data_int <= to_unsigned(49542,16); cos_data_int <= to_unsigned(7893,16);
     when "111001111110" => sin_data_int <= to_unsigned(49504,16); cos_data_int <= to_unsigned(7867,16);
     when "111001111111" => sin_data_int <= to_unsigned(49465,16); cos_data_int <= to_unsigned(7842,16);
     when "111010000000" => sin_data_int <= to_unsigned(49427,16); cos_data_int <= to_unsigned(7816,16);
     when "111010000001" => sin_data_int <= to_unsigned(49389,16); cos_data_int <= to_unsigned(7790,16);
     when "111010000010" => sin_data_int <= to_unsigned(49351,16); cos_data_int <= to_unsigned(7765,16);
     when "111010000011" => sin_data_int <= to_unsigned(49312,16); cos_data_int <= to_unsigned(7739,16);
     when "111010000100" => sin_data_int <= to_unsigned(49274,16); cos_data_int <= to_unsigned(7714,16);
     when "111010000101" => sin_data_int <= to_unsigned(49235,16); cos_data_int <= to_unsigned(7689,16);
     when "111010000110" => sin_data_int <= to_unsigned(49197,16); cos_data_int <= to_unsigned(7664,16);
     when "111010000111" => sin_data_int <= to_unsigned(49158,16); cos_data_int <= to_unsigned(7638,16);
     when "111010001000" => sin_data_int <= to_unsigned(49120,16); cos_data_int <= to_unsigned(7613,16);
     when "111010001001" => sin_data_int <= to_unsigned(49081,16); cos_data_int <= to_unsigned(7588,16);
     when "111010001010" => sin_data_int <= to_unsigned(49043,16); cos_data_int <= to_unsigned(7563,16);
     when "111010001011" => sin_data_int <= to_unsigned(49004,16); cos_data_int <= to_unsigned(7538,16);
     when "111010001100" => sin_data_int <= to_unsigned(48965,16); cos_data_int <= to_unsigned(7513,16);
     when "111010001101" => sin_data_int <= to_unsigned(48926,16); cos_data_int <= to_unsigned(7489,16);
     when "111010001110" => sin_data_int <= to_unsigned(48888,16); cos_data_int <= to_unsigned(7464,16);
     when "111010001111" => sin_data_int <= to_unsigned(48849,16); cos_data_int <= to_unsigned(7439,16);
     when "111010010000" => sin_data_int <= to_unsigned(48810,16); cos_data_int <= to_unsigned(7414,16);
     when "111010010001" => sin_data_int <= to_unsigned(48771,16); cos_data_int <= to_unsigned(7390,16);
     when "111010010010" => sin_data_int <= to_unsigned(48732,16); cos_data_int <= to_unsigned(7365,16);
     when "111010010011" => sin_data_int <= to_unsigned(48693,16); cos_data_int <= to_unsigned(7341,16);
     when "111010010100" => sin_data_int <= to_unsigned(48654,16); cos_data_int <= to_unsigned(7316,16);
     when "111010010101" => sin_data_int <= to_unsigned(48615,16); cos_data_int <= to_unsigned(7292,16);
     when "111010010110" => sin_data_int <= to_unsigned(48576,16); cos_data_int <= to_unsigned(7268,16);
     when "111010010111" => sin_data_int <= to_unsigned(48537,16); cos_data_int <= to_unsigned(7244,16);
     when "111010011000" => sin_data_int <= to_unsigned(48498,16); cos_data_int <= to_unsigned(7219,16);
     when "111010011001" => sin_data_int <= to_unsigned(48458,16); cos_data_int <= to_unsigned(7195,16);
     when "111010011010" => sin_data_int <= to_unsigned(48419,16); cos_data_int <= to_unsigned(7171,16);
     when "111010011011" => sin_data_int <= to_unsigned(48380,16); cos_data_int <= to_unsigned(7147,16);
     when "111010011100" => sin_data_int <= to_unsigned(48341,16); cos_data_int <= to_unsigned(7123,16);
     when "111010011101" => sin_data_int <= to_unsigned(48301,16); cos_data_int <= to_unsigned(7099,16);
     when "111010011110" => sin_data_int <= to_unsigned(48262,16); cos_data_int <= to_unsigned(7076,16);
     when "111010011111" => sin_data_int <= to_unsigned(48223,16); cos_data_int <= to_unsigned(7052,16);
     when "111010100000" => sin_data_int <= to_unsigned(48183,16); cos_data_int <= to_unsigned(7028,16);
     when "111010100001" => sin_data_int <= to_unsigned(48144,16); cos_data_int <= to_unsigned(7005,16);
     when "111010100010" => sin_data_int <= to_unsigned(48104,16); cos_data_int <= to_unsigned(6981,16);
     when "111010100011" => sin_data_int <= to_unsigned(48065,16); cos_data_int <= to_unsigned(6957,16);
     when "111010100100" => sin_data_int <= to_unsigned(48025,16); cos_data_int <= to_unsigned(6934,16);
     when "111010100101" => sin_data_int <= to_unsigned(47985,16); cos_data_int <= to_unsigned(6911,16);
     when "111010100110" => sin_data_int <= to_unsigned(47946,16); cos_data_int <= to_unsigned(6887,16);
     when "111010100111" => sin_data_int <= to_unsigned(47906,16); cos_data_int <= to_unsigned(6864,16);
     when "111010101000" => sin_data_int <= to_unsigned(47866,16); cos_data_int <= to_unsigned(6841,16);
     when "111010101001" => sin_data_int <= to_unsigned(47826,16); cos_data_int <= to_unsigned(6818,16);
     when "111010101010" => sin_data_int <= to_unsigned(47787,16); cos_data_int <= to_unsigned(6795,16);
     when "111010101011" => sin_data_int <= to_unsigned(47747,16); cos_data_int <= to_unsigned(6772,16);
     when "111010101100" => sin_data_int <= to_unsigned(47707,16); cos_data_int <= to_unsigned(6749,16);
     when "111010101101" => sin_data_int <= to_unsigned(47667,16); cos_data_int <= to_unsigned(6726,16);
     when "111010101110" => sin_data_int <= to_unsigned(47627,16); cos_data_int <= to_unsigned(6703,16);
     when "111010101111" => sin_data_int <= to_unsigned(47587,16); cos_data_int <= to_unsigned(6680,16);
     when "111010110000" => sin_data_int <= to_unsigned(47547,16); cos_data_int <= to_unsigned(6657,16);
     when "111010110001" => sin_data_int <= to_unsigned(47507,16); cos_data_int <= to_unsigned(6635,16);
     when "111010110010" => sin_data_int <= to_unsigned(47467,16); cos_data_int <= to_unsigned(6612,16);
     when "111010110011" => sin_data_int <= to_unsigned(47427,16); cos_data_int <= to_unsigned(6590,16);
     when "111010110100" => sin_data_int <= to_unsigned(47387,16); cos_data_int <= to_unsigned(6567,16);
     when "111010110101" => sin_data_int <= to_unsigned(47346,16); cos_data_int <= to_unsigned(6545,16);
     when "111010110110" => sin_data_int <= to_unsigned(47306,16); cos_data_int <= to_unsigned(6522,16);
     when "111010110111" => sin_data_int <= to_unsigned(47266,16); cos_data_int <= to_unsigned(6500,16);
     when "111010111000" => sin_data_int <= to_unsigned(47226,16); cos_data_int <= to_unsigned(6478,16);
     when "111010111001" => sin_data_int <= to_unsigned(47185,16); cos_data_int <= to_unsigned(6456,16);
     when "111010111010" => sin_data_int <= to_unsigned(47145,16); cos_data_int <= to_unsigned(6434,16);
     when "111010111011" => sin_data_int <= to_unsigned(47104,16); cos_data_int <= to_unsigned(6412,16);
     when "111010111100" => sin_data_int <= to_unsigned(47064,16); cos_data_int <= to_unsigned(6390,16);
     when "111010111101" => sin_data_int <= to_unsigned(47024,16); cos_data_int <= to_unsigned(6368,16);
     when "111010111110" => sin_data_int <= to_unsigned(46983,16); cos_data_int <= to_unsigned(6346,16);
     when "111010111111" => sin_data_int <= to_unsigned(46942,16); cos_data_int <= to_unsigned(6324,16);
     when "111011000000" => sin_data_int <= to_unsigned(46902,16); cos_data_int <= to_unsigned(6302,16);
     when "111011000001" => sin_data_int <= to_unsigned(46861,16); cos_data_int <= to_unsigned(6281,16);
     when "111011000010" => sin_data_int <= to_unsigned(46821,16); cos_data_int <= to_unsigned(6259,16);
     when "111011000011" => sin_data_int <= to_unsigned(46780,16); cos_data_int <= to_unsigned(6238,16);
     when "111011000100" => sin_data_int <= to_unsigned(46739,16); cos_data_int <= to_unsigned(6216,16);
     when "111011000101" => sin_data_int <= to_unsigned(46699,16); cos_data_int <= to_unsigned(6195,16);
     when "111011000110" => sin_data_int <= to_unsigned(46658,16); cos_data_int <= to_unsigned(6173,16);
     when "111011000111" => sin_data_int <= to_unsigned(46617,16); cos_data_int <= to_unsigned(6152,16);
     when "111011001000" => sin_data_int <= to_unsigned(46576,16); cos_data_int <= to_unsigned(6131,16);
     when "111011001001" => sin_data_int <= to_unsigned(46535,16); cos_data_int <= to_unsigned(6110,16);
     when "111011001010" => sin_data_int <= to_unsigned(46494,16); cos_data_int <= to_unsigned(6089,16);
     when "111011001011" => sin_data_int <= to_unsigned(46453,16); cos_data_int <= to_unsigned(6068,16);
     when "111011001100" => sin_data_int <= to_unsigned(46413,16); cos_data_int <= to_unsigned(6047,16);
     when "111011001101" => sin_data_int <= to_unsigned(46372,16); cos_data_int <= to_unsigned(6026,16);
     when "111011001110" => sin_data_int <= to_unsigned(46330,16); cos_data_int <= to_unsigned(6005,16);
     when "111011001111" => sin_data_int <= to_unsigned(46289,16); cos_data_int <= to_unsigned(5984,16);
     when "111011010000" => sin_data_int <= to_unsigned(46248,16); cos_data_int <= to_unsigned(5963,16);
     when "111011010001" => sin_data_int <= to_unsigned(46207,16); cos_data_int <= to_unsigned(5943,16);
     when "111011010010" => sin_data_int <= to_unsigned(46166,16); cos_data_int <= to_unsigned(5922,16);
     when "111011010011" => sin_data_int <= to_unsigned(46125,16); cos_data_int <= to_unsigned(5901,16);
     when "111011010100" => sin_data_int <= to_unsigned(46084,16); cos_data_int <= to_unsigned(5881,16);
     when "111011010101" => sin_data_int <= to_unsigned(46042,16); cos_data_int <= to_unsigned(5861,16);
     when "111011010110" => sin_data_int <= to_unsigned(46001,16); cos_data_int <= to_unsigned(5840,16);
     when "111011010111" => sin_data_int <= to_unsigned(45960,16); cos_data_int <= to_unsigned(5820,16);
     when "111011011000" => sin_data_int <= to_unsigned(45918,16); cos_data_int <= to_unsigned(5800,16);
     when "111011011001" => sin_data_int <= to_unsigned(45877,16); cos_data_int <= to_unsigned(5780,16);
     when "111011011010" => sin_data_int <= to_unsigned(45836,16); cos_data_int <= to_unsigned(5760,16);
     when "111011011011" => sin_data_int <= to_unsigned(45794,16); cos_data_int <= to_unsigned(5739,16);
     when "111011011100" => sin_data_int <= to_unsigned(45753,16); cos_data_int <= to_unsigned(5720,16);
     when "111011011101" => sin_data_int <= to_unsigned(45711,16); cos_data_int <= to_unsigned(5700,16);
     when "111011011110" => sin_data_int <= to_unsigned(45670,16); cos_data_int <= to_unsigned(5680,16);
     when "111011011111" => sin_data_int <= to_unsigned(45628,16); cos_data_int <= to_unsigned(5660,16);
     when "111011100000" => sin_data_int <= to_unsigned(45587,16); cos_data_int <= to_unsigned(5640,16);
     when "111011100001" => sin_data_int <= to_unsigned(45545,16); cos_data_int <= to_unsigned(5621,16);
     when "111011100010" => sin_data_int <= to_unsigned(45503,16); cos_data_int <= to_unsigned(5601,16);
     when "111011100011" => sin_data_int <= to_unsigned(45462,16); cos_data_int <= to_unsigned(5582,16);
     when "111011100100" => sin_data_int <= to_unsigned(45420,16); cos_data_int <= to_unsigned(5562,16);
     when "111011100101" => sin_data_int <= to_unsigned(45378,16); cos_data_int <= to_unsigned(5543,16);
     when "111011100110" => sin_data_int <= to_unsigned(45337,16); cos_data_int <= to_unsigned(5523,16);
     when "111011100111" => sin_data_int <= to_unsigned(45295,16); cos_data_int <= to_unsigned(5504,16);
     when "111011101000" => sin_data_int <= to_unsigned(45253,16); cos_data_int <= to_unsigned(5485,16);
     when "111011101001" => sin_data_int <= to_unsigned(45211,16); cos_data_int <= to_unsigned(5466,16);
     when "111011101010" => sin_data_int <= to_unsigned(45169,16); cos_data_int <= to_unsigned(5447,16);
     when "111011101011" => sin_data_int <= to_unsigned(45127,16); cos_data_int <= to_unsigned(5428,16);
     when "111011101100" => sin_data_int <= to_unsigned(45085,16); cos_data_int <= to_unsigned(5409,16);
     when "111011101101" => sin_data_int <= to_unsigned(45043,16); cos_data_int <= to_unsigned(5390,16);
     when "111011101110" => sin_data_int <= to_unsigned(45001,16); cos_data_int <= to_unsigned(5371,16);
     when "111011101111" => sin_data_int <= to_unsigned(44959,16); cos_data_int <= to_unsigned(5352,16);
     when "111011110000" => sin_data_int <= to_unsigned(44917,16); cos_data_int <= to_unsigned(5334,16);
     when "111011110001" => sin_data_int <= to_unsigned(44875,16); cos_data_int <= to_unsigned(5315,16);
     when "111011110010" => sin_data_int <= to_unsigned(44833,16); cos_data_int <= to_unsigned(5297,16);
     when "111011110011" => sin_data_int <= to_unsigned(44791,16); cos_data_int <= to_unsigned(5278,16);
     when "111011110100" => sin_data_int <= to_unsigned(44749,16); cos_data_int <= to_unsigned(5260,16);
     when "111011110101" => sin_data_int <= to_unsigned(44707,16); cos_data_int <= to_unsigned(5241,16);
     when "111011110110" => sin_data_int <= to_unsigned(44664,16); cos_data_int <= to_unsigned(5223,16);
     when "111011110111" => sin_data_int <= to_unsigned(44622,16); cos_data_int <= to_unsigned(5205,16);
     when "111011111000" => sin_data_int <= to_unsigned(44580,16); cos_data_int <= to_unsigned(5187,16);
     when "111011111001" => sin_data_int <= to_unsigned(44537,16); cos_data_int <= to_unsigned(5168,16);
     when "111011111010" => sin_data_int <= to_unsigned(44495,16); cos_data_int <= to_unsigned(5150,16);
     when "111011111011" => sin_data_int <= to_unsigned(44453,16); cos_data_int <= to_unsigned(5132,16);
     when "111011111100" => sin_data_int <= to_unsigned(44410,16); cos_data_int <= to_unsigned(5115,16);
     when "111011111101" => sin_data_int <= to_unsigned(44368,16); cos_data_int <= to_unsigned(5097,16);
     when "111011111110" => sin_data_int <= to_unsigned(44325,16); cos_data_int <= to_unsigned(5079,16);
     when "111011111111" => sin_data_int <= to_unsigned(44283,16); cos_data_int <= to_unsigned(5061,16);
     when "111100000000" => sin_data_int <= to_unsigned(44241,16); cos_data_int <= to_unsigned(5044,16);
     when "111100000001" => sin_data_int <= to_unsigned(44198,16); cos_data_int <= to_unsigned(5026,16);
     when "111100000010" => sin_data_int <= to_unsigned(44155,16); cos_data_int <= to_unsigned(5009,16);
     when "111100000011" => sin_data_int <= to_unsigned(44113,16); cos_data_int <= to_unsigned(4991,16);
     when "111100000100" => sin_data_int <= to_unsigned(44070,16); cos_data_int <= to_unsigned(4974,16);
     when "111100000101" => sin_data_int <= to_unsigned(44028,16); cos_data_int <= to_unsigned(4956,16);
     when "111100000110" => sin_data_int <= to_unsigned(43985,16); cos_data_int <= to_unsigned(4939,16);
     when "111100000111" => sin_data_int <= to_unsigned(43942,16); cos_data_int <= to_unsigned(4922,16);
     when "111100001000" => sin_data_int <= to_unsigned(43900,16); cos_data_int <= to_unsigned(4905,16);
     when "111100001001" => sin_data_int <= to_unsigned(43857,16); cos_data_int <= to_unsigned(4888,16);
     when "111100001010" => sin_data_int <= to_unsigned(43814,16); cos_data_int <= to_unsigned(4871,16);
     when "111100001011" => sin_data_int <= to_unsigned(43771,16); cos_data_int <= to_unsigned(4854,16);
     when "111100001100" => sin_data_int <= to_unsigned(43728,16); cos_data_int <= to_unsigned(4837,16);
     when "111100001101" => sin_data_int <= to_unsigned(43686,16); cos_data_int <= to_unsigned(4820,16);
     when "111100001110" => sin_data_int <= to_unsigned(43643,16); cos_data_int <= to_unsigned(4803,16);
     when "111100001111" => sin_data_int <= to_unsigned(43600,16); cos_data_int <= to_unsigned(4787,16);
     when "111100010000" => sin_data_int <= to_unsigned(43557,16); cos_data_int <= to_unsigned(4770,16);
     when "111100010001" => sin_data_int <= to_unsigned(43514,16); cos_data_int <= to_unsigned(4754,16);
     when "111100010010" => sin_data_int <= to_unsigned(43471,16); cos_data_int <= to_unsigned(4737,16);
     when "111100010011" => sin_data_int <= to_unsigned(43428,16); cos_data_int <= to_unsigned(4721,16);
     when "111100010100" => sin_data_int <= to_unsigned(43385,16); cos_data_int <= to_unsigned(4704,16);
     when "111100010101" => sin_data_int <= to_unsigned(43342,16); cos_data_int <= to_unsigned(4688,16);
     when "111100010110" => sin_data_int <= to_unsigned(43299,16); cos_data_int <= to_unsigned(4672,16);
     when "111100010111" => sin_data_int <= to_unsigned(43256,16); cos_data_int <= to_unsigned(4656,16);
     when "111100011000" => sin_data_int <= to_unsigned(43213,16); cos_data_int <= to_unsigned(4640,16);
     when "111100011001" => sin_data_int <= to_unsigned(43169,16); cos_data_int <= to_unsigned(4624,16);
     when "111100011010" => sin_data_int <= to_unsigned(43126,16); cos_data_int <= to_unsigned(4608,16);
     when "111100011011" => sin_data_int <= to_unsigned(43083,16); cos_data_int <= to_unsigned(4592,16);
     when "111100011100" => sin_data_int <= to_unsigned(43040,16); cos_data_int <= to_unsigned(4576,16);
     when "111100011101" => sin_data_int <= to_unsigned(42997,16); cos_data_int <= to_unsigned(4560,16);
     when "111100011110" => sin_data_int <= to_unsigned(42953,16); cos_data_int <= to_unsigned(4545,16);
     when "111100011111" => sin_data_int <= to_unsigned(42910,16); cos_data_int <= to_unsigned(4529,16);
     when "111100100000" => sin_data_int <= to_unsigned(42867,16); cos_data_int <= to_unsigned(4514,16);
     when "111100100001" => sin_data_int <= to_unsigned(42823,16); cos_data_int <= to_unsigned(4498,16);
     when "111100100010" => sin_data_int <= to_unsigned(42780,16); cos_data_int <= to_unsigned(4483,16);
     when "111100100011" => sin_data_int <= to_unsigned(42737,16); cos_data_int <= to_unsigned(4467,16);
     when "111100100100" => sin_data_int <= to_unsigned(42693,16); cos_data_int <= to_unsigned(4452,16);
     when "111100100101" => sin_data_int <= to_unsigned(42650,16); cos_data_int <= to_unsigned(4437,16);
     when "111100100110" => sin_data_int <= to_unsigned(42606,16); cos_data_int <= to_unsigned(4422,16);
     when "111100100111" => sin_data_int <= to_unsigned(42563,16); cos_data_int <= to_unsigned(4407,16);
     when "111100101000" => sin_data_int <= to_unsigned(42519,16); cos_data_int <= to_unsigned(4392,16);
     when "111100101001" => sin_data_int <= to_unsigned(42476,16); cos_data_int <= to_unsigned(4377,16);
     when "111100101010" => sin_data_int <= to_unsigned(42432,16); cos_data_int <= to_unsigned(4362,16);
     when "111100101011" => sin_data_int <= to_unsigned(42389,16); cos_data_int <= to_unsigned(4347,16);
     when "111100101100" => sin_data_int <= to_unsigned(42345,16); cos_data_int <= to_unsigned(4332,16);
     when "111100101101" => sin_data_int <= to_unsigned(42301,16); cos_data_int <= to_unsigned(4318,16);
     when "111100101110" => sin_data_int <= to_unsigned(42258,16); cos_data_int <= to_unsigned(4303,16);
     when "111100101111" => sin_data_int <= to_unsigned(42214,16); cos_data_int <= to_unsigned(4289,16);
     when "111100110000" => sin_data_int <= to_unsigned(42170,16); cos_data_int <= to_unsigned(4274,16);
     when "111100110001" => sin_data_int <= to_unsigned(42127,16); cos_data_int <= to_unsigned(4260,16);
     when "111100110010" => sin_data_int <= to_unsigned(42083,16); cos_data_int <= to_unsigned(4245,16);
     when "111100110011" => sin_data_int <= to_unsigned(42039,16); cos_data_int <= to_unsigned(4231,16);
     when "111100110100" => sin_data_int <= to_unsigned(41995,16); cos_data_int <= to_unsigned(4217,16);
     when "111100110101" => sin_data_int <= to_unsigned(41952,16); cos_data_int <= to_unsigned(4203,16);
     when "111100110110" => sin_data_int <= to_unsigned(41908,16); cos_data_int <= to_unsigned(4189,16);
     when "111100110111" => sin_data_int <= to_unsigned(41864,16); cos_data_int <= to_unsigned(4175,16);
     when "111100111000" => sin_data_int <= to_unsigned(41820,16); cos_data_int <= to_unsigned(4161,16);
     when "111100111001" => sin_data_int <= to_unsigned(41776,16); cos_data_int <= to_unsigned(4147,16);
     when "111100111010" => sin_data_int <= to_unsigned(41732,16); cos_data_int <= to_unsigned(4133,16);
     when "111100111011" => sin_data_int <= to_unsigned(41688,16); cos_data_int <= to_unsigned(4119,16);
     when "111100111100" => sin_data_int <= to_unsigned(41645,16); cos_data_int <= to_unsigned(4106,16);
     when "111100111101" => sin_data_int <= to_unsigned(41601,16); cos_data_int <= to_unsigned(4092,16);
     when "111100111110" => sin_data_int <= to_unsigned(41557,16); cos_data_int <= to_unsigned(4079,16);
     when "111100111111" => sin_data_int <= to_unsigned(41513,16); cos_data_int <= to_unsigned(4065,16);
     when "111101000000" => sin_data_int <= to_unsigned(41469,16); cos_data_int <= to_unsigned(4052,16);
     when "111101000001" => sin_data_int <= to_unsigned(41424,16); cos_data_int <= to_unsigned(4038,16);
     when "111101000010" => sin_data_int <= to_unsigned(41380,16); cos_data_int <= to_unsigned(4025,16);
     when "111101000011" => sin_data_int <= to_unsigned(41336,16); cos_data_int <= to_unsigned(4012,16);
     when "111101000100" => sin_data_int <= to_unsigned(41292,16); cos_data_int <= to_unsigned(3999,16);
     when "111101000101" => sin_data_int <= to_unsigned(41248,16); cos_data_int <= to_unsigned(3986,16);
     when "111101000110" => sin_data_int <= to_unsigned(41204,16); cos_data_int <= to_unsigned(3973,16);
     when "111101000111" => sin_data_int <= to_unsigned(41160,16); cos_data_int <= to_unsigned(3960,16);
     when "111101001000" => sin_data_int <= to_unsigned(41116,16); cos_data_int <= to_unsigned(3947,16);
     when "111101001001" => sin_data_int <= to_unsigned(41071,16); cos_data_int <= to_unsigned(3934,16);
     when "111101001010" => sin_data_int <= to_unsigned(41027,16); cos_data_int <= to_unsigned(3922,16);
     when "111101001011" => sin_data_int <= to_unsigned(40983,16); cos_data_int <= to_unsigned(3909,16);
     when "111101001100" => sin_data_int <= to_unsigned(40939,16); cos_data_int <= to_unsigned(3896,16);
     when "111101001101" => sin_data_int <= to_unsigned(40894,16); cos_data_int <= to_unsigned(3884,16);
     when "111101001110" => sin_data_int <= to_unsigned(40850,16); cos_data_int <= to_unsigned(3871,16);
     when "111101001111" => sin_data_int <= to_unsigned(40806,16); cos_data_int <= to_unsigned(3859,16);
     when "111101010000" => sin_data_int <= to_unsigned(40761,16); cos_data_int <= to_unsigned(3847,16);
     when "111101010001" => sin_data_int <= to_unsigned(40717,16); cos_data_int <= to_unsigned(3834,16);
     when "111101010010" => sin_data_int <= to_unsigned(40673,16); cos_data_int <= to_unsigned(3822,16);
     when "111101010011" => sin_data_int <= to_unsigned(40628,16); cos_data_int <= to_unsigned(3810,16);
     when "111101010100" => sin_data_int <= to_unsigned(40584,16); cos_data_int <= to_unsigned(3798,16);
     when "111101010101" => sin_data_int <= to_unsigned(40539,16); cos_data_int <= to_unsigned(3786,16);
     when "111101010110" => sin_data_int <= to_unsigned(40495,16); cos_data_int <= to_unsigned(3774,16);
     when "111101010111" => sin_data_int <= to_unsigned(40450,16); cos_data_int <= to_unsigned(3762,16);
     when "111101011000" => sin_data_int <= to_unsigned(40406,16); cos_data_int <= to_unsigned(3751,16);
     when "111101011001" => sin_data_int <= to_unsigned(40361,16); cos_data_int <= to_unsigned(3739,16);
     when "111101011010" => sin_data_int <= to_unsigned(40317,16); cos_data_int <= to_unsigned(3727,16);
     when "111101011011" => sin_data_int <= to_unsigned(40272,16); cos_data_int <= to_unsigned(3716,16);
     when "111101011100" => sin_data_int <= to_unsigned(40228,16); cos_data_int <= to_unsigned(3704,16);
     when "111101011101" => sin_data_int <= to_unsigned(40183,16); cos_data_int <= to_unsigned(3693,16);
     when "111101011110" => sin_data_int <= to_unsigned(40139,16); cos_data_int <= to_unsigned(3682,16);
     when "111101011111" => sin_data_int <= to_unsigned(40094,16); cos_data_int <= to_unsigned(3670,16);
     when "111101100000" => sin_data_int <= to_unsigned(40049,16); cos_data_int <= to_unsigned(3659,16);
     when "111101100001" => sin_data_int <= to_unsigned(40005,16); cos_data_int <= to_unsigned(3648,16);
     when "111101100010" => sin_data_int <= to_unsigned(39960,16); cos_data_int <= to_unsigned(3637,16);
     when "111101100011" => sin_data_int <= to_unsigned(39915,16); cos_data_int <= to_unsigned(3626,16);
     when "111101100100" => sin_data_int <= to_unsigned(39871,16); cos_data_int <= to_unsigned(3615,16);
     when "111101100101" => sin_data_int <= to_unsigned(39826,16); cos_data_int <= to_unsigned(3604,16);
     when "111101100110" => sin_data_int <= to_unsigned(39781,16); cos_data_int <= to_unsigned(3593,16);
     when "111101100111" => sin_data_int <= to_unsigned(39737,16); cos_data_int <= to_unsigned(3582,16);
     when "111101101000" => sin_data_int <= to_unsigned(39692,16); cos_data_int <= to_unsigned(3572,16);
     when "111101101001" => sin_data_int <= to_unsigned(39647,16); cos_data_int <= to_unsigned(3561,16);
     when "111101101010" => sin_data_int <= to_unsigned(39602,16); cos_data_int <= to_unsigned(3551,16);
     when "111101101011" => sin_data_int <= to_unsigned(39557,16); cos_data_int <= to_unsigned(3540,16);
     when "111101101100" => sin_data_int <= to_unsigned(39513,16); cos_data_int <= to_unsigned(3530,16);
     when "111101101101" => sin_data_int <= to_unsigned(39468,16); cos_data_int <= to_unsigned(3519,16);
     when "111101101110" => sin_data_int <= to_unsigned(39423,16); cos_data_int <= to_unsigned(3509,16);
     when "111101101111" => sin_data_int <= to_unsigned(39378,16); cos_data_int <= to_unsigned(3499,16);
     when "111101110000" => sin_data_int <= to_unsigned(39333,16); cos_data_int <= to_unsigned(3489,16);
     when "111101110001" => sin_data_int <= to_unsigned(39288,16); cos_data_int <= to_unsigned(3479,16);
     when "111101110010" => sin_data_int <= to_unsigned(39243,16); cos_data_int <= to_unsigned(3469,16);
     when "111101110011" => sin_data_int <= to_unsigned(39198,16); cos_data_int <= to_unsigned(3459,16);
     when "111101110100" => sin_data_int <= to_unsigned(39153,16); cos_data_int <= to_unsigned(3449,16);
     when "111101110101" => sin_data_int <= to_unsigned(39108,16); cos_data_int <= to_unsigned(3439,16);
     when "111101110110" => sin_data_int <= to_unsigned(39063,16); cos_data_int <= to_unsigned(3430,16);
     when "111101110111" => sin_data_int <= to_unsigned(39018,16); cos_data_int <= to_unsigned(3420,16);
     when "111101111000" => sin_data_int <= to_unsigned(38973,16); cos_data_int <= to_unsigned(3410,16);
     when "111101111001" => sin_data_int <= to_unsigned(38928,16); cos_data_int <= to_unsigned(3401,16);
     when "111101111010" => sin_data_int <= to_unsigned(38883,16); cos_data_int <= to_unsigned(3392,16);
     when "111101111011" => sin_data_int <= to_unsigned(38838,16); cos_data_int <= to_unsigned(3382,16);
     when "111101111100" => sin_data_int <= to_unsigned(38793,16); cos_data_int <= to_unsigned(3373,16);
     when "111101111101" => sin_data_int <= to_unsigned(38748,16); cos_data_int <= to_unsigned(3364,16);
     when "111101111110" => sin_data_int <= to_unsigned(38703,16); cos_data_int <= to_unsigned(3355,16);
     when "111101111111" => sin_data_int <= to_unsigned(38658,16); cos_data_int <= to_unsigned(3345,16);
     when "111110000000" => sin_data_int <= to_unsigned(38613,16); cos_data_int <= to_unsigned(3336,16);
     when "111110000001" => sin_data_int <= to_unsigned(38568,16); cos_data_int <= to_unsigned(3327,16);
     when "111110000010" => sin_data_int <= to_unsigned(38522,16); cos_data_int <= to_unsigned(3319,16);
     when "111110000011" => sin_data_int <= to_unsigned(38477,16); cos_data_int <= to_unsigned(3310,16);
     when "111110000100" => sin_data_int <= to_unsigned(38432,16); cos_data_int <= to_unsigned(3301,16);
     when "111110000101" => sin_data_int <= to_unsigned(38387,16); cos_data_int <= to_unsigned(3292,16);
     when "111110000110" => sin_data_int <= to_unsigned(38342,16); cos_data_int <= to_unsigned(3284,16);
     when "111110000111" => sin_data_int <= to_unsigned(38296,16); cos_data_int <= to_unsigned(3275,16);
     when "111110001000" => sin_data_int <= to_unsigned(38251,16); cos_data_int <= to_unsigned(3267,16);
     when "111110001001" => sin_data_int <= to_unsigned(38206,16); cos_data_int <= to_unsigned(3258,16);
     when "111110001010" => sin_data_int <= to_unsigned(38161,16); cos_data_int <= to_unsigned(3250,16);
     when "111110001011" => sin_data_int <= to_unsigned(38115,16); cos_data_int <= to_unsigned(3242,16);
     when "111110001100" => sin_data_int <= to_unsigned(38070,16); cos_data_int <= to_unsigned(3234,16);
     when "111110001101" => sin_data_int <= to_unsigned(38025,16); cos_data_int <= to_unsigned(3226,16);
     when "111110001110" => sin_data_int <= to_unsigned(37980,16); cos_data_int <= to_unsigned(3218,16);
     when "111110001111" => sin_data_int <= to_unsigned(37934,16); cos_data_int <= to_unsigned(3210,16);
     when "111110010000" => sin_data_int <= to_unsigned(37889,16); cos_data_int <= to_unsigned(3202,16);
     when "111110010001" => sin_data_int <= to_unsigned(37844,16); cos_data_int <= to_unsigned(3194,16);
     when "111110010010" => sin_data_int <= to_unsigned(37798,16); cos_data_int <= to_unsigned(3186,16);
     when "111110010011" => sin_data_int <= to_unsigned(37753,16); cos_data_int <= to_unsigned(3178,16);
     when "111110010100" => sin_data_int <= to_unsigned(37707,16); cos_data_int <= to_unsigned(3171,16);
     when "111110010101" => sin_data_int <= to_unsigned(37662,16); cos_data_int <= to_unsigned(3163,16);
     when "111110010110" => sin_data_int <= to_unsigned(37617,16); cos_data_int <= to_unsigned(3156,16);
     when "111110010111" => sin_data_int <= to_unsigned(37571,16); cos_data_int <= to_unsigned(3148,16);
     when "111110011000" => sin_data_int <= to_unsigned(37526,16); cos_data_int <= to_unsigned(3141,16);
     when "111110011001" => sin_data_int <= to_unsigned(37480,16); cos_data_int <= to_unsigned(3134,16);
     when "111110011010" => sin_data_int <= to_unsigned(37435,16); cos_data_int <= to_unsigned(3126,16);
     when "111110011011" => sin_data_int <= to_unsigned(37389,16); cos_data_int <= to_unsigned(3119,16);
     when "111110011100" => sin_data_int <= to_unsigned(37344,16); cos_data_int <= to_unsigned(3112,16);
     when "111110011101" => sin_data_int <= to_unsigned(37298,16); cos_data_int <= to_unsigned(3105,16);
     when "111110011110" => sin_data_int <= to_unsigned(37253,16); cos_data_int <= to_unsigned(3098,16);
     when "111110011111" => sin_data_int <= to_unsigned(37207,16); cos_data_int <= to_unsigned(3091,16);
     when "111110100000" => sin_data_int <= to_unsigned(37162,16); cos_data_int <= to_unsigned(3085,16);
     when "111110100001" => sin_data_int <= to_unsigned(37116,16); cos_data_int <= to_unsigned(3078,16);
     when "111110100010" => sin_data_int <= to_unsigned(37071,16); cos_data_int <= to_unsigned(3071,16);
     when "111110100011" => sin_data_int <= to_unsigned(37025,16); cos_data_int <= to_unsigned(3065,16);
     when "111110100100" => sin_data_int <= to_unsigned(36980,16); cos_data_int <= to_unsigned(3058,16);
     when "111110100101" => sin_data_int <= to_unsigned(36934,16); cos_data_int <= to_unsigned(3052,16);
     when "111110100110" => sin_data_int <= to_unsigned(36889,16); cos_data_int <= to_unsigned(3045,16);
     when "111110100111" => sin_data_int <= to_unsigned(36843,16); cos_data_int <= to_unsigned(3039,16);
     when "111110101000" => sin_data_int <= to_unsigned(36797,16); cos_data_int <= to_unsigned(3033,16);
     when "111110101001" => sin_data_int <= to_unsigned(36752,16); cos_data_int <= to_unsigned(3027,16);
     when "111110101010" => sin_data_int <= to_unsigned(36706,16); cos_data_int <= to_unsigned(3021,16);
     when "111110101011" => sin_data_int <= to_unsigned(36661,16); cos_data_int <= to_unsigned(3015,16);
     when "111110101100" => sin_data_int <= to_unsigned(36615,16); cos_data_int <= to_unsigned(3009,16);
     when "111110101101" => sin_data_int <= to_unsigned(36569,16); cos_data_int <= to_unsigned(3003,16);
     when "111110101110" => sin_data_int <= to_unsigned(36524,16); cos_data_int <= to_unsigned(2997,16);
     when "111110101111" => sin_data_int <= to_unsigned(36478,16); cos_data_int <= to_unsigned(2991,16);
     when "111110110000" => sin_data_int <= to_unsigned(36432,16); cos_data_int <= to_unsigned(2986,16);
     when "111110110001" => sin_data_int <= to_unsigned(36387,16); cos_data_int <= to_unsigned(2980,16);
     when "111110110010" => sin_data_int <= to_unsigned(36341,16); cos_data_int <= to_unsigned(2974,16);
     when "111110110011" => sin_data_int <= to_unsigned(36295,16); cos_data_int <= to_unsigned(2969,16);
     when "111110110100" => sin_data_int <= to_unsigned(36250,16); cos_data_int <= to_unsigned(2964,16);
     when "111110110101" => sin_data_int <= to_unsigned(36204,16); cos_data_int <= to_unsigned(2958,16);
     when "111110110110" => sin_data_int <= to_unsigned(36158,16); cos_data_int <= to_unsigned(2953,16);
     when "111110110111" => sin_data_int <= to_unsigned(36112,16); cos_data_int <= to_unsigned(2948,16);
     when "111110111000" => sin_data_int <= to_unsigned(36067,16); cos_data_int <= to_unsigned(2943,16);
     when "111110111001" => sin_data_int <= to_unsigned(36021,16); cos_data_int <= to_unsigned(2938,16);
     when "111110111010" => sin_data_int <= to_unsigned(35975,16); cos_data_int <= to_unsigned(2933,16);
     when "111110111011" => sin_data_int <= to_unsigned(35929,16); cos_data_int <= to_unsigned(2928,16);
     when "111110111100" => sin_data_int <= to_unsigned(35884,16); cos_data_int <= to_unsigned(2923,16);
     when "111110111101" => sin_data_int <= to_unsigned(35838,16); cos_data_int <= to_unsigned(2918,16);
     when "111110111110" => sin_data_int <= to_unsigned(35792,16); cos_data_int <= to_unsigned(2914,16);
     when "111110111111" => sin_data_int <= to_unsigned(35746,16); cos_data_int <= to_unsigned(2909,16);
     when "111111000000" => sin_data_int <= to_unsigned(35701,16); cos_data_int <= to_unsigned(2904,16);
     when "111111000001" => sin_data_int <= to_unsigned(35655,16); cos_data_int <= to_unsigned(2900,16);
     when "111111000010" => sin_data_int <= to_unsigned(35609,16); cos_data_int <= to_unsigned(2896,16);
     when "111111000011" => sin_data_int <= to_unsigned(35563,16); cos_data_int <= to_unsigned(2891,16);
     when "111111000100" => sin_data_int <= to_unsigned(35517,16); cos_data_int <= to_unsigned(2887,16);
     when "111111000101" => sin_data_int <= to_unsigned(35471,16); cos_data_int <= to_unsigned(2883,16);
     when "111111000110" => sin_data_int <= to_unsigned(35426,16); cos_data_int <= to_unsigned(2879,16);
     when "111111000111" => sin_data_int <= to_unsigned(35380,16); cos_data_int <= to_unsigned(2875,16);
     when "111111001000" => sin_data_int <= to_unsigned(35334,16); cos_data_int <= to_unsigned(2871,16);
     when "111111001001" => sin_data_int <= to_unsigned(35288,16); cos_data_int <= to_unsigned(2867,16);
     when "111111001010" => sin_data_int <= to_unsigned(35242,16); cos_data_int <= to_unsigned(2863,16);
     when "111111001011" => sin_data_int <= to_unsigned(35196,16); cos_data_int <= to_unsigned(2859,16);
     when "111111001100" => sin_data_int <= to_unsigned(35150,16); cos_data_int <= to_unsigned(2855,16);
     when "111111001101" => sin_data_int <= to_unsigned(35105,16); cos_data_int <= to_unsigned(2852,16);
     when "111111001110" => sin_data_int <= to_unsigned(35059,16); cos_data_int <= to_unsigned(2848,16);
     when "111111001111" => sin_data_int <= to_unsigned(35013,16); cos_data_int <= to_unsigned(2845,16);
     when "111111010000" => sin_data_int <= to_unsigned(34967,16); cos_data_int <= to_unsigned(2841,16);
     when "111111010001" => sin_data_int <= to_unsigned(34921,16); cos_data_int <= to_unsigned(2838,16);
     when "111111010010" => sin_data_int <= to_unsigned(34875,16); cos_data_int <= to_unsigned(2835,16);
     when "111111010011" => sin_data_int <= to_unsigned(34829,16); cos_data_int <= to_unsigned(2831,16);
     when "111111010100" => sin_data_int <= to_unsigned(34783,16); cos_data_int <= to_unsigned(2828,16);
     when "111111010101" => sin_data_int <= to_unsigned(34737,16); cos_data_int <= to_unsigned(2825,16);
     when "111111010110" => sin_data_int <= to_unsigned(34691,16); cos_data_int <= to_unsigned(2822,16);
     when "111111010111" => sin_data_int <= to_unsigned(34646,16); cos_data_int <= to_unsigned(2819,16);
     when "111111011000" => sin_data_int <= to_unsigned(34600,16); cos_data_int <= to_unsigned(2816,16);
     when "111111011001" => sin_data_int <= to_unsigned(34554,16); cos_data_int <= to_unsigned(2814,16);
     when "111111011010" => sin_data_int <= to_unsigned(34508,16); cos_data_int <= to_unsigned(2811,16);
     when "111111011011" => sin_data_int <= to_unsigned(34462,16); cos_data_int <= to_unsigned(2808,16);
     when "111111011100" => sin_data_int <= to_unsigned(34416,16); cos_data_int <= to_unsigned(2806,16);
     when "111111011101" => sin_data_int <= to_unsigned(34370,16); cos_data_int <= to_unsigned(2803,16);
     when "111111011110" => sin_data_int <= to_unsigned(34324,16); cos_data_int <= to_unsigned(2801,16);
     when "111111011111" => sin_data_int <= to_unsigned(34278,16); cos_data_int <= to_unsigned(2798,16);
     when "111111100000" => sin_data_int <= to_unsigned(34232,16); cos_data_int <= to_unsigned(2796,16);
     when "111111100001" => sin_data_int <= to_unsigned(34186,16); cos_data_int <= to_unsigned(2794,16);
     when "111111100010" => sin_data_int <= to_unsigned(34140,16); cos_data_int <= to_unsigned(2792,16);
     when "111111100011" => sin_data_int <= to_unsigned(34094,16); cos_data_int <= to_unsigned(2790,16);
     when "111111100100" => sin_data_int <= to_unsigned(34048,16); cos_data_int <= to_unsigned(2788,16);
     when "111111100101" => sin_data_int <= to_unsigned(34002,16); cos_data_int <= to_unsigned(2786,16);
     when "111111100110" => sin_data_int <= to_unsigned(33956,16); cos_data_int <= to_unsigned(2784,16);
     when "111111100111" => sin_data_int <= to_unsigned(33910,16); cos_data_int <= to_unsigned(2782,16);
     when "111111101000" => sin_data_int <= to_unsigned(33864,16); cos_data_int <= to_unsigned(2780,16);
     when "111111101001" => sin_data_int <= to_unsigned(33818,16); cos_data_int <= to_unsigned(2779,16);
     when "111111101010" => sin_data_int <= to_unsigned(33772,16); cos_data_int <= to_unsigned(2777,16);
     when "111111101011" => sin_data_int <= to_unsigned(33726,16); cos_data_int <= to_unsigned(2776,16);
     when "111111101100" => sin_data_int <= to_unsigned(33680,16); cos_data_int <= to_unsigned(2774,16);
     when "111111101101" => sin_data_int <= to_unsigned(33634,16); cos_data_int <= to_unsigned(2773,16);
     when "111111101110" => sin_data_int <= to_unsigned(33588,16); cos_data_int <= to_unsigned(2771,16);
     when "111111101111" => sin_data_int <= to_unsigned(33542,16); cos_data_int <= to_unsigned(2770,16);
     when "111111110000" => sin_data_int <= to_unsigned(33496,16); cos_data_int <= to_unsigned(2769,16);
     when "111111110001" => sin_data_int <= to_unsigned(33450,16); cos_data_int <= to_unsigned(2768,16);
     when "111111110010" => sin_data_int <= to_unsigned(33404,16); cos_data_int <= to_unsigned(2767,16);
     when "111111110011" => sin_data_int <= to_unsigned(33358,16); cos_data_int <= to_unsigned(2766,16);
     when "111111110100" => sin_data_int <= to_unsigned(33312,16); cos_data_int <= to_unsigned(2765,16);
     when "111111110101" => sin_data_int <= to_unsigned(33266,16); cos_data_int <= to_unsigned(2764,16);
     when "111111110110" => sin_data_int <= to_unsigned(33220,16); cos_data_int <= to_unsigned(2764,16);
     when "111111110111" => sin_data_int <= to_unsigned(33174,16); cos_data_int <= to_unsigned(2763,16);
     when "111111111000" => sin_data_int <= to_unsigned(33128,16); cos_data_int <= to_unsigned(2762,16);
     when "111111111001" => sin_data_int <= to_unsigned(33082,16); cos_data_int <= to_unsigned(2762,16);
     when "111111111010" => sin_data_int <= to_unsigned(33036,16); cos_data_int <= to_unsigned(2761,16);
     when "111111111011" => sin_data_int <= to_unsigned(32990,16); cos_data_int <= to_unsigned(2761,16);
     when "111111111100" => sin_data_int <= to_unsigned(32944,16); cos_data_int <= to_unsigned(2761,16);
     when "111111111101" => sin_data_int <= to_unsigned(32898,16); cos_data_int <= to_unsigned(2760,16);
     when "111111111110" => sin_data_int <= to_unsigned(32852,16); cos_data_int <= to_unsigned(2760,16);
     when "111111111111" => sin_data_int <= to_unsigned(32806,16); cos_data_int <= to_unsigned(2760,16);

        when others             =>NULL;


     end case;
    theta_int <= theta_int + scaler;
  end if;
end process;


end behavioral;
